@@ -24,8 +24,10 @@ typedef struct _SPDesktopWidgetClass SPDesktopWidgetClass;
 #define SP_IS_DESKTOP_WIDGET(o) (GTK_CHECK_TYPE ((o), SP_TYPE_DESKTOP_WIDGET))
 #define SP_IS_DESKTOP_WIDGET_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_DESKTOP_WIDGET))
 
-#include <libart_lgpl/art_rect.h>
+#include <libnr/nr-rect.h>
+#include <libnr/nr-matrix.h>
 #include "helper/helper-forward.h"
+#include "helper/units.h"
 #include "forward.h"
 #include "view.h"
 
@@ -41,11 +43,13 @@ struct _SPDesktop {
 	SPSelection *selection;
 	SPEventContext *event_context;
 
+	unsigned int dkey;
+
 	SPCanvasItem *acetate;
 	SPCanvasGroup *main;
 	SPCanvasGroup *grid;
   	SPCanvasGroup *guides;
-	SPCanvasGroup *drawing;
+	SPCanvasItem *drawing;
 	SPCanvasGroup *sketch;
 	SPCanvasGroup *controls;
 	SPCanvasItem *page;
@@ -68,6 +72,13 @@ struct _SPDesktopClass {
 	void (* modified) (SPDesktop *desktop, guint flags);
 };
 
+#define SP_DESKTOP_SCROLL_LIMIT 4000.0
+#define SP_DESKTOP_ZOOM_INC 1.414213562
+#define SP_DESKTOP_ZOOM_MAX 32.0
+#define SP_DESKTOP_ZOOM_MIN 0.03125
+#define SP_DESKTOP_ZOOM(d) NR_MATRIX_DF_EXPANSION(NR_MATRIX_D_FROM_DOUBLE ((d)->d2w))
+#define SP_MOUSEMOVE_STEP 40
+
 void sp_desktop_set_active (SPDesktop *desktop, gboolean active);
 
 #ifndef __SP_DESKTOP_C__
@@ -82,38 +93,50 @@ void sp_desktop_activate_guides (SPDesktop *desktop, gboolean activate);
 
 void sp_desktop_change_document (SPDesktop *desktop, SPDocument * document);
 
-/* Zooming, viewport, position & similar */
-#define SP_DESKTOP_SCROLL_LIMIT 10000.0
-#define SP_DESKTOP_ZOOM_INC 1.414213562
-#define SP_DESKTOP_ZOOM_MAX 16.0
-#define SP_DESKTOP_ZOOM_MIN 0.0625
-
-gdouble sp_desktop_zoom_factor (SPDesktop * desktop);
-
-void sp_desktop_scroll_world (SPDesktop * desktop, gint dx, gint dy);
-ArtDRect *sp_desktop_get_visible_area (SPDesktop * desktop, ArtDRect * area);
-void sp_desktop_show_region (SPDesktop * desktop, gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint border);
-void sp_desktop_zoom_relative (SPDesktop * desktop, gdouble zoom, gdouble cx, gdouble cy);
-void sp_desktop_zoom_absolute (SPDesktop * desktop, gdouble zoom, gdouble cx, gdouble cy);
-
 /* fixme: */
 void sp_desktop_toggle_borders (GtkWidget * widget);
 
 /* Context */
-void sp_desktop_set_event_context (SPDesktop *desktop, GtkType type, const guchar *config);
+void sp_desktop_set_event_context (SPDesktop *desktop, GtkType type, const unsigned char *config);
+void sp_desktop_push_event_context (SPDesktop *desktop, GtkType type, const unsigned char *config, unsigned int key);
+void sp_desktop_pop_event_context (SPDesktop *desktop, unsigned int key);
 
 #define SP_COORDINATES_UNDERLINE_X (1 << 0)
 #define SP_COORDINATES_UNDERLINE_Y (1 << 1)
 
 void sp_desktop_set_coordinate_status (SPDesktop *desktop, gdouble x, gdouble y, guint underline);
 
+NRRectF *sp_desktop_get_display_area (SPDesktop *dt, NRRectF *area);
+
+void sp_desktop_set_display_area (SPDesktop *dt, float x0, float y0, float x1, float y1, float border);
+void sp_desktop_zoom_absolute (SPDesktop *dt, float cx, float cy, float zoom);
+void sp_desktop_zoom_relative (SPDesktop *dt, float cx, float cy, float zoom);
+void sp_desktop_zoom_page (SPDesktop *dt);
+void sp_desktop_zoom_drawing (SPDesktop *dt);
+void sp_desktop_zoom_selection (SPDesktop *dt);
+void sp_desktop_scroll_absolute_center_desktop (SPDesktop *dt, float x, float y);
+void sp_desktop_scroll_relative_canvas (SPDesktop *dt, float dx, float dy);
+
+const SPUnit *sp_desktop_get_default_unit (SPDesktop *dt);
+
+/* Pure utility */
+
+unsigned int sp_desktop_write_length (SPDesktop *dt, unsigned char *c, unsigned int length, float val);
+
+/* SPDesktopWidget */
+
 struct _SPDesktopWidget {
 	SPViewWidget viewwidget;
 
+	unsigned int update : 1;
+
+	unsigned int decorations : 1;
+	unsigned int statusbar : 1;
+
 	SPDesktop *desktop;
 
-	gint decorations : 1;
-	GtkWidget *table;
+        GtkWidget *mbtn;
+
 	GtkWidget *hscrollbar, *vscrollbar;
 
 	/* Rulers */
@@ -121,15 +144,17 @@ struct _SPDesktopWidget {
 	double dt2r;
 	double rx0, ry0;
 
-        GtkWidget *active;
-        GtkWidget *inactive;
-        GtkWidget *menubutton;   
-        GtkWidget *select_status, *coord_status;
+	GtkWidget *sticky_zoom;
+	GtkWidget *coord_status;
+        GtkWidget *select_status;
+        GtkWidget *zoom_status;
+	gulong zoom_update;
 
         gint coord_status_id, select_status_id;
-        GtkWidget *zoom;
 
 	SPCanvas *canvas;
+
+	GtkAdjustment *hadj, *vadj;
 };
 
 struct _SPDesktopWidgetClass {
