@@ -13,16 +13,23 @@
  * Copyright (C) Lauris Kaplinski 2000-2002
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <math.h>
 #include <string.h>
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-paper.h>
 
-#include "../helper/unit-menu.h"
+#include <libarikkei/arikkei-strlib.h>
+
+#include <glib.h>
+
+#include "macros.h"
+
+#include "helper/sp-intl.h"
+#include "helper/window.h"
+#include "helper/unit-menu.h"
+
 #include "../sodipodi.h"
 #include "../document.h"
 #include "../desktop.h"
@@ -37,7 +44,9 @@
  */ 
 
 static GtkWidget *dialog = NULL;
+#if 0
 static GList *papers = NULL;
+#endif
 
 static GtkWidget *sp_doc_dialog_new (void);
 static void sp_doc_dialog_activate_desktop (Sodipodi *sodipodi, SPDesktop *desktop, GtkWidget *dialog);
@@ -45,8 +54,10 @@ static void sp_doc_dialog_desactivate_desktop (Sodipodi *sodipodi, SPDesktop *de
 static void sp_doc_dialog_update (GtkWidget *dialog, SPDocument *doc);
 
 static void
-sp_doc_dialog_destroy (GtkObject *object, gpointer data)
+sp_doc_dialog_destroy (GObject *object, gpointer data)
 {
+	sp_signal_disconnect_by_data (SODIPODI, object);
+
 	dialog = NULL;
 }
 
@@ -55,15 +66,15 @@ sp_document_dialog (void)
 {
 	if (!dialog) {
 		dialog = sp_doc_dialog_new ();
-		gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-				    GTK_SIGNAL_FUNC (sp_doc_dialog_destroy), NULL);
+		g_signal_connect (G_OBJECT (dialog), "destroy", G_CALLBACK (sp_doc_dialog_destroy), NULL);
 	}
 
 	gtk_widget_show (dialog);
 }
 
+#if 0
 static void
-sp_doc_dialog_paper_selected (GtkWidget *widget, const GnomePaper *paper)
+sp_doc_dialog_paper_selected (GtkWidget *widget, const GnomePrintPaper *paper)
 {
 	GtkWidget *ww, *hw;
 
@@ -83,11 +94,11 @@ sp_doc_dialog_paper_selected (GtkWidget *widget, const GnomePaper *paper)
 		if (!pt) pt = sp_unit_get_by_abbreviation ("pt");
 		us = gtk_object_get_data (GTK_OBJECT (dialog), "units");
 		unit = sp_unit_selector_get_unit (us);
-		w = gnome_paper_pswidth (paper);
+		w = paper->width;
 		a = gtk_object_get_data (GTK_OBJECT (dialog), "width");
 		sp_convert_distance (&w, pt, unit);
 		gtk_adjustment_set_value (a, w);
-		h = gnome_paper_psheight (paper);
+		h = paper->height;
 		a = gtk_object_get_data (GTK_OBJECT (dialog), "height");
 		sp_convert_distance (&h, pt, unit);
 		gtk_adjustment_set_value (a, h);
@@ -96,6 +107,21 @@ sp_doc_dialog_paper_selected (GtkWidget *widget, const GnomePaper *paper)
 		gtk_widget_set_sensitive (hw, TRUE);
 	}
 }
+#else
+static void
+sp_doc_dialog_paper_selected (GtkWidget *widget, gpointer data)
+{
+	GtkWidget *ww, *hw;
+
+	if (gtk_object_get_data (GTK_OBJECT (dialog), "update")) return;
+
+	ww = gtk_object_get_data (GTK_OBJECT (dialog), "widthsb");
+	hw = gtk_object_get_data (GTK_OBJECT (dialog), "heightsb");
+
+	gtk_widget_set_sensitive (ww, TRUE);
+	gtk_widget_set_sensitive (hw, TRUE);
+}
+#endif
 
 static void
 sp_doc_dialog_whatever_changed (GtkAdjustment *adjustment, GtkWidget *dialog)
@@ -105,7 +131,7 @@ sp_doc_dialog_whatever_changed (GtkAdjustment *adjustment, GtkWidget *dialog)
 	SPRepr *repr;
 	SPUnitSelector *us;
 	const guchar *key;
-	guchar c[32];
+	guchar c0[32], c[32];
 
 	if (gtk_object_get_data (GTK_OBJECT (dialog), "update")) return;
 
@@ -117,8 +143,8 @@ sp_doc_dialog_whatever_changed (GtkAdjustment *adjustment, GtkWidget *dialog)
 	key = gtk_object_get_data (GTK_OBJECT (adjustment), "key");
 	us = gtk_object_get_data (GTK_OBJECT (adjustment), "unit_selector");
 
-	g_snprintf (c, 32, "%g%s", adjustment->value, sp_unit_selector_get_unit (us)->abbr);
-
+	arikkei_dtoa_simple (c0, 32, adjustment->value, 6, 0, FALSE);
+	g_snprintf (c, 32, "%s%s", c0, sp_unit_selector_get_unit (us)->abbr);
 	sp_repr_set_attr (repr, key, c);
 
 	sp_document_done (doc);
@@ -129,10 +155,8 @@ sp_doc_dialog_new (void)
 {
 	GtkWidget *dialog, *nb, *vb, *hb, *l, *om, *m, *i, *f, *t, *us, *sb;
 	GtkObject *a;
-	GList *ll;
 
-	dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Document settings"));
+	dialog = sp_window_new (_("Document settings"), FALSE);
 
 	nb = gtk_notebook_new ();
 	gtk_widget_show (nb);
@@ -164,20 +188,19 @@ sp_doc_dialog_new (void)
 
 	m = gtk_menu_new ();
 	gtk_widget_show (m);
-	if (!papers) papers = gnome_paper_name_list ();
+#if 0
+	if (!papers) papers = gnome_print_paper_get_list ();
 	for (ll = papers; ll != NULL; ll = ll->next) {
-		const GnomePaper * paper;
-		paper = gnome_paper_with_name (ll->data);
-		i = gtk_menu_item_new_with_label (gnome_paper_name (paper));
+		const GnomePrintPaper * paper = (GnomePrintPaper *)ll->data;
+		i = gtk_menu_item_new_with_label (paper->name);
 		gtk_widget_show (i);
-		gtk_signal_connect (GTK_OBJECT (i), "activate",
-				    GTK_SIGNAL_FUNC (sp_doc_dialog_paper_selected), (gpointer) paper);
+		g_signal_connect (G_OBJECT (i), "activate", G_CALLBACK (sp_doc_dialog_paper_selected), (gpointer) paper);
 		gtk_menu_append (GTK_MENU (m), i);
 	}
+#endif
 	i = gtk_menu_item_new_with_label (_("Custom"));
 	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate",
-			    GTK_SIGNAL_FUNC (sp_doc_dialog_paper_selected), NULL);
+	g_signal_connect (G_OBJECT (i), "activate", G_CALLBACK (sp_doc_dialog_paper_selected), NULL);
 	gtk_menu_prepend (GTK_MENU (m), i);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
 
@@ -215,7 +238,7 @@ sp_doc_dialog_new (void)
 	gtk_widget_show (sb);
 	gtk_table_attach (GTK_TABLE (t), sb, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_object_set_data (GTK_OBJECT (dialog), "widthsb", sb);
-	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_doc_dialog_whatever_changed), dialog);
+	g_signal_connect (G_OBJECT (a), "value_changed", G_CALLBACK (sp_doc_dialog_whatever_changed), dialog);
 
 	l = gtk_label_new (_("Height:"));
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
@@ -230,13 +253,11 @@ sp_doc_dialog_new (void)
 	gtk_widget_show (sb);
 	gtk_table_attach (GTK_TABLE (t), sb, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_object_set_data (GTK_OBJECT (dialog), "heightsb", sb);
-	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_doc_dialog_whatever_changed), dialog);
+	g_signal_connect (G_OBJECT (a), "value_changed", G_CALLBACK (sp_doc_dialog_whatever_changed), dialog);
 
 	/* fixme: We should listen namedview changes here as well */
-	gtk_signal_connect_while_alive (GTK_OBJECT (SODIPODI), "activate_desktop",
-					GTK_SIGNAL_FUNC (sp_doc_dialog_activate_desktop), dialog, GTK_OBJECT (dialog));
-	gtk_signal_connect_while_alive (GTK_OBJECT (SODIPODI), "desactivate_desktop",
-					GTK_SIGNAL_FUNC (sp_doc_dialog_desactivate_desktop), dialog, GTK_OBJECT (dialog));
+	g_signal_connect (G_OBJECT (SODIPODI), "activate_desktop", G_CALLBACK (sp_doc_dialog_activate_desktop), dialog);
+	g_signal_connect (G_OBJECT (SODIPODI), "desactivate_desktop", G_CALLBACK (sp_doc_dialog_desactivate_desktop), dialog);
 	sp_doc_dialog_update (dialog, SP_ACTIVE_DOCUMENT);
 
 	return dialog;
@@ -277,15 +298,18 @@ sp_doc_dialog_update (GtkWidget *dialog, SPDocument *doc)
 
 		pos = 1;
 
+#if 0
 		for (l = papers; l != NULL; l = l->next) {
 			gdouble pw, ph;
-			const GnomePaper *paper;
-			paper = gnome_paper_with_name ((gchar *) l->data);
-			pw = gnome_paper_pswidth (paper);
-			ph = gnome_paper_psheight (paper);
+			const GnomePrintPaper *paper = (GnomePrintPaper *)l->data;
+			pw = paper->width;
+			ph = paper->height;
 			if ((fabs (docw - pw) < 1.0) && (fabs (doch - ph) < 1.0)) break;
 			pos += 1;
 		}
+#else
+		l = NULL;
+#endif
 
 		ww = gtk_object_get_data (GTK_OBJECT (dialog), "widthsb");
 		hw = gtk_object_get_data (GTK_OBJECT (dialog), "heightsb");
