@@ -12,14 +12,14 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
+#include <math.h>
 #include <string.h>
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 #include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkframe.h>
@@ -28,8 +28,9 @@
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkhseparator.h>
-#include <libgnomeui/gnome-file-entry.h>
 
+#include "helper/sp-intl.h"
+#include "helper/window.h"
 #include "helper/unit-menu.h"
 #include "sodipodi.h"
 #include "document.h"
@@ -66,23 +67,31 @@ sp_export_dialog_destroy (GtkObject *object, gpointer data)
 }
 
 static void
-sp_export_spinbutton_new (GtkTable *t, int x, int y, const unsigned char *ll, const unsigned char *lr,
-			  GtkAdjustment *a, int step, int digits, unsigned int sensitive)
+sp_export_spinbutton_new (unsigned char *key, float val, float min, float max, float step, float page, GtkWidget *us,
+			  GtkWidget *t, int x, int y, const unsigned char *ll, const unsigned char *lr,
+			  int digits, unsigned int sensitive,
+			  GCallback cb, GtkWidget *dlg)
 {
 	GtkWidget *l, *sb;
+	GtkObject *a;
 	int pos;
 
+	a = gtk_adjustment_new (val, min, max, step, page, page);
+	gtk_object_set_data (a, "key", key);
+	gtk_object_set_data (GTK_OBJECT (dlg), key, a);
+	if (us) sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
+
 	pos = 0;
 
 	if (ll) {
 		l = gtk_label_new (ll);
 		gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-		gtk_table_attach (t, l, x + pos, x + pos + 1, y, y + 1, 0, 0, 0, 0);
+		gtk_table_attach (GTK_TABLE (t), l, x + pos, x + pos + 1, y, y + 1, 0, 0, 0, 0);
 		gtk_widget_set_sensitive (l, sensitive);
 		pos += 1;
 	}
 
-	sb = gtk_spin_button_new (a, step, digits);
+	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1.0, digits);
 	gtk_table_attach (GTK_TABLE (t), sb, x + pos, x + pos + 1, y, y + 1, 0, 0, 0, 0);
 	gtk_widget_set_usize (sb, 64, -1);
 	gtk_widget_set_sensitive (sb, sensitive);
@@ -91,10 +100,12 @@ sp_export_spinbutton_new (GtkTable *t, int x, int y, const unsigned char *ll, co
 	if (lr) {
 		l = gtk_label_new (lr);
 		gtk_misc_set_alignment (GTK_MISC (l), 0.0, 0.5);
-		gtk_table_attach (t, l, x + pos, x + pos + 1, y, y + 1, 0, 0, 0, 0);
+		gtk_table_attach (GTK_TABLE (t), l, x + pos, x + pos + 1, y, y + 1, 0, 0, 0, 0);
 		gtk_widget_set_sensitive (l, sensitive);
 		pos += 1;
 	}
+
+	if (cb) gtk_signal_connect (a, "value_changed", cb, dlg);
 }
 
 void
@@ -102,10 +113,8 @@ sp_export_dialog (void)
 {
 	if (!dlg) {
 		GtkWidget *vb, *f, *t, *hb, *us, *l, *fe, *hs, *b;
-		GtkObject *a;
 
-		dlg = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title (GTK_WINDOW (dlg), _("Export bitmap"));
+		dlg = sp_window_new (_("Export bitmap"), FALSE);
 		gtk_signal_connect (GTK_OBJECT (dlg), "destroy", GTK_SIGNAL_FUNC (sp_export_dialog_destroy), NULL);
 
 		vb = gtk_vbox_new (FALSE, 4);
@@ -147,45 +156,23 @@ sp_export_dialog (void)
 		gtk_box_pack_end (GTK_BOX (hb), l, FALSE, FALSE, 0);
 		gtk_object_set_data (GTK_OBJECT (dlg), "units", us);
 
-		a = gtk_adjustment_new (0.0, -10000.0, 10000.0, 0.1, 1.0, 1.0);
-		gtk_object_set_data (GTK_OBJECT (a), "key", "x0");
-		gtk_object_set_data (GTK_OBJECT (dlg), "x0", a);
-		sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
-		sp_export_spinbutton_new (GTK_TABLE (t), 0, 1, _("x0:"), NULL, GTK_ADJUSTMENT (a), 1.0, 2, 1);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_area_x_value_changed), dlg);
+		sp_export_spinbutton_new ("x0", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, t, 0, 1, _("x0:"), NULL, 2, 1,
+					  G_CALLBACK (sp_export_area_x_value_changed), dlg);
 
-		a = gtk_adjustment_new (0.0, -10000.0, 10000.0, 0.1, 1.0, 1.0);
-		gtk_object_set_data (GTK_OBJECT (a), "key", "x1");
-		gtk_object_set_data (GTK_OBJECT (dlg), "x1", a);
-		sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
-		sp_export_spinbutton_new (GTK_TABLE (t), 2, 1, _("x1:"), NULL, GTK_ADJUSTMENT (a), 1.0, 2, 1);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_area_x_value_changed), dlg);
+		sp_export_spinbutton_new ("x1", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, t, 2, 1, _("x1:"), NULL, 2, 1,
+					  G_CALLBACK (sp_export_area_x_value_changed), dlg);
 
-		a = gtk_adjustment_new (0.0, -10000.0, 10000.0, 0.1, 1.0, 1.0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "width", a);
-		sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
-		sp_export_spinbutton_new (GTK_TABLE (t), 4, 1, _("Width:"), NULL, GTK_ADJUSTMENT (a), 1.0, 2, 1);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_area_width_value_changed), dlg);
+		sp_export_spinbutton_new ("width", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, t, 4, 1, _("Width:"), NULL, 2, 1,
+					  G_CALLBACK (sp_export_area_width_value_changed), dlg);
 
-		a = gtk_adjustment_new (0.0, -10000.0, 10000.0, 0.1, 1.0, 1.0);
-		gtk_object_set_data (GTK_OBJECT (a), "key", "y0");
-		gtk_object_set_data (GTK_OBJECT (dlg), "y0", a);
-		sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
-		sp_export_spinbutton_new (GTK_TABLE (t), 0, 2, _("y0:"), NULL, GTK_ADJUSTMENT (a), 1.0, 2, 1);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_area_y_value_changed), dlg);
+		sp_export_spinbutton_new ("y0", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, t, 0, 2, _("y0:"), NULL, 2, 1,
+					  G_CALLBACK (sp_export_area_y_value_changed), dlg);
 
-		a = gtk_adjustment_new (0.0, -10000.0, 10000.0, 0.1, 1.0, 1.0);
-		gtk_object_set_data (GTK_OBJECT (a), "key", "y1");
-		gtk_object_set_data (GTK_OBJECT (dlg), "y1", a);
-		sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
-		sp_export_spinbutton_new (GTK_TABLE (t), 2, 2, _("y1:"), NULL, GTK_ADJUSTMENT (a), 1.0, 2, 1);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_area_y_value_changed), dlg);
+		sp_export_spinbutton_new ("y1", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, t, 2, 2, _("y1:"), NULL, 2, 1,
+					  G_CALLBACK (sp_export_area_y_value_changed), dlg);
 
-		a = gtk_adjustment_new (0.0, -10000.0, 10000.0, 0.1, 1.0, 1.0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "height", a);
-		sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
-		sp_export_spinbutton_new (GTK_TABLE (t), 4, 2, _("Height:"), NULL, GTK_ADJUSTMENT (a), 1.0, 2, 1);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_area_height_value_changed), dlg);
+		sp_export_spinbutton_new ("height", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, t, 4, 2, _("Height:"), NULL, 2, 1,
+					  G_CALLBACK (sp_export_area_height_value_changed), dlg);
 
 		gtk_widget_show_all (f);
 
@@ -198,28 +185,46 @@ sp_export_dialog (void)
 		gtk_container_set_border_width (GTK_CONTAINER (t), 4);
 		gtk_container_add (GTK_CONTAINER (f), t);
 
-		a = gtk_adjustment_new (16.0, -10000.0, 10000.0, 1, 10.0, 10.0);
-		sp_export_spinbutton_new (GTK_TABLE (t), 0, 0, _("Width:"), _("pixels"), GTK_ADJUSTMENT (a), 1.0, 0, 1);
-		gtk_object_set_data (GTK_OBJECT (dlg), "bmwidth", a);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_bitmap_width_value_changed), dlg);
+		sp_export_spinbutton_new ("bmwidth", 16.0, 16.0, 1000000.0, 1.0, 10.0, NULL, t, 0, 0,
+					  _("Width:"), _("pixels"), 0, 1,
+					  G_CALLBACK (sp_export_bitmap_width_value_changed), dlg);
 
-		a = gtk_adjustment_new (72.0, 1.0, 9600.0, 0.1, 1.0, 1.0);
-		sp_export_spinbutton_new (GTK_TABLE (t), 3, 0, NULL, _("dpi"), GTK_ADJUSTMENT (a), 1.0, 2, 1);
-		gtk_object_set_data (GTK_OBJECT (dlg), "xdpi", a);
-		gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_export_xdpi_value_changed), dlg);
+		sp_export_spinbutton_new ("xdpi", 72.0, 1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 0,
+					  NULL, _("dpi"), 2, 1,
+					  G_CALLBACK (sp_export_xdpi_value_changed), dlg);
 
-		a = gtk_adjustment_new (16.0, -10000.0, 10000.0, 1, 10.0, 10.0);
-		sp_export_spinbutton_new (GTK_TABLE (t), 0, 1, _("Height:"), _("pixels"), GTK_ADJUSTMENT (a), 1.0, 0, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "bmheight", a);
+		sp_export_spinbutton_new ("bmheight", 16.0, 16.0, 1000000.0, 1, 10.0, NULL, t, 0, 1,
+					  _("Height:"), _("pixels"), 0, 0,
+					  NULL, dlg);
 
-		a = gtk_adjustment_new (72.0, 1.0, 9600.0, 0.1, 1.0, 1.0);
-		sp_export_spinbutton_new (GTK_TABLE (t), 3, 1, NULL, _("dpi"), GTK_ADJUSTMENT (a), 1.0, 2, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "ydpi", a);
+		sp_export_spinbutton_new ("ydpi", 72.0, 1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 1,
+					  NULL, _("dpi"), 2, 0,
+					  NULL, dlg);
 
 		gtk_widget_show_all (f);
 
 		/* File entry */
+#if 0
 		fe = gnome_file_entry_new ("export", _("Export png file"));
+#else
+		/* fixme: */
+		fe = gtk_entry_new ();
+		if (SP_ACTIVE_DOCUMENT && SP_DOCUMENT_URI (SP_ACTIVE_DOCUMENT)) {
+			const unsigned char *name, *dot;
+			unsigned char c[1024];
+			int len;
+			name = SP_DOCUMENT_NAME (SP_ACTIVE_DOCUMENT);
+			len = strlen (name);
+			dot = strrchr (name, '.');
+			if (dot && (dot > name)) len = dot - name;
+			len = MIN (len, 1019);
+			memcpy (c, name, len);
+			memcpy (c + len, ".png", 4);
+			c[len + 4] = 0;
+			gtk_entry_set_text (GTK_ENTRY (fe), c);
+
+		}
+#endif
 		gtk_widget_show (fe);
 		gtk_box_pack_start (GTK_BOX (vb), fe, FALSE, FALSE, 0);
 		gtk_object_set_data (GTK_OBJECT (dlg), "filename", fe);
@@ -260,7 +265,7 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
 		gtk_object_set_data (base, "area", (gpointer) key);
 		if (SP_ACTIVE_DESKTOP) {
 			SPDocument *doc;
-			ArtDRect bbox;
+			NRRectF bbox;
 			doc = SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP);
 			if (!strcmp (key, "page")) {
 				bbox.x0 = 0.0;
@@ -282,21 +287,28 @@ sp_export_export_clicked (GtkButton *button, GtkObject *base)
 {
 	GtkWidget *fe;
 	const unsigned char *filename;
-	float x0, y0, x1, y1, width, height;
+	float x0, y0, x1, y1;
+	int width, height;
 
 	if (!SP_ACTIVE_DESKTOP) return;
 
 	fe = gtk_object_get_data (base, "filename");
+#if 0
 	filename = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (fe), FALSE);
+#else
+	filename = gtk_entry_get_text (GTK_ENTRY (fe));
+#endif
 
 	x0 = sp_export_value_get_pt (base, "x0");
 	y0 = sp_export_value_get_pt (base, "y0");
 	x1 = sp_export_value_get_pt (base, "x1");
 	y1 = sp_export_value_get_pt (base, "y1");
-	width = sp_export_value_get (base, "bmwidth");
-	height = sp_export_value_get (base, "bmheight");
+	width = (int) (sp_export_value_get (base, "bmwidth") + 0.5);
+	height = (int) (sp_export_value_get (base, "bmheight") + 0.5);
 
-	sp_export_png_file (SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP), filename, x0, y0, x1, y1, width, height, 0x00000000);
+	if ((x1 > x0) && (y1 > y0) && (width > 0) && (height > 0)) {
+		sp_export_png_file (SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP), filename, x0, y0, x1, y1, width, height, 0x00000000);
+	}
 }
 
 static void
@@ -312,7 +324,7 @@ sp_export_area_x_value_changed (GtkAdjustment *adj, GtkObject *base)
 	x1 = sp_export_value_get_pt (base, "x1");
 	xdpi = sp_export_value_get (base, "xdpi");
 
-	width = (x1 - x0) * xdpi / 72.0;
+	width = floor ((x1 - x0) * xdpi / 72.0 + 0.5);
 
 	if (width < SP_EXPORT_MIN_SIZE) {
 		const unsigned char *key;
@@ -346,7 +358,7 @@ sp_export_area_y_value_changed (GtkAdjustment *adj, GtkObject *base)
 	y1 = sp_export_value_get_pt (base, "y1");
 	ydpi = sp_export_value_get (base, "ydpi");
 
-	height = (y1 - y0) * ydpi / 72.0;
+	height = floor ((y1 - y0) * ydpi / 72.0 + 0.5);
 
 	if (height < SP_EXPORT_MIN_SIZE) {
 		const unsigned char *key;
@@ -380,7 +392,7 @@ sp_export_area_width_value_changed (GtkAdjustment *adj, GtkObject *base)
 	x1 = sp_export_value_get_pt (base, "x1");
 	xdpi = sp_export_value_get (base, "xdpi");
 	width = sp_export_value_get_pt (base, "width");
-	bmwidth = width * xdpi / 72.0;
+	bmwidth = floor (width * xdpi / 72.0 + 0.5);
 
 	if (bmwidth < SP_EXPORT_MIN_SIZE) {
 		bmwidth = SP_EXPORT_MIN_SIZE;
@@ -407,7 +419,7 @@ sp_export_area_height_value_changed (GtkAdjustment *adj, GtkObject *base)
 	y1 = sp_export_value_get_pt (base, "y1");
 	ydpi = sp_export_value_get (base, "ydpi");
 	height = sp_export_value_get_pt (base, "height");
-	bmheight = height * ydpi / 72.0;
+	bmheight = floor (height * ydpi / 72.0 + 0.5);
 
 	if (bmheight < SP_EXPORT_MIN_SIZE) {
 		bmheight = SP_EXPORT_MIN_SIZE;
