@@ -15,25 +15,29 @@
 
 #define noSP_FS_VERBOSE
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <string.h>
+
+#include <libarikkei/arikkei-strlib.h>
+
 #include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtkmisc.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkmenuitem.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkmenuitem.h>
 
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-
 #include <libnr/nr-values.h>
 
-#include <libart_lgpl/art_svp.h>
-#include <libart_lgpl/art_svp_wind.h>
-
-#include "../helper/art-utils.h"
+#include "../helper/sp-intl.h"
+#include "../helper/window.h"
 #include "../svg/svg.h"
 #include "../widgets/sp-widget.h"
 #include "../widgets/paint-selector.h"
@@ -42,6 +46,7 @@
 #include "../document.h"
 #include "../desktop-handles.h"
 #include "../selection.h"
+#include "../sp-item.h"
 #include "../sp-gradient.h"
 #include "../sodipodi.h"
 
@@ -78,16 +83,15 @@ sp_fill_style_dialog (void)
 	if (!dialog) {
 		GtkWidget *fs;
 
-		dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Fill style"));
-		gtk_signal_connect (GTK_OBJECT (dialog), "destroy", GTK_SIGNAL_FUNC (sp_fill_style_dialog_destroy), NULL);
+		dialog = sp_window_new (_("Fill style"), TRUE);
+		g_signal_connect (G_OBJECT (dialog), "destroy", G_CALLBACK (sp_fill_style_dialog_destroy), NULL);
 
 		fs = sp_fill_style_widget_new ();
 		gtk_widget_show (fs);
 		gtk_container_add (GTK_CONTAINER (dialog), fs);
-	}
 
-	gtk_widget_show (dialog);
+		gtk_widget_show (dialog);
+	}
 }
 
 GtkWidget *
@@ -104,11 +108,11 @@ sp_fill_style_widget_new (void)
 	psel = sp_paint_selector_new ();
 	gtk_widget_show (psel);
 	gtk_box_pack_start (GTK_BOX (vb), psel, TRUE, TRUE, 0);
-	gtk_object_set_data (GTK_OBJECT (spw), "paint-selector", psel);
+	g_object_set_data (G_OBJECT (spw), "paint-selector", psel);
 
-	gtk_signal_connect (GTK_OBJECT (psel), "mode_changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_mode_changed), spw);
-	gtk_signal_connect (GTK_OBJECT (psel), "dragged", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_dragged), spw);
-	gtk_signal_connect (GTK_OBJECT (psel), "changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_changed), spw);
+	g_signal_connect (G_OBJECT (psel), "mode_changed", G_CALLBACK (sp_fill_style_widget_paint_mode_changed), spw);
+	g_signal_connect (G_OBJECT (psel), "dragged", G_CALLBACK (sp_fill_style_widget_paint_dragged), spw);
+	g_signal_connect (G_OBJECT (psel), "changed", G_CALLBACK (sp_fill_style_widget_paint_changed), spw);
 
 	hb = gtk_hbox_new (FALSE, 4);
 	gtk_widget_show (hb);
@@ -122,7 +126,7 @@ sp_fill_style_widget_new (void)
 	om = gtk_option_menu_new ();
 	gtk_widget_show (om);
 	gtk_box_pack_start (GTK_BOX (hb), om, FALSE, FALSE, 0);
-	gtk_object_set_data (GTK_OBJECT (spw), "fill-rule", om);
+	g_object_set_data (G_OBJECT (spw), "fill-rule", om);
 
 	/* 0 - nonzero 1 - evenodd */
 	m = gtk_menu_new ();
@@ -131,26 +135,43 @@ sp_fill_style_widget_new (void)
 	mi = gtk_menu_item_new_with_label (_("nonzero"));
 	gtk_widget_show (mi);
 	gtk_menu_append (GTK_MENU (m), mi);
-	gtk_object_set_data (GTK_OBJECT (mi), "fill-rule", "nonzero");
-	gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_fill_style_widget_fill_rule_activate), spw);
+	g_object_set_data (G_OBJECT (mi), "fill-rule", "nonzero");
+	g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (sp_fill_style_widget_fill_rule_activate), spw);
 	mi = gtk_menu_item_new_with_label (_("evenodd"));
 	gtk_widget_show (mi);
 	gtk_menu_append (GTK_MENU (m), mi);
-	gtk_object_set_data (GTK_OBJECT (mi), "fill-rule", "evenodd");
-	gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_fill_style_widget_fill_rule_activate), spw);
+	g_object_set_data (G_OBJECT (mi), "fill-rule", "evenodd");
+	g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (sp_fill_style_widget_fill_rule_activate), spw);
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
 
-	gtk_signal_connect (GTK_OBJECT (spw), "construct", GTK_SIGNAL_FUNC (sp_fill_style_widget_construct), psel);
-	gtk_signal_connect (GTK_OBJECT (spw), "modify_selection", GTK_SIGNAL_FUNC (sp_fill_style_widget_modify_selection), psel);
-	gtk_signal_connect (GTK_OBJECT (spw), "change_selection", GTK_SIGNAL_FUNC (sp_fill_style_widget_change_selection), psel);
-	gtk_signal_connect (GTK_OBJECT (spw), "attr_changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_attr_changed), psel);
+	g_signal_connect (G_OBJECT (spw), "construct", G_CALLBACK (sp_fill_style_widget_construct), psel);
+	g_signal_connect (G_OBJECT (spw), "modify_selection", G_CALLBACK (sp_fill_style_widget_modify_selection), psel);
+	g_signal_connect (G_OBJECT (spw), "change_selection", G_CALLBACK (sp_fill_style_widget_change_selection), psel);
+	g_signal_connect (G_OBJECT (spw), "attr_changed", G_CALLBACK (sp_fill_style_widget_attr_changed), psel);
 
 	sp_fill_style_widget_update (SP_WIDGET (spw), SP_ACTIVE_DESKTOP ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL);
 
 	return spw;
 }
 
+void
+sp_fill_style_widget_system_color_set (GtkWidget *widget, SPColor *color, float opacity)
+{
+	SPPaintSelector *psel;
+
+	psel = g_object_get_data (G_OBJECT (widget), "paint-selector");
+
+	switch (psel->mode) {
+	case SP_PAINT_SELECTOR_MODE_COLOR_RGB:
+	case SP_PAINT_SELECTOR_MODE_COLOR_CMYK:
+		sp_paint_selector_system_color_set (psel, color, opacity);
+		break;
+	default:
+		break;
+	}
+}
+
 static void
 sp_fill_style_widget_construct (SPWidget *spw, SPPaintSelector *psel)
 {
@@ -197,26 +218,24 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 	SPObject *object;
 	SPGradient *vector;
 	gfloat c[5];
-	ArtDRect bbox;
 	SPLinearGradient *lg;
 	SPRadialGradient *rg;
-	gdouble ctm[6];
 #if 0
 	NRPointF p0, p1, p2;
 #endif
 	NRMatrixF fctm, gs2d;
 	NRRectF fbb;
 
-	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+	if (g_object_get_data (G_OBJECT (spw), "update")) return;
 
-	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
 
-	psel = gtk_object_get_data (GTK_OBJECT (spw), "paint-selector");
+	psel = g_object_get_data (G_OBJECT (spw), "paint-selector");
 
 	if (!sel || sp_selection_is_empty (sel)) {
 		/* No objects, set empty */
 		sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_EMPTY);
-		gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+		g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 		return;
 	}
 
@@ -230,7 +249,7 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 		if (nextmode != pselmode) {
 			/* Multiple styles */
 			sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
-			gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+			g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 			return;
 		}
 	}
@@ -262,29 +281,19 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 			if (sp_gradient_get_vector (SP_GRADIENT (SP_OBJECT_STYLE_FILL_SERVER (next)), FALSE) != vector) {
 				/* Multiple vectors */
 				sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
-				gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+				g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 				return;
 			}
 		}
 		/* fixme: Probably we should set multiple mode here too */
 		sp_paint_selector_set_gradient_linear (psel, vector);
-		sp_selection_bbox_document (sel, &bbox);
-		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
+		sp_selection_bbox_document (sel, &fbb);
+		sp_paint_selector_set_gradient_bbox (psel, fbb.x0, fbb.y0, fbb.x1, fbb.y1);
 
 		/* fixme: This is plain wrong */
 		lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object));
-		sp_item_invoke_bbox (SP_ITEM (object), &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
-		sp_item_i2doc_affine (SP_ITEM (object), ctm);
-		fctm.c[0] = ctm[0];
-		fctm.c[1] = ctm[1];
-		fctm.c[2] = ctm[2];
-		fctm.c[3] = ctm[3];
-		fctm.c[4] = ctm[4];
-		fctm.c[5] = ctm[5];
-		fbb.x0 = bbox.x0;
-		fbb.y0 = bbox.y0;
-		fbb.x1 = bbox.x1;
-		fbb.y1 = bbox.y1;
+		sp_item_invoke_bbox (SP_ITEM (object), &fbb, NULL, TRUE);
+		sp_item_i2doc_affine (SP_ITEM (object), &fctm);
 		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (lg), &fctm, &fbb, &gs2d);
 		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
 		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (lg), SP_GRADIENT_SPREAD (lg));
@@ -300,28 +309,18 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 			if (sp_gradient_get_vector (SP_GRADIENT (SP_OBJECT_STYLE_FILL_SERVER (next)), FALSE) != vector) {
 				/* Multiple vectors */
 				sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
-				gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+				g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 				return;
 			}
 		}
 		/* fixme: Probably we should set multiple mode here too */
 		sp_paint_selector_set_gradient_radial (psel, vector);
-		sp_selection_bbox_document (sel, &bbox);
-		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
+		sp_selection_bbox_document (sel, &fbb);
+		sp_paint_selector_set_gradient_bbox (psel, fbb.x0, fbb.y0, fbb.x1, fbb.y1);
 		/* fixme: This is plain wrong */
 		rg = SP_RADIALGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object));
-		sp_item_invoke_bbox (SP_ITEM (object), &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
-		sp_item_i2doc_affine (SP_ITEM (object), ctm);
-		fctm.c[0] = ctm[0];
-		fctm.c[1] = ctm[1];
-		fctm.c[2] = ctm[2];
-		fctm.c[3] = ctm[3];
-		fctm.c[4] = ctm[4];
-		fctm.c[5] = ctm[5];
-		fbb.x0 = bbox.x0;
-		fbb.y0 = bbox.y0;
-		fbb.x1 = bbox.x1;
-		fbb.y1 = bbox.y1;
+		sp_item_invoke_bbox (SP_ITEM (object), &fbb, NULL, TRUE);
+		sp_item_i2doc_affine (SP_ITEM (object), &fctm);
 		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (rg), &fctm, &fbb, &gs2d);
 		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
 		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (rg), SP_GRADIENT_SPREAD (rg));
@@ -332,10 +331,10 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 		break;
 	}
 
-	fillrule = gtk_object_get_data (GTK_OBJECT (spw), "fill-rule");
-	gtk_option_menu_set_history (GTK_OPTION_MENU (fillrule), (SP_OBJECT_STYLE (object)->fill_rule.computed == ART_WIND_RULE_NONZERO) ? 0 : 1);
+	fillrule = g_object_get_data (G_OBJECT (spw), "fill-rule");
+	gtk_option_menu_set_history (GTK_OPTION_MENU (fillrule), SP_OBJECT_STYLE (object)->fill_rule.computed);
 
-	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 }
 
 static void
@@ -347,11 +346,14 @@ sp_fill_style_widget_update_repr (SPWidget *spw, SPRepr *repr)
 	SPStyle *style;
 	gfloat c[5];
 
-	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+	if (g_object_get_data (G_OBJECT (spw), "update")) return;
 
-	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
 
-	psel = gtk_object_get_data (GTK_OBJECT (spw), "paint-selector");
+#ifdef SP_FS_VERBOSE
+	g_print ("FillStyleWidget: Set update flag\n");
+#endif
+	psel = g_object_get_data (G_OBJECT (spw), "paint-selector");
 
 	style = sp_style_new ();
 	sp_style_read_from_repr (style, repr);
@@ -383,18 +385,21 @@ sp_fill_style_widget_update_repr (SPWidget *spw, SPRepr *repr)
 		break;
 	}
 
-	fillrule = gtk_object_get_data (GTK_OBJECT (spw), "fill-rule");
-	gtk_option_menu_set_history (GTK_OPTION_MENU (fillrule), (style->fill_rule.computed == ART_WIND_RULE_NONZERO) ? 0 : 1);
+	fillrule = g_object_get_data (G_OBJECT (spw), "fill-rule");
+	gtk_option_menu_set_history (GTK_OPTION_MENU (fillrule), style->fill_rule.computed);
 
 	sp_style_unref (style);
 
-	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+#ifdef SP_FS_VERBOSE
+	g_print ("FillStyleWidget: Cleared update flag\n");
+#endif
 }
 
 static void
 sp_fill_style_widget_paint_mode_changed (SPPaintSelector *psel, SPPaintSelectorMode mode, SPWidget *spw)
 {
-	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+	if (g_object_get_data (G_OBJECT (spw), "update")) return;
 
 	/* fixme: Does this work? */
 	/* fixme: Not really, here we have to get old color back from object */
@@ -410,7 +415,8 @@ sp_fill_style_widget_paint_dragged (SPPaintSelector *psel, SPWidget *spw)
 	gfloat c[5];
 
 	if (!spw->sodipodi) return;
-	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+	if (g_object_get_data (G_OBJECT (spw), "update")) return;
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
 #ifdef SP_FS_VERBOSE
 	g_print ("FillStyleWidget: paint dragged\n");
 #endif
@@ -458,6 +464,7 @@ sp_fill_style_widget_paint_dragged (SPPaintSelector *psel, SPWidget *spw)
 		g_warning ("file %s: line %d: Paint selector should not be in mode %d", __FILE__, __LINE__, psel->mode);
 		break;
 	}
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 }
 
 static void
@@ -468,15 +475,19 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 	SPCSSAttr *css;
 	gfloat rgba[4], cmyka[5];
 	SPGradient *vector;
-	guchar b[64];
+	guchar b[64], c0[32], c1[32], c2[32], c3[32];
 
-	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+	if (g_object_get_data (G_OBJECT (spw), "update")) return;
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
 #ifdef SP_FS_VERBOSE
 	g_print ("FillStyleWidget: paint changed\n");
 #endif
 	if (spw->sodipodi) {
 		/* fixme: */
-		if (!SP_WIDGET_DOCUMENT (spw)) return;
+		if (!SP_WIDGET_DOCUMENT (spw)) {
+			g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+			return;
+		}
 		reprs = NULL;
 		items = sp_widget_get_item_list (spw);
 		for (i = items; i != NULL; i = i->next) {
@@ -497,7 +508,7 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 		sp_repr_css_set_property (css, "fill", "none");
 		for (r = reprs; r != NULL; r = r->next) {
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
-			sp_repr_set_attr_recursive ((SPRepr *) r->data, "fill-cmyk", NULL);
+			sp_repr_set_attr_recursive ((SPRepr *) r->data, "sodipodi:fill-cmyk", NULL);
 		}
 		sp_repr_css_attr_unref (css);
 		if (spw->sodipodi) sp_document_done (SP_WIDGET_DOCUMENT (spw));
@@ -507,11 +518,11 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 		sp_paint_selector_get_rgba_floatv (psel, rgba);
 		sp_svg_write_color (b, 64, SP_RGBA32_F_COMPOSE (rgba[0], rgba[1], rgba[2], 0.0));
 		sp_repr_css_set_property (css, "fill", b);
-		g_snprintf (b, 64, "%g", rgba[3]);
+		arikkei_dtoa_simple (b, 64, rgba[3], 4, 0, FALSE);
 		sp_repr_css_set_property (css, "fill-opacity", b);
 		for (r = reprs; r != NULL; r = r->next) {
+			sp_repr_set_attr_recursive ((SPRepr *) r->data, "sodipodi:fill-cmyk", NULL);
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
-			sp_repr_set_attr_recursive ((SPRepr *) r->data, "fill-cmyk", NULL);
 		}
 		sp_repr_css_attr_unref (css);
 		if (spw->sodipodi) sp_document_done (SP_WIDGET_DOCUMENT (spw));
@@ -522,12 +533,16 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 		sp_color_cmyk_to_rgb_floatv (rgba, cmyka[0], cmyka[1], cmyka[2], cmyka[3]);
 		sp_svg_write_color (b, 64, SP_RGBA32_F_COMPOSE (rgba[0], rgba[1], rgba[2], 0.0));
 		sp_repr_css_set_property (css, "fill", b);
-		g_snprintf (b, 64, "%g", cmyka[4]);
+		arikkei_dtoa_simple (b, 64, cmyka[4], 4, 0, FALSE);
 		sp_repr_css_set_property (css, "fill-opacity", b);
-		g_snprintf (b, 64, "(%g %g %g %g)", cmyka[0], cmyka[1], cmyka[2], cmyka[3]);
+		arikkei_dtoa_simple (c0, 32, cmyka[0], 4, 0, FALSE);
+		arikkei_dtoa_simple (c1, 32, cmyka[1], 4, 0, FALSE);
+		arikkei_dtoa_simple (c2, 32, cmyka[2], 4, 0, FALSE);
+		arikkei_dtoa_simple (c3, 32, cmyka[3], 4, 0, FALSE);
+		g_snprintf (b, 64, "(%s %s %s %s)", c0, c1, c2, c3);
 		for (r = reprs; r != NULL; r = r->next) {
+			sp_repr_set_attr_recursive ((SPRepr *) r->data, "sodipodi:fill-cmyk", b);
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
-			sp_repr_set_attr_recursive ((SPRepr *) r->data, "fill-cmyk", b);
 		}
 		sp_repr_css_attr_unref (css);
 		if (spw->sodipodi) sp_document_done (SP_WIDGET_DOCUMENT (spw));
@@ -580,6 +595,8 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 	}
 
 	g_slist_free (reprs);
+
+	g_object_set_data (G_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 }
 
 static void
@@ -589,7 +606,7 @@ sp_fill_style_widget_fill_rule_activate (GtkWidget *w, SPWidget *spw)
 	GSList *reprs;
 	SPCSSAttr *css;
 
-	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+	if (g_object_get_data (G_OBJECT (spw), "update")) return;
 
 	if (spw->sodipodi) {
 		reprs = NULL;
@@ -603,7 +620,7 @@ sp_fill_style_widget_fill_rule_activate (GtkWidget *w, SPWidget *spw)
 	}
 
 	css = sp_repr_css_attr_new ();
-	sp_repr_css_set_property (css, "fill-rule", gtk_object_get_data (GTK_OBJECT (w), "fill-rule"));
+	sp_repr_css_set_property (css, "fill-rule", g_object_get_data (G_OBJECT (w), "fill-rule"));
 	for (r = reprs; r != NULL; r = r->next) {
 		sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
 	}
