@@ -11,14 +11,17 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include "../sp-anchor.h"
+#include "helper/sp-intl.h"
+#include "helper/window.h"
+#include "macros.h"
+#include "sp-anchor.h"
 #include "sp-attribute-widget.h"
 #include "object-attributes.h"
 
@@ -81,11 +84,17 @@ static const SPAttrDesc rect_desc[] = {
 };
 
 static void
-object_destroyed (GtkObject *object, GtkWidget *widget)
+object_released (GtkObject *object, GtkWidget *widget)
 {
 	gtk_widget_destroy (widget);
 }
 
+static void
+window_destroyed (GtkObject *window, GtkObject *object)
+{
+	sp_signal_disconnect_by_data (object, window);
+}
+
 static void
 sp_object_attr_show_dialog (SPObject *object, const SPAttrDesc *desc, const guchar *tag)
 {
@@ -105,17 +114,16 @@ sp_object_attr_show_dialog (SPObject *object, const SPAttrDesc *desc, const guch
 		attrs[i] = desc[i].attribute;
 	}
 
-	w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	title = g_strdup_printf (_("%s attributes"), tag);
-	gtk_window_set_title (GTK_WINDOW (w), title);
+	w = sp_window_new (title, TRUE);
 	g_free (title);
 
 	t = sp_attribute_table_new (object, len, labels, attrs);
 	gtk_widget_show (t);
 	gtk_container_add (GTK_CONTAINER (w), t);
 
-	gtk_signal_connect_while_alive (GTK_OBJECT (object), "destroy",
-					GTK_SIGNAL_FUNC (object_destroyed), w, GTK_OBJECT (w));
+	g_signal_connect (G_OBJECT (w), "destroy", G_CALLBACK (window_destroyed), object);
+	g_signal_connect (G_OBJECT (object), "release", G_CALLBACK (object_released), w);
 
 	gtk_widget_show (w);
 }
