@@ -13,14 +13,13 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <glib.h>
 #include <libart_lgpl/art_affine.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 #include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkhbox.h>
@@ -29,10 +28,13 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkhseparator.h>
-#include <libgnomeui/gnome-pixmap.h>
-
-#include "../widgets/sp-widget.h"
-#include "../sodipodi.h"
+#include <gtk/gtkimage.h>
+#include "helper/sp-intl.h"
+#include "helper/window.h"
+#include "widgets/icon.h"
+#include "widgets/sp-widget.h"
+#include "macros.h"
+#include "sodipodi.h"
 #include "fill-style.h"
 #include "stroke-style.h"
 
@@ -43,6 +45,7 @@ static GtkWidget *dlg = NULL;
 static void
 sp_object_properties_dialog_destroy (GtkObject *object, gpointer data)
 {
+	sp_signal_disconnect_by_data (SODIPODI, dlg);
 	dlg = NULL;
 }
 
@@ -81,14 +84,33 @@ sp_object_properties_style_activate (GtkMenuItem *menuitem, const guchar *key)
 	}
 }
 
+static void
+sp_object_properties_color_set (Sodipodi *sodipodi, SPColor *color, double opacity, GObject *dlg)
+{
+	GtkNotebook *nb;
+	int pnum;
+
+	nb = g_object_get_data (dlg, "notebook");
+	pnum = gtk_notebook_get_current_page (nb);
+
+	if (pnum == 0) {
+		GtkWidget *fs;
+		fs = g_object_get_data (dlg, "fill");
+		sp_fill_style_widget_system_color_set (fs, color, opacity);
+	} else if (pnum == 1) {
+		GtkWidget *ss;
+		ss = g_object_get_data (dlg, "stroke-paint");
+		sp_stroke_style_paint_system_color_set (ss, color, opacity);
+	}
+}
+
 void
 sp_object_properties_dialog (void)
 {
 	if (!dlg) {
 		GtkWidget *vb, *nb, *hb, *l, *px, *page, *hs, *om, *m, *mi;
 
-		dlg = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title (GTK_WINDOW (dlg), _("Object style"));
+		dlg = sp_window_new (_("Object style"), TRUE);
 		gtk_signal_connect (GTK_OBJECT (dlg), "destroy", GTK_SIGNAL_FUNC (sp_object_properties_dialog_destroy), dlg);
 
 		vb = gtk_vbox_new (FALSE, 0);
@@ -106,7 +128,7 @@ sp_object_properties_dialog (void)
 		l = gtk_label_new (_("Fill"));
 		gtk_widget_show (l);
 		gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
-		px = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/properties_fill.xpm");
+		px = sp_icon_new (SP_ICON_SIZE_BUTTON, "properties_fill");
 		gtk_widget_show (px);
 		gtk_box_pack_start (GTK_BOX (hb), px, FALSE, FALSE, 0);
 		page = sp_fill_style_widget_new ();
@@ -120,7 +142,7 @@ sp_object_properties_dialog (void)
 		l = gtk_label_new (_("Stroke paint"));
 		gtk_widget_show (l);
 		gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
-		px = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/properties_stroke.xpm");
+		px = sp_icon_new (SP_ICON_SIZE_BUTTON, "properties_stroke");
 		gtk_widget_show (px);
 		gtk_box_pack_start (GTK_BOX (hb), px, FALSE, FALSE, 0);
 		page = sp_stroke_style_paint_widget_new ();
@@ -134,7 +156,7 @@ sp_object_properties_dialog (void)
 		l = gtk_label_new (_("Stroke style"));
 		gtk_widget_show (l);
 		gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
-		px = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/properties_stroke.xpm");
+		px = sp_icon_new (SP_ICON_SIZE_BUTTON, "properties_stroke");
 		gtk_widget_show (px);
 		gtk_box_pack_start (GTK_BOX (hb), px, FALSE, FALSE, 0);
 		page = sp_stroke_style_line_widget_new ();
@@ -160,56 +182,52 @@ sp_object_properties_dialog (void)
 		gtk_box_pack_start (GTK_BOX (hb), om, TRUE, TRUE, 0);
 
 		m = gtk_menu_new ();
-		gtk_widget_show (m);
 
 		mi = gtk_menu_item_new_with_label (_("Selected objects"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), NULL);
 		mi = gtk_menu_item_new_with_label (_("All shape tools"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.shapes");
 		mi = gtk_menu_item_new_with_label (_("Rectangle tool"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.shapes.rect");
 		mi = gtk_menu_item_new_with_label (_("Arc tool"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.shapes.arc");
 		mi = gtk_menu_item_new_with_label (_("Star tool"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.shapes.star");
 		mi = gtk_menu_item_new_with_label (_("Spiral tool"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.shapes.spiral");
 		mi = gtk_menu_item_new_with_label (_("Freehand and pen"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.freehand");
 		mi = gtk_menu_item_new_with_label (_("Calligraphic line"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.calligraphic");
 		mi = gtk_menu_item_new_with_label (_("Text"));
-		gtk_widget_show (mi);
 		gtk_menu_append (GTK_MENU (m), mi);
 		gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_object_properties_style_activate), "tools.text");
 
-		gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
-	}
+		gtk_widget_show_all (m);
 
-	gtk_widget_show (dlg);
+		gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
+
+		g_signal_connect (G_OBJECT (SODIPODI), "color_set", G_CALLBACK (sp_object_properties_color_set), dlg);
+
+		gtk_widget_show (dlg);
+	} else {
+		gtk_window_present (GTK_WINDOW (dlg));
+	}
 }
 
 void sp_object_properties_stroke (void)
 {
 	GtkWidget *nb;
 
-	if (!dlg) sp_object_properties_dialog ();
+	sp_object_properties_dialog ();
 
 	nb = gtk_object_get_data (GTK_OBJECT (dlg), "notebook");
 
@@ -220,7 +238,7 @@ void sp_object_properties_fill (void)
 {
 	GtkWidget *nb;
 
-	if (!dlg) sp_object_properties_dialog ();
+	sp_object_properties_dialog ();
 
 	nb = gtk_object_get_data (GTK_OBJECT (dlg), "notebook");
 
@@ -233,6 +251,8 @@ void sp_object_properties_fill (void)
 
 #include <math.h>
 
+#include <libnr/nr-matrix.h>
+
 #include <gtk/gtkframe.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkspinbutton.h>
@@ -252,28 +272,30 @@ static void sp_selection_layout_widget_change_selection (SPWidget *spw, SPSelect
 static void sp_selection_layout_widget_update (SPWidget *spw, SPSelection *sel);
 static void sp_object_layout_any_value_changed (GtkAdjustment *adj, SPWidget *spw);
 
+static GtkWidget *layout_dialog = NULL;
+
 static void
-sp_object_layout_dialog_destroy (GtkObject *object, GtkWidget **reference)
+sp_object_layout_dialog_destroy (GtkObject *object, gpointer data)
 {
-	*reference = NULL;
+	layout_dialog = NULL;
 }
 
 void
 sp_object_properties_layout (void)
 {
-	static GtkWidget *dialog = NULL;
 
-	if (!dialog) {
+	if (!layout_dialog) {
 		GtkWidget *w;
-		dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Object position and size"));
-		gtk_signal_connect (GTK_OBJECT (dialog), "destroy", GTK_SIGNAL_FUNC (sp_object_layout_dialog_destroy), &dialog);
+		layout_dialog = sp_window_new (_("Object Size and Position"), TRUE);
+		gtk_signal_connect (GTK_OBJECT (layout_dialog), "destroy",
+				    GTK_SIGNAL_FUNC (sp_object_layout_dialog_destroy), NULL);
 		w = sp_selection_layout_widget_new ();
 		gtk_widget_show (w);
-		gtk_container_add (GTK_CONTAINER (dialog), w);
+		gtk_container_add (GTK_CONTAINER (layout_dialog), w);
+		gtk_widget_show (layout_dialog);
+	} else {
+		gtk_window_present (GTK_WINDOW (layout_dialog));
 	}
-
-	gtk_widget_show (dialog);
 }
 
 GtkWidget *
@@ -285,88 +307,71 @@ sp_selection_layout_widget_new (void)
 	spw = sp_widget_new_global (SODIPODI);
 
 	vb = gtk_vbox_new (FALSE, 4);
-	gtk_widget_show (vb);
 	gtk_container_add (GTK_CONTAINER (spw), vb);
 
-	f = gtk_frame_new (_("Position and size"));
-	gtk_widget_show (f);
+	f = gtk_frame_new (_("Size and Position"));
 	gtk_container_set_border_width (GTK_CONTAINER (f), 4);
 	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
 	gtk_object_set_data (GTK_OBJECT (spw), "frame", f);
 
 	t = gtk_table_new (5, 3, FALSE);
-	gtk_widget_show (t);
 	gtk_container_set_border_width (GTK_CONTAINER (t), 4);
 	gtk_table_set_row_spacings (GTK_TABLE (t), 4);
 	gtk_table_set_col_spacings (GTK_TABLE (t), 4);
 	gtk_container_add (GTK_CONTAINER (f), t);
 
 	l = gtk_label_new (_("Units:"));
-	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (t), l, 0, 2, 0, 1, GTK_FILL, 0, 0, 0);
 	us = sp_unit_selector_new (SP_UNIT_ABSOLUTE);
-	gtk_widget_show (us);
 	gtk_table_attach (GTK_TABLE (t), us, 2, 3, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_object_set_data (GTK_OBJECT (spw), "units", us);
 
-	px = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/arrows_hor.xpm");
-	gtk_widget_show (px);
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "arrows_hor");
 	gtk_table_attach (GTK_TABLE (t), px, 0, 1, 1, 2, 0, 0, 0, 0);
 	l = gtk_label_new (_("X:"));
-	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (t), l, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
 	a = gtk_adjustment_new (0.0, -1e6, 1e6, 0.1, 10.0, 10.0);
 	sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
 	gtk_object_set_data (GTK_OBJECT (spw), "X", a);
 	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
-	gtk_widget_show (sb);
 	gtk_table_attach (GTK_TABLE (t), sb, 2, 3, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_object_layout_any_value_changed), spw);
 
-	px = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/arrows_ver.xpm");
-	gtk_widget_show (px);
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "arrows_ver");
 	gtk_table_attach (GTK_TABLE (t), px, 0, 1, 2, 3, 0, 0, 0, 0);
 	l = gtk_label_new (_("Y:"));
-	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (t), l, 1, 2, 2, 3, GTK_FILL, 0, 0, 0);
 	a = gtk_adjustment_new (0.0, -1e6, 1e6, 0.1, 10.0, 10.0);
 	sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
 	gtk_object_set_data (GTK_OBJECT (spw), "Y", a);
 	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
-	gtk_widget_show (sb);
 	gtk_table_attach (GTK_TABLE (t), sb, 2, 3, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_object_layout_any_value_changed), spw);
 
-	px = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/dimension_hor.xpm");
-	gtk_widget_show (px);
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "dimension_hor");
 	gtk_table_attach (GTK_TABLE (t), px, 0, 1, 3, 4, 0, 0, 0, 0);
 	l = gtk_label_new (_("Width:"));
-	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (t), l, 1, 2, 3, 4, GTK_FILL, 0, 0, 0);
 	a = gtk_adjustment_new (0.0, -1e6, 1e6, 0.1, 10.0, 10.0);
 	sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
 	gtk_object_set_data (GTK_OBJECT (spw), "width", a);
 	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
-	gtk_widget_show (sb);
 	gtk_table_attach (GTK_TABLE (t), sb, 2, 3, 3, 4, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_object_layout_any_value_changed), spw);
 
-	px = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/dimension_ver.xpm");
-	gtk_widget_show (px);
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "dimension_ver");
 	gtk_table_attach (GTK_TABLE (t), px, 0, 1, 4, 5, 0, 0, 0, 0);
 	l = gtk_label_new (_("Height:"));
-	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (t), l, 1, 2, 4, 5, GTK_FILL, 0, 0, 0);
 	a = gtk_adjustment_new (0.0, -1e6, 1e6, 0.1, 10.0, 10.0);
 	sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), GTK_ADJUSTMENT (a));
 	gtk_object_set_data (GTK_OBJECT (spw), "height", a);
 	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
-	gtk_widget_show (sb);
 	gtk_table_attach (GTK_TABLE (t), sb, 2, 3, 4, 5, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_object_layout_any_value_changed), spw);
 
@@ -375,6 +380,8 @@ sp_selection_layout_widget_new (void)
 
 	sp_selection_layout_widget_update (SP_WIDGET (spw), SP_ACTIVE_DESKTOP ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL);
 
+	gtk_widget_show_all (vb);
+
 	return spw;
 }
 
@@ -404,7 +411,7 @@ sp_selection_layout_widget_update (SPWidget *spw, SPSelection *sel)
 	f = gtk_object_get_data (GTK_OBJECT (spw), "frame");
 
 	if (sel && !sp_selection_is_empty (sel)) {
-		ArtDRect bbox;
+		NRRectF bbox;
 
 		sp_selection_bbox (sel, &bbox);
 
@@ -442,17 +449,22 @@ sp_object_layout_any_value_changed (GtkAdjustment *adj, SPWidget *spw)
 	GtkWidget *us;
 	GtkAdjustment *a;
 	const SPUnit *unit;
-	ArtDRect bbox;
+	NRRectF bbox;
 	gdouble x0, y0, x1, y1;
 	SPSelection *sel;
 
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
 
-	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
-
 	sel = SP_WIDGET_SELECTION (spw);
 	us = gtk_object_get_data (GTK_OBJECT (spw), "units");
 	unit = sp_unit_selector_get_unit (SP_UNIT_SELECTOR (us));
+	if (sp_unit_selector_update_test ((SPUnitSelector *) us)) {
+		/*
+		 * When only units are being changed, don't treat changes
+		 * to adjuster values as object changes.
+		 */
+		return;
+	}
 
 	sp_selection_bbox (sel, &bbox);
 	g_return_if_fail (bbox.x1 - bbox.x0 > 1e-9);
@@ -467,20 +479,21 @@ sp_object_layout_any_value_changed (GtkAdjustment *adj, SPWidget *spw)
 	a = gtk_object_get_data (GTK_OBJECT (spw), "height");
 	y1 = y0 + sp_units_get_points (a->value, unit);
 
-	if ((fabs (x0 - bbox.x0) > 1e-6) || (fabs (y0 - bbox.y0) > 1e-6) || (fabs (x1 - bbox.x1) > 1e-6) || (fabs (y1 - bbox.y1) > 1e-6)) {
-		gdouble p2o[6], o2n[6], scale[6], s[6], t[6];
-
-		art_affine_translate (p2o, -bbox.x0, -bbox.y0);
-		art_affine_scale (scale, (x1 - x0) / (bbox.x1 - bbox.x0), (y1 - y0) / (bbox.y1 - bbox.y0));
-		art_affine_translate (o2n, x0, y0);
-		art_affine_multiply (s , p2o, scale);
-		art_affine_multiply (t , s, o2n);
-		sp_selection_apply_affine (sel, t);
-#if 1
+	if ((fabs (x0 - bbox.x0) > 1e-3) ||
+	    (fabs (y0 - bbox.y0) > 1e-3) ||
+	    (fabs (x1 - bbox.x1) > 1e-3) ||
+	    (fabs (y1 - bbox.y1) > 1e-3)) {
+		NRMatrixF p2o, o2n, scale, s, t;
+		gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (TRUE));
+		nr_matrix_f_set_translate (&p2o, -bbox.x0, -bbox.y0);
+		nr_matrix_f_set_scale (&scale, (x1 - x0) / (bbox.x1 - bbox.x0), (y1 - y0) / (bbox.y1 - bbox.y0));
+		nr_matrix_f_set_translate (&o2n, x0, y0);
+		nr_matrix_multiply_fff (&s , &p2o, &scale);
+		nr_matrix_multiply_fff (&t , &s, &o2n);
+		/* No duplicate, not repeatable */
+		sp_selection_apply_affine (sel, &t, FALSE, FALSE);
 		sp_document_done (SP_WIDGET_DOCUMENT (spw));
-#endif
+		gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 	}
-
-	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 }
 
