@@ -14,37 +14,46 @@
 
 #define noSP_SS_VERBOSE
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <string.h>
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
+
+#include <libarikkei/arikkei-strlib.h>
 
 #include <libnr/nr-values.h>
+#include <libnr/nr-matrix.h>
 
+#include <glib.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtkadjustment.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtkmisc.h>
+#include <gtk/gtklabel.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtkradiobutton.h>
-#include <libgnomeui/gnome-stock.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkimage.h>
 
-#include "../helper/art-utils.h"
-#include "../helper/unit-menu.h"
+#include "helper/sp-intl.h"
+#include "helper/unit-menu.h"
 #include "../svg/svg.h"
-#include "../widgets/sp-widget.h"
-#include <widgets/paint-selector.h>
-#include <widgets/dash-selector.h>
-#include "../style.h"
+#include "widgets/icon.h"
+#include "widgets/sp-widget.h"
+#include "widgets/paint-selector.h"
+#include "widgets/dash-selector.h"
+#include "enums.h"
+#include "style.h"
 #include "../gradient-chemistry.h"
 #include "../document.h"
 #include "../desktop-handles.h"
 #include "../selection.h"
+#include "../sp-item.h"
 #include "../sp-gradient.h"
 #include "../sodipodi.h"
 
@@ -94,6 +103,23 @@ sp_stroke_style_paint_widget_new (void)
 	return spw;
 }
 
+void
+sp_stroke_style_paint_system_color_set (GtkWidget *widget, SPColor *color, float opacity)
+{
+	SPPaintSelector *psel;
+
+	psel = g_object_get_data (G_OBJECT (widget), "paint-selector");
+
+	switch (psel->mode) {
+	case SP_PAINT_SELECTOR_MODE_COLOR_RGB:
+	case SP_PAINT_SELECTOR_MODE_COLOR_CMYK:
+		sp_paint_selector_system_color_set (psel, color, opacity);
+		break;
+	default:
+		break;
+	}
+}
+
 static void
 sp_stroke_style_paint_construct (SPWidget *spw, SPPaintSelector *psel)
 {
@@ -139,10 +165,8 @@ sp_stroke_style_paint_update (SPWidget *spw, SPSelection *sel)
 	SPObject *object;
 	SPGradient *vector;
 	gfloat c[5];
-	ArtDRect bbox;
 	SPLinearGradient *lg;
 	SPRadialGradient *rg;
-	gdouble ctm[6];
 #if 0
 	NRPointF p0, p1;
 #endif
@@ -211,22 +235,12 @@ sp_stroke_style_paint_update (SPWidget *spw, SPSelection *sel)
 		/* fixme: Probably we should set multiple mode here too */
 		sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
 		sp_paint_selector_set_gradient_linear (psel, vector);
-		sp_selection_bbox_document (sel, &bbox);
-		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
+		sp_selection_bbox_document (sel, &fbb);
+		sp_paint_selector_set_gradient_bbox (psel, fbb.x0, fbb.y0, fbb.x1, fbb.y1);
 		/* fixme: This is plain wrong */
 		lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (object));
-		sp_item_invoke_bbox (SP_ITEM (object), &bbox, NR_MATRIX_D_IDENTITY.c);
-		sp_item_i2doc_affine (SP_ITEM (object), ctm);
-		fctm.c[0] = ctm[0];
-		fctm.c[1] = ctm[1];
-		fctm.c[2] = ctm[2];
-		fctm.c[3] = ctm[3];
-		fctm.c[4] = ctm[4];
-		fctm.c[5] = ctm[5];
-		fbb.x0 = bbox.x0;
-		fbb.y0 = bbox.y0;
-		fbb.x1 = bbox.x1;
-		fbb.y1 = bbox.y1;
+		sp_item_invoke_bbox (SP_ITEM (object), &fbb, NULL, TRUE);
+		sp_item_i2doc_affine (SP_ITEM (object), &fctm);
 		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (lg), &fctm, &fbb, &gs2d);
 		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
 		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (lg), SP_GRADIENT_SPREAD (lg));
@@ -248,22 +262,12 @@ sp_stroke_style_paint_update (SPWidget *spw, SPSelection *sel)
 		}
 		/* fixme: Probably we should set multiple mode here too */
 		sp_paint_selector_set_gradient_radial (psel, vector);
-		sp_selection_bbox_document (sel, &bbox);
-		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
+		sp_selection_bbox_document (sel, &fbb);
+		sp_paint_selector_set_gradient_bbox (psel, fbb.x0, fbb.y0, fbb.x1, fbb.y1);
 		/* fixme: This is plain wrong */
 		rg = SP_RADIALGRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (object));
-		sp_item_invoke_bbox (SP_ITEM (object), &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
-		sp_item_i2doc_affine (SP_ITEM (object), ctm);
-		fctm.c[0] = ctm[0];
-		fctm.c[1] = ctm[1];
-		fctm.c[2] = ctm[2];
-		fctm.c[3] = ctm[3];
-		fctm.c[4] = ctm[4];
-		fctm.c[5] = ctm[5];
-		fbb.x0 = bbox.x0;
-		fbb.y0 = bbox.y0;
-		fbb.x1 = bbox.x1;
-		fbb.y1 = bbox.y1;
+		sp_item_invoke_bbox (SP_ITEM (object), &fbb, NULL, TRUE);
+		sp_item_i2doc_affine (SP_ITEM (object), &fctm);
 		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (rg), &fctm, &fbb, &gs2d);
 		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
 		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (rg), SP_GRADIENT_SPREAD (rg));
@@ -403,7 +407,7 @@ sp_stroke_style_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 	SPCSSAttr *css;
 	gfloat rgba[4], cmyka[5];
 	SPGradient *vector;
-	guchar b[64];
+	guchar b[64], c0[32], c1[32], c2[32], c3[32];
 
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
 #ifdef SP_SS_VERBOSE
@@ -432,7 +436,7 @@ sp_stroke_style_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 		sp_repr_css_set_property (css, "stroke", "none");
 		for (r = reprs; r != NULL; r = r->next) {
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
-			sp_repr_set_attr_recursive ((SPRepr *) r->data, "stroke-cmyk", NULL);
+			sp_repr_set_attr_recursive ((SPRepr *) r->data, "sodipodi:stroke-cmyk", NULL);
 		}
 		sp_repr_css_attr_unref (css);
 		if (spw->sodipodi) sp_document_done (SP_WIDGET_DOCUMENT (spw));
@@ -442,11 +446,11 @@ sp_stroke_style_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 		sp_paint_selector_get_rgba_floatv (psel, rgba);
 		sp_svg_write_color (b, 64, SP_RGBA32_F_COMPOSE (rgba[0], rgba[1], rgba[2], 0.0));
 		sp_repr_css_set_property (css, "stroke", b);
-		g_snprintf (b, 64, "%g", rgba[3]);
+		arikkei_dtoa_simple (b, 64, rgba[3], 4, 0, FALSE);
 		sp_repr_css_set_property (css, "stroke-opacity", b);
 		for (r = reprs; r != NULL; r = r->next) {
+			sp_repr_set_attr_recursive ((SPRepr *) r->data, "sodipodi:stroke-cmyk", NULL);
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
-			sp_repr_set_attr_recursive ((SPRepr *) r->data, "stroke-cmyk", NULL);
 		}
 		sp_repr_css_attr_unref (css);
 		if (spw->sodipodi) sp_document_done (SP_WIDGET_DOCUMENT (spw));
@@ -457,12 +461,16 @@ sp_stroke_style_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 		sp_color_cmyk_to_rgb_floatv (rgba, cmyka[0], cmyka[1], cmyka[2], cmyka[3]);
 		sp_svg_write_color (b, 64, SP_RGBA32_F_COMPOSE (rgba[0], rgba[1], rgba[2], 0.0));
 		sp_repr_css_set_property (css, "stroke", b);
-		g_snprintf (b, 64, "%g", cmyka[4]);
+		arikkei_dtoa_simple (b, 64, cmyka[4], 4, 0, FALSE);
 		sp_repr_css_set_property (css, "stroke-opacity", b);
-		g_snprintf (b, 64, "(%g %g %g %g)", cmyka[0], cmyka[1], cmyka[2], cmyka[3]);
+		arikkei_dtoa_simple (c0, 32, cmyka[0], 4, 0, FALSE);
+		arikkei_dtoa_simple (c1, 32, cmyka[1], 4, 0, FALSE);
+		arikkei_dtoa_simple (c2, 32, cmyka[2], 4, 0, FALSE);
+		arikkei_dtoa_simple (c3, 32, cmyka[3], 4, 0, FALSE);
+		g_snprintf (b, 64, "(%s %s %s %s)", c0, c1, c2, c3);
 		for (r = reprs; r != NULL; r = r->next) {
+			sp_repr_set_attr_recursive ((SPRepr *) r->data, "sodipodi:stroke-cmyk", b);
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
-			sp_repr_set_attr_recursive ((SPRepr *) r->data, "stroke-cmyk", b);
 		}
 		sp_repr_css_attr_unref (css);
 		if (spw->sodipodi) sp_document_done (SP_WIDGET_DOCUMENT (spw));
@@ -593,7 +601,7 @@ sp_stroke_style_line_widget_new (void)
 	gtk_object_set_data (GTK_OBJECT (spw), "join-miter", tb);
 	gtk_object_set_data (GTK_OBJECT (tb), "join", "miter");
 	gtk_signal_connect (GTK_OBJECT (tb), "toggled", GTK_SIGNAL_FUNC (sp_stroke_style_any_toggled), spw);
-	px = gnome_stock_pixmap_widget (spw, SODIPODI_GLADEDIR "/join_miter.xpm");
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "join_miter");
 	gtk_widget_show (px);
 	gtk_container_add (GTK_CONTAINER (tb), px);
 
@@ -604,7 +612,7 @@ sp_stroke_style_line_widget_new (void)
 	gtk_object_set_data (GTK_OBJECT (spw), "join-round", tb);
 	gtk_object_set_data (GTK_OBJECT (tb), "join", "round");
 	gtk_signal_connect (GTK_OBJECT (tb), "toggled", GTK_SIGNAL_FUNC (sp_stroke_style_any_toggled), spw);
-	px = gnome_stock_pixmap_widget (spw, SODIPODI_GLADEDIR "/join_round.xpm");
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "join_round");
 	gtk_widget_show (px);
 	gtk_container_add (GTK_CONTAINER (tb), px);
 
@@ -615,7 +623,7 @@ sp_stroke_style_line_widget_new (void)
 	gtk_object_set_data (GTK_OBJECT (spw), "join-bevel", tb);
 	gtk_object_set_data (GTK_OBJECT (tb), "join", "bevel");
 	gtk_signal_connect (GTK_OBJECT (tb), "toggled", GTK_SIGNAL_FUNC (sp_stroke_style_any_toggled), spw);
-	px = gnome_stock_pixmap_widget (spw, SODIPODI_GLADEDIR "/join_bevel.xpm");
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "join_bevel");
 	gtk_widget_show (px);
 	gtk_container_add (GTK_CONTAINER (tb), px);
 
@@ -636,7 +644,7 @@ sp_stroke_style_line_widget_new (void)
 	gtk_object_set_data (GTK_OBJECT (spw), "cap-butt", tb);
 	gtk_object_set_data (GTK_OBJECT (tb), "cap", "butt");
 	gtk_signal_connect (GTK_OBJECT (tb), "toggled", GTK_SIGNAL_FUNC (sp_stroke_style_any_toggled), spw);
-	px = gnome_stock_pixmap_widget (spw, SODIPODI_GLADEDIR "/cap_butt.xpm");
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "cap_butt");
 	gtk_widget_show (px);
 	gtk_container_add (GTK_CONTAINER (tb), px);
 
@@ -647,7 +655,7 @@ sp_stroke_style_line_widget_new (void)
 	gtk_object_set_data (GTK_OBJECT (spw), "cap-round", tb);
 	gtk_object_set_data (GTK_OBJECT (tb), "cap", "round");
 	gtk_signal_connect (GTK_OBJECT (tb), "toggled", GTK_SIGNAL_FUNC (sp_stroke_style_any_toggled), spw);
-	px = gnome_stock_pixmap_widget (spw, SODIPODI_GLADEDIR "/cap_round.xpm");
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "cap_round");
 	gtk_widget_show (px);
 	gtk_container_add (GTK_CONTAINER (tb), px);
 
@@ -658,7 +666,7 @@ sp_stroke_style_line_widget_new (void)
 	gtk_object_set_data (GTK_OBJECT (spw), "cap-square", tb);
 	gtk_object_set_data (GTK_OBJECT (tb), "cap", "square");
 	gtk_signal_connect (GTK_OBJECT (tb), "toggled", GTK_SIGNAL_FUNC (sp_stroke_style_any_toggled), spw);
-	px = gnome_stock_pixmap_widget (spw, SODIPODI_GLADEDIR "/cap_square.xpm");
+	px = sp_icon_new (SP_ICON_SIZE_BUTTON, "cap_square");
 	gtk_widget_show (px);
 	gtk_container_add (GTK_CONTAINER (tb), px);
 
@@ -756,11 +764,11 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 	avgwidth = 0.0;
 	stroked = TRUE;
 	for (l = objects; l != NULL; l = l->next) {
-		gdouble i2d[6];
+		NRMatrixF i2d;
 		gdouble dist;
-		sp_item_i2d_affine (SP_ITEM (l->data), i2d);
+		sp_item_i2d_affine (SP_ITEM (l->data), &i2d);
 		object = SP_OBJECT (l->data);
-		dist = sp_distance_d_matrix_d_transform (object->style->stroke_width.computed, i2d);
+		dist = object->style->stroke_width.computed * NR_MATRIX_DF_EXPANSION (&i2d);
 #ifdef SP_SS_VERBOSE
 		g_print ("%g in user is %g on desktop\n", object->style->stroke_width.computed, dist);
 #endif
@@ -794,13 +802,13 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 	}
 
 	switch (jointype) {
-	case ART_PATH_STROKE_JOIN_MITER:
+	case SP_STROKE_LINEJOIN_MITER:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "join-miter");
 		break;
-	case ART_PATH_STROKE_JOIN_ROUND:
+	case SP_STROKE_LINEJOIN_ROUND:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "join-round");
 		break;
-	case ART_PATH_STROKE_JOIN_BEVEL:
+	case SP_STROKE_LINEJOIN_BEVEL:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "join-bevel");
 		break;
 	default:
@@ -810,13 +818,13 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 	sp_stroke_style_set_join_buttons (spw, tb);
 
 	switch (captype) {
-	case ART_PATH_STROKE_CAP_BUTT:
+	case SP_STROKE_LINECAP_BUTT:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "cap-butt");
 		break;
-	case ART_PATH_STROKE_CAP_ROUND:
+	case SP_STROKE_LINECAP_ROUND:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "cap-round");
 		break;
-	case ART_PATH_STROKE_CAP_SQUARE:
+	case SP_STROKE_LINECAP_SQUARE:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "cap-square");
 		break;
 	default:
@@ -879,13 +887,13 @@ sp_stroke_style_line_update_repr (SPWidget *spw, SPRepr *repr)
 
 	/* Join & Cap */
 	switch (style->stroke_linejoin.value) {
-	case ART_PATH_STROKE_JOIN_MITER:
+	case SP_STROKE_LINEJOIN_MITER:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "join-miter");
 		break;
-	case ART_PATH_STROKE_JOIN_ROUND:
+	case SP_STROKE_LINEJOIN_ROUND:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "join-round");
 		break;
-	case ART_PATH_STROKE_JOIN_BEVEL:
+	case SP_STROKE_LINEJOIN_BEVEL:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "join-bevel");
 		break;
 	default:
@@ -895,13 +903,13 @@ sp_stroke_style_line_update_repr (SPWidget *spw, SPRepr *repr)
 	sp_stroke_style_set_join_buttons (spw, tb);
 
 	switch (style->stroke_linecap.value) {
-	case ART_PATH_STROKE_CAP_BUTT:
+	case SP_STROKE_LINECAP_BUTT:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "cap-butt");
 		break;
-	case ART_PATH_STROKE_CAP_ROUND:
+	case SP_STROKE_LINECAP_ROUND:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "cap-round");
 		break;
-	case ART_PATH_STROKE_CAP_SQUARE:
+	case SP_STROKE_LINECAP_SQUARE:
 		tb = gtk_object_get_data (GTK_OBJECT (spw), "cap-square");
 		break;
 	default:
@@ -938,7 +946,7 @@ sp_stroke_style_set_scaled_dash (SPCSSAttr *css, int ndash, double *dash, double
 		int i, pos;
 		pos = 0;
 		for (i = 0; i < ndash; i++) {
-			pos += g_snprintf (c + pos, 1022 - pos, "%g", dash[i] * scale);
+			pos += arikkei_dtoa_simple (c + pos, 1022 - pos, dash[i] * scale, 6, 0, FALSE);
 			if ((i < (ndash - 1)) && (pos < 1020)) {
 				c[pos] = ',';
 				pos += 1;
@@ -946,7 +954,7 @@ sp_stroke_style_set_scaled_dash (SPCSSAttr *css, int ndash, double *dash, double
 		}
 		c[pos] = 0;
 		sp_repr_css_set_property (css, "stroke-dasharray", c);
-		g_snprintf (c, 1024, "%g", offset * scale);
+		arikkei_dtoa_simple (c, 1024, offset * scale, 6, 0, FALSE);
 		sp_repr_css_set_property (css, "stroke-dashoffset", c);
 	} else {
 		sp_repr_css_set_property (css, "stroke-dasharray", "none");
@@ -987,7 +995,7 @@ sp_stroke_style_scale_line (SPWidget *spw)
 
 	if (items) {
 		for (i = items; i != NULL; i = i->next) {
-			double i2d[6], d2i[6];
+			NRMatrixF i2d, d2i;
 			double length, dist;
 			double *dash, offset;
 			int ndash;
@@ -995,10 +1003,10 @@ sp_stroke_style_scale_line (SPWidget *spw)
 			sp_dash_selector_get_dash (dsel, &ndash, &dash, &offset);
 			/* Set stroke width */
 			sp_convert_distance (&length, sp_unit_selector_get_unit (us), SP_PS_UNIT);
-			sp_item_i2d_affine (SP_ITEM (i->data), i2d);
-			art_affine_invert (d2i, i2d);
-			dist = sp_distance_d_matrix_d_transform (length, d2i);
-			g_snprintf (c, 32, "%g", sp_distance_d_matrix_d_transform (length, d2i));
+			sp_item_i2d_affine (SP_ITEM (i->data), &i2d);
+			nr_matrix_f_invert (&d2i, &i2d);
+			dist = length * NR_MATRIX_DF_EXPANSION (&d2i);
+			arikkei_dtoa_simple (c, 32, dist, 4, 0, FALSE);
 			sp_repr_css_set_property (css, "stroke-width", c);
 			/* Set dash */
 			sp_stroke_style_set_scaled_dash (css, ndash, dash, offset, dist);
@@ -1013,7 +1021,7 @@ sp_stroke_style_scale_line (SPWidget *spw)
 			length = wadj->value;
 			sp_dash_selector_get_dash (dsel, &ndash, &dash, &offset);
 			sp_convert_distance (&length, sp_unit_selector_get_unit (us), SP_PS_UNIT);
-			g_snprintf (c, 32, "%g", length * 1.25);
+			arikkei_dtoa_simple (c, 32, length * 1.25, 6, 0, FALSE);
 			sp_repr_css_set_property (css, "stroke-width", c);
 			sp_stroke_style_set_scaled_dash (css, ndash, dash, offset, length);
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
