@@ -12,28 +12,37 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <string.h>
+
+#include <libarikkei/arikkei-strlib.h>
+
 #include <libnrtype/nr-type-directory.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 #include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtktable.h>
-#include <gtk/gtktext.h>
+#include <gtk/gtktextview.h>
 #include <gtk/gtkradiobutton.h>
 #include <gtk/gtkhseparator.h>
-#include <libgnomeui/gnome-stock.h>
-#include <gal/widgets/e-unicode.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkcombo.h>
 
-#include "../widgets/font-selector.h"
+#include "macros.h"
+#include "helper/sp-intl.h"
+#include "helper/window.h"
+#include "widgets/icon.h"
+#include "widgets/font-selector.h"
 #include "../forward.h"
 #include "../sodipodi.h"
 #include "../document.h"
@@ -53,26 +62,30 @@ static void sp_text_edit_dialog_close (GtkButton *button, GtkWidget *dlg);
 
 static void sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean style, gboolean content);
 
-static void sp_text_edit_dialog_text_changed (GtkText *txt, GtkWidget *dlg);
+static void sp_text_edit_dialog_text_changed (GtkTextBuffer *tb, GtkWidget *dlg);
 static void sp_text_edit_dialog_font_changed (SPFontSelector *fontsel, NRFont *font, GtkWidget *dlg);
 static void sp_text_edit_dialog_any_toggled (GtkToggleButton *tb, GtkWidget *dlg);
+static void sp_text_edit_dialog_line_spacing_changed (GtkEditable *editable, GtkWidget *dlg);
 
 static SPText *sp_ted_get_selected_text_item (void);
+static unsigned sp_ted_get_selected_text_count (void);
+
+
+static const unsigned char *spacings[] = {"90%", "100%", "110%", "120%", "133%", "150%", "200%", NULL};
 
 static GtkWidget *dlg = NULL;
 
 static void
 sp_text_edit_dialog_destroy (GtkObject *object, gpointer data)
 {
-	gtk_signal_disconnect_by_data (GTK_OBJECT (SODIPODI), dlg);
+	sp_signal_disconnect_by_data (SODIPODI, dlg);
 	dlg = NULL;
 }
 
 static gint
 sp_text_edit_dialog_delete (GtkWidget *dlg, GdkEvent *event, gpointer data)
 {
-	gtk_object_destroy (GTK_OBJECT (dlg));
-
+	gtk_widget_destroy (GTK_WIDGET (dlg));
 	return TRUE;
 }
 
@@ -80,155 +93,160 @@ void
 sp_text_edit_dialog (void)
 {
 	if (!dlg) {
-		GtkWidget *mainvb, *nb, *vb, *hb, *txt, *fontsel, *preview, *f, *tbl, *l, *px, *b, *hs;
+		GtkWidget *mainvb, *nb, *vb, *hb, *txt, *fontsel, *preview, *f, *tbl, *l, *px, *c, *b, *hs;
+		GtkTextBuffer *tb;
+		GList *sl;
+		int i;
 
-		dlg = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title (GTK_WINDOW (dlg), _("Text properties"));
+		dlg = sp_window_new (_("Text properties"), TRUE);
 		gtk_window_set_policy (GTK_WINDOW (dlg), TRUE, TRUE, FALSE);
-		gtk_signal_connect (GTK_OBJECT (dlg), "destroy", GTK_SIGNAL_FUNC (sp_text_edit_dialog_destroy), dlg);
-		gtk_signal_connect (GTK_OBJECT (dlg), "delete_event", GTK_SIGNAL_FUNC (sp_text_edit_dialog_delete), dlg);
+		g_signal_connect (G_OBJECT (dlg), "destroy", G_CALLBACK (sp_text_edit_dialog_destroy), dlg);
+		g_signal_connect (G_OBJECT (dlg), "delete_event", G_CALLBACK (sp_text_edit_dialog_delete), dlg);
 
 		mainvb = gtk_vbox_new (FALSE, 0);
-		gtk_widget_show (mainvb);
 		gtk_container_add (GTK_CONTAINER (dlg), mainvb);
 
 		nb = gtk_notebook_new ();
-		gtk_widget_show (nb);
 		gtk_container_set_border_width (GTK_CONTAINER (nb), 4);
 		gtk_box_pack_start (GTK_BOX (mainvb), nb, TRUE, TRUE, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "notebook", nb);
+		g_object_set_data (G_OBJECT (dlg), "notebook", nb);
 
 		/* Vbox inside notebook */
 		vb = gtk_vbox_new (FALSE, 0);
-		gtk_widget_show (vb);
 
-		txt = gtk_text_new (NULL, NULL);
-		gtk_widget_show (txt);
-		gtk_text_set_editable (GTK_TEXT (txt), TRUE);
-		gtk_signal_connect (GTK_OBJECT (txt), "changed", GTK_SIGNAL_FUNC (sp_text_edit_dialog_text_changed), dlg);
-		gtk_box_pack_start (GTK_BOX (vb), txt, TRUE, TRUE, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "text", txt);
+		/* Textview */
+		f = gtk_frame_new (NULL);
+		gtk_frame_set_shadow_type (GTK_FRAME (f), GTK_SHADOW_IN);
+		tb = gtk_text_buffer_new (NULL);
+		txt = gtk_text_view_new_with_buffer (tb);
+		gtk_widget_set_size_request (txt, -1, 64);
+		gtk_text_view_set_editable (GTK_TEXT_VIEW (txt), TRUE);
+		gtk_container_add (GTK_CONTAINER (f), txt);
+		gtk_box_pack_start (GTK_BOX (vb), f, TRUE, TRUE, 0);
+		g_signal_connect (G_OBJECT (tb), "changed", G_CALLBACK (sp_text_edit_dialog_text_changed), dlg);
+		g_object_set_data (G_OBJECT (dlg), "text", tb);
+		g_object_set_data (G_OBJECT (dlg), "textw", txt);
 
 		/* HBox containing font selection and layout */
 		hb = gtk_hbox_new (FALSE, 0);
-		gtk_widget_show (hb);
 		gtk_box_pack_start (GTK_BOX (vb), hb, TRUE, TRUE, 0);
 
 		fontsel = sp_font_selector_new ();
-		gtk_widget_show (fontsel);
-		gtk_signal_connect (GTK_OBJECT (fontsel), "font_set", GTK_SIGNAL_FUNC (sp_text_edit_dialog_font_changed), dlg);
+		g_signal_connect (G_OBJECT (fontsel), "font_set", G_CALLBACK (sp_text_edit_dialog_font_changed), dlg);
 		gtk_box_pack_start (GTK_BOX (hb), fontsel, TRUE, TRUE, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "fontsel", fontsel);
+		g_object_set_data (G_OBJECT (dlg), "fontsel", fontsel);
 
 		/* Layout */
 		f = gtk_frame_new (_("Layout"));
-		gtk_widget_show (f);
 		gtk_box_pack_start (GTK_BOX (hb), f, FALSE, FALSE, 4);
 
-		tbl = gtk_table_new (2, 4, FALSE);
-		gtk_widget_show (tbl);
+		tbl = gtk_table_new (3, 4, FALSE);
 		gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
 		gtk_table_set_col_spacings (GTK_TABLE (tbl), 4);
 		gtk_container_add (GTK_CONTAINER (f), tbl);
 
 		l = gtk_label_new (_("Alignment:"));
-		gtk_widget_show (l);
 		gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 		gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 0, 1, 0, 0, 4, 0);
-		px = gnome_stock_pixmap_widget (dlg, GNOME_STOCK_PIXMAP_ALIGN_LEFT);
-		gtk_widget_show (px);
+		px = gtk_image_new_from_stock (GTK_STOCK_JUSTIFY_LEFT, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		b = gtk_radio_button_new (NULL);
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_text_edit_dialog_any_toggled), dlg);
+		g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (sp_text_edit_dialog_any_toggled), dlg);
 		gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (b), FALSE);
 		gtk_container_add (GTK_CONTAINER (b), px);
 		gtk_table_attach (GTK_TABLE (tbl), b, 1, 2, 0, 1, 0, 0, 0, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "text_anchor_start", b);
-		px = gnome_stock_pixmap_widget (dlg, GNOME_STOCK_PIXMAP_ALIGN_CENTER);
-		gtk_widget_show (px);
+		g_object_set_data (G_OBJECT (dlg), "text_anchor_start", b);
+		px = gtk_image_new_from_stock (GTK_STOCK_JUSTIFY_CENTER, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		b = gtk_radio_button_new (gtk_radio_button_group (GTK_RADIO_BUTTON (b)));
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_text_edit_dialog_any_toggled), dlg);
+		g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (sp_text_edit_dialog_any_toggled), dlg);
 		gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (b), FALSE);
 		gtk_container_add (GTK_CONTAINER (b), px);
 		gtk_table_attach (GTK_TABLE (tbl), b, 2, 3, 0, 1, 0, 0, 0, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "text_anchor_middle", b);
-		px = gnome_stock_pixmap_widget (dlg, GNOME_STOCK_PIXMAP_ALIGN_RIGHT);
-		gtk_widget_show (px);
+		g_object_set_data (G_OBJECT (dlg), "text_anchor_middle", b);
+		px = gtk_image_new_from_stock (GTK_STOCK_JUSTIFY_RIGHT, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		b = gtk_radio_button_new (gtk_radio_button_group (GTK_RADIO_BUTTON (b)));
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_text_edit_dialog_any_toggled), dlg);
+		g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (sp_text_edit_dialog_any_toggled), dlg);
 		gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (b), FALSE);
 		gtk_container_add (GTK_CONTAINER (b), px);
 		gtk_table_attach (GTK_TABLE (tbl), b, 3, 4, 0, 1, 0, 0, 0, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "text_anchor_end", b);
+		g_object_set_data (G_OBJECT (dlg), "text_anchor_end", b);
 
 		l = gtk_label_new (_("Orientation:"));
-		gtk_widget_show (l);
 		gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 		gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 1, 2, 0, 0, 4, 0);
-		px = gnome_stock_pixmap_widget (dlg, SODIPODI_GLADEDIR "/writing_mode_lr.xpm");
-		gtk_widget_show (px);
+		px = sp_icon_new (SP_ICON_SIZE_BUTTON, "writing_mode_lr");
 		b = gtk_radio_button_new (NULL);
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_text_edit_dialog_any_toggled), dlg);
+		g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (sp_text_edit_dialog_any_toggled), dlg);
 		gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (b), FALSE);
 		gtk_container_add (GTK_CONTAINER (b), px);
 		gtk_table_attach (GTK_TABLE (tbl), b, 1, 2, 1, 2, 0, 0, 0, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "writing_mode_lr", b);
-		px = gnome_stock_pixmap_widget (dlg, SODIPODI_GLADEDIR "/writing_mode_tb.xpm");
-		gtk_widget_show (px);
+		g_object_set_data (G_OBJECT (dlg), "writing_mode_lr", b);
+		px = sp_icon_new (SP_ICON_SIZE_BUTTON, "writing_mode_tb");
 		b = gtk_radio_button_new (gtk_radio_button_group (GTK_RADIO_BUTTON (b)));
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_text_edit_dialog_any_toggled), dlg);
+		g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (sp_text_edit_dialog_any_toggled), dlg);
 		gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (b), FALSE);
 		gtk_container_add (GTK_CONTAINER (b), px);
 		gtk_table_attach (GTK_TABLE (tbl), b, 2, 3, 1, 2, 0, 0, 0, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "writing_mode_tb", b);
+		g_object_set_data (G_OBJECT (dlg), "writing_mode_tb", b);
 
+		l = gtk_label_new (_("Line spacing:"));
+		gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+		gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, 0, 4, 0);
+		c = gtk_combo_new ();
+		gtk_combo_set_value_in_list ((GtkCombo *) c, FALSE, FALSE);
+		gtk_combo_set_use_arrows ((GtkCombo *) c, TRUE);
+		gtk_combo_set_use_arrows_always ((GtkCombo *) c, TRUE);
+		gtk_widget_set_size_request (c, 64, -1);
+		/* Setup strings */
+		sl = NULL;
+		for (i = 0; spacings[i]; i++) {
+			sl = g_list_prepend (sl, (void *) spacings[i]);
+		}
+		sl = g_list_reverse (sl);
+		gtk_combo_set_popdown_strings ((GtkCombo *) c, sl);
+		g_list_free (sl);
+		g_signal_connect ((GObject *) ((GtkCombo *) c)->entry, "changed", (GCallback) sp_text_edit_dialog_line_spacing_changed, dlg);
+		gtk_table_attach (GTK_TABLE (tbl), c, 1, 4, 2, 3, 0, 0, 4, 0);
+		g_object_set_data (G_OBJECT (dlg), "line_spacing", c);
+
+		/* Font preview */
 		preview = sp_font_preview_new ();
-		gtk_widget_show (preview);
 		gtk_box_pack_start (GTK_BOX (vb), preview, TRUE, TRUE, 4);
-		gtk_object_set_data (GTK_OBJECT (dlg), "preview", preview);
+		g_object_set_data (G_OBJECT (dlg), "preview", preview);
 
 		l = gtk_label_new (_("Text and font"));
-		gtk_widget_show (l);
 		gtk_notebook_append_page (GTK_NOTEBOOK (nb), vb, l);
 
 		/* Buttons */
 		hs = gtk_hseparator_new ();
-		gtk_widget_show (hs);
 		gtk_box_pack_start (GTK_BOX (mainvb), hs, FALSE, FALSE, 0);
 
 		hb = gtk_hbox_new (FALSE, 4);
-		gtk_widget_show (hb);
 		gtk_container_set_border_width (GTK_CONTAINER (hb), 4);
 		gtk_box_pack_start (GTK_BOX (mainvb), hb, FALSE, FALSE, 0);
 
 		b = gtk_button_new_with_label (_("Set as default"));
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_text_edit_dialog_set_default), dlg);
+		g_signal_connect (G_OBJECT (b), "clicked", G_CALLBACK (sp_text_edit_dialog_set_default), dlg);
 		gtk_box_pack_start (GTK_BOX (hb), b, FALSE, FALSE, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "default", b);
+		g_object_set_data (G_OBJECT (dlg), "default", b);
 
-		b = gnome_stock_button (GNOME_STOCK_BUTTON_CLOSE);
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_text_edit_dialog_close), dlg);
+		b = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+		g_signal_connect (G_OBJECT (b), "clicked", G_CALLBACK (sp_text_edit_dialog_close), dlg);
 		gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 0);
 
-		b = gnome_stock_button (GNOME_STOCK_BUTTON_APPLY);
-		gtk_widget_show (b);
-		gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_text_edit_dialog_apply), dlg);
+		b = gtk_button_new_from_stock (GTK_STOCK_APPLY);
+		g_signal_connect (G_OBJECT (b), "clicked", G_CALLBACK (sp_text_edit_dialog_apply), dlg);
 		gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 0);
-		gtk_object_set_data (GTK_OBJECT (dlg), "apply", b);
+		g_object_set_data (G_OBJECT (dlg), "apply", b);
+
+		g_signal_connect (G_OBJECT (SODIPODI), "modify_selection", G_CALLBACK (sp_text_edit_dialog_modify_selection), dlg);
+		g_signal_connect (G_OBJECT (SODIPODI), "change_selection", G_CALLBACK (sp_text_edit_dialog_change_selection), dlg);
+
+		sp_text_edit_dialog_read_selection (dlg, TRUE, TRUE);
+
+		gtk_widget_show_all (dlg);
 	}
 
-	gtk_signal_connect (GTK_OBJECT (SODIPODI), "modify_selection", GTK_SIGNAL_FUNC (sp_text_edit_dialog_modify_selection), dlg);
-	gtk_signal_connect (GTK_OBJECT (SODIPODI), "change_selection", GTK_SIGNAL_FUNC (sp_text_edit_dialog_change_selection), dlg);
-
-	sp_text_edit_dialog_read_selection (dlg, TRUE, TRUE);
-
-	gtk_widget_show (dlg);
+	gtk_window_present ((GtkWindow *) dlg);
 }
 
 static void
@@ -251,28 +269,31 @@ sp_text_edit_dialog_change_selection (Sodipodi *sodipodi, SPSelection *sel, GtkW
 static void
 sp_text_edit_dialog_update_object (SPText *text, SPRepr *repr)
 {
-	gtk_object_set_data (GTK_OBJECT (dlg), "blocked", GINT_TO_POINTER (TRUE));
+	g_object_set_data (G_OBJECT (dlg), "blocked", GINT_TO_POINTER (TRUE));
 
 	if (text) {
-		GtkWidget *textw;
+		GtkTextBuffer *tb;
+		GtkTextIter start, end;
 		guchar *str;
 
-		textw = gtk_object_get_data (GTK_OBJECT (dlg), "text");
+		tb = g_object_get_data (G_OBJECT (dlg), "text");
 
 		/* Content */
-		str = e_utf8_gtk_editable_get_text (GTK_EDITABLE (textw));
+		gtk_text_buffer_get_bounds (tb, &start, &end);
+		str = gtk_text_buffer_get_text (tb, &start, &end, TRUE);
 		sp_text_set_repr_text_multiline (text, str);
 		g_free (str);
 	}
 
 	if (repr) {
-		GtkWidget *fontsel, *preview, *b;
+		GtkWidget *fontsel, *preview, *b, *combo;
 		SPCSSAttr *css;
 		NRFont *font;
 		guchar c[256];
+		const char *sstr;
 
-		fontsel = gtk_object_get_data (GTK_OBJECT (dlg), "fontsel");
-		preview = gtk_object_get_data (GTK_OBJECT (dlg), "preview");
+		fontsel = g_object_get_data (G_OBJECT (dlg), "fontsel");
+		preview = g_object_get_data (G_OBJECT (dlg), "preview");
 
 		css = sp_repr_css_attr_new ();
 
@@ -286,27 +307,30 @@ sp_text_edit_dialog_update_object (SPText *text, SPRepr *repr)
 		nr_typeface_attribute_get (NR_FONT_TYPEFACE (font), "style", c, 256);
 		g_strdown (c);
 		sp_repr_css_set_property (css, "font-style", c);
-		snprintf (c, 64, "%g", NR_FONT_SIZE (font));
+		arikkei_dtoa_simple (c, 64, NR_FONT_SIZE (font), 6, 0, FALSE);
 		sp_repr_css_set_property (css, "font-size", c);
 		nr_font_unref (font);
 		/* Layout */
-		b = gtk_object_get_data (GTK_OBJECT (dlg), "text_anchor_start");
+		b = g_object_get_data (G_OBJECT (dlg), "text_anchor_start");
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) {
 			sp_repr_css_set_property (css, "text-anchor", "start");
 		} else {
-			b = gtk_object_get_data (GTK_OBJECT (dlg), "text_anchor_middle");
+			b = g_object_get_data (G_OBJECT (dlg), "text_anchor_middle");
 			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) {
 				sp_repr_css_set_property (css, "text-anchor", "middle");
 			} else {
 				sp_repr_css_set_property (css, "text-anchor", "end");
 			}
 		}
-		b = gtk_object_get_data (GTK_OBJECT (dlg), "writing_mode_lr");
+		b = g_object_get_data (G_OBJECT (dlg), "writing_mode_lr");
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) {
 			sp_repr_css_set_property (css, "writing-mode", "lr");
 		} else {
 			sp_repr_css_set_property (css, "writing-mode", "tb");
 		}
+		combo = g_object_get_data ((GObject *) dlg, "line_spacing");
+		sstr = gtk_entry_get_text ((GtkEntry *) ((GtkCombo *) (combo))->entry);
+		sp_repr_set_attr (repr, "sodipodi:linespacing", sstr);
 
 		sp_repr_css_change (repr, css, "style");
 		sp_repr_css_attr_unref (css);
@@ -317,7 +341,7 @@ sp_text_edit_dialog_update_object (SPText *text, SPRepr *repr)
 		sp_document_ensure_up_to_date (SP_OBJECT_DOCUMENT (text));
 	}
 
-	gtk_object_set_data (GTK_OBJECT (dlg), "blocked", NULL);
+	g_object_set_data (G_OBJECT (dlg), "blocked", NULL);
 }
 
 static void
@@ -326,7 +350,7 @@ sp_text_edit_dialog_set_default (GtkButton *button, GtkWidget *dlg)
 	GtkWidget *def;
 	SPRepr *repr;
 
-	def = gtk_object_get_data (GTK_OBJECT (dlg), "default");
+	def = g_object_get_data (G_OBJECT (dlg), "default");
 
 	repr = sodipodi_get_repr (SODIPODI, "tools.text");
 
@@ -341,33 +365,43 @@ sp_text_edit_dialog_apply (GtkButton *button, GtkWidget *dlg)
 	GtkWidget *apply, *def;
 	SPText *text;
 	SPRepr *repr;
+	const GSList *item;
+	unsigned items;
+	
+	g_object_set_data (G_OBJECT (dlg), "blocked", GINT_TO_POINTER (TRUE));
 
-	gtk_object_set_data (GTK_OBJECT (dlg), "blocked", GINT_TO_POINTER (TRUE));
+	apply = g_object_get_data (G_OBJECT (dlg), "apply");
+	def = g_object_get_data (G_OBJECT (dlg), "default");
 
-	apply = gtk_object_get_data (GTK_OBJECT (dlg), "apply");
-	def = gtk_object_get_data (GTK_OBJECT (dlg), "default");
-
-	text = sp_ted_get_selected_text_item ();
-	if (text) {
+	text = NULL;
+	items = 0;
+	item = sp_selection_item_list(SP_DT_SELECTION (SP_ACTIVE_DESKTOP));
+	
+	for (; item != NULL; item = item->next) {
+		if (!SP_IS_TEXT (item->data)) continue;
+		text = SP_TEXT(item->data);
 		repr = SP_OBJECT_REPR (text);
-	} else {
-		repr = sodipodi_get_repr (SODIPODI, "tools.text");
+		sp_text_edit_dialog_update_object (NULL, repr);
+		++items;
 	}
-
-	sp_text_edit_dialog_update_object (text, repr);
-
-	if (!text) {
+	
+	if (items == 1) {
+		sp_text_edit_dialog_update_object (text, NULL);
+	} else if (items == 0) {
+		repr = sodipodi_get_repr (SODIPODI, "tools.text");
+		sp_text_edit_dialog_update_object (NULL, repr);
 		gtk_widget_set_sensitive (def, FALSE);
 	}
+
 	gtk_widget_set_sensitive (apply, FALSE);
 
-	gtk_object_set_data (GTK_OBJECT (dlg), "blocked", NULL);
+	g_object_set_data (G_OBJECT (dlg), "blocked", NULL);
 }
 
 static void
 sp_text_edit_dialog_close (GtkButton *button, GtkWidget *dlg)
 {
-	gtk_object_destroy (GTK_OBJECT (dlg));
+	gtk_widget_destroy (GTK_WIDGET (dlg));
 }
 
 static guchar *
@@ -446,48 +480,53 @@ static void
 sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean dostyle, gboolean docontent)
 {
 	GtkWidget *notebook, *textw, *fontsel, *preview, *apply, *def;
+	GtkTextBuffer *tb;
 	SPText *text;
+	SPRepr *repr;
 	SPStyle *style;
+	unsigned items;
 
-	if (gtk_object_get_data (GTK_OBJECT (dlg), "blocked")) return;
-	gtk_object_set_data (GTK_OBJECT (dlg), "blocked", GINT_TO_POINTER (TRUE));
+	if (g_object_get_data (G_OBJECT (dlg), "blocked")) return;
+	g_object_set_data (G_OBJECT (dlg), "blocked", GINT_TO_POINTER (TRUE));
 
-	notebook = gtk_object_get_data (GTK_OBJECT (dlg), "notebook");
-	textw = gtk_object_get_data (GTK_OBJECT (dlg), "text");
-	fontsel = gtk_object_get_data (GTK_OBJECT (dlg), "fontsel");
-	preview = gtk_object_get_data (GTK_OBJECT (dlg), "preview");
-	apply = gtk_object_get_data (GTK_OBJECT (dlg), "apply");
-	def = gtk_object_get_data (GTK_OBJECT (dlg), "default");
+	notebook = g_object_get_data (G_OBJECT (dlg), "notebook");
+	textw = g_object_get_data (G_OBJECT (dlg), "textw");
+	tb = g_object_get_data (G_OBJECT (dlg), "text");
+	fontsel = g_object_get_data (G_OBJECT (dlg), "fontsel");
+	preview = g_object_get_data (G_OBJECT (dlg), "preview");
+	apply = g_object_get_data (G_OBJECT (dlg), "apply");
+	def = g_object_get_data (G_OBJECT (dlg), "default");
 
 	text = sp_ted_get_selected_text_item ();
 
 	if (text) {
-		gtk_widget_set_sensitive (textw, TRUE);
+		items = sp_ted_get_selected_text_count ();
+		if (items == 1) {
+			gtk_widget_set_sensitive (textw, TRUE);
+		} else {
+			gtk_widget_set_sensitive (textw, FALSE);
+		}
 		gtk_widget_set_sensitive (apply, FALSE);
 		gtk_widget_set_sensitive (def, TRUE);
 		style = SP_OBJECT_STYLE (text);
 		if (docontent) {
 			guchar *str;
 			str = sp_text_get_string_multiline (text);
-			if (str && *str) {
-				gchar *gtkstr;
+			if (str) {
 				int pos;
 				pos = 0;
-				gtk_text_freeze (GTK_TEXT (textw));
-				gtk_editable_delete_text (GTK_EDITABLE (textw), 0, -1);
-				gtkstr = e_utf8_to_gtk_string (textw, str);
-				gtk_editable_insert_text (GTK_EDITABLE (textw), gtkstr, strlen (gtkstr), &pos);
-				g_free (gtkstr);
-				gtk_text_thaw (GTK_TEXT (textw));
+				if (items == 1) {
+					gtk_text_buffer_set_text (tb, str, strlen (str));
+				}
 				sp_font_preview_set_phrase (SP_FONT_PREVIEW (preview), str);
 				g_free (str);
 			} else {
-				gtk_editable_delete_text (GTK_EDITABLE (textw), 0, -1);
+				gtk_text_buffer_set_text (tb, "", 0);
 				sp_font_preview_set_phrase (SP_FONT_PREVIEW (preview), NULL);
 			}
 		}
+		repr = SP_OBJECT_REPR (text);
 	} else {
-		SPRepr *repr;
 		gtk_widget_set_sensitive (textw, FALSE);
 		gtk_widget_set_sensitive (apply, FALSE);
 		gtk_widget_set_sensitive (def, FALSE);
@@ -505,8 +544,9 @@ sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean dostyle, gboolean d
 	if (dostyle) {
 		NRTypeFace *tf;
 		NRFont *font;
-		GtkWidget *b;
+		GtkWidget *b, *combo;
 		guchar *c;
+		const unsigned char *sstr;
 
 		c = sp_text_edit_dialog_font_style_to_lookup (style);
 		tf = nr_type_directory_lookup_fuzzy (style->text->font_family.value, c);
@@ -520,41 +560,46 @@ sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean dostyle, gboolean d
 		}
 
 		if (style->text_anchor.computed == SP_CSS_TEXT_ANCHOR_START) {
-			b = gtk_object_get_data (GTK_OBJECT (dlg), "text_anchor_start");
+			b = g_object_get_data (G_OBJECT (dlg), "text_anchor_start");
 		} else if (style->text_anchor.computed == SP_CSS_TEXT_ANCHOR_MIDDLE) {
-			b = gtk_object_get_data (GTK_OBJECT (dlg), "text_anchor_middle");
+			b = g_object_get_data (G_OBJECT (dlg), "text_anchor_middle");
 		} else {
-			b = gtk_object_get_data (GTK_OBJECT (dlg), "text_anchor_end");
+			b = g_object_get_data (G_OBJECT (dlg), "text_anchor_end");
 		}
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), TRUE);
 		if (style->writing_mode.computed == SP_CSS_WRITING_MODE_LR) {
-			b = gtk_object_get_data (GTK_OBJECT (dlg), "writing_mode_lr");
+			b = g_object_get_data (G_OBJECT (dlg), "writing_mode_lr");
 		} else {
-			b = gtk_object_get_data (GTK_OBJECT (dlg), "writing_mode_tb");
+			b = g_object_get_data (G_OBJECT (dlg), "writing_mode_tb");
 		}
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), TRUE);
+		combo = g_object_get_data (G_OBJECT (dlg), "line_spacing");
+		sstr = (repr) ? sp_repr_attr (repr, "sodipodi:linespacing") : NULL;
+		gtk_entry_set_text ((GtkEntry *) ((GtkCombo *) (combo))->entry, (sstr) ? sstr : (const unsigned char *) "100%");
 	}
 
-	gtk_object_set_data (GTK_OBJECT (dlg), "blocked", NULL);
+	g_object_set_data (G_OBJECT (dlg), "blocked", NULL);
 }
 
 static void
-sp_text_edit_dialog_text_changed (GtkText *txt, GtkWidget *dlg)
+sp_text_edit_dialog_text_changed (GtkTextBuffer *tb, GtkWidget *dlg)
 {
 	GtkWidget *textw, *preview, *apply, *def;
 	SPText *text;
+	GtkTextIter start, end;
 	gchar *str;
 
-	if (gtk_object_get_data (GTK_OBJECT (dlg), "blocked")) return;
+	if (g_object_get_data (G_OBJECT (dlg), "blocked")) return;
 
 	text = sp_ted_get_selected_text_item ();
 
-	textw = gtk_object_get_data (GTK_OBJECT (dlg), "text");
-	preview = gtk_object_get_data (GTK_OBJECT (dlg), "preview");
-	apply = gtk_object_get_data (GTK_OBJECT (dlg), "apply");
-	def = gtk_object_get_data (GTK_OBJECT (dlg), "default");
+	textw = g_object_get_data (G_OBJECT (dlg), "textw");
+	preview = g_object_get_data (G_OBJECT (dlg), "preview");
+	apply = g_object_get_data (G_OBJECT (dlg), "apply");
+	def = g_object_get_data (G_OBJECT (dlg), "default");
 
-	str = e_utf8_gtk_editable_get_text (GTK_EDITABLE (textw));
+	gtk_text_buffer_get_bounds (tb, &start, &end);
+	str = gtk_text_buffer_get_text (tb, &start, &end, TRUE);
 	if (str && *str) {
 		sp_font_preview_set_phrase (SP_FONT_PREVIEW (preview), str);
 	} else {
@@ -574,13 +619,13 @@ sp_text_edit_dialog_font_changed (SPFontSelector *fsel, NRFont *font, GtkWidget
 	GtkWidget *preview, *apply, *def;
 	SPText *text;
 
-	if (gtk_object_get_data (GTK_OBJECT (dlg), "blocked")) return;
+	if (g_object_get_data (G_OBJECT (dlg), "blocked")) return;
 
 	text = sp_ted_get_selected_text_item ();
 
-	preview = gtk_object_get_data (GTK_OBJECT (dlg), "preview");
-	apply = gtk_object_get_data (GTK_OBJECT (dlg), "apply");
-	def = gtk_object_get_data (GTK_OBJECT (dlg), "default");
+	preview = g_object_get_data (G_OBJECT (dlg), "preview");
+	apply = g_object_get_data (G_OBJECT (dlg), "apply");
+	def = g_object_get_data (G_OBJECT (dlg), "default");
 
 	sp_font_preview_set_font (SP_FONT_PREVIEW (preview), font);
 
@@ -596,12 +641,31 @@ sp_text_edit_dialog_any_toggled (GtkToggleButton *tb, GtkWidget *dlg)
 	GtkWidget *apply, *def;
 	SPText *text;
 
-	if (gtk_object_get_data (GTK_OBJECT (dlg), "blocked")) return;
+	if (g_object_get_data (G_OBJECT (dlg), "blocked")) return;
 
 	text = sp_ted_get_selected_text_item ();
 
-	apply = gtk_object_get_data (GTK_OBJECT (dlg), "apply");
-	def = gtk_object_get_data (GTK_OBJECT (dlg), "default");
+	apply = g_object_get_data (G_OBJECT (dlg), "apply");
+	def = g_object_get_data (G_OBJECT (dlg), "default");
+
+	if (text) {
+		gtk_widget_set_sensitive (apply, TRUE);
+	}
+	gtk_widget_set_sensitive (def, TRUE);
+}
+
+static void
+sp_text_edit_dialog_line_spacing_changed (GtkEditable *editable, GtkWidget *dlg)
+{
+	GtkWidget *apply, *def;
+	SPText *text;
+
+	if (g_object_get_data ((GObject *) dlg, "blocked")) return;
+
+	text = sp_ted_get_selected_text_item ();
+
+	apply = g_object_get_data (G_OBJECT (dlg), "apply");
+	def = g_object_get_data (G_OBJECT (dlg), "default");
 
 	if (text) {
 		gtk_widget_set_sensitive (apply, TRUE);
@@ -612,9 +676,31 @@ sp_text_edit_dialog_any_toggled (GtkToggleButton *tb, GtkWidget *dlg)
 static SPText *
 sp_ted_get_selected_text_item (void)
 {
-	SPItem *item;
+	const GSList *item;
+	
 	if (!SP_ACTIVE_DESKTOP) return NULL;
-	item = sp_selection_item (SP_DT_SELECTION (SP_ACTIVE_DESKTOP));
-	if (item && SP_IS_TEXT (item)) return SP_TEXT (item);
+	
+	item = sp_selection_item_list(SP_DT_SELECTION (SP_ACTIVE_DESKTOP));
+	for (; item != NULL; item = item->next) {
+		if (SP_IS_TEXT(item->data)) return SP_TEXT (item->data);
+	}
+	
 	return NULL;
 }
+
+static unsigned
+sp_ted_get_selected_text_count (void)
+{
+	const GSList *item;
+	unsigned items;
+
+	if (!SP_ACTIVE_DESKTOP) return 0;
+
+	items = 0;
+	item = sp_selection_item_list(SP_DT_SELECTION (SP_ACTIVE_DESKTOP));
+	for (; item != NULL; item = item->next) {
+		if (SP_IS_TEXT(item->data)) ++items;
+	}
+	
+	return items;
+}
