@@ -12,14 +12,13 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <string.h>
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 #include <gtk/gtkmain.h>
-#include <gtk/gtkwindow.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkhpaned.h>
@@ -30,19 +29,25 @@
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtktooltips.h>
-#include <libgnomeui/gnome-stock.h>
-#include <gal/widgets/e-unicode.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkarrow.h>
 
-#include "../sodipodi.h"
-#include "../document.h"
-#include "../desktop-handles.h"
-#include "../selection.h"
+#include "helper/sp-intl.h"
+#include "helper/window.h"
+#include "macros.h"
+#include "system.h"
+#include "sodipodi.h"
+#include "document.h"
+#include "desktop-handles.h"
+#include "selection.h"
+#include "sp-item.h"
 
 #include "../xml/repr-private.h"
 
-#include "../widgets/sp-xmlview-tree.h"
-#include "../widgets/sp-xmlview-content.h"
-#include "../widgets/sp-xmlview-attr-list.h"
+#include "widgets/icon.h"
+#include "widgets/sp-xmlview-tree.h"
+#include "widgets/sp-xmlview-content.h"
+#include "widgets/sp-xmlview-attr-list.h"
 
 typedef struct _EditableDest {
 	GtkEditable * editable;
@@ -52,7 +57,7 @@ typedef struct _EditableDest {
 static GtkWidget * dialog = NULL;
 static GtkTooltips * tooltips = NULL;
 static GtkEditable * attr_name = NULL;
-static GtkEditable * attr_value = NULL;
+static GtkTextView *attr_value = NULL;
 static SPXMLViewTree * tree = NULL;
 static SPXMLViewAttrList * attributes = NULL;
 static SPXMLViewContent * content = NULL;
@@ -94,7 +99,7 @@ static void on_tree_unselect_row_hide (GtkCTree * tree, GtkCTreeNode * node, gin
 static void on_attr_select_row (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data);
 static void on_attr_unselect_row (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data);
 
-static void on_attr_select_row_enable_if_not_id (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data);
+static void on_attr_select_row_enable (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data);
 static void on_attr_unselect_row_disable (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data);
 
 static void on_attr_select_row_set_name_content (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data);
@@ -104,7 +109,7 @@ static void on_attr_unselect_row_clear_text (GtkCList *list, gint row, gint colu
 static void on_editable_changed_enable_if_valid_xml_name (GtkEditable * editable, gpointer data);
 
 static void on_desktop_selection_changed (SPSelection * selection);
-static void on_desktop_destroy (SPDesktop * desktop, gpointer data);
+static gboolean on_desktop_shutdown (SPDesktop * desktop, gpointer data);
 static void on_document_uri_set (SPDocument * document, const guchar * uri, gpointer data);
 
 static void on_clicked_get_editable_text (GtkWidget * widget, gpointer data);
@@ -139,10 +144,10 @@ sp_xml_tree_dialog (void)
 		tooltips = gtk_tooltips_new ();
 		gtk_tooltips_enable (tooltips);
 
-		dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_container_set_border_width (GTK_CONTAINER (dialog), 4);
+		dialog = sp_window_new ("", TRUE);
+		gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
 		gtk_window_set_default_size (GTK_WINDOW (dialog), 640, 384);
-		gtk_signal_connect (GTK_OBJECT (dialog), "destroy", on_destroy, NULL);
+		g_signal_connect (G_OBJECT (dialog), "destroy", G_CALLBACK (on_destroy), NULL);
 
 		paned = gtk_hpaned_new ();
 		gtk_paned_set_position (GTK_PANED (paned), 256);
@@ -155,55 +160,123 @@ sp_xml_tree_dialog (void)
 
 		tree = SP_XMLVIEW_TREE (sp_xmlview_tree_new (NULL, NULL, NULL));
 		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (tree), _("Drag to reorder nodes"), NULL);
-		gtk_signal_connect (GTK_OBJECT (tree), "tree_select_row", (GtkSignalFunc) on_tree_select_row, NULL);
-		gtk_signal_connect (GTK_OBJECT (tree), "tree_unselect_row", (GtkSignalFunc) on_tree_unselect_row, NULL);
-		gtk_signal_connect_after (GTK_OBJECT (tree), "tree_move", (GtkSignalFunc) after_tree_move, NULL);
+		g_signal_connect (G_OBJECT (tree), "tree_select_row", G_CALLBACK (on_tree_select_row), NULL);
+		g_signal_connect (G_OBJECT (tree), "tree_unselect_row", G_CALLBACK (on_tree_unselect_row), NULL);
+		g_signal_connect_after (G_OBJECT (tree), "tree_move", G_CALLBACK (after_tree_move), NULL);
 
-		toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
-		gtk_container_set_border_width (GTK_CONTAINER (toolbar), 4);
+		/* fixme: replace gtk_signal_connect_while_alive() with something else... */
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, _("New element node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/add_xml_element_node.xpm"), cmd_new_element_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_element, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		toolbar = gtk_toolbar_new ();
+		gtk_toolbar_set_style (GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
+		gtk_container_set_border_width (GTK_CONTAINER (toolbar), 0);
+
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
+						  NULL,
+						  _("New element node"),
+						  NULL,
+						  sp_icon_new (SP_ICON_SIZE_BUTTON, "add_xml_element_node"),
+						  G_CALLBACK (cmd_new_element_node),
+						  NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree),
+						"tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_element),
+						button,
+						GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree),
+						"tree_unselect_row",
+						G_CALLBACK (on_tree_unselect_row_disable),
+						button,
+						GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, _("New text node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/add_xml_text_node.xpm"), cmd_new_text_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_element, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
+						  NULL, _("New text node"), NULL,
+						  sp_icon_new (SP_ICON_SIZE_BUTTON, "add_xml_text_node"),
+						  G_CALLBACK (cmd_new_text_node),
+						  NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree),
+						"tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_element),
+						button,
+						GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree),
+						"tree_unselect_row",
+						G_CALLBACK (on_tree_unselect_row_disable),
+						button,
+						GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, _("Duplicate node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/duplicate_xml_node.xpm"), cmd_duplicate_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_non_root, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
+						  NULL, _("Duplicate node"), NULL,
+						  sp_icon_new (SP_ICON_SIZE_BUTTON, "duplicate_xml_node"),
+						  G_CALLBACK (cmd_duplicate_node),
+						  NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree),
+						"tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_non_root),
+						button,
+						GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						G_CALLBACK (on_tree_unselect_row_disable),
+						button, GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
 		gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, _("Delete node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/delete_xml_node.xpm"), cmd_delete_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_non_root, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, _("Delete node"), NULL,
+						  sp_icon_new (SP_ICON_SIZE_BUTTON, "delete_xml_node"),
+						  G_CALLBACK (cmd_delete_node), NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_non_root),
+						button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						G_CALLBACK (on_tree_unselect_row_disable),
+						button, GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
 		gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "<", _("Unindent node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/unindent_xml_node.xpm"), cmd_unindent_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_has_grandparent, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "<", _("Unindent node"), NULL,
+						  gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_IN),
+						  G_CALLBACK (cmd_unindent_node), NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_has_grandparent),
+						button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						G_CALLBACK (on_tree_unselect_row_disable),
+						button, GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), ">", _("Indent node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/indent_xml_node.xpm"), cmd_indent_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_indentable, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), ">", _("Indent node"), NULL,
+						  gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_IN),
+						  G_CALLBACK (cmd_indent_node), NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_indentable),
+						button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						(GCallback) on_tree_unselect_row_disable, button, GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "^", _("Raise node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/raise_xml_node.xpm"), cmd_raise_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_not_first_child, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "^", _("Raise node"), NULL,
+						  gtk_arrow_new (GTK_ARROW_UP, GTK_SHADOW_IN),
+						  G_CALLBACK (cmd_raise_node), NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_not_first_child),
+						button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						G_CALLBACK (on_tree_unselect_row_disable),
+						button, GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "v", _("Lower node"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/lower_xml_node.xpm"), cmd_lower_node, NULL);
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_enable_if_not_last_child, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "v", _("Lower node"), NULL,
+						  gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_IN),
+						  G_CALLBACK (cmd_lower_node), NULL);
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row",
+						G_CALLBACK (on_tree_select_row_enable_if_not_last_child),
+						button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						G_CALLBACK (on_tree_unselect_row_disable),
+						button, GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
 		gtk_box_pack_start (GTK_BOX (box), toolbar, FALSE, TRUE, 0); 
@@ -225,17 +298,23 @@ sp_xml_tree_dialog (void)
 		gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (attr_container), TRUE, TRUE, 0);
 
 		attributes = SP_XMLVIEW_ATTR_LIST (sp_xmlview_attr_list_new (NULL));
-		gtk_signal_connect (GTK_OBJECT (attributes), "select_row", on_attr_select_row, NULL);
-		gtk_signal_connect (GTK_OBJECT (attributes), "unselect_row", on_attr_unselect_row, NULL);
+		g_signal_connect (G_OBJECT (attributes), "select_row", G_CALLBACK (on_attr_select_row), NULL);
+		g_signal_connect (G_OBJECT (attributes), "unselect_row", G_CALLBACK (on_attr_unselect_row), NULL);
 
-		toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
-		gtk_container_set_border_width (GTK_CONTAINER (toolbar), 4);
+		toolbar = gtk_toolbar_new ();
+		gtk_toolbar_set_style (GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
+		gtk_container_set_border_width (GTK_CONTAINER (toolbar), 0);
 
-		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, _("Delete attribute"), NULL, gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/delete_xml_attribute.xpm"), cmd_delete_attr, NULL);
+		button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, _("Delete attribute"), NULL,
+						  sp_icon_new (SP_ICON_SIZE_BUTTON, "delete_xml_attribute"),
+						  (GCallback) cmd_delete_attr, NULL);
 
-		gtk_signal_connect_while_alive (GTK_OBJECT (attributes), "select_row", on_attr_select_row_enable_if_not_id, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (attributes), "unselect_row", on_attr_unselect_row_disable, button, GTK_OBJECT (button));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_disable, button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (attributes), "select_row",
+						(GCallback) on_attr_select_row_enable, button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (attributes), "unselect_row",
+						(GCallback) on_attr_unselect_row_disable, button, GTK_OBJECT (button));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						(GCallback) on_tree_unselect_row_disable, button, GTK_OBJECT (button));
 		gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
 		gtk_box_pack_start (GTK_BOX (attr_container), GTK_WIDGET (toolbar), FALSE, TRUE, 0);
@@ -257,33 +336,33 @@ sp_xml_tree_dialog (void)
 
 		attr_name = GTK_EDITABLE (gtk_entry_new ());
 		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (attr_name), _("Attribute name"), NULL);
-		gtk_signal_connect (GTK_OBJECT (attributes), "select_row", on_attr_select_row_set_name_content, attr_name);
-		gtk_signal_connect (GTK_OBJECT (attributes), "unselect_row", on_attr_unselect_row_clear_text, attr_name);
-		gtk_signal_connect (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_clear_text, attr_name);
+		gtk_signal_connect (GTK_OBJECT (attributes), "select_row", (GCallback) on_attr_select_row_set_name_content, attr_name);
+		gtk_signal_connect (GTK_OBJECT (attributes), "unselect_row", (GCallback) on_attr_unselect_row_clear_text, attr_name);
+		gtk_signal_connect (GTK_OBJECT (tree), "tree_unselect_row", (GCallback) on_tree_unselect_row_clear_text, attr_name);
 		gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (attr_name), TRUE, TRUE, 0);
 
 		set_attr = gtk_button_new ();
 		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (set_attr), _("Set attribute"), NULL);
-		gtk_container_add (GTK_CONTAINER (set_attr), gnome_stock_pixmap_widget (dialog, SODIPODI_PIXMAPDIR "/set.xpm"));
-		gtk_signal_connect (GTK_OBJECT (set_attr), "clicked", cmd_set_attr, NULL);
-		gtk_signal_connect (GTK_OBJECT (attr_name), "changed", on_editable_changed_enable_if_valid_xml_name, set_attr);
+		gtk_container_add (GTK_CONTAINER (set_attr), sp_icon_new (SP_ICON_SIZE_BUTTON, "set"));
+		gtk_signal_connect (GTK_OBJECT (set_attr), "clicked", (GCallback) cmd_set_attr, NULL);
+		gtk_signal_connect (GTK_OBJECT (attr_name), "changed", (GCallback) on_editable_changed_enable_if_valid_xml_name, set_attr);
 		gtk_widget_set_sensitive (GTK_WIDGET (set_attr), FALSE);
 
 		gtk_box_pack_start (GTK_BOX (box2), set_attr, FALSE, FALSE, 0);
 
 		sw = gtk_scrolled_window_new (NULL, NULL);
-		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_box_pack_start (GTK_BOX (toolbar), sw, TRUE, TRUE, 0);
 
-		attr_value = GTK_EDITABLE (gtk_text_new (NULL, NULL));
+		attr_value = (GtkTextView *) gtk_text_view_new ();
 		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (attr_value), _("Attribute value"), NULL);
-		gtk_signal_connect (GTK_OBJECT (attributes), "select_row", on_attr_select_row_set_value_content, attr_value);
-		gtk_signal_connect (GTK_OBJECT (attributes), "unselect_row", on_attr_unselect_row_clear_text, attr_value);
-		gtk_signal_connect (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_clear_text, attr_value);
-		gtk_editable_set_editable (GTK_EDITABLE (attr_value), TRUE);
+		gtk_signal_connect (GTK_OBJECT (attributes), "select_row", (GCallback) on_attr_select_row_set_value_content, attr_value);
+		gtk_signal_connect (GTK_OBJECT (attributes), "unselect_row", (GCallback) on_attr_unselect_row_clear_text, attr_value);
+		gtk_signal_connect (GTK_OBJECT (tree), "tree_unselect_row", (GCallback) on_tree_unselect_row_clear_text, attr_value);
+		gtk_text_view_set_editable (attr_value, TRUE);
 		gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (attr_value));
 
-		gtk_paned_pack2 (GTK_PANED(attr_subpaned_container), GTK_WIDGET (toolbar), FALSE, TRUE);
+		gtk_paned_pack2 (GTK_PANED (attr_subpaned_container), GTK_WIDGET (toolbar), FALSE, TRUE);
 
 		/* text */
 
@@ -300,12 +379,16 @@ sp_xml_tree_dialog (void)
 
 		gtk_widget_show_all (GTK_WIDGET (dialog));
 
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_show_if_element, attr_container, GTK_OBJECT (attr_container));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_hide, attr_container, GTK_OBJECT (attr_container));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row",
+						(GCallback) on_tree_select_row_show_if_element, attr_container, GTK_OBJECT (attr_container));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						(GCallback) on_tree_unselect_row_hide, attr_container, GTK_OBJECT (attr_container));
 		gtk_widget_hide (attr_container);
 
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row", on_tree_select_row_show_if_text, text_container, GTK_OBJECT (text_container));
-		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row", on_tree_unselect_row_hide, text_container, GTK_OBJECT (text_container));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_select_row",
+						(GCallback) on_tree_select_row_show_if_text, text_container, GTK_OBJECT (text_container));
+		gtk_signal_connect_while_alive (GTK_OBJECT (tree), "tree_unselect_row",
+						(GCallback) on_tree_unselect_row_hide, text_container, GTK_OBJECT (text_container));
 		gtk_widget_hide (text_container);
 	} else {
 		gdk_window_raise (GTK_WIDGET (dialog)->window);
@@ -320,13 +403,15 @@ set_tree_desktop (SPDesktop * desktop)
 {
 	if ( desktop == current_desktop ) return;
 	if (current_desktop) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DT_SELECTION (current_desktop)), dialog);
-		gtk_signal_disconnect_by_data (GTK_OBJECT (current_desktop), dialog);
+		if (SP_DT_SELECTION (current_desktop)) {
+			sp_signal_disconnect_by_data (SP_DT_SELECTION (current_desktop), dialog);
+		}
+		sp_signal_disconnect_by_data (current_desktop, dialog);
 	}
 	current_desktop = desktop;
 	if (desktop) {
-		gtk_signal_connect (GTK_OBJECT (desktop), "destroy", on_desktop_destroy, dialog);
-		gtk_signal_connect (GTK_OBJECT (SP_DT_SELECTION (desktop)), "changed", on_desktop_selection_changed, dialog);
+		g_signal_connect (G_OBJECT (desktop), "shutdown", G_CALLBACK (on_desktop_shutdown), dialog);
+		g_signal_connect (G_OBJECT (SP_DT_SELECTION (desktop)), "changed", G_CALLBACK (on_desktop_selection_changed), dialog);
 		set_tree_document (SP_DT_DOCUMENT (desktop));
 	} else {
 		set_tree_document (NULL);
@@ -338,11 +423,11 @@ set_tree_document (SPDocument * document)
 {
 	if ( document == current_document ) return;
 	if (current_document) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (current_document), dialog);
+		sp_signal_disconnect_by_data (current_document, dialog);
 	}
 	current_document = document;
 	if (current_document) {
-		gtk_signal_connect (GTK_OBJECT (current_document), "uri_set", on_document_uri_set, dialog);
+		g_signal_connect (G_OBJECT (current_document), "uri_set", G_CALLBACK (on_document_uri_set), dialog);
 		on_document_uri_set (current_document, SP_DOCUMENT_URI (current_document), dialog);
 		set_tree_repr (sp_document_repr_root (current_document));
 	} else {
@@ -443,7 +528,7 @@ set_dt_select (SPRepr *repr)
 	}
 
 	blocked++;
-	if ( object && SP_IS_ITEM (object) ) {
+	if (object && SP_IS_ITEM (object)) {
 		sp_selection_set_item (selection, SP_ITEM (object));
 	} else {
 		sp_selection_empty (selection);
@@ -495,9 +580,11 @@ on_tree_unselect_row (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointe
 void
 after_tree_move (GtkCTree * tree, GtkCTreeNode * node, GtkCTreeNode * new_parent, GtkCTreeNode * new_sibling, gpointer data)
 {
-	SPRepr * repr;
-	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
-	sp_document_done (current_document);
+	if (GTK_CTREE_ROW (node)->parent == new_parent && GTK_CTREE_ROW (node)->sibling == new_sibling) {
+		sp_document_done (current_document);
+	} else {
+		sp_document_cancel (current_document);
+	}
 }
 
 void
@@ -575,7 +662,13 @@ on_tree_unselect_row_hide (GtkCTree * tree, GtkCTreeNode * node, gint column, gp
 void
 on_tree_unselect_row_clear_text (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
 {
-	gtk_editable_delete_text (GTK_EDITABLE (data), 0, -1);
+	if (GTK_IS_EDITABLE (data)) {
+		gtk_editable_delete_text (GTK_EDITABLE (data), 0, -1);
+	} else if (GTK_IS_TEXT_VIEW (data)) {
+		GtkTextBuffer *tb;
+		tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data));
+		gtk_text_buffer_set_text (tb, "", 0);
+	}
 }
 
 void
@@ -590,16 +683,6 @@ on_attr_unselect_row (GtkCList *list, gint row, gint column, GdkEventButton *eve
 	selected_attr = 0;
 }
 
-void
-on_attr_select_row_enable_if_not_id (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
-{
-	if (g_quark_from_string ("id") != sp_xmlview_attr_list_get_row_key (list, row)) {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
-	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
-	}
-}
-
 void
 on_attr_select_row_set_name_content (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
 {
@@ -617,27 +700,64 @@ on_attr_select_row_set_name_content (GtkCList *list, gint row, gint column, GdkE
 void
 on_attr_select_row_set_value_content (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
 {
-	GtkEditable * editable;
-	const guchar * name, * value;
-	gint pos;
-	editable = GTK_EDITABLE (data);
+	GtkTextBuffer *tb;
+	const guchar *name, *value;
+	tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data));
 	name = g_quark_to_string (sp_xmlview_attr_list_get_row_key (list, row));
 	value = sp_repr_attr (selected_repr, name);
-	gtk_editable_delete_text (editable, 0, -1);
-	pos = 0;
-	gtk_editable_insert_text (editable, value, strlen(value), &pos);
+	gtk_text_buffer_set_text (tb, value, strlen (value));
 }
 
 void
 on_tree_select_row_enable_if_indentable (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
 {
-	GtkCTreeNode * prev, * parent;
+	SPRepr * repr, * prev;
+	gboolean indentable;
+	indentable = FALSE;
+	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
+	if ( repr->parent && repr != repr->parent->children ) {
+		g_assert (repr->parent->children);
+		for ( prev = repr->parent->children ; prev != repr ;
+		      prev = prev->next );
+		if (SP_REPR_TYPE (prev) == SP_XML_ELEMENT_NODE) {
+			indentable = TRUE;
+		}
+	}
+	gtk_widget_set_sensitive (GTK_WIDGET (data), indentable);
+}
+
+void
+on_tree_select_row_enable_if_not_first_child (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
+{
+	SPRepr * repr;
+	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
+	if ( repr->parent && repr != repr->parent->children ) {
+		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
+	} else {
+		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
+	}
+}
+
+void
+on_tree_select_row_enable_if_not_last_child (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
+{
+	SPRepr * repr;
+	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
+	if ( repr->parent && repr->parent->parent && repr->next ) {
+		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
+	} else {
+		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
+	}
+}
+
+void
+on_tree_select_row_enable_if_has_grandparent (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
+{
+	GtkCTreeNode * parent, * grandparent;
 	parent = GTK_CTREE_ROW (node)->parent;
-	prev = GTK_CTREE_NODE_PREV (node);
-	if ( parent && prev && GTK_CTREE_ROW (prev)->parent == parent ) {
-		SPRepr * prev_repr;
-		prev_repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), prev);
-		if (SP_REPR_TYPE (prev_repr) == SP_XML_ELEMENT_NODE) {
+	if (parent) {
+		grandparent = GTK_CTREE_ROW (parent)->parent;
+		if (grandparent) {
 			gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
 		} else {
 			gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
@@ -648,41 +768,9 @@ on_tree_select_row_enable_if_indentable (GtkCTree * tree, GtkCTreeNode * node, g
 }
 
 void
-on_tree_select_row_enable_if_not_first_child (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
+on_attr_select_row_enable (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
 {
-	GtkCTreeNode * prev, * parent;
-	parent = GTK_CTREE_ROW (node)->parent;
-	prev = GTK_CTREE_NODE_PREV (node);
-	if ( parent && prev && GTK_CTREE_ROW (prev)->parent == parent ) {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
-	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
-	}
-}
-
-void
-on_tree_select_row_enable_if_not_last_child (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
-{
-	GtkCTreeNode * parent, * next;
-	parent = GTK_CTREE_ROW (node)->parent;
-	next = GTK_CTREE_NODE_NEXT (node);
-	if ( parent && next && GTK_CTREE_ROW (next)->parent == parent ) {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
-	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
-	}
-}
-
-void
-on_tree_select_row_enable_if_has_grandparent (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
-{
-	GtkCTreeNode * parent;
-	parent = GTK_CTREE_ROW (node)->parent;
-	if ( parent && GTK_CTREE_ROW (parent)->parent ) {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
-	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
-	}
+	gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
 }
 
 void
@@ -694,7 +782,13 @@ on_attr_unselect_row_disable (GtkCList *list, gint row, gint column, GdkEventBut
 void
 on_attr_unselect_row_clear_text (GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
 {
-	gtk_editable_delete_text (GTK_EDITABLE (data), 0, -1);
+	if (GTK_IS_EDITABLE (data)) {
+		gtk_editable_delete_text (GTK_EDITABLE (data), 0, -1);
+	} else if (GTK_IS_TEXT_VIEW (data)) {
+		GtkTextBuffer *tb;
+		tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data));
+		gtk_text_buffer_set_text (tb, "", 0);
+	}
 }
 
 void
@@ -720,11 +814,13 @@ on_desktop_selection_changed (SPSelection * selection)
 	blocked--;
 }
 
-void
-on_desktop_destroy (SPDesktop * desktop, gpointer data)
+gboolean
+on_desktop_shutdown (SPDesktop * desktop, gpointer data)
 {
 	g_assert (dialog != NULL);
 	gtk_widget_destroy (dialog);
+
+	return FALSE;
 }
 
 void
@@ -752,7 +848,7 @@ cmd_new_element_node (GtkObject * object, gpointer data)
 
 	g_assert (selected_repr != NULL);
 
-	window = gtk_window_new (GTK_WINDOW_DIALOG);
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 4);
 	gtk_window_set_title (GTK_WINDOW (window), _("New element node..."));
 	gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);
@@ -775,14 +871,14 @@ cmd_new_element_node (GtkObject * object, gpointer data)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, TRUE, 0);
 
 	cancel = gtk_button_new_with_label (_("Cancel"));
-	gtk_signal_connect_object (GTK_OBJECT (cancel), "clicked", gtk_widget_destroy, GTK_OBJECT (window));
+	gtk_signal_connect_object (GTK_OBJECT (cancel), "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (window));
 	gtk_container_add (GTK_CONTAINER (bbox), cancel);
 
 	create = gtk_button_new_with_label (_("Create"));
 	gtk_widget_set_sensitive (GTK_WIDGET (create), FALSE);
-	gtk_signal_connect (GTK_OBJECT (entry), "changed", on_editable_changed_enable_if_valid_xml_name, create);
-	gtk_signal_connect (GTK_OBJECT (create), "clicked", on_clicked_get_editable_text, &name);
-	gtk_signal_connect_object (GTK_OBJECT (create), "clicked", gtk_widget_destroy, GTK_OBJECT (window));
+	gtk_signal_connect (GTK_OBJECT (entry), "changed", G_CALLBACK (on_editable_changed_enable_if_valid_xml_name), create);
+	gtk_signal_connect (GTK_OBJECT (create), "clicked", G_CALLBACK (on_clicked_get_editable_text), &name);
+	gtk_signal_connect_object (GTK_OBJECT (create), "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (window));
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (create), GTK_CAN_DEFAULT | GTK_RECEIVES_DEFAULT);
 	gtk_container_add (GTK_CONTAINER (bbox), create);
 	
@@ -865,14 +961,16 @@ cmd_delete_attr (GtkObject * object, gpointer data)
 void
 cmd_set_attr (GtkObject * object, gpointer data)
 {
+	GtkTextIter start, end;
 	gchar * name;
 	gchar * value;
 	gint row;
 
 	g_assert (selected_repr != NULL);
 
-	name = e_utf8_gtk_editable_get_text (attr_name);
-	value = e_utf8_gtk_editable_get_text (attr_value);
+	name = gtk_editable_get_chars (attr_name, 0, -1);
+	gtk_text_buffer_get_bounds (gtk_text_view_get_buffer (attr_value), &start, &end);
+	value = gtk_text_buffer_get_text (gtk_text_view_get_buffer (attr_value), &start, &end, TRUE);
 
 	sp_repr_set_attr (selected_repr, name, value);
 
@@ -932,8 +1030,8 @@ cmd_lower_node (GtkObject * object, gpointer data)
 void
 cmd_indent_node (GtkObject * object, gpointer data)
 {
-	SPRepr * prev, * parent, * repr;
-	gboolean success;
+	int ok;
+	SPRepr * prev, * parent, * repr, * ref;
 
 	repr = selected_repr;
 	g_assert (repr != NULL);
@@ -948,24 +1046,29 @@ cmd_indent_node (GtkObject * object, gpointer data)
 	g_return_if_fail (prev != NULL);
 	g_return_if_fail (SP_REPR_TYPE (prev) == SP_XML_ELEMENT_NODE);
 
-	sp_repr_ref (repr);
-	success = sp_repr_remove_child (parent, repr);
-	if (success) {
-		sp_repr_append_child (prev, repr);
+	if (prev->children) {
+		for ( ref = prev->children ; ref->next ; ref = ref->next );
+	} else {
+		ref = NULL;
 	}
-	sp_repr_unref (repr);
 
-	sp_document_done (current_document);
+	ok = sp_repr_remove_child (parent, repr) &&
+	     sp_repr_add_child (prev, repr, ref);
 
-	set_tree_select (repr);
-	set_dt_select (repr);
+	if (ok) {
+		sp_document_done (current_document);
+		set_tree_select (repr);
+		set_dt_select (repr);
+	} else {
+		sp_document_cancel (current_document);
+	}
 }
 
 void
 cmd_unindent_node (GtkObject * object, gpointer data)
 {
 	SPRepr * grandparent, * parent, * repr;
-	gboolean success;
+	int ok;
 
 	repr = selected_repr;
 	g_assert (repr != NULL);
@@ -974,17 +1077,15 @@ cmd_unindent_node (GtkObject * object, gpointer data)
 	grandparent = sp_repr_parent (parent);
 	g_return_if_fail (grandparent);
 	
-	sp_repr_ref (repr);
-	success = sp_repr_remove_child (parent, repr);
-	if (success) {
-		success = sp_repr_add_child (grandparent, repr, parent);
-		g_assert (success);
+	ok = sp_repr_remove_child (parent, repr) &&
+	     sp_repr_add_child (grandparent, repr, parent);
+	
+	if (ok) {
+		sp_document_done (current_document);
+		set_tree_select (repr);
+		set_dt_select (repr);
+	} else {
+		sp_document_cancel (current_document);
 	}
-	sp_repr_unref (repr);
-
-	sp_document_done (current_document);
-
-	set_tree_select (repr);
-	set_dt_select (repr);
 }
 
