@@ -17,6 +17,7 @@
 #include <string.h>
 #include <libnr/nr-rect.h>
 #include <libnr/nr-matrix.h>
+#include <libnr/nr-blit.h>
 #include <libart_lgpl/art_misc.h>
 #include <libart_lgpl/art_bpath.h>
 #include <libart_lgpl/art_vpath.h>
@@ -24,40 +25,35 @@
 #include <libart_lgpl/art_vpath_bpath.h>
 #include <libart_lgpl/art_rect_svp.h>
 #include <libart_lgpl/art_svp_vpath.h>
+#include <libart_lgpl/art_svp_vpath_stroke.h>
 #include <libart_lgpl/art_svp_wind.h>
 #include <libart_lgpl/art_svp_point.h>
 #include <libart_lgpl/art_gray_svp.h>
-#include "../helper/nr-buffers.h"
-#include "../helper/nr-plain-stuff.h"
-#include "../helper/art-utils.h"
 #include "../style.h"
 #include "nr-arena.h"
 #include "nr-arena-glyphs.h"
 
 static void nr_arena_glyphs_class_init (NRArenaGlyphsClass *klass);
 static void nr_arena_glyphs_init (NRArenaGlyphs *glyphs);
-static void nr_arena_glyphs_destroy (GtkObject *object);
+static void nr_arena_glyphs_finalize (NRObject *object);
 
 static guint nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
-static guint nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b);
-static NRArenaItem *nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky);
+static guint nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb);
+static NRArenaItem *nr_arena_glyphs_pick (NRArenaItem *item, double x, double y, double delta, unsigned int sticky);
 
 static NRArenaItemClass *glyphs_parent_class;
 
-GtkType
+unsigned int
 nr_arena_glyphs_get_type (void)
 {
-	static GtkType type = 0;
+	static unsigned int type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"NRArenaGlyphs",
-			sizeof (NRArenaGlyphs),
-			sizeof (NRArenaGlyphsClass),
-			(GtkClassInitFunc) nr_arena_glyphs_class_init,
-			(GtkObjectInitFunc) nr_arena_glyphs_init,
-			NULL, NULL, NULL
-		};
-		type = gtk_type_unique (NR_TYPE_ARENA_ITEM, &info);
+		type = nr_object_register_type (NR_TYPE_ARENA_ITEM,
+						"NRArenaGlyphs",
+						sizeof (NRArenaGlyphsClass),
+						sizeof (NRArenaGlyphs),
+						(void (*) (NRObjectClass *)) nr_arena_glyphs_class_init,
+						(void (*) (NRObject *)) nr_arena_glyphs_init);
 	}
 	return type;
 }
@@ -65,15 +61,15 @@ nr_arena_glyphs_get_type (void)
 static void
 nr_arena_glyphs_class_init (NRArenaGlyphsClass *klass)
 {
-	GtkObjectClass *object_class;
+	NRObjectClass *object_class;
 	NRArenaItemClass *item_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (NRObjectClass *) klass;
 	item_class = (NRArenaItemClass *) klass;
 
-	glyphs_parent_class = gtk_type_class (NR_TYPE_ARENA_ITEM);
+	glyphs_parent_class = (NRArenaItemClass *) ((NRObjectClass *) klass)->parent;
 
-	object_class->destroy = nr_arena_glyphs_destroy;
+	object_class->finalize = nr_arena_glyphs_finalize;
 
 	item_class->update = nr_arena_glyphs_update;
 	item_class->clip = nr_arena_glyphs_clip;
@@ -90,7 +86,7 @@ nr_arena_glyphs_init (NRArenaGlyphs *glyphs)
 }
 
 static void
-nr_arena_glyphs_destroy (GtkObject *object)
+nr_arena_glyphs_finalize (NRObject *object)
 {
 	NRArenaGlyphs *glyphs;
 
@@ -118,8 +114,7 @@ nr_arena_glyphs_destroy (GtkObject *object)
 		glyphs->curve = sp_curve_unref (glyphs->curve);
 	}
 
-	if (GTK_OBJECT_CLASS (glyphs_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (glyphs_parent_class)->destroy) (object);
+	((NRObjectClass *) glyphs_parent_class)->finalize (object);
 }
 
 static guint
@@ -154,14 +149,7 @@ nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 
 	if (glyphs->style->fill.type != SP_PAINT_TYPE_NONE) {
 		NRRectF area;
-
-		t.c[0] = gc->affine[0];
-		t.c[1] = gc->affine[1];
-		t.c[2] = gc->affine[2];
-		t.c[3] = gc->affine[3];
-		t.c[4] = gc->affine[4];
-		t.c[5] = gc->affine[5];
-		nr_matrix_multiply_fff (&t, &glyphs->transform, &t);
+		nr_matrix_multiply_ffd (&t, &glyphs->transform, &gc->transform);
 		rfont = nr_rasterfont_new (glyphs->font, &t);
 		if (glyphs->rfont) glyphs->rfont = nr_rasterfont_unref (glyphs->rfont);
 		glyphs->rfont = rfont;
@@ -176,7 +164,7 @@ nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 
 	if (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE) {
 		/* Build state data */
-		abp = art_bpath_affine_transform (glyphs->curve->bpath, gc->affine);
+		abp = art_bpath_affine_transform (glyphs->curve->bpath, NR_MATRIX_D_TO_DOUBLE (&gc->transform));
 		vp = art_bez_path_to_vec (abp, 0.25);
 		art_free (abp);
 		pvp = art_vpath_perturb (vp);
@@ -184,14 +172,13 @@ nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 
 		if (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE) {
 			gdouble width;
-			width = sp_distance_d_matrix_d_transform (glyphs->style->stroke_width.computed, gc->affine);
-			if (width > 0.125) {
-				glyphs->stroke_svp = art_svp_vpath_stroke (pvp,
-									   glyphs->style->stroke_linejoin.value,
-									   glyphs->style->stroke_linecap.value,
-									   width,
-									   glyphs->style->stroke_miterlimit.value, 0.25);
-			}
+			width = glyphs->style->stroke_width.computed * NR_MATRIX_DF_EXPANSION (&gc->transform);
+			width = MAX (0.125, width);
+			glyphs->stroke_svp = art_svp_vpath_stroke (pvp,
+								   glyphs->style->stroke_linejoin.value,
+								   glyphs->style->stroke_linecap.value,
+								   width,
+								   glyphs->style->stroke_miterlimit.value, 0.25);
 		}
 
 		art_free (pvp);
@@ -210,7 +197,7 @@ nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 }
 
 static guint
-nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b)
+nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 {
 	NRArenaGlyphs *glyphs;
 
@@ -230,7 +217,7 @@ nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 }
 
 static NRArenaItem *
-nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky)
+nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, unsigned int sticky)
 {
 	NRArenaGlyphs *glyphs;
 
@@ -255,10 +242,10 @@ nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gb
 }
 
 void
-nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, gboolean private, NRFont *font, gint glyph, const NRMatrixF *transform)
+nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, unsigned int private, NRFont *font, gint glyph, const NRMatrixF *transform)
 {
-	g_return_if_fail (glyphs != NULL);
-	g_return_if_fail (NR_IS_ARENA_GLYPHS (glyphs));
+	nr_return_if_fail (glyphs != NULL);
+	nr_return_if_fail (NR_IS_ARENA_GLYPHS (glyphs));
 
 	nr_arena_item_request_render (NR_ARENA_ITEM (glyphs));
 
@@ -298,8 +285,8 @@ nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, gboolean privat
 void
 nr_arena_glyphs_set_style (NRArenaGlyphs *glyphs, SPStyle *style)
 {
-	g_return_if_fail (glyphs != NULL);
-	g_return_if_fail (NR_IS_ARENA_GLYPHS (glyphs));
+	nr_return_if_fail (glyphs != NULL);
+	nr_return_if_fail (NR_IS_ARENA_GLYPHS (glyphs));
 
 	if (style) sp_style_ref (style);
 	if (glyphs->style) sp_style_unref (glyphs->style);
@@ -325,29 +312,29 @@ nr_arena_glyphs_fill_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m)
 }
 
 static guint
-nr_arena_glyphs_stroke_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRBuffer *b)
+nr_arena_glyphs_stroke_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m)
 {
 	NRArenaItem *item;
 
 	item = NR_ARENA_ITEM (glyphs);
 
 	if (glyphs->stroke_svp && nr_rect_l_test_intersect (area, &item->bbox)) {
-		NRBuffer *gb;
+		NRPixBlock gb;
 		gint x, y;
-		gb = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
-		art_gray_svp_aa (glyphs->stroke_svp, area->x0, area->y0, area->x1, area->y1, gb->px, gb->rs);
+		nr_pixblock_setup_fast (&gb, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
+		art_gray_svp_aa (glyphs->stroke_svp, area->x0, area->y0, area->x1, area->y1, NR_PIXBLOCK_PX (&gb), gb.rs);
 		for (y = area->y0; y < area->y1; y++) {
 			guchar *d, *s;
-			d = b->px + (y - area->y0) * b->rs;
-			s = gb->px + (y - area->y0) * b->rs;
+			d = NR_PIXBLOCK_PX (m) + (y - area->y0) * m->rs;
+			s = NR_PIXBLOCK_PX (&gb) + (y - area->y0) * gb.rs;
 			for (x = area->x0; x < area->x1; x++) {
 				*d = (*d) + ((255 - *d) * (*s) / 255);
 				d += 1;
 				s += 1;
 			}
 		}
-		nr_buffer_free (gb);
-		b->empty = FALSE;
+		nr_pixblock_release (&gb);
+		m->empty = FALSE;
 	}
 
 	return item->state;
@@ -355,28 +342,26 @@ nr_arena_glyphs_stroke_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRBuffer *b)
 
 static void nr_arena_glyphs_group_class_init (NRArenaGlyphsGroupClass *klass);
 static void nr_arena_glyphs_group_init (NRArenaGlyphsGroup *group);
-static void nr_arena_glyphs_group_destroy (GtkObject *object);
+static void nr_arena_glyphs_group_finalize (NRObject *object);
 
 static guint nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
-static guint nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b);
-static NRArenaItem *nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky);
+static unsigned int nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags);
+static unsigned int nr_arena_glyphs_group_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb);
+static NRArenaItem *nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, unsigned int sticky);
 
 static NRArenaGroupClass *group_parent_class;
 
-GtkType
+unsigned int
 nr_arena_glyphs_group_get_type (void)
 {
-	static GtkType type = 0;
+	static unsigned int type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"NRArenaGlyphsGroup",
-			sizeof (NRArenaGlyphsGroup),
-			sizeof (NRArenaGlyphsGroupClass),
-			(GtkClassInitFunc) nr_arena_glyphs_group_class_init,
-			(GtkObjectInitFunc) nr_arena_glyphs_group_init,
-			NULL, NULL, NULL
-		};
-		type = gtk_type_unique (NR_TYPE_ARENA_GROUP, &info);
+		type = nr_object_register_type (NR_TYPE_ARENA_GROUP,
+						"NRArenaGlyphsGroup",
+						sizeof (NRArenaGlyphsGroupClass),
+						sizeof (NRArenaGlyphsGroup),
+						(void (*) (NRObjectClass *)) nr_arena_glyphs_group_class_init,
+						(void (*) (NRObject *)) nr_arena_glyphs_group_init);
 	}
 	return type;
 }
@@ -384,18 +369,19 @@ nr_arena_glyphs_group_get_type (void)
 static void
 nr_arena_glyphs_group_class_init (NRArenaGlyphsGroupClass *klass)
 {
-	GtkObjectClass *object_class;
+	NRObjectClass *object_class;
 	NRArenaItemClass *item_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (NRObjectClass *) klass;
 	item_class = (NRArenaItemClass *) klass;
 
-	group_parent_class = gtk_type_class (NR_TYPE_ARENA_GROUP);
+	group_parent_class = (NRArenaGroupClass *) ((NRObjectClass *) klass)->parent;
 
-	object_class->destroy = nr_arena_glyphs_group_destroy;
+	object_class->finalize = nr_arena_glyphs_group_finalize;
 
 	item_class->update = nr_arena_glyphs_group_update;
 	item_class->render = nr_arena_glyphs_group_render;
+	item_class->clip = nr_arena_glyphs_group_clip;
 	item_class->pick = nr_arena_glyphs_group_pick;
 }
 
@@ -403,15 +389,15 @@ static void
 nr_arena_glyphs_group_init (NRArenaGlyphsGroup *group)
 {
 	group->style = NULL;
-	group->paintbox.x0 = group->paintbox.y0 = 0.0;
-	group->paintbox.x1 = group->paintbox.y1 = 1.0;
+	group->paintbox.x0 = group->paintbox.y0 = 0.0F;
+	group->paintbox.x1 = group->paintbox.y1 = 1.0F;
 
 	group->fill_painter = NULL;
 	group->stroke_painter = NULL;
 }
 
 static void
-nr_arena_glyphs_group_destroy (GtkObject *object)
+nr_arena_glyphs_group_finalize (NRObject *object)
 {
 	NRArenaGlyphsGroup *group;
 
@@ -432,8 +418,7 @@ nr_arena_glyphs_group_destroy (GtkObject *object)
 		group->style = NULL;
 	}
 
-	if (GTK_OBJECT_CLASS (group_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (group_parent_class)->destroy) (object);
+		((NRObjectClass *) group_parent_class)->finalize (object);
 }
 
 static guint
@@ -453,12 +438,17 @@ nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint
 		group->stroke_painter = NULL;
 	}
 
+	item->render_opacity = TRUE;
 	if (group->style->fill.type == SP_PAINT_TYPE_PAINTSERVER) {
-		group->fill_painter = sp_paint_server_painter_new (SP_STYLE_FILL_SERVER (group->style), gc->affine, &group->paintbox);
+		group->fill_painter = sp_paint_server_painter_new (SP_STYLE_FILL_SERVER (group->style),
+								   NR_MATRIX_D_TO_DOUBLE (&gc->transform), &group->paintbox);
+	item->render_opacity = FALSE;
 	}
 
 	if (group->style->stroke.type == SP_PAINT_TYPE_PAINTSERVER) {
-		group->stroke_painter = sp_paint_server_painter_new (SP_STYLE_STROKE_SERVER (group->style), gc->affine, &group->paintbox);
+		group->stroke_painter = sp_paint_server_painter_new (SP_STYLE_STROKE_SERVER (group->style),
+								     NR_MATRIX_D_TO_DOUBLE (&gc->transform), &group->paintbox);
+	item->render_opacity = FALSE;
 	}
 
 	if (((NRArenaItemClass *) group_parent_class)->update)
@@ -469,8 +459,8 @@ nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint
 
 /* This sucks - as soon, as we have inheritable renderprops, do something with that opacity */
 
-static guint
-nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b)
+static unsigned int
+nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags)
 {
 	NRArenaGroup *group;
 	NRArenaGlyphsGroup *ggroup;
@@ -487,61 +477,54 @@ nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 	/* Fill */
 	if (style->fill.type != SP_PAINT_TYPE_NONE) {
 		NRPixBlock mb;
-		NRBuffer *m;
 		guint32 rgba;
-		m = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
-		nr_pixblock_setup_extern (&mb, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, m->px, m->rs, FALSE, FALSE);
+		nr_pixblock_setup_fast (&mb, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
 		/* Render children fill mask */
 		for (child = group->children; child != NULL; child = child->next) {
 			ret = nr_arena_glyphs_fill_mask (NR_ARENA_GLYPHS (child), area, &mb);
 			if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) {
 				nr_pixblock_release (&mb);
-				nr_buffer_free (m);
 				return ret;
 			}
 		}
-		m->empty = mb.empty;
-		nr_pixblock_release (&mb);
 		/* Composite into buffer */
 		switch (style->fill.type) {
 		case SP_PAINT_TYPE_COLOR:
 			rgba = sp_color_get_rgba32_falpha (&style->fill.value.color,
 							   SP_SCALE24_TO_FLOAT (style->fill_opacity.value) *
 							   SP_SCALE24_TO_FLOAT (style->opacity.value));
-			nr_render_buf_mask_rgba32 (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0, m, 0, 0, rgba);
-			b->empty = FALSE;
+			nr_blit_pixblock_mask_rgba32 (pb, &mb, rgba);
+			pb->empty = FALSE;
 			break;
 		case SP_PAINT_TYPE_PAINTSERVER:
 			if (ggroup->fill_painter) {
-				NRBuffer *pb;
+				NRPixBlock cb;
 				/* Need separate gradient buffer */
-				pb = nr_buffer_get (NR_IMAGE_R8G8B8A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, FALSE);
-				ggroup->fill_painter->fill (ggroup->fill_painter, pb->px, area->x0, area->y0, pb->w, pb->h, pb->rs);
-				pb->empty = FALSE;
+				nr_pixblock_setup_fast (&cb, NR_PIXBLOCK_MODE_R8G8B8A8N, area->x0, area->y0, area->x1, area->y1, TRUE);
+				ggroup->fill_painter->fill (ggroup->fill_painter, &cb);
+				cb.empty = FALSE;
 				/* Composite */
-				nr_render_buf_buf_mask (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0,
-							pb, 0, 0,
-							m, 0, 0);
-				b->empty = FALSE;
-				nr_buffer_free (pb);
+				nr_blit_pixblock_pixblock_mask (pb, &cb, &mb);
+				pb->empty = FALSE;
+				nr_pixblock_release (&cb);
 			}
 			break;
 		default:
 			break;
 		}
-		nr_buffer_free (m);
+		nr_pixblock_release (&mb);
 	}
 
 	/* Stroke */
 	if (style->stroke.type != SP_PAINT_TYPE_NONE) {
-		NRBuffer *m;
+		NRPixBlock m;
 		guint32 rgba;
-		m = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
+		nr_pixblock_setup_fast (&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
 		/* Render children stroke mask */
 		for (child = group->children; child != NULL; child = child->next) {
-			ret = nr_arena_glyphs_stroke_mask (NR_ARENA_GLYPHS (child), area, m);
+			ret = nr_arena_glyphs_stroke_mask (NR_ARENA_GLYPHS (child), area, &m);
 			if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) {
-				nr_buffer_free (m);
+				nr_pixblock_release (&m);
 				return ret;
 			}
 		}
@@ -551,35 +534,55 @@ nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 			rgba = sp_color_get_rgba32_falpha (&style->stroke.value.color,
 							   SP_SCALE24_TO_FLOAT (style->stroke_opacity.value) *
 							   SP_SCALE24_TO_FLOAT (style->opacity.value));
-			nr_render_buf_mask_rgba32 (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0, m, 0, 0, rgba);
-			b->empty = FALSE;
+			nr_blit_pixblock_mask_rgba32 (pb, &m, rgba);
+			pb->empty = FALSE;
 			break;
 		case SP_PAINT_TYPE_PAINTSERVER:
 			if (ggroup->stroke_painter) {
-				NRBuffer *pb;
+				NRPixBlock cb;
 				/* Need separate gradient buffer */
-				pb = nr_buffer_get (NR_IMAGE_R8G8B8A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, FALSE);
-				ggroup->stroke_painter->fill (ggroup->stroke_painter, pb->px, area->x0, area->y0, pb->w, pb->h, pb->rs);
-				pb->empty = FALSE;
+				nr_pixblock_setup_fast (&cb, NR_PIXBLOCK_MODE_R8G8B8A8N, area->x0, area->y0, area->x1, area->y1, TRUE);
+				ggroup->stroke_painter->fill (ggroup->stroke_painter, &cb);
+				cb.empty = FALSE;
 				/* Composite */
-				nr_render_buf_buf_mask (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0,
-							pb, 0, 0,
-							m, 0, 0);
-				b->empty = FALSE;
-				nr_buffer_free (pb);
+				nr_blit_pixblock_pixblock_mask (pb, &cb, &m);
+				pb->empty = FALSE;
+				nr_pixblock_release (&cb);
 			}
 			break;
 		default:
 			break;
 		}
-		nr_buffer_free (m);
+		nr_pixblock_release (&m);
+	}
+
+	return ret;
+}
+
+static unsigned int
+nr_arena_glyphs_group_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
+{
+	NRArenaGroup *group;
+	NRArenaGlyphsGroup *ggroup;
+	NRArenaItem *child;
+	guint ret;
+
+	group = NR_ARENA_GROUP (item);
+	ggroup = NR_ARENA_GLYPHS_GROUP (item);
+
+	ret = item->state;
+
+	/* Render children fill mask */
+	for (child = group->children; child != NULL; child = child->next) {
+		ret = nr_arena_glyphs_fill_mask (NR_ARENA_GLYPHS (child), area, pb);
+		if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) return ret;
 	}
 
 	return ret;
 }
 
 static NRArenaItem *
-nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky)
+nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, unsigned int sticky)
 {
 	NRArenaGroup *group;
 	NRArenaItem *picked;
@@ -643,8 +646,8 @@ nr_arena_glyphs_group_set_style (NRArenaGlyphsGroup *sg, SPStyle *style)
 	NRArenaGroup *group;
 	NRArenaItem *child;
 
-	g_return_if_fail (sg != NULL);
-	g_return_if_fail (NR_IS_ARENA_GLYPHS_GROUP (sg));
+	nr_return_if_fail (sg != NULL);
+	nr_return_if_fail (NR_IS_ARENA_GLYPHS_GROUP (sg));
 
 	group = NR_ARENA_GROUP (sg);
 
@@ -653,7 +656,7 @@ nr_arena_glyphs_group_set_style (NRArenaGlyphsGroup *sg, SPStyle *style)
 	sg->style = style;
 
 	for (child = group->children; child != NULL; child = child->next) {
-		g_return_if_fail (NR_IS_ARENA_GLYPHS (child));
+		nr_return_if_fail (NR_IS_ARENA_GLYPHS (child));
 		nr_arena_glyphs_set_style (NR_ARENA_GLYPHS (child), sg->style);
 	}
 
@@ -663,18 +666,19 @@ nr_arena_glyphs_group_set_style (NRArenaGlyphsGroup *sg, SPStyle *style)
 void
 nr_arena_glyphs_group_set_paintbox (NRArenaGlyphsGroup *gg, const ArtDRect *pbox)
 {
-	g_return_if_fail (gg != NULL);
-	g_return_if_fail (NR_IS_ARENA_GLYPHS_GROUP (gg));
-	g_return_if_fail (pbox != NULL);
-	g_return_if_fail (pbox->x1 > pbox->x0);
-	g_return_if_fail (pbox->y1 > pbox->y0);
+	nr_return_if_fail (gg != NULL);
+	nr_return_if_fail (NR_IS_ARENA_GLYPHS_GROUP (gg));
+	nr_return_if_fail (pbox != NULL);
 
 	if ((pbox->x0 < pbox->x1) && (pbox->y0 < pbox->y1)) {
-		memcpy (&gg->paintbox, pbox, sizeof (ArtDRect));
+		gg->paintbox.x0 = pbox->x0;
+		gg->paintbox.y0 = pbox->y0;
+		gg->paintbox.x1 = pbox->x1;
+		gg->paintbox.y1 = pbox->y1;
 	} else {
 		/* fixme: We kill warning, although not sure what to do here (Lauris) */
-		gg->paintbox.x0 = gg->paintbox.y0 = 0.0;
-		gg->paintbox.x1 = gg->paintbox.y1 = 256.0;
+		gg->paintbox.x0 = gg->paintbox.y0 = 0.0F;
+		gg->paintbox.x1 = gg->paintbox.y1 = 256.0F;
 	}
 
 	nr_arena_item_request_update (NR_ARENA_ITEM (gg), NR_ARENA_ITEM_STATE_ALL, FALSE);
