@@ -14,38 +14,40 @@
  */
 
 #define NR_TYPE_ARENA_SHAPE (nr_arena_shape_get_type ())
-#define NR_ARENA_SHAPE(obj) (GTK_CHECK_CAST ((obj), NR_TYPE_ARENA_SHAPE, NRArenaShape))
-#define NR_ARENA_SHAPE_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), NR_TYPE_ARENA_SHAPE, NRArenaShapeClass))
-#define NR_IS_ARENA_SHAPE(obj) (GTK_CHECK_TYPE ((obj), NR_TYPE_ARENA_SHAPE))
-#define NR_IS_ARENA_SHAPE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), NR_TYPE_ARENA_SHAPE))
+#define NR_ARENA_SHAPE(obj) (NR_CHECK_INSTANCE_CAST ((obj), NR_TYPE_ARENA_SHAPE, NRArenaShape))
+#define NR_IS_ARENA_SHAPE(obj) (NR_CHECK_INSTANCE_TYPE ((obj), NR_TYPE_ARENA_SHAPE))
 
+#include <libnr/nr-svp.h>
+
+#include "helper/curve.h"
+#include "forward.h"
+#include "sp-paint-server.h"
 #include "nr-arena-item.h"
-#include "cpath-component.h"
-#include "../forward.h"
-#include "../sp-paint-server.h"
 
 struct _NRArenaShape {
 	NRArenaItem item;
 	/* Shape data */
 	SPCurve *curve;
 	SPStyle *style;
-	ArtDRect paintbox;
+	NRRectF paintbox;
 	/* State data */
 	NRMatrixD ctm;
 	SPPainter *fill_painter;
 	SPPainter *stroke_painter;
-	ArtSVP *fill_svp;
-	ArtSVP *stroke_svp;
+	NRSVP *fill_svp;
+	NRSVP *stroke_svp;
+	/* Markers */
+	NRArenaItem *markers;
 };
 
 struct _NRArenaShapeClass {
 	NRArenaItemClass parent_class;
 };
 
-GtkType nr_arena_shape_get_type (void);
+unsigned int nr_arena_shape_get_type (void);
 
-void nr_arena_shape_set_path (NRArenaShape *shape, SPCurve *curve, gboolean private, const gdouble *affine);
+void nr_arena_shape_set_path (NRArenaShape *shape, SPCurve *curve, unsigned int private, const double *affine);
 void nr_arena_shape_set_style (NRArenaShape *shape, SPStyle *style);
-void nr_arena_shape_set_paintbox (NRArenaShape *shape, const ArtDRect *pbox);
+void nr_arena_shape_set_paintbox (NRArenaShape *shape, const NRRectF *pbox);
 
 #endif
