@@ -15,20 +15,22 @@
 #define noSP_DOCUMENT_DEBUG_IDLE
 #define noSP_DOCUMENT_DEBUG_UNDO
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
+#include <glib.h>
 #include <gtk/gtkmain.h>
-#include <gtk/gtksignal.h>
 #include <xml/repr.h>
-#include "marshal.h"
+#include "helper/sp-marshal.h"
+#include "helper/sp-intl.h"
 #include "sodipodi-private.h"
 #include "sp-object-repr.h"
 #include "sp-root.h"
 #include "sp-namedview.h"
 #include "document-private.h"
 #include "desktop.h"
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 
 #define SP_NAMESPACE_SVG "http://www.w3.org/2000/svg"
 #define SP_NAMESPACE_SODIPODI "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
@@ -37,7 +39,7 @@
 #define A4_WIDTH_STR "210mm"
 #define A4_HEIGHT_STR "297mm"
 
-#define SP_DOCUMENT_UPDATE_PRIORITY 10
+#define SP_DOCUMENT_UPDATE_PRIORITY (G_PRIORITY_HIGH_IDLE - 1)
 
 enum {
 	MODIFIED,
@@ -47,8 +49,8 @@ enum {
 };
 
 static void sp_document_class_init (SPDocumentClass * klass);
-static void sp_document_init (SPDocument * document);
-static void sp_document_destroy (GtkObject * object);
+static void sp_document_init (SPDocument *document);
+static void sp_document_dispose (GObject *object);
 
 static gint sp_document_idle_handler (gpointer data);
 
@@ -58,58 +60,63 @@ gboolean sp_document_resource_list_free (gpointer key, gpointer value, gpointer
 static gboolean sp_document_warn_undo_stack (SPDocument *doc);
 #endif
 
-static GtkObjectClass * parent_class;
-static guint signals[LAST_SIGNAL] = { 0 };
+static GObjectClass * parent_class;
+static guint signals[LAST_SIGNAL] = {0};
 static gint doc_count = 0;
 
-GtkType
+GType
 sp_document_get_type (void)
 {
-	static GtkType document_type = 0;
-	if (!document_type) {
-		GtkTypeInfo document_info = {
-			"SPDocument",
-			sizeof (SPDocument),
+	static GType type = 0;
+	if (!type) {
+		GTypeInfo info = {
 			sizeof (SPDocumentClass),
-			(GtkClassInitFunc) sp_document_class_init,
-			(GtkObjectInitFunc) sp_document_init,
-			NULL, NULL, NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_document_class_init,
+			NULL, NULL,
+			sizeof (SPDocument),
+			4,
+			(GInstanceInitFunc) sp_document_init,
 		};
-		document_type = gtk_type_unique (gtk_object_get_type (), &document_info);
+		type = g_type_register_static (G_TYPE_OBJECT, "SPDocument", &info, 0);
 	}
-	return document_type;
+	return type;
 }
 
 static void
 sp_document_class_init (SPDocumentClass * klass)
 {
-	GtkObjectClass * object_class;
+	GObjectClass * object_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 
-	parent_class = gtk_type_class (GTK_TYPE_OBJECT);
+	parent_class = g_type_class_peek_parent (klass);
 
-	signals[MODIFIED] = gtk_signal_new ("modified",
-					    GTK_RUN_FIRST,
-					    object_class->type,
-					    GTK_SIGNAL_OFFSET (SPDocumentClass, modified),
-					    gtk_marshal_NONE__UINT,
-					    GTK_TYPE_NONE, 1, GTK_TYPE_UINT);
-	signals[URI_SET] =  gtk_signal_new ("uri_set",
-					    GTK_RUN_FIRST,
-					    object_class->type,
-					    GTK_SIGNAL_OFFSET (SPDocumentClass, uri_set),
-					    gtk_marshal_NONE__STRING,
-					    GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
-	signals[RESIZED] =  gtk_signal_new ("resized",
-					    GTK_RUN_FIRST,
-					    object_class->type,
-					    GTK_SIGNAL_OFFSET (SPDocumentClass, resized),
+	signals[MODIFIED] = g_signal_new ("modified",
+					    G_TYPE_FROM_CLASS(klass),
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (SPDocumentClass, modified),
+					    NULL, NULL,
+					    sp_marshal_NONE__UINT,
+					    G_TYPE_NONE, 1,
+					    G_TYPE_UINT);
+	signals[URI_SET] =    g_signal_new ("uri_set",
+					    G_TYPE_FROM_CLASS(klass),
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (SPDocumentClass, uri_set),
+					    NULL, NULL,
+					    sp_marshal_NONE__POINTER,
+					    G_TYPE_NONE, 1,
+					    G_TYPE_POINTER);
+	signals[RESIZED] =    g_signal_new ("resized",
+					    G_TYPE_FROM_CLASS(klass),
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (SPDocumentClass, uri_set),
+					    NULL, NULL,
 					    sp_marshal_NONE__DOUBLE_DOUBLE,
-					    GTK_TYPE_NONE, 2, GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
-
-	object_class->destroy = sp_document_destroy;
+					    G_TYPE_NONE, 2,
+					    G_TYPE_DOUBLE, G_TYPE_DOUBLE);
+	object_class->dispose = sp_document_dispose;
 }
 
 static void
@@ -117,7 +124,8 @@ sp_document_init (SPDocument *doc)
 {
 	SPDocumentPrivate *p;
 
-	doc->public = FALSE;
+	doc->advertize = FALSE;
+	doc->keepalive = FALSE;
 
 	doc->modified_id = 0;
 
@@ -131,53 +139,55 @@ sp_document_init (SPDocument *doc)
 
 	p = g_new (SPDocumentPrivate, 1);
 
-
 	p->iddef = g_hash_table_new (g_str_hash, g_str_equal);
 
-	p->aspect = SPXMidYMid;
-	p->clip = FALSE;
-
 	p->resources = g_hash_table_new (g_str_hash, g_str_equal);
 
 	p->sensitive = FALSE;
+	p->partial = NULL;
+	p->undo_size = 0;
 	p->undo = NULL;
 	p->redo = NULL;
-	p->actions = NULL;
 
-	doc->private = p;
+	doc->priv = p;
 }
 
 static void
-sp_document_destroy (GtkObject *object)
+sp_document_dispose (GObject *object)
 {
 	SPDocument *doc;
-	SPDocumentPrivate * private;
+	SPDocumentPrivate * priv;
 
 	doc = (SPDocument *) object;
-	private = doc->private;
+	priv = doc->priv;
 
-	if (private) {
+	if (priv) {
 		sodipodi_remove_document (doc);
 
-		if (private->actions) {
-			sp_action_free_list (private->actions);
+		if (priv->partial) {
+			sp_repr_free_log (priv->partial);
+			priv->partial = NULL;
 		}
 
 		sp_document_clear_redo (doc);
 		sp_document_clear_undo (doc);
 
-		if (doc->root) gtk_object_unref (GTK_OBJECT (doc->root));
+		if (doc->root) {
+			sp_object_invoke_release (doc->root);
+			g_object_unref (G_OBJECT (doc->root));
+			doc->root = NULL;
+		}
 
-		if (private->iddef) g_hash_table_destroy (private->iddef);
+		if (priv->iddef) g_hash_table_destroy (priv->iddef);
 
 		if (doc->rdoc) sp_repr_document_unref (doc->rdoc);
 
 		/* Free resources */
-		g_hash_table_foreach_remove (private->resources, sp_document_resource_list_free, doc);
-		g_hash_table_destroy (private->resources);
+		g_hash_table_foreach_remove (priv->resources, sp_document_resource_list_free, doc);
+		g_hash_table_destroy (priv->resources);
 
-		g_free (private);
-		doc->private = NULL;
+		g_free (priv);
+		doc->priv = NULL;
 	}
 
 	if (doc->name) {
@@ -193,68 +203,52 @@ sp_document_destroy (GtkObject *object)
 		doc->uri = NULL;
 	}
 
-	if (doc->modified_id) gtk_idle_remove (doc->modified_id);
+	if (doc->modified_id) {
+		gtk_idle_remove (doc->modified_id);
+		doc->modified_id = 0;
+	}
 
-	sodipodi_unref ();
+	if (doc->keepalive) {
+		sodipodi_unref ();
+		doc->keepalive = FALSE;
+	}
 
-	if (((GtkObjectClass *) (parent_class))->destroy)
-		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-SPDocument *
-sp_document_new (const gchar *uri, gboolean public)
+static SPDocument *
+sp_document_create (SPReprDoc *rdoc,
+		    const unsigned char *uri,
+		    const unsigned char *base,
+		    const unsigned char *name,
+		    unsigned int advertize,
+		    unsigned int keepalive)
 {
 	SPDocument *document;
-	SPReprDoc *rdoc;
 	SPRepr *rroot;
 	guint version;
 
-	if (uri != NULL) {
-		/* Try to fetch repr from file */
-		rdoc = sp_repr_read_file (uri);
-		/* If file cannot be loaded, return NULL without warning */
-		if (rdoc == NULL) return NULL;
-		rroot = sp_repr_document_root (rdoc);
-		/* If xml file is not svg, return NULL without warning */
-		/* fixme: destroy document */
-		if (strcmp (sp_repr_name (rroot), "svg") != 0) return NULL;
-	} else {
-		rdoc = sp_repr_document_new ("svg");
-		rroot = sp_repr_document_root (rdoc);
-	}
+	rroot = sp_repr_document_root (rdoc);
 
-	document = gtk_type_new (SP_TYPE_DOCUMENT);
-	g_return_val_if_fail (document != NULL, NULL);
+	document = g_object_new (SP_TYPE_DOCUMENT, NULL);
 
-	document->public = public;
+	document->advertize = advertize;
+	document->keepalive = keepalive;
 
 	document->rdoc = rdoc;
 	document->rroot = rroot;
 
-	if (uri) {
-		guchar *s, *p;
-		document->uri = g_strdup (uri);
-		/* fixme: Think, what this means for images (Lauris) */
-		s = g_strdup (uri);
-		p = strrchr (s, '/');
-		if (p) {
-			document->name = g_strdup (p + 1);
-			p[1] = '\0';
-			document->base = g_strdup (s);
-		} else {
-			document->base = NULL;
-			document->name = g_strdup (document->uri);
-		}
-		g_free (s);
-	} else {
-		document->uri = NULL;
-		document->base = NULL;
-		document->name = g_strdup_printf (_("New document %d"), ++doc_count);
-	}
+	document->uri = g_strdup (uri);
+	document->base = g_strdup (base);
+	document->name = g_strdup (name);
 
 	document->root = sp_object_repr_build_tree (document, rroot);
-	g_return_val_if_fail (document->root != NULL, NULL);
-	g_return_val_if_fail (SP_IS_ROOT (document->root), NULL);
+
+#if 0
+	/* fixme: Is this correct here? (Lauris) */
+	sp_document_ensure_up_to_date (document);
+#endif
+
 	version = SP_ROOT (document->root)->sodipodi;
 
 	/* fixme: Not sure about this, but lets assume ::build updates */
@@ -285,14 +279,13 @@ sp_document_new (const gchar *uri, gboolean public)
 	}
 
 	/* Namedviews */
-	if (!SP_ROOT (document->root)->namedviews) {
+	if (!sp_item_group_get_child_by_name ((SPGroup *) document->root, NULL, "sodipodi:namedview")) {
 		SPRepr *r;
 		r = sodipodi_get_repr (SODIPODI, "template.sodipodi:namedview");
 		if (!r) r = sp_repr_new ("sodipodi:namedview");
 		sp_repr_set_attr (r, "id", "base");
 		sp_repr_add_child (rroot, r, NULL);
 		sp_repr_unref (r);
-		g_assert (SP_ROOT (document->root)->namedviews);
 	}
 
 	/* Defs */
@@ -304,7 +297,9 @@ sp_document_new (const gchar *uri, gboolean public)
 		g_assert (SP_ROOT (document->root)->defs);
 	}
 
-	sodipodi_ref ();
+	if (keepalive) {
+		sodipodi_ref ();
+	}
 
 	sp_document_set_undo_sensitive (document, TRUE);
 
@@ -314,90 +309,71 @@ sp_document_new (const gchar *uri, gboolean public)
 }
 
 SPDocument *
-sp_document_new_from_mem (const gchar *buffer, gint length, gboolean public)
+sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive)
 {
-	SPDocument *document;
+	SPDocument *doc;
+	SPReprDoc *rdoc;
+	unsigned char *base, *name;
+
+	if (uri) {
+		SPRepr *rroot;
+		unsigned char *s, *p;
+		/* Try to fetch repr from file */
+		rdoc = sp_repr_read_file (uri, SP_SVG_NS_URI);
+		/* If file cannot be loaded, return NULL without warning */
+		if (rdoc == NULL) return NULL;
+		rroot = sp_repr_document_root (rdoc);
+		/* If xml file is not svg, return NULL without warning */
+		/* fixme: destroy document */
+		if (strcmp (sp_repr_name (rroot), "svg") != 0) return NULL;
+		s = g_strdup (uri);
+		p = strrchr (s, '/');
+		if (p) {
+			name = g_strdup (p + 1);
+			p[1] = '\0';
+			base = g_strdup (s);
+		} else {
+			base = NULL;
+			name = g_strdup (uri);
+		}
+		g_free (s);
+	} else {
+		rdoc = sp_repr_document_new ("svg");
+		base = NULL;
+		name = g_strdup_printf (_("New document %d"), ++doc_count);
+	}
+
+	doc = sp_document_create (rdoc, uri, base, name, advertize, keepalive);
+
+	if (base) g_free (base);
+	if (name) g_free (name);
+
+	return doc;
+}
+
+SPDocument *
+sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int advertize, unsigned int keepalive)
+{
+	SPDocument *doc;
 	SPReprDoc *rdoc;
 	SPRepr *rroot;
-	guint version;
+	unsigned char *name;
 
-	rdoc = sp_repr_read_mem (buffer, length);
+	rdoc = sp_repr_read_mem (buffer, length, SP_SVG_NS_URI);
 
 	/* If it cannot be loaded, return NULL without warning */
 	if (rdoc == NULL) return NULL;
 
 	rroot = sp_repr_document_root (rdoc);
-
 	/* If xml file is not svg, return NULL without warning */
 	/* fixme: destroy document */
 	if (strcmp (sp_repr_name (rroot), "svg") != 0) return NULL;
 
-	document = gtk_type_new (SP_TYPE_DOCUMENT);
-	g_return_val_if_fail (document != NULL, NULL);
+	name = g_strdup_printf (_("Memory document %d"), ++doc_count);
 
-	document->public = public;
+	doc = sp_document_create (rdoc, NULL, NULL, name, advertize, keepalive);
 
-	document->rdoc = rdoc;
-	document->rroot = rroot;
-
-	document->uri = NULL;
-	document->base = NULL;
-	document->name = g_strdup_printf (_("Memory document %d"), ++doc_count);
-
-	document->root = sp_object_repr_build_tree (document, rroot);
-	g_return_val_if_fail (document->root != NULL, NULL);
-	g_return_val_if_fail (SP_IS_ROOT (document->root), NULL);
-	version = SP_ROOT (document->root)->sodipodi;
-
-	/* fixme: docbase */
-
-	/* fixme: Not sure about this, but lets assume ::build updates */
-	sp_repr_set_attr (rroot, "sodipodi:version", VERSION);
-	/* fixme: Again, I moved these here to allow version determining in ::build (Lauris) */
-	/* A quick hack to get namespaces into doc */
-	sp_repr_set_attr (rroot, "xmlns", SP_NAMESPACE_SVG);
-	sp_repr_set_attr (rroot, "xmlns:sodipodi", SP_NAMESPACE_SODIPODI);
-	sp_repr_set_attr (rroot, "xmlns:xlink", SP_NAMESPACE_XLINK);
-	/* End of quick hack */
-	/* Quick hack 2 - get default image size into document */
-	if (!sp_repr_attr (rroot, "width")) sp_repr_set_attr (rroot, "width", A4_WIDTH_STR);
-	if (!sp_repr_attr (rroot, "height")) sp_repr_set_attr (rroot, "height", A4_HEIGHT_STR);
-	/* End of quick hack 2 */
-	if ((version > 0) && (version < 25)) {
-		/* Clear ancient spec violating attributes */
-		sp_repr_set_attr (rroot, "SP-DOCNAME", NULL);
-		sp_repr_set_attr (rroot, "SP-DOCBASE", NULL);
-		sp_repr_set_attr (rroot, "docname", NULL);
-		sp_repr_set_attr (rroot, "docbase", NULL);
-	}
-
-	/* Namedviews */
-	if (!SP_ROOT (document->root)->namedviews) {
-		SPRepr *r;
-		r = sodipodi_get_repr (SODIPODI, "template.sodipodi:namedview");
-		if (!r) r = sp_repr_new ("sodipodi:namedview");
-		sp_repr_set_attr (r, "id", "base");
-		sp_repr_add_child (rroot, r, 0);
-		sp_repr_unref (r);
-		g_assert (SP_ROOT (document->root)->namedviews);
-	}
-
-	/* Defs */
-	if (!SP_ROOT (document->root)->defs) {
-		SPRepr *r;
-		r = sp_repr_new ("defs");
-		sp_repr_add_child (rroot, r, NULL);
-		sp_repr_unref (r);
-		g_assert (SP_ROOT (document->root)->defs);
-	}
-
-	sodipodi_ref ();
-
-	sp_document_set_undo_sensitive (document, TRUE);
-
-	sodipodi_add_document (document);
-
-	return document;
+	return doc;
 }
 
 SPDocument *
@@ -406,7 +382,7 @@ sp_document_ref (SPDocument *doc)
 	g_return_val_if_fail (doc != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
 
-	gtk_object_ref (GTK_OBJECT (doc));
+	g_object_ref (G_OBJECT (doc));
 
 	return doc;
 }
@@ -417,7 +393,7 @@ sp_document_unref (SPDocument *doc)
 	g_return_val_if_fail (doc != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
 
-	gtk_object_unref (GTK_OBJECT (doc));
+	g_object_unref (G_OBJECT (doc));
 
 	return NULL;
 }
@@ -427,7 +403,7 @@ sp_document_width (SPDocument * document)
 {
 	g_return_val_if_fail (document != NULL, 0.0);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), 0.0);
-	g_return_val_if_fail (document->private != NULL, 0.0);
+	g_return_val_if_fail (document->priv != NULL, 0.0);
 	g_return_val_if_fail (document->root != NULL, 0.0);
 
 	return SP_ROOT (document->root)->width.computed / 1.25;
@@ -438,7 +414,7 @@ sp_document_height (SPDocument * document)
 {
 	g_return_val_if_fail (document != NULL, 0.0);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), 0.0);
-	g_return_val_if_fail (document->private != NULL, 0.0);
+	g_return_val_if_fail (document->priv != NULL, 0.0);
 	g_return_val_if_fail (document->root != NULL, 0.0);
 
 	return SP_ROOT (document->root)->height.computed / 1.25;
@@ -484,7 +460,7 @@ sp_document_set_uri (SPDocument *doc, const guchar *uri)
 		doc->name = g_strdup (doc->uri);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (doc), signals [URI_SET], doc->uri);
+	g_signal_emit (G_OBJECT (doc), signals [URI_SET], 0, doc->uri);
 }
 
 void
@@ -495,9 +471,10 @@ sp_document_set_size_px (SPDocument *doc, gdouble width, gdouble height)
 	g_return_if_fail (width > 0.001);
 	g_return_if_fail (height > 0.001);
 
-	gtk_signal_emit (GTK_OBJECT (doc), signals [RESIZED], width / 1.25, height / 1.25);
+	g_signal_emit (G_OBJECT (doc), signals [RESIZED], 0, width / 1.25, height / 1.25);
 }
 
+#if 0
 /* named views */
 
 const GSList *
@@ -529,21 +506,22 @@ sp_document_namedview (SPDocument * document, const gchar * id)
 
 	return NULL;
 }
+#endif
 
 void
 sp_document_def_id (SPDocument * document, const gchar * id, SPObject * object)
 {
-	g_assert (g_hash_table_lookup (document->private->iddef, id) == NULL);
+	g_assert (g_hash_table_lookup (document->priv->iddef, id) == NULL);
 
-	g_hash_table_insert (document->private->iddef, (gchar *) id, object);
+	g_hash_table_insert (document->priv->iddef, (gchar *) id, object);
 }
 
 void
 sp_document_undef_id (SPDocument * document, const gchar * id)
 {
-	g_assert (g_hash_table_lookup (document->private->iddef, id) != NULL);
+	g_assert (g_hash_table_lookup (document->priv->iddef, id) != NULL);
 
-	g_hash_table_remove (document->private->iddef, id);
+	g_hash_table_remove (document->priv->iddef, id);
 }
 
 SPObject *
@@ -553,7 +531,7 @@ sp_document_lookup_id (SPDocument *doc, const gchar *id)
 	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
 	g_return_val_if_fail (id != NULL, NULL);
 
-	return g_hash_table_lookup (doc->private->iddef, id);
+	return g_hash_table_lookup (doc->priv->iddef, id);
 }
 
 /* Object modification root handler */
@@ -569,66 +547,97 @@ sp_document_request_modified (SPDocument *doc)
 gint
 sp_document_ensure_up_to_date (SPDocument *doc)
 {
+	int lc;
+	lc = 16;
+	while (doc->root->uflags || doc->root->mflags) {
+		lc -= 1;
+		if (lc < 0) {
+			g_warning ("More than 16 iterations while updating document '%s'", doc->uri);
+			if (doc->modified_id) {
+				/* Remove handler */
+				gtk_idle_remove (doc->modified_id);
+				doc->modified_id = 0;
+			}
+			return FALSE;
+		}
+		/* Process updates */
+		if (doc->root->uflags) {
+			SPItemCtx ctx;
+			ctx.ctx.flags = 0;
+			nr_matrix_d_set_identity (&ctx.i2doc);
+			/* Set up viewport in case svg has it defined as percentages */
+			ctx.vp.x0 = 0.0;
+			ctx.vp.y0 = 0.0;
+			ctx.vp.x1 = 21.0 / 2.54 * 72.0 * 1.25;
+			ctx.vp.y1 = 29.7 / 2.54 * 72.0 * 1.25;
+			nr_matrix_d_set_identity (&ctx.i2vp);
+			sp_object_invoke_update (doc->root, (SPCtx *) &ctx, 0);
+		}
+		/* Emit "modified" signal on objects */
+		sp_object_invoke_modified (doc->root, 0);
+		/* Emit our own "modified" signal */
+		g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
+			       SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
+	}
 	if (doc->modified_id) {
 		/* Remove handler */
 		gtk_idle_remove (doc->modified_id);
 		doc->modified_id = 0;
-		/* Emit "modified" signal on objects */
-		sp_object_modified (doc->root, 0);
-		/* Emit our own "modified" signal */
-		gtk_signal_emit (GTK_OBJECT (doc), signals [MODIFIED],
-				 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
-		return TRUE;
 	}
-
-	return FALSE;
+	return TRUE;
 }
 
 static gint
 sp_document_idle_handler (gpointer data)
 {
 	SPDocument *doc;
+	int repeat;
 
 	doc = SP_DOCUMENT (data);
 
-	doc->modified_id = 0;
-
-#ifdef SP_DOCUMENT_DEBUG_UNDO
-	/* ------------------------- */
-	if (doc->private->actions) {
-		static gboolean warn = TRUE;
-		if (warn) {
-			warn = sp_document_warn_undo_stack (doc);
-		}
-	}
-	/* ------------------------- */
-#endif
-
 #ifdef SP_DOCUMENT_DEBUG_IDLE
 	g_print ("->\n");
 #endif
 
+	/* Process updates */
+	if (doc->root->uflags) {
+		SPItemCtx ctx;
+		ctx.ctx.flags = 0;
+		nr_matrix_d_set_identity (&ctx.i2doc);
+		/* Set up viewport in case svg has it defined as percentages */
+		ctx.vp.x0 = 0.0;
+		ctx.vp.y0 = 0.0;
+		ctx.vp.x1 = 21.0 / 2.54 * 72.0 * 1.25;
+		ctx.vp.y1 = 29.7 / 2.54 * 72.0 * 1.25;
+		nr_matrix_d_set_identity (&ctx.i2vp);
+		sp_object_invoke_update (doc->root, (SPCtx *) &ctx, 0);
+		/* if (doc->root->uflags & SP_OBJECT_MODIFIED_FLAG) return TRUE; */
+	}
+
 	/* Emit "modified" signal on objects */
-	sp_object_modified (doc->root, 0);
+	sp_object_invoke_modified (doc->root, 0);
 
 #ifdef SP_DOCUMENT_DEBUG_IDLE
 	g_print ("\n->");
 #endif
 
 	/* Emit our own "modified" signal */
-	gtk_signal_emit (GTK_OBJECT (doc), signals [MODIFIED],
+	g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
 			 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
 
 #ifdef SP_DOCUMENT_DEBUG_IDLE
 	g_print (" S ->\n");
 #endif
-	return FALSE;
+
+	repeat = (doc->root->uflags || doc->root->mflags);
+	if (!repeat) doc->modified_id = 0;
+	return repeat;
 }
 
 SPObject *
 sp_document_add_repr (SPDocument *document, SPRepr *repr)
 {
-	GtkType type;
+	GType type;
 
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
@@ -636,15 +645,29 @@ sp_document_add_repr (SPDocument *document, SPRepr *repr)
 
 	type = sp_repr_type_lookup (repr);
 
-	if (gtk_type_is_a (type, SP_TYPE_ITEM)) {
-		sp_repr_append_child (document->rroot, repr);
-	} else if (gtk_type_is_a (type, SP_TYPE_OBJECT)) {
-		sp_repr_append_child (SP_OBJECT_REPR (SP_ROOT (document->root)->defs), repr);
+	if (g_type_is_a (type, SP_TYPE_ITEM)) {
+		sp_repr_append_child (sp_document_repr_root(document), repr);
+	} else if (g_type_is_a (type, SP_TYPE_OBJECT)) {
+		sp_repr_append_child (SP_OBJECT_REPR (SP_DOCUMENT_DEFS(document)), repr);
 	}
 
 	return sp_document_lookup_id (document, sp_repr_attr (repr, "id"));
 }
 
+/* Returns the sequence number of object */
+
+unsigned int
+sp_document_object_sequence_get (SPDocument *doc, SPObject *object)
+{
+	unsigned int seq;
+
+	seq = 0;
+
+	sp_object_invoke_sequence (doc->root, object, &seq);
+
+	return seq;
+}
+
 /*
  * Return list of items, contained in box
  *
@@ -653,24 +676,24 @@ sp_document_add_repr (SPDocument *document, SPRepr *repr)
  */
 
 GSList *
-sp_document_items_in_box (SPDocument * document, ArtDRect * box)
+sp_document_items_in_box (SPDocument *document, NRRectD *box)
 {
 	SPGroup * group;
 	SPItem * child;
 	SPObject * o;
-	ArtDRect b;
+	NRRectF b;
 	GSList * s;
 
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (document->private != NULL, NULL);
+	g_return_val_if_fail (document->priv != NULL, NULL);
 	g_return_val_if_fail (box != NULL, NULL);
 
 	group = SP_GROUP (document->root);
 
 	s = NULL;
 
-	for (o = group->children; o != NULL; o = o->next) {
+	for (o = ((SPObject *) group)->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
 			child = SP_ITEM (o);
 			sp_item_bbox_desktop (child, &b);
@@ -692,24 +715,24 @@ sp_document_items_in_box (SPDocument * document, ArtDRect * box)
  */
 
 GSList *
-sp_document_partial_items_in_box (SPDocument * document, ArtDRect * box)
+sp_document_partial_items_in_box (SPDocument *document, NRRectD *box)
 {
 	SPGroup * group;
 	SPItem * child;
 	SPObject * o;
-	ArtDRect b;
+	NRRectF b;
 	GSList * s;
 
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (document->private != NULL, NULL);
+	g_return_val_if_fail (document->priv != NULL, NULL);
 	g_return_val_if_fail (box != NULL, NULL);
 
 	group = SP_GROUP (document->root);
 
 	s = NULL;
 
-	for (o = group->children; o != NULL; o = o->next) {
+	for (o = ((SPObject *) group)->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
 			child = SP_ITEM (o);
 			sp_item_bbox_desktop (child, &b);
@@ -740,10 +763,10 @@ sp_document_add_resource (SPDocument *document, const guchar *key, SPObject *obj
 	g_return_val_if_fail (object != NULL, FALSE);
 	g_return_val_if_fail (SP_IS_OBJECT (object), FALSE);
 
-	rlist = g_hash_table_lookup (document->private->resources, key);
+	rlist = g_hash_table_lookup (document->priv->resources, key);
 	g_return_val_if_fail (!g_slist_find (rlist, object), FALSE);
 	rlist = g_slist_prepend (rlist, object);
-	g_hash_table_insert (document->private->resources, (gpointer) key, rlist);
+	g_hash_table_insert (document->priv->resources, (gpointer) key, rlist);
 
 	return TRUE;
 }
@@ -760,11 +783,11 @@ sp_document_remove_resource (SPDocument *document, const guchar *key, SPObject *
 	g_return_val_if_fail (object != NULL, FALSE);
 	g_return_val_if_fail (SP_IS_OBJECT (object), FALSE);
 
-	rlist = g_hash_table_lookup (document->private->resources, key);
+	rlist = g_hash_table_lookup (document->priv->resources, key);
 	g_return_val_if_fail (rlist != NULL, FALSE);
 	g_return_val_if_fail (g_slist_find (rlist, object), FALSE);
 	rlist = g_slist_remove (rlist, object);
-	g_hash_table_insert (document->private->resources, (gpointer) key, rlist);
+	g_hash_table_insert (document->priv->resources, (gpointer) key, rlist);
 
 	return TRUE;
 }
@@ -777,7 +800,7 @@ sp_document_get_resource_list (SPDocument *document, const guchar *key)
 	g_return_val_if_fail (key != NULL, NULL);
 	g_return_val_if_fail (*key != '\0', NULL);
 
-	return g_hash_table_lookup (document->private->resources, key);
+	return g_hash_table_lookup (document->priv->resources, key);
 }
 
 /* Helpers */
@@ -789,98 +812,28 @@ sp_document_resource_list_free (gpointer key, gpointer value, gpointer data)
 	return TRUE;
 }
 
-#ifdef SP_DOCUMENT_DEBUG_UNDO
-
-#include <libgnomeui/gnome-stock.h>
-#include <libgnomeui/gnome-dialog.h>
-
-static GSList *
-sp_action_print_pending_list (SPAction *action)
+guint
+sp_document_get_version (SPDocument *document, guint version_type)
 {
-	GSList *al;
-	gchar *s;
+	SPRoot *root;
 
-	al = NULL;
+	g_return_val_if_fail (document != NULL, 0);
+	g_return_val_if_fail (SP_IS_DOCUMENT (document), 0);
+	g_return_val_if_fail (document->root != NULL, 0);
 
-	while (action) {
-		s = g_strdup_printf ("SPAction: Id %s\n", action->id);
-		al = g_slist_prepend (al, s);
-		switch (action->type) {
-		case SP_ACTION_ADD:
-			s = g_strdup_printf ("SPAction: Add ref %s\n", action->act.add.ref);
-			al = g_slist_prepend (al, s);
-			break;
-		case SP_ACTION_DEL:
-			s = g_strdup_printf ("SPAction: Del ref %s\n", action->act.del.ref);
-			al = g_slist_prepend (al, s);
-			break;
-		case SP_ACTION_CHGATTR:
-			s = g_strdup_printf ("SPAction: ChgAttr %s: %s -> %s\n",
-				 g_quark_to_string (action->act.chgattr.key),
-				 action->act.chgattr.oldval,
-				 action->act.chgattr.newval);
-			al = g_slist_prepend (al, s);
-			break;
-		case SP_ACTION_CHGCONTENT:
-			s = g_strdup_printf ("SPAction: ChgContent %s -> %s\n",
-				 action->act.chgcontent.oldval,
-				 action->act.chgcontent.newval);
-			al = g_slist_prepend (al, s);
-			break;
-		case SP_ACTION_CHGORDER:
-			s = g_strdup_printf ("SPAction: ChgOrder %s: %s -> %s\n",
-				 action->act.chgorder.child,
-				 action->act.chgorder.oldref,
-				 action->act.chgorder.newref);
-			al = g_slist_prepend (al, s);
-			break;
-		default:
-			s = g_strdup_printf ("SPAction: Invalid action type %d\n", action->type);
-			al = g_slist_prepend (al, s);
-			break;
-		}
-		action = action->next;
+	root = SP_ROOT (document->root);
+
+	switch (version_type)
+	{
+	case SP_VERSION_SVG:
+		return root->svg;
+	case SP_VERSION_SODIPODI:
+		return root->sodipodi;
+	case SP_VERSION_ORIGINAL:
+		return root->original;
+	default:
+		g_assert_not_reached ();
 	}
 
-	return al;
+	return 0;
 }
-
-static gboolean
-sp_document_warn_undo_stack (SPDocument *doc)
-{
-	GnomeDialog *dlg;
-	GtkWidget *l, *t;
-	GSList *al;
-
-	dlg = (GnomeDialog *) gnome_dialog_new ("Stale undo stack warning", GNOME_STOCK_BUTTON_OK, NULL);
-
-	l = gtk_label_new ("WARNING");
-	gtk_widget_show (l);
-	gtk_box_pack_start (GTK_BOX (dlg->vbox), l, FALSE, FALSE, 8);
-
-	l = gtk_label_new ("Last operation did not flush undo stack");
-	gtk_widget_show (l);
-	gtk_box_pack_start (GTK_BOX (dlg->vbox), l, FALSE, FALSE, 8);
-
-	l = gtk_label_new ("Please report following data to developers");
-	gtk_widget_show (l);
-	gtk_box_pack_start (GTK_BOX (dlg->vbox), l, FALSE, FALSE, 8);
-
-	t = gtk_text_new (NULL, NULL);
-	gtk_widget_show (t);
-	gtk_box_pack_start (GTK_BOX (dlg->vbox), t, FALSE, FALSE, 8);
-
-	al = sp_action_print_pending_list (doc->private->actions);
-	while (al) {
-		gtk_text_set_point (GTK_TEXT (t), 0);
-		gtk_text_insert (GTK_TEXT (t), NULL, NULL, NULL, al->data, strlen (al->data));
-		g_free (al->data);
-		al = g_slist_remove (al, al->data);
-	}
-
-	gnome_dialog_run_and_close (dlg);
-
-	return TRUE;
-}
-
-#endif
