@@ -15,23 +15,26 @@
 
 #define DRAW_VERBOSE
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <math.h>
 #include <string.h>
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
+#include "macros.h"
 #include "xml/repr.h"
 #include "svg/svg.h"
-#include "helper/art-utils.h"
+#include "helper/sp-intl.h"
 #include "helper/curve.h"
 #include "helper/bezier-utils.h"
 #include "helper/sodipodi-ctrl.h"
 #include "helper/sp-ctrlline.h"
 #include "helper/canvas-bpath.h"
 
+#include "enums.h"
 #include "sodipodi.h"
 #include "document.h"
 #include "sp-path.h"
@@ -54,13 +57,14 @@ struct _SPDrawAnchor {
 	SPCurve *curve;
 	guint start : 1;
 	guint active : 1;
-	ArtPoint dp, wp;
+	NRPointF dp;
+	NRPointF wp;
 	SPCanvasItem *ctrl;
 };
 
 static void sp_draw_context_class_init (SPDrawContextClass *klass);
 static void sp_draw_context_init (SPDrawContext *dc);
-static void sp_draw_context_finalize (GtkObject *object);
+static void sp_draw_context_dispose (GObject *object);
 
 static void sp_draw_context_setup (SPEventContext *ec);
 static void sp_draw_context_set (SPEventContext *ec, const guchar *key, const guchar *value);
@@ -96,35 +100,34 @@ static SPEventContextClass *draw_parent_class;
 GtkType
 sp_draw_context_get_type (void)
 {
-	static GtkType type = 0;
-
+	static GType type = 0;
 	if (!type) {
-		static const GtkTypeInfo info = {
-			"SPDrawContext",
-			sizeof (SPDrawContext),
+		GTypeInfo info = {
 			sizeof (SPDrawContextClass),
-			(GtkClassInitFunc) sp_draw_context_class_init,
-			(GtkObjectInitFunc) sp_draw_context_init,
-			NULL, NULL, NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_draw_context_class_init,
+			NULL, NULL,
+			sizeof (SPDrawContext),
+			4,
+			(GInstanceInitFunc) sp_draw_context_init,
 		};
-		type = gtk_type_unique (SP_TYPE_EVENT_CONTEXT, &info);
+		type = g_type_register_static (SP_TYPE_EVENT_CONTEXT, "SPDrawContext", &info, 0);
 	}
-
 	return type;
 }
 
 static void
 sp_draw_context_class_init (SPDrawContextClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	SPEventContextClass *ec_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *)klass;
 	ec_class = (SPEventContextClass *) klass;
 
-	draw_parent_class = gtk_type_class (SP_TYPE_EVENT_CONTEXT);
+	draw_parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = sp_draw_context_finalize;
+	object_class->dispose = sp_draw_context_dispose;
 
 	ec_class->setup = sp_draw_context_setup;
 	ec_class->set = sp_draw_context_set;
@@ -145,21 +148,25 @@ sp_draw_context_init (SPDrawContext *dc)
 }
 
 static void
-sp_draw_context_finalize (GtkObject *object)
+sp_draw_context_dispose (GObject *object)
 {
 	SPDrawContext *dc;
 
 	dc = SP_DRAW_CONTEXT (object);
 
-	if (dc->grab) sp_canvas_item_ungrab (dc->grab, gdk_time_get ());
+	if (dc->grab) {
+		sp_canvas_item_ungrab (dc->grab, GDK_CURRENT_TIME);
+		dc->grab = NULL;
+	}
 
 	if (dc->selection) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (dc->selection), dc);
+		sp_signal_disconnect_by_data (dc->selection, dc);
+		dc->selection = NULL;
 	}
 
 	spdc_free_colors (dc);
 
-	GTK_OBJECT_CLASS (draw_parent_class)->finalize (object);
+	G_OBJECT_CLASS (draw_parent_class)->dispose (object);
 }
 
 static void
@@ -171,24 +178,24 @@ sp_draw_context_setup (SPEventContext *ec)
 	dc = SP_DRAW_CONTEXT (ec);
 	dt = ec->desktop;
 
-	if (SP_EVENT_CONTEXT_CLASS (draw_parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (draw_parent_class)->setup (ec);
+	if (((SPEventContextClass *) draw_parent_class)->setup)
+		((SPEventContextClass *) draw_parent_class)->setup (ec);
 
 	dc->selection = SP_DT_SELECTION (dt);
 
 	/* Connect signals to track selection changes */
-	gtk_signal_connect (GTK_OBJECT (dc->selection), "changed", GTK_SIGNAL_FUNC (spdc_selection_changed), dc);
-	gtk_signal_connect (GTK_OBJECT (dc->selection), "modified", GTK_SIGNAL_FUNC (spdc_selection_modified), dc);
+	g_signal_connect (G_OBJECT (dc->selection), "changed", G_CALLBACK (spdc_selection_changed), dc);
+	g_signal_connect (G_OBJECT (dc->selection), "modified", G_CALLBACK (spdc_selection_modified), dc);
 
 	/* Create red bpath */
 	dc->red_bpath = sp_canvas_bpath_new (SP_DT_SKETCH (ec->desktop), NULL);
-	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (dc->red_bpath), dc->red_color, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
+	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (dc->red_bpath), dc->red_color, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 	/* Create red curve */
 	dc->red_curve = sp_curve_new_sized (4);
 
 	/* Create blue bpath */
 	dc->blue_bpath = sp_canvas_bpath_new (SP_DT_SKETCH (ec->desktop), NULL);
-	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (dc->blue_bpath), dc->blue_color, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
+	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (dc->blue_bpath), dc->blue_color, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 	/* Create blue curve */
 	dc->blue_curve = sp_curve_new_sized (8);
 
@@ -208,11 +215,11 @@ sp_draw_context_finish (SPEventContext *ec)
 	dc = SP_DRAW_CONTEXT (ec);
 
 	if (dc->grab) {
-		sp_canvas_item_ungrab (dc->grab, gdk_time_get ());
+		sp_canvas_item_ungrab (dc->grab, GDK_CURRENT_TIME);
 	}
 
 	if (dc->selection) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (dc->selection), dc);
+		sp_signal_disconnect_by_data (dc->selection, dc);
 		dc->selection = NULL;
 	}
 
@@ -241,8 +248,8 @@ sp_draw_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	case GDK_KEY_PRESS:
 		/* fixme: */
 		switch (event->key.keyval) {
-		case GDK_plus:
-		case GDK_KP_Add:
+		case GDK_A:
+		case GDK_a:
 			if (dc->attach) {
 				spdc_set_attach (dc, FALSE);
 			} else {
@@ -259,8 +266,8 @@ sp_draw_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	}
 
 	if (!ret) {
-		if (SP_EVENT_CONTEXT_CLASS (draw_parent_class)->root_handler)
-			ret = SP_EVENT_CONTEXT_CLASS (draw_parent_class)->root_handler (ec, event);
+		if (((SPEventContextClass *) draw_parent_class)->root_handler)
+			ret = ((SPEventContextClass *) draw_parent_class)->root_handler (ec, event);
 	}
 
 	return ret;
@@ -273,12 +280,12 @@ spdc_set_attach (SPDrawContext *dc, gboolean attach)
 		dc->attach = TRUE;
 		spdc_attach_selection (dc, dc->selection);
 		sp_view_set_status (SP_VIEW (SP_EVENT_CONTEXT_DESKTOP (dc)),
-				    _("Appending to selection. Press '+' to toggle Append/New."), FALSE);
+				    _("Appending to selection. Press 'a' to toggle Append/New."), FALSE);
 	} else {
 		dc->attach = FALSE;
 		spdc_detach_selection (dc, dc->selection);
 		sp_view_set_status (SP_VIEW (SP_EVENT_CONTEXT_DESKTOP (dc)),
-				    _("Creating new curve. Press '+' to toggle Append/New."), FALSE);
+				    _("Creating new curve. Press 'a' to toggle Append/New."), FALSE);
 	}
 }
 
@@ -289,7 +296,7 @@ spdc_set_attach (SPDrawContext *dc, gboolean attach)
 static void
 spdc_selection_changed (SPSelection *sel, SPDrawContext *dc)
 {
-	g_print ("Selection changed in draw context\n");
+	/* g_print ("Selection changed in draw context\n"); */
 	if (dc->attach) {
 		spdc_attach_selection (dc, sel);
 	}
@@ -300,7 +307,7 @@ spdc_selection_changed (SPSelection *sel, SPDrawContext *dc)
 static void
 spdc_selection_modified (SPSelection *sel, guint flags, SPDrawContext *dc)
 {
-	g_print ("Selection modified in draw context\n");
+	/* g_print ("Selection modified in draw context\n"); */
 	if (dc->attach) {
 		spdc_attach_selection (dc, sel);
 	}
@@ -318,18 +325,20 @@ spdc_attach_selection (SPDrawContext *dc, SPSelection *sel)
 
 	item = sp_selection_item (dc->selection);
 
-	if (item && SP_IS_PATH (item) && SP_PATH (item)->independent) {
+	if (item && SP_IS_PATH (item)) {
 		SPCurve *norm;
-		gdouble i2dt[6];
+		NRMatrixF i2dt;
+		NRMatrixD i2dtd;
 		GSList *l;
 		/* Create new white data */
 		/* Item */
 		dc->white_item = item;
 		/* Curve list */
 		/* We keep it in desktop coordinates to eliminate calculation errors */
-		norm = sp_path_normalized_bpath (SP_PATH (item));
-		sp_item_i2d_affine (dc->white_item, i2dt);
-		norm = sp_curve_transform (norm, i2dt);
+		norm = sp_shape_get_curve (SP_SHAPE (item));
+		sp_item_i2d_affine (dc->white_item, &i2dt);
+		nr_matrix_d_from_f (&i2dtd, &i2dt);
+		norm = sp_curve_transform (norm, NR_MATRIX_D_TO_DOUBLE (&i2dtd));
 		g_return_if_fail (norm != NULL);
 		dc->white_curves = sp_curve_split (norm);
 		sp_curve_unref (norm);
@@ -362,19 +371,44 @@ spdc_detach_selection (SPDrawContext *dc, SPSelection *sel)
 	dc->ea = NULL;
 }
 
+#define NUMBER_OF_TURNS 12
+
 static void
-spdc_endpoint_snap (SPDrawContext *dc, ArtPoint *p, guint state)
+spdc_endpoint_snap (SPDrawContext *dc, NRPointF *p, guint state)
 {
 	if (state & GDK_CONTROL_MASK) {
 		/* Constrained motion */
-		if (fabs (p->x - dc->p[0].x) > fabs (p->y - dc->p[0].y)) {
-			/* Horizontal */
-			p->y = dc->p[0].y;
-			sp_desktop_horizontal_snap (SP_EVENT_CONTEXT_DESKTOP (dc), p);
-		} else {
-			/* Vertical */
-			p->x = dc->p[0].x;
-			sp_desktop_vertical_snap (SP_EVENT_CONTEXT_DESKTOP (dc), p);
+		/* mirrored by fabs, so this corresponds to 15 degrees */
+		double bx = 0, by = 0; /* best solution */
+		double bn = 1e18; /* best normal */
+		double bdot = 0;
+		double vx = 0, vy = 1;
+		double r00 = cos (M_PI / NUMBER_OF_TURNS), r01 = sin (M_PI / NUMBER_OF_TURNS);
+		double r10 = -r01, r11 = r00;
+		double dx = p->x - dc->p[0].x;
+		double dy = p->y - dc->p[0].y;
+		int i;
+		for(i = 0; i < NUMBER_OF_TURNS; i++) {
+			double ndot = fabs(vy*dx-vx*dy);
+			double tx = r00*vx + r01*vy;
+			double ty = r10*vx + r11*vy;
+			if (ndot < bn) { 
+				/* I think it is better numerically to use the normal, rather than the */
+				/* dot product to assess solutions, but I haven't proven it */
+				bn = ndot;
+				bx = vx;
+				by = vy;
+				bdot = vx*dx + vy*dy;
+			}
+			vx = tx;
+			vy = ty;
+		}
+
+		if (fabs (bdot) > 0) {
+			p->x = dc->p[0].x + bdot * bx;
+			p->y = dc->p[0].y + bdot * by;
+			/* Snap it along best vector */
+			sp_desktop_vector_snap (SP_EVENT_CONTEXT_DESKTOP (dc), p, bx, by);
 		}
 	} else {
 		/* Free */
@@ -404,17 +438,17 @@ spdc_concat_colors_and_flush (SPDrawContext *dc, gboolean forceclosed)
 		dc->green_bpaths = g_slist_remove (dc->green_bpaths, dc->green_bpaths->data);
 	}
 	/* Blue */
-	sp_curve_append_continuous (c, dc->blue_curve, 1e-9);
+	sp_curve_append_continuous (c, dc->blue_curve, 0.0625);
 	sp_curve_reset (dc->blue_curve);
 	sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->blue_bpath), NULL);
 	/* Red */
-	sp_curve_append_continuous (c, dc->red_curve, 1e-9);
+	sp_curve_append_continuous (c, dc->red_curve, 0.0625);
 	sp_curve_reset (dc->red_curve);
 	sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), NULL);
 
 	/* Step A - test, whether we ended on green anchor */
 	if (forceclosed || (dc->green_anchor && dc->green_anchor->active)) {
-		g_print ("We hit green anchor, closing Green-Blue-Red\n");
+		/* g_print ("We hit green anchor, closing Green-Blue-Red\n"); */
 		sp_curve_closepath_current (c);
 		/* Closed path, just flush */
 		spdc_flush_white (dc, c);
@@ -424,15 +458,15 @@ spdc_concat_colors_and_flush (SPDrawContext *dc, gboolean forceclosed)
 
 	/* Step B - both start and end anchored to same curve */
 	if (dc->sa && dc->ea && (dc->sa->curve == dc->ea->curve)) {
-		g_print ("We hit bot start and end of single curve, closing paths\n");
+		/* g_print ("We hit bot start and end of single curve, closing paths\n"); */
 		if (dc->sa->start) {
 			SPCurve *r;
-			g_print ("Reversing curve\n");
+			/* g_print ("Reversing curve\n"); */
 			r = sp_curve_reverse (c);
 			sp_curve_unref (c);
 			c = r;
 		}
-		sp_curve_append_continuous (dc->sa->curve, c, 1e-9);
+		sp_curve_append_continuous (dc->sa->curve, c, 0.0625);
 		sp_curve_unref (c);
 		sp_curve_closepath_current (dc->sa->curve);
 		spdc_flush_white (dc, NULL);
@@ -442,17 +476,17 @@ spdc_concat_colors_and_flush (SPDrawContext *dc, gboolean forceclosed)
 	/* Step C - test start */
 	if (dc->sa) {
 		SPCurve *s;
-		g_print ("Curve start hit anchor\n");
+		/* g_print ("Curve start hit anchor\n"); */
 		s = dc->sa->curve;
 		dc->white_curves = g_slist_remove (dc->white_curves, s);
 		if (dc->sa->start) {
 			SPCurve *r;
-			g_print ("Reversing curve\n");
+			/* g_print ("Reversing curve\n"); */
 			r = sp_curve_reverse (s);
 			sp_curve_unref (s);
 			s = r;
 		}
-		sp_curve_append_continuous (s, c, 1e-9);
+		sp_curve_append_continuous (s, c, 0.0625);
 		sp_curve_unref (c);
 		c = s;
 	}
@@ -460,17 +494,17 @@ spdc_concat_colors_and_flush (SPDrawContext *dc, gboolean forceclosed)
 	/* Step D - test end */
 	if (dc->ea) {
 		SPCurve *e;
-		g_print ("Curve end hit anchor\n");
+		/* g_print ("Curve end hit anchor\n"); */
 		e = dc->ea->curve;
 		dc->white_curves = g_slist_remove (dc->white_curves, e);
 		if (!dc->ea->start) {
 			SPCurve *r;
-			g_print ("Reversing curve\n");
+			/* g_print ("Reversing curve\n"); */
 			r = sp_curve_reverse (e);
 			sp_curve_unref (e);
 			e = r;
 		}
-		sp_curve_append_continuous (c, e, 1e-9);
+		sp_curve_append_continuous (c, e, 0.0625);
 		sp_curve_unref (e);
 	}
 
@@ -509,9 +543,11 @@ spdc_flush_white (SPDrawContext *dc, SPCurve *gc)
 
 	/* Now we have to go back to item coordinates at last */
 	if (dc->white_item) {
-		gdouble d2item[6];
-		sp_item_dt2i_affine (dc->white_item, SP_EVENT_CONTEXT_DESKTOP (dc), d2item);
-		c = sp_curve_transform (c, d2item);
+		NRMatrixF d2itemf;
+		NRMatrixD d2itemd;
+		sp_item_dt2i_affine (dc->white_item, SP_EVENT_CONTEXT_DESKTOP (dc), &d2itemf);
+		nr_matrix_d_from_f (&d2itemd, &d2itemf);
+		c = sp_curve_transform (c, NR_MATRIX_D_TO_DOUBLE (&d2itemd));
 	} else {
 		gdouble d2item[6];
 		sp_desktop_dt2root_affine (SP_EVENT_CONTEXT_DESKTOP (dc), (NRMatrixD *) d2item);
@@ -594,7 +630,7 @@ test_inside (SPDrawContext *dc, gdouble wx, gdouble wy)
 static void
 fit_and_split (SPDrawContext * dc)
 {
-	ArtPoint b[4];
+	NRPointF b[4];
 	gdouble tolerance;
 
 	g_assert (dc->npoints > 1);
@@ -613,13 +649,13 @@ fit_and_split (SPDrawContext * dc)
 		SPCanvasItem *cshape;
 		/* Fit and draw and copy last point */
 		g_assert (!sp_curve_empty (dc->red_curve));
-		sp_curve_append_continuous (dc->green_curve, dc->red_curve, 1e-9);
+		sp_curve_append_continuous (dc->green_curve, dc->red_curve, 0.0625);
 		curve = sp_curve_copy (dc->red_curve);
 
 		/* fixme: */
 		cshape = sp_canvas_bpath_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), curve);
 		sp_curve_unref (curve);
-		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cshape), dc->green_color, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
+		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cshape), dc->green_color, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 
 		dc->green_bpaths = g_slist_prepend (dc->green_bpaths, cshape);
 
@@ -722,7 +758,8 @@ static SPDrawAnchor *
 sp_draw_anchor_new (SPDrawContext *dc, SPCurve *curve, gboolean start, gdouble dx, gdouble dy)
 {
 	SPDrawAnchor *a;
-	NRPointF fp;
+
+	/* g_print ("Creating anchor at %g %g\n", dx, dy); */
 
 	a = g_new (SPDrawAnchor, 1);
 
@@ -732,9 +769,7 @@ sp_draw_anchor_new (SPDrawContext *dc, SPCurve *curve, gboolean start, gdouble d
 	a->active = FALSE;
 	a->dp.x = dx;
 	a->dp.y = dy;
-	sp_desktop_d2w_xy_point (SP_EVENT_CONTEXT_DESKTOP (dc), &fp, dx, dy);
-	a->wp.x = fp.x;
-	a->wp.y = fp.y;
+	sp_desktop_d2w_xy_point (SP_EVENT_CONTEXT_DESKTOP (dc), &a->wp, dx, dy);
 	a->ctrl = sp_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (dc)), SP_TYPE_CTRL,
 					 "size", 4.0,
 					 "filled", 0,
@@ -782,49 +817,48 @@ sp_draw_anchor_test (SPDrawAnchor *anchor, gdouble wx, gdouble wy, gboolean acti
 
 static void sp_pencil_context_class_init (SPPencilContextClass *klass);
 static void sp_pencil_context_init (SPPencilContext *dc);
-static void sp_pencil_context_finalize (GtkObject *object);
+static void sp_pencil_context_dispose (GObject *object);
 
 static gint sp_pencil_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 
-static void spdc_set_startpoint (SPPencilContext *dc, ArtPoint *p, guint state);
-static void spdc_set_endpoint (SPPencilContext *dc, ArtPoint *p, guint state);
-static void spdc_finish_endpoint (SPPencilContext *dc, ArtPoint *p, gboolean snap, guint state);
-static void spdc_add_freehand_point (SPPencilContext *dc, ArtPoint *p, guint state);
+static void spdc_set_startpoint (SPPencilContext *dc, NRPointF *p, guint state);
+static void spdc_set_endpoint (SPPencilContext *dc, NRPointF *p, guint state);
+static void spdc_finish_endpoint (SPPencilContext *dc, NRPointF *p, gboolean snap, guint state);
+static void spdc_add_freehand_point (SPPencilContext *dc, NRPointF *p, guint state);
 
 static SPDrawContextClass *pencil_parent_class;
 
 GtkType
 sp_pencil_context_get_type (void)
 {
-	static GtkType type = 0;
-
+	static GType type = 0;
 	if (!type) {
-		static const GtkTypeInfo info = {
-			"SPPencilContext",
-			sizeof (SPPencilContext),
+		GTypeInfo info = {
 			sizeof (SPPencilContextClass),
-			(GtkClassInitFunc) sp_pencil_context_class_init,
-			(GtkObjectInitFunc) sp_pencil_context_init,
-			NULL, NULL, NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_pencil_context_class_init,
+			NULL, NULL,
+			sizeof (SPPencilContext),
+			4,
+			(GInstanceInitFunc) sp_pencil_context_init,
 		};
-		type = gtk_type_unique (SP_TYPE_DRAW_CONTEXT, &info);
+		type = g_type_register_static (SP_TYPE_DRAW_CONTEXT, "SPPencilContext", &info, 0);
 	}
-
 	return type;
 }
 
 static void
 sp_pencil_context_class_init (SPPencilContextClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	SPEventContextClass *event_context_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 	event_context_class = (SPEventContextClass *) klass;
 
-	pencil_parent_class = gtk_type_class (SP_TYPE_DRAW_CONTEXT);
+	pencil_parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = sp_pencil_context_finalize;
+	object_class->dispose = sp_pencil_context_dispose;
 
 	event_context_class->root_handler = sp_pencil_context_root_handler;
 }
@@ -836,13 +870,13 @@ sp_pencil_context_init (SPPencilContext *pc)
 }
 
 static void
-sp_pencil_context_finalize (GtkObject *object)
+sp_pencil_context_dispose (GObject *object)
 {
 	SPPencilContext *pc;
 
 	pc = SP_PENCIL_CONTEXT (object);
 
-	GTK_OBJECT_CLASS (pencil_parent_class)->finalize (object);
+	G_OBJECT_CLASS (pencil_parent_class)->dispose (object);
 }
 
 gint
@@ -851,7 +885,7 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	SPDrawContext *dc;
 	SPPencilContext *pc;
 	SPDesktop *dt;
-	ArtPoint p;
+	NRPointF p;
 	gint ret;
 	SPDrawAnchor *anchor;
 	NRPointF fp;
@@ -974,7 +1008,7 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				}
 				dc->ea = anchor;
 				/* Write curves to object */
-				g_print ("Finishing freehand\n");
+				/* g_print ("Finishing freehand\n"); */
 				spdc_concat_colors_and_flush (dc, FALSE);
 				dc->sa = NULL;
 				dc->ea = NULL;
@@ -1003,8 +1037,8 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	}
 
 	if (!ret) {
-		if (SP_EVENT_CONTEXT_CLASS (pencil_parent_class)->root_handler)
-			ret = SP_EVENT_CONTEXT_CLASS (pencil_parent_class)->root_handler (ec, event);
+		if (((SPEventContextClass *) pencil_parent_class)->root_handler)
+			ret = ((SPEventContextClass *) pencil_parent_class)->root_handler (ec, event);
 	}
 
 	return ret;
@@ -1015,12 +1049,14 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
  */
 
 static void
-spdc_set_startpoint (SPPencilContext *pc, ArtPoint *p, guint state)
+spdc_set_startpoint (SPPencilContext *pc, NRPointF *p, guint state)
 {
 	SPDrawContext *dc;
 
 	dc = SP_DRAW_CONTEXT (pc);
 
+	sp_desktop_free_snap (SP_EVENT_CONTEXT_DESKTOP (dc), p);
+
 	dc->npoints = 0;
 	dc->p[dc->npoints++] = *p;
 }
@@ -1036,7 +1072,7 @@ spdc_set_startpoint (SPPencilContext *pc, ArtPoint *p, guint state)
  */
 
 static void
-spdc_set_endpoint (SPPencilContext *pc, ArtPoint *p, guint state)
+spdc_set_endpoint (SPPencilContext *pc, NRPointF *p, guint state)
 {
 	SPDrawContext *dc;
 
@@ -1062,7 +1098,7 @@ spdc_set_endpoint (SPPencilContext *pc, ArtPoint *p, guint state)
  */
 
 static void
-spdc_finish_endpoint (SPPencilContext *pc, ArtPoint *p, gboolean snap, guint state)
+spdc_finish_endpoint (SPPencilContext *pc, NRPointF *p, gboolean snap, guint state)
 {
 	SPDrawContext *dc;
 
@@ -1070,11 +1106,11 @@ spdc_finish_endpoint (SPPencilContext *pc, ArtPoint *p, gboolean snap, guint sta
 
 	if (SP_CURVE_LENGTH (dc->red_curve) < 2) {
 		/* Just a click, reset red curve and continue */
-		g_print ("Finishing empty red curve\n");
+		/* g_print ("Finishing empty red curve\n"); */
 		sp_curve_reset (dc->red_curve);
 		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), NULL);
 	} else if (SP_CURVE_LENGTH (dc->red_curve) > 2) {
-		g_warning ("Red curve length is %d", SP_CURVE_LENGTH (dc->red_curve));
+		/* g_warning ("Red curve length is %d", SP_CURVE_LENGTH (dc->red_curve)); */
 		sp_curve_reset (dc->red_curve);
 		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), NULL);
 	} else {
@@ -1089,12 +1125,12 @@ spdc_finish_endpoint (SPPencilContext *pc, ArtPoint *p, gboolean snap, guint sta
 		e = SP_CURVE_SEGMENT (dc->red_curve, 1);
 		if ((e->x3 == s->x3) && (e->y3 == s->y3)) {
 			/* Empty line, reset red curve and continue */
-			g_print ("Finishing zero length red curve\n");
+			/* g_print ("Finishing zero length red curve\n"); */
 			sp_curve_reset (dc->red_curve);
 			sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), NULL);
 		} else {
 			/* Write curves to object */
-			g_print ("Finishing real red curve\n");
+			/* g_print ("Finishing real red curve\n"); */
 			spdc_concat_colors_and_flush (dc, FALSE);
 			dc->sa = NULL;
 			dc->ea = NULL;
@@ -1103,7 +1139,7 @@ spdc_finish_endpoint (SPPencilContext *pc, ArtPoint *p, gboolean snap, guint sta
 }
 
 static void
-spdc_add_freehand_point (SPPencilContext *pc, ArtPoint *p, guint state)
+spdc_add_freehand_point (SPPencilContext *pc, NRPointF *p, guint state)
 {
 	SPDrawContext *dc;
 
@@ -1121,16 +1157,16 @@ spdc_add_freehand_point (SPPencilContext *pc, ArtPoint *p, guint state)
 
 static void sp_pen_context_class_init (SPPenContextClass *klass);
 static void sp_pen_context_init (SPPenContext *pc);
-static void sp_pen_context_finalize (GtkObject *object);
+static void sp_pen_context_dispose (GObject *object);
 
 static void sp_pen_context_setup (SPEventContext *ec);
 static void sp_pen_context_finish (SPEventContext *ec);
 static void sp_pen_context_set (SPEventContext *ec, const guchar *key, const guchar *val);
 static gint sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event);
 
-static void spdc_pen_set_point (SPPenContext *pc, ArtPoint *p, guint state);
-static void spdc_pen_set_ctrl (SPPenContext *pc, ArtPoint *p, guint state);
-static void spdc_pen_finish_segment (SPPenContext *pc, ArtPoint *p, guint state);
+static void spdc_pen_set_point (SPPenContext *pc, NRPointF *p, guint state);
+static void spdc_pen_set_ctrl (SPPenContext *pc, NRPointF *p, guint state);
+static void spdc_pen_finish_segment (SPPenContext *pc, NRPointF *p, guint state);
 
 static void spdc_pen_finish (SPPenContext *pc, gboolean closed);
 
@@ -1139,35 +1175,34 @@ static SPDrawContextClass *pen_parent_class;
 GtkType
 sp_pen_context_get_type (void)
 {
-	static GtkType type = 0;
-
+	static GType type = 0;
 	if (!type) {
-		static const GtkTypeInfo info = {
-			"SPPenContext",
-			sizeof (SPPenContext),
+		GTypeInfo info = {
 			sizeof (SPPenContextClass),
-			(GtkClassInitFunc) sp_pen_context_class_init,
-			(GtkObjectInitFunc) sp_pen_context_init,
-			NULL, NULL, NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_pen_context_class_init,
+			NULL, NULL,
+			sizeof (SPPenContext),
+			4,
+			(GInstanceInitFunc) sp_pen_context_init,
 		};
-		type = gtk_type_unique (SP_TYPE_DRAW_CONTEXT, &info);
+		type = g_type_register_static (SP_TYPE_DRAW_CONTEXT, "SPPenContext", &info, 0);
 	}
-
 	return type;
 }
 
 static void
 sp_pen_context_class_init (SPPenContextClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	SPEventContextClass *event_context_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 	event_context_class = (SPEventContextClass *) klass;
 
-	pen_parent_class = gtk_type_class (SP_TYPE_DRAW_CONTEXT);
+	pen_parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = sp_pen_context_finalize;
+	object_class->dispose = sp_pen_context_dispose;
 
 	event_context_class->setup = sp_pen_context_setup;
 	event_context_class->finish = sp_pen_context_finish;
@@ -1189,18 +1224,30 @@ sp_pen_context_init (SPPenContext *pc)
 }
 
 static void
-sp_pen_context_finalize (GtkObject *object)
+sp_pen_context_dispose (GObject *object)
 {
 	SPPenContext *pc;
 
 	pc = SP_PEN_CONTEXT (object);
 
-	if (pc->c0) gtk_object_destroy (GTK_OBJECT (pc->c0));
-	if (pc->c1) gtk_object_destroy (GTK_OBJECT (pc->c1));
-	if (pc->cl0) gtk_object_destroy (GTK_OBJECT (pc->cl0));
-	if (pc->cl1) gtk_object_destroy (GTK_OBJECT (pc->cl1));
+	if (pc->c0) {
+		gtk_object_destroy (GTK_OBJECT (pc->c0));
+		pc->c0 = NULL;
+	}
+	if (pc->c1) {
+		gtk_object_destroy (GTK_OBJECT (pc->c1));
+		pc->c1 = NULL;
+	}
+	if (pc->cl0) {
+		gtk_object_destroy (GTK_OBJECT (pc->cl0));
+		pc->cl0 = NULL;
+	}
+	if (pc->cl1) {
+		gtk_object_destroy (GTK_OBJECT (pc->cl1));
+		pc->cl1 = NULL;
+	}
 
-	GTK_OBJECT_CLASS (pen_parent_class)->finalize (object);
+	G_OBJECT_CLASS (pen_parent_class)->dispose (object);
 }
 
 static void
@@ -1210,8 +1257,8 @@ sp_pen_context_setup (SPEventContext *ec)
 
 	pc = SP_PEN_CONTEXT (ec);
 
-	if (SP_EVENT_CONTEXT_CLASS (pen_parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (pen_parent_class)->setup (ec);
+	if (((SPEventContextClass *) pen_parent_class)->setup)
+		((SPEventContextClass *) pen_parent_class)->setup (ec);
 
 	/* Pen indicators */
 	pc->c0 = sp_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRL, "shape", SP_CTRL_SHAPE_CIRCLE,
@@ -1236,8 +1283,8 @@ sp_pen_context_finish (SPEventContext *ec)
 {
 	spdc_pen_finish (SP_PEN_CONTEXT (ec), FALSE);
 
-	if (SP_EVENT_CONTEXT_CLASS (pen_parent_class)->finish)
-		SP_EVENT_CONTEXT_CLASS (pen_parent_class)->finish (ec);
+	if (((SPEventContextClass *) pen_parent_class)->finish)
+		((SPEventContextClass *) pen_parent_class)->finish (ec);
 }
 
 static void
@@ -1262,7 +1309,7 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	SPDrawContext *dc;
 	SPPenContext *pc;
 	SPDesktop *dt;
-	ArtPoint p;
+	NRPointF p;
 	gint ret;
 	SPDrawAnchor *anchor;
 	NRPointF fp;
@@ -1334,10 +1381,10 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 					ret = TRUE;
 					break;
 				case SP_PEN_CONTEXT_CONTROL:
-					g_warning ("Button down in CONTROL state");
+					/* g_warning ("Button down in CONTROL state"); */
 					break;
 				case SP_PEN_CONTEXT_CLOSE:
-					g_warning ("Button down in CLOSE state");
+					/* g_warning ("Button down in CLOSE state"); */
 					break;
 				default:
 					break;
@@ -1362,6 +1409,10 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		p.y = fp.y;
 		/* Test, whether we hit any anchor */
 		anchor = test_inside (dc, event->button.x, event->button.y);
+		if (!anchor) {
+			/* Snap only if not hitting anchor */
+			spdc_endpoint_snap (dc, &p, event->motion.state);
+		}
 
 		switch (pc->mode) {
 		case SP_PEN_CONTEXT_MODE_CLICK:
@@ -1529,7 +1580,7 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				ret = TRUE;
 				break;
 			} else {
-				ArtPoint pt;
+				NRPointF pt;
 				ArtBpath *p;
 				gint e;
 				/* Reset red curve */
@@ -1541,7 +1592,7 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				p = SP_CURVE_BPATH (dc->green_curve);
 				e = SP_CURVE_LENGTH (dc->green_curve);
 				if (e < 2) {
-					g_warning ("Green curve length is %d", e);
+					/* g_warning ("Green curve length is %d", e); */
 					break;
 				}
 				dc->p[0].x = p[e - 2].x3;
@@ -1575,15 +1626,15 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	}
 
 	if (!ret) {
-		if (SP_EVENT_CONTEXT_CLASS (pen_parent_class)->root_handler)
-			ret = SP_EVENT_CONTEXT_CLASS (pen_parent_class)->root_handler (ec, event);
+		if (((SPEventContextClass *) pen_parent_class)->root_handler)
+			return ((SPEventContextClass *) pen_parent_class)->root_handler (ec, event);
 	}
 
 	return ret;
 }
 
 static void
-spdc_pen_set_point (SPPenContext *pc, ArtPoint *p, guint state)
+spdc_pen_set_point (SPPenContext *pc, NRPointF *p, guint state)
 {
 	SPDrawContext *dc;
 
@@ -1602,13 +1653,19 @@ spdc_pen_set_point (SPPenContext *pc, ArtPoint *p, guint state)
 		dc->npoints = 5;
 		sp_curve_reset (dc->red_curve);
 		sp_curve_moveto (dc->red_curve, dc->p[0].x, dc->p[0].y);
-		sp_curve_curveto (dc->red_curve, dc->p[1].x, dc->p[1].y, dc->p[2].x, dc->p[2].y, dc->p[3].x, dc->p[3].y);
+		if ((pc->onlycurves) ||
+		    (dc->p[1].x != dc->p[0].x) ||
+		    (dc->p[1].y != dc->p[0].y)) {
+			sp_curve_curveto (dc->red_curve, dc->p[1].x, dc->p[1].y, dc->p[2].x, dc->p[2].y, dc->p[3].x, dc->p[3].y);
+		} else {
+			sp_curve_lineto (dc->red_curve, dc->p[3].x, dc->p[3].y);
+		}
 		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), dc->red_curve);
 	}
 }
 
 static void
-spdc_pen_set_ctrl (SPPenContext *pc, ArtPoint *p, guint state)
+spdc_pen_set_ctrl (SPPenContext *pc, NRPointF *p, guint state)
 {
 	SPDrawContext *dc;
 
@@ -1644,12 +1701,12 @@ spdc_pen_set_ctrl (SPPenContext *pc, ArtPoint *p, guint state)
 		sp_ctrl_moveto (SP_CTRL (pc->c1), dc->p[4].x, dc->p[4].y);
 		sp_ctrlline_set_coords (SP_CTRLLINE (pc->cl1), dc->p[3].x, dc->p[3].y, dc->p[4].x, dc->p[4].y);
 	} else {
-		g_warning ("Something bad happened - npoints is %d", dc->npoints);
+		/* g_warning ("Something bad happened - npoints is %d", dc->npoints); */
 	}
 }
 
 static void
-spdc_pen_finish_segment (SPPenContext *pc, ArtPoint *p, guint state)
+spdc_pen_finish_segment (SPPenContext *pc, NRPointF *p, guint state)
 {
 	SPDrawContext *dc;
 
@@ -1659,12 +1716,12 @@ spdc_pen_finish_segment (SPPenContext *pc, ArtPoint *p, guint state)
 		SPCurve *curve;
 		SPCanvasItem *cshape;
 
-		sp_curve_append_continuous (dc->green_curve, dc->red_curve, 1e-9);
+		sp_curve_append_continuous (dc->green_curve, dc->red_curve, 0.0625);
 		curve = sp_curve_copy (dc->red_curve);
 		/* fixme: */
 		cshape = sp_canvas_bpath_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), curve);
 		sp_curve_unref (curve);
-		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cshape), dc->green_color, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
+		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cshape), dc->green_color, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 
 		dc->green_bpaths = g_slist_prepend (dc->green_bpaths, cshape);
 
@@ -1683,7 +1740,7 @@ spdc_pen_finish (SPPenContext *pc, gboolean closed)
 
 	dc = SP_DRAW_CONTEXT (pc);
 
-	g_print ("Finishing pen\n");
+	/* g_print ("Finishing pen\n"); */
 
 	sp_curve_reset (dc->red_curve);
 	spdc_concat_colors_and_flush (dc, closed);
