@@ -30,12 +30,28 @@
 #define noDYNA_DRAW_VERBOSE
 #define NORMALIZED_COORDINATE
 
+#ifdef DYNA_DRAW_DEBUG
+#define BEZIER_ASSERT(b) { \
+      g_assert ((b[0].x > -8000.0) && (b[0].x < 8000.0)); \
+      g_assert ((b[0].y > -8000.0) && (b[0].y < 8000.0)); \
+      g_assert ((b[1].x > -8000.0) && (b[1].x < 8000.0)); \
+      g_assert ((b[1].y > -8000.0) && (b[1].y < 8000.0)); \
+      g_assert ((b[2].x > -8000.0) && (b[2].x < 8000.0)); \
+      g_assert ((b[2].y > -8000.0) && (b[2].y < 8000.0)); \
+      g_assert ((b[3].x > -8000.0) && (b[3].x < 8000.0)); \
+      g_assert ((b[3].y > -8000.0) && (b[3].y < 8000.0)); \
+      }
+#else
+#define BEZIER_ASSERT(b)
+#endif
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <math.h>
 #include <string.h>
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtktable.h>
@@ -43,11 +59,13 @@
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtklabel.h>
 #include "svg/svg.h"
+#include "helper/sp-intl.h"
 #include "helper/curve.h"
 #include "helper/canvas-bpath.h"
 #include "helper/sodipodi-ctrl.h"
 #include "helper/bezier-utils.h"
 
+#include "enums.h"
 #include "sodipodi.h"
 #include "document.h"
 #include "selection.h"
@@ -77,7 +95,7 @@
 
 static void sp_dyna_draw_context_class_init (SPDynaDrawContextClass *klass);
 static void sp_dyna_draw_context_init (SPDynaDrawContext *ddc);
-static void sp_dyna_draw_context_destroy (GtkObject *object);
+static void sp_dyna_draw_context_dispose (GObject *object);
 
 static void sp_dyna_draw_context_setup (SPEventContext *ec);
 static void sp_dyna_draw_context_set (SPEventContext *ec, const guchar *key, const guchar *val);
@@ -106,35 +124,34 @@ static SPEventContextClass *parent_class;
 GtkType
 sp_dyna_draw_context_get_type (void)
 {
-	static GtkType type = 0;
-
+	static GType type = 0;
 	if (!type) {
-		static const GtkTypeInfo info = {
-			"SPDynaDrawContext",
-			sizeof (SPDynaDrawContext),
+		GTypeInfo info = {
 			sizeof (SPDynaDrawContextClass),
-			(GtkClassInitFunc) sp_dyna_draw_context_class_init,
-			(GtkObjectInitFunc) sp_dyna_draw_context_init,
-			NULL, NULL, NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_dyna_draw_context_class_init,
+			NULL, NULL,
+			sizeof (SPDynaDrawContext),
+			4,
+			(GInstanceInitFunc) sp_dyna_draw_context_init,
 		};
-		type = gtk_type_unique (SP_TYPE_EVENT_CONTEXT, &info);
+		type = g_type_register_static (SP_TYPE_EVENT_CONTEXT, "SPDynaDrawContext", &info, 0);
 	}
-
 	return type;
 }
 
 static void
 sp_dyna_draw_context_class_init (SPDynaDrawContextClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	SPEventContextClass *event_context_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 	event_context_class = (SPEventContextClass *) klass;
 
-	parent_class = gtk_type_class (SP_TYPE_EVENT_CONTEXT);
+	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->destroy = sp_dyna_draw_context_destroy;
+	object_class->dispose = sp_dyna_draw_context_dispose;
 
 	event_context_class->setup = sp_dyna_draw_context_setup;
 	event_context_class->set = sp_dyna_draw_context_set;
@@ -190,7 +207,7 @@ sp_dyna_draw_context_init (SPDynaDrawContext *ddc)
 }
 
 static void
-sp_dyna_draw_context_destroy (GtkObject *object)
+sp_dyna_draw_context_dispose (GObject *object)
 {
 	SPDynaDrawContext *ddc;
 
@@ -212,7 +229,7 @@ sp_dyna_draw_context_destroy (GtkObject *object)
 		ddc->currentshape = NULL;
 	}
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -222,8 +239,8 @@ sp_dyna_draw_context_setup (SPEventContext *ec)
 
 	ddc = SP_DYNA_DRAW_CONTEXT (ec);
 
-	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (ec);
+	if (((SPEventContextClass *) parent_class)->setup)
+		((SPEventContextClass *) parent_class)->setup (ec);
 
 	ddc->accumulated = sp_curve_new_sized (32);
 	ddc->currentcurve = sp_curve_new_sized (4);
@@ -233,10 +250,10 @@ sp_dyna_draw_context_setup (SPEventContext *ec)
 
 	/* style should be changed when dc->use_calligraphc is touched */  
 	ddc->currentshape = sp_canvas_item_new (SP_DT_SKETCH (ec->desktop), SP_TYPE_CANVAS_BPATH, NULL);
-	sp_canvas_bpath_set_fill (SP_CANVAS_BPATH (ddc->currentshape), DDC_RED_RGBA, ART_WIND_RULE_ODDEVEN);
-	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (ddc->currentshape), 0x00000000, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
+	sp_canvas_bpath_set_fill (SP_CANVAS_BPATH (ddc->currentshape), DDC_RED_RGBA, SP_WIND_RULE_EVENODD);
+	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (ddc->currentshape), 0x00000000, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 	/* fixme: Cannot we cascade it to root more clearly? */
-	gtk_signal_connect (GTK_OBJECT (ddc->currentshape), "event", GTK_SIGNAL_FUNC (sp_desktop_root_handler), ec->desktop);
+	g_signal_connect (G_OBJECT (ddc->currentshape), "event", G_CALLBACK (sp_desktop_root_handler), ec->desktop);
 
 	sp_event_context_read (ec, "mass");
 	sp_event_context_read (ec, "drag");
@@ -253,21 +270,21 @@ sp_dyna_draw_context_set (SPEventContext *ec, const guchar *key, const guchar *v
 	ddc = SP_DYNA_DRAW_CONTEXT (ec);
 
 	if (!strcmp (key, "mass")) {
-		dval = (val) ? atof (val) : 0.2;
+		dval = (val) ? sp_svg_atof (val) : 0.2;
 		ddc->mass = CLAMP (dval, -1000.0, 1000.0);
 	} else if (!strcmp (key, "drag")) {
-		dval = (val) ? atof (val) : DRAG_DEFAULT;
+		dval = (val) ? sp_svg_atof (val) : DRAG_DEFAULT;
 		ddc->drag = CLAMP (dval, DRAG_MIN, DRAG_MAX);
 	} else if (!strcmp (key, "angle")) {
-		dval = (val) ? atof (val) : 0.5;
+		dval = (val) ? sp_svg_atof (val) : 0.5;
 		dval = fmod (dval, 360.0);
 		ddc->angle = (dval > 0) ? dval : dval + 360.0;
 	} else if (!strcmp (key, "width")) {
-		dval = (val) ? atof (val) : 0.1;
+		dval = (val) ? sp_svg_atof (val) : 0.1;
 		ddc->width = CLAMP (dval, -1000.0, 1000.0);
 	}
 
-	g_print ("DDC: %g %g %g %g\n", ddc->mass, ddc->drag, ddc->angle, ddc->width);
+	/* g_print ("DDC: %g %g %g %g\n", ddc->mass, ddc->drag, ddc->angle, ddc->width); */
 }
 
 static double
@@ -285,14 +302,14 @@ sp_dyna_draw_get_npoint (const SPDynaDrawContext *dc,
                          gdouble           *ny)
 {
 #ifdef NORMALIZED_COORDINATE
-  ArtDRect drect;
+	NRRectF drect;
 
-  sp_desktop_get_visible_area (SP_EVENT_CONTEXT(dc)->desktop, &drect);
-  *nx = (vx - drect.x0)/(drect.x1 - drect.x0);
-  *ny = (vy - drect.y0)/(drect.y1 - drect.y0);
+	sp_desktop_get_display_area (SP_EVENT_CONTEXT(dc)->desktop, &drect);
+	*nx = (vx - drect.x0)/(drect.x1 - drect.x0);
+	*ny = (vy - drect.y0)/(drect.y1 - drect.y0);
 #else
-  *nx = vx;
-  *ny = vy;
+	*nx = vx;
+	*ny = vy;
 #endif
 
 /*    g_print ("NP:: vx:%g vy:%g => nx:%g ny:%g\n", vx, vy, *nx, *ny); */
@@ -308,14 +325,14 @@ sp_dyna_draw_get_vpoint (const SPDynaDrawContext *dc,
                          gdouble           *vy)
 {
 #ifdef NORMALIZED_COORDINATE
-  ArtDRect drect;
+	NRRectF drect;
 
-  sp_desktop_get_visible_area (SP_EVENT_CONTEXT(dc)->desktop, &drect);
-  *vx = nx * (drect.x1 - drect.x0) + drect.x0;
-  *vy = ny * (drect.y1 - drect.y0) + drect.y0;
+	sp_desktop_get_display_area (SP_EVENT_CONTEXT(dc)->desktop, &drect);
+	*vx = nx * (drect.x1 - drect.x0) + drect.x0;
+	*vy = ny * (drect.y1 - drect.y0) + drect.y0;
 #else
-  *vx = nx;
-  *vy = ny;
+	*vx = nx;
+	*vy = ny;
 #endif
 
 /*    g_print ("VP:: nx:%g ny:%g => vx:%g vy:%g\n", nx, ny, *vx, *vy); */
@@ -328,14 +345,14 @@ sp_dyna_draw_get_curr_vpoint (const SPDynaDrawContext * dc,
                               gdouble *vy)
 {
 #ifdef NORMALIZED_COORDINATE
-  ArtDRect drect;
+	NRRectF drect;
 
-  sp_desktop_get_visible_area (SP_EVENT_CONTEXT(dc)->desktop, &drect);
-  *vx = dc->curx * (drect.x1 - drect.x0) + drect.x0;
-  *vy = dc->cury * (drect.y1 - drect.y0) + drect.y0;
+	sp_desktop_get_display_area (SP_EVENT_CONTEXT(dc)->desktop, &drect);
+	*vx = dc->curx * (drect.x1 - drect.x0) + drect.x0;
+	*vy = dc->cury * (drect.y1 - drect.y0) + drect.y0;
 #else
-  *vx = dc->curx;
-  *vy = dc->cury;
+	*vx = dc->curx;
+	*vy = dc->cury;
 #endif
 }
 
@@ -418,87 +435,91 @@ sp_dyna_draw_apply (SPDynaDrawContext * dc, double x, double y)
 static void
 sp_dyna_draw_brush (SPDynaDrawContext *dc)
 {
+	g_assert (dc->npoints >= 0 && dc->npoints < SAMPLING_SIZE);
 
-  g_assert (dc->npoints >= 0 && dc->npoints < SAMPLING_SIZE);
+	if (dc->use_calligraphic) {
+		/* calligraphics */
+		double width;
+		double delx, dely;
+		NRPointF *vp;
+		double xd, yd;
 
-  if (dc->use_calligraphic)
-    {
-      /* calligraphics */
-      double width;
-      double delx, dely;
-      ArtPoint *vp;
-
-      /* fixme: */
+		/* fixme: */
 #ifdef NORMALIZED_COORDINATE
-      width = (0.05 - dc->vel) * dc->width;
+		width = (0.05 - dc->vel) * dc->width;
 #else
-      width = (100.0 - dc->vel) * dc->width;
+		width = (100.0 - dc->vel) * dc->width;
 #endif
-      if (width < DYNA_MIN_WIDTH)
-        width = DYNA_MIN_WIDTH;
-      delx = dc->angx * width;
-      dely = dc->angy * width;
+		if (width < DYNA_MIN_WIDTH)
+			width = DYNA_MIN_WIDTH;
+		delx = dc->angx * width;
+		dely = dc->angy * width;
 
 #if 0
-      g_print ("brush:: [w:%g] [dc->w:%g] [dc->vel:%g]\n", width, dc->width, dc->vel);
-      g_print("brush:: [del: %g, %g]\n", delx, dely);
+		g_print ("brush:: [w:%g] [dc->w:%g] [dc->vel:%g]\n", width, dc->width, dc->vel);
+		g_print("brush:: [del: %g, %g]\n", delx, dely);
 #endif
 
-      vp = &dc->point1[dc->npoints];
-      sp_dyna_draw_get_vpoint (dc, dc->curx + delx, dc->cury + dely,
-                               &vp->x, &vp->y);
-      vp = &dc->point2[dc->npoints];
-      sp_dyna_draw_get_vpoint (dc, dc->curx - delx, dc->cury - dely,
-                               &vp->x, &vp->y);
+		vp = &dc->point1[dc->npoints];
+		sp_dyna_draw_get_vpoint (dc, dc->curx + delx, dc->cury + dely, &xd, &yd);
+		vp->x = xd;
+		vp->y = yd;
+		vp = &dc->point2[dc->npoints];
+		sp_dyna_draw_get_vpoint (dc, dc->curx - delx, dc->cury - dely, &xd, &yd);
+		vp->x = xd;
+		vp->y = yd;
 
-      dc->delx = delx;
-      dc->dely = dely;
-    }
-  else
-    {
-      sp_dyna_draw_get_curr_vpoint (dc, &dc->point1[dc->npoints].x,
-                                    &dc->point1[dc->npoints].y);
-    }
+		dc->delx = delx;
+		dc->dely = dely;
+	} else {
+		double xd, yd;
+		sp_dyna_draw_get_curr_vpoint (dc, &xd, &yd);
+		dc->point1[dc->npoints].x = xd;
+		dc->point1[dc->npoints].y = yd;
+	}
 
-  dc->npoints ++;
+	dc->npoints++;
 }
 
 static gint
 sp_dyna_draw_timeout_handler (gpointer data)
 {
-  SPDynaDrawContext *dc;
-  SPDesktop *desktop;
-  SPCanvas *canvas;
-  gint x, y;
-  ArtPoint p;
-  NRPointF fp;
+	SPDynaDrawContext *dc;
+	SPDesktop *desktop;
+	SPCanvas *canvas;
+	double xd, yd;
+	int x, y;
+	NRPointF p;
 
-  dc = SP_DYNA_DRAW_CONTEXT (data);
-  desktop = SP_EVENT_CONTEXT(dc)->desktop;
-  canvas = SP_CANVAS (SP_DT_CANVAS (desktop));
+	dc = SP_DYNA_DRAW_CONTEXT (data);
+	desktop = SP_EVENT_CONTEXT(dc)->desktop;
+	canvas = SP_CANVAS (SP_DT_CANVAS (desktop));
 
-  dc->dragging = TRUE;
-  dc->dynahand = TRUE;
+	dc->dragging = TRUE;
+	dc->dynahand = TRUE;
   
-  gtk_widget_get_pointer (GTK_WIDGET(canvas), &x, &y);
-  sp_canvas_window_to_world (canvas, (double)x, (double)y, &p.x, &p.y);
-  sp_desktop_w2d_xy_point (desktop, &fp, p.x, p.y);
-  p.x = fp.x;
-  p.y = fp.y;
-/*    g_print ("(%d %d)=>(%g %g)\n", x, y, p.x, p.y); */
-  if (! sp_dyna_draw_apply (dc, p.x, p.y))
-    return TRUE;
-  sp_dyna_draw_get_curr_vpoint (dc, &p.x, &p.y);
+	gtk_widget_get_pointer (GTK_WIDGET(canvas), &x, &y);
+	sp_canvas_window_to_world (canvas, x, y, &xd, &yd);
+	p.x = xd;
+	p.y = yd;
+	sp_desktop_w2d_xy_point (desktop, &p, p.x, p.y);
+	/* g_print ("(%d %d)=>(%g %g)\n", x, y, p.x, p.y); */
+	if (! sp_dyna_draw_apply (dc, p.x, p.y)) {
+		return TRUE;
+	}
+	sp_dyna_draw_get_curr_vpoint (dc, &xd, &yd);
+	p.x = xd;
+	p.y = yd;
 
-  sp_desktop_free_snap (desktop, &p);
+	sp_desktop_free_snap (desktop, &p);
   
-  if ((dc->curx != dc->lastx) || (dc->cury != dc->lasty))
-    {
-      sp_dyna_draw_brush (dc);
-      g_assert (dc->npoints > 0);
-      fit_and_split (dc, FALSE);
-    }
-  return TRUE;
+	if ((dc->curx != dc->lastx) || (dc->cury != dc->lasty)) {
+		sp_dyna_draw_brush (dc);
+		g_assert (dc->npoints > 0);
+		fit_and_split (dc, FALSE);
+	}
+
+	return TRUE;
 }
 
 gint
@@ -507,9 +528,8 @@ sp_dyna_draw_context_root_handler (SPEventContext * event_context,
 {
 	SPDynaDrawContext *dc;
 	SPDesktop *desktop;
-	ArtPoint p;
+	NRPointF p;
 	gint ret;
-	NRPointF fp;
 
 	dc = SP_DYNA_DRAW_CONTEXT (event_context);
 	desktop = event_context->desktop;
@@ -519,13 +539,13 @@ sp_dyna_draw_context_root_handler (SPEventContext * event_context,
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
-			sp_desktop_w2d_xy_point (desktop, &fp, event->button.x, event->button.y);
-			p.x = fp.x;
-			p.y = fp.y;
+			double xd, yd;
+			sp_desktop_w2d_xy_point (desktop, &p, event->button.x, event->button.y);
 			sp_dyna_draw_reset (dc, p.x, p.y);
 			sp_dyna_draw_apply (dc, p.x, p.y);
-			sp_dyna_draw_get_curr_vpoint (dc, &p.x, &p.y);
-
+			sp_dyna_draw_get_curr_vpoint (dc, &xd, &yd);
+			p.x = xd;
+			p.y = yd;
 			sp_desktop_free_snap (desktop, &p);
 			sp_curve_reset (dc->accumulated);
 			if (dc->repr) {
@@ -556,14 +576,12 @@ sp_dyna_draw_context_root_handler (SPEventContext * event_context,
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (!dc->use_timeout && (event->motion.state & GDK_BUTTON1_MASK)) {
-			NRPointF fp;
+			double xd, yd;
 
 			dc->dragging = TRUE;
 			dc->dynahand = TRUE;
 
-			sp_desktop_w2d_xy_point (desktop, &fp, event->motion.x, event->motion.y);
-			p.x = fp.x;
-			p.y = fp.y;
+			sp_desktop_w2d_xy_point (desktop, &p, event->motion.x, event->motion.y);
 #if 0
 			g_print ("(%g %g)=>(%g %g)\n", event->motion.x, event->motion.y, p.x, p.y);
 #endif
@@ -571,7 +589,9 @@ sp_dyna_draw_context_root_handler (SPEventContext * event_context,
 				ret = TRUE;
 				break;
 			}
-			sp_dyna_draw_get_curr_vpoint (dc, &p.x, &p.y);
+			sp_dyna_draw_get_curr_vpoint (dc, &xd, &yd);
+			p.x = xd;
+			p.y = yd;
 
 			sp_desktop_free_snap (desktop, &p);
 
@@ -629,8 +649,8 @@ sp_dyna_draw_context_root_handler (SPEventContext * event_context,
 	}
 
 	if (!ret) {
-		if (SP_EVENT_CONTEXT_CLASS (parent_class)->root_handler)
-			ret = SP_EVENT_CONTEXT_CLASS (parent_class)-> root_handler (event_context, event);
+		if (((SPEventContextClass *) parent_class)->root_handler)
+			ret = ((SPEventContextClass *) parent_class)-> root_handler (event_context, event);
 	}
 
 	return ret;
@@ -759,17 +779,6 @@ accumulate_calligraphic (SPDynaDrawContext * dc)
     }
 }
 
-#define BEZIER_ASSERT(b) { \
-      g_assert ((b[0].x > -8000.0) && (b[0].x < 8000.0)); \
-      g_assert ((b[0].y > -8000.0) && (b[0].y < 8000.0)); \
-      g_assert ((b[1].x > -8000.0) && (b[1].x < 8000.0)); \
-      g_assert ((b[1].y > -8000.0) && (b[1].y < 8000.0)); \
-      g_assert ((b[2].x > -8000.0) && (b[2].x < 8000.0)); \
-      g_assert ((b[2].y > -8000.0) && (b[2].y < 8000.0)); \
-      g_assert ((b[3].x > -8000.0) && (b[3].x < 8000.0)); \
-      g_assert ((b[3].y > -8000.0) && (b[3].y < 8000.0)); \
-      }
-
 static void
 fit_and_split (SPDynaDrawContext *dc,
                gboolean           release)
@@ -788,7 +797,7 @@ static void
 fit_and_split_line (SPDynaDrawContext *dc,
                     gboolean           release)
 {
-	ArtPoint b[4];
+	NRPointF b[4];
 	gdouble tolerance;
 
 	tolerance = SP_EVENT_CONTEXT (dc)->desktop->w2d[0] * TOLERANCE_LINE;
@@ -824,10 +833,10 @@ fit_and_split_line (SPDynaDrawContext *dc,
 		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (cbp), curve);
 		sp_curve_unref (curve);
 		/* fixme: We have to parse style color somehow */
-		sp_canvas_bpath_set_fill (SP_CANVAS_BPATH (cbp), DDC_GREEN_RGBA, ART_WIND_RULE_ODDEVEN);
-		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cbp), 0x000000ff, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
+		sp_canvas_bpath_set_fill (SP_CANVAS_BPATH (cbp), DDC_GREEN_RGBA, SP_WIND_RULE_EVENODD);
+		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cbp), 0x000000ff, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 		/* fixme: Cannot we cascade it to root more clearly? */
-		gtk_signal_connect (GTK_OBJECT (cbp), "event", GTK_SIGNAL_FUNC (sp_desktop_root_handler), SP_EVENT_CONTEXT (dc)->desktop);
+		g_signal_connect (G_OBJECT (cbp), "event", G_CALLBACK (sp_desktop_root_handler), SP_EVENT_CONTEXT (dc)->desktop);
 
 		dc->segments = g_slist_prepend (dc->segments, cbp);
 
@@ -860,7 +869,7 @@ fit_and_split_calligraphics (SPDynaDrawContext *dc, gboolean release)
 #define BEZIER_MAX_DEPTH  4
 #define BEZIER_MAX_LENGTH (BEZIER_SIZE * (2 << (BEZIER_MAX_DEPTH-1)))
 		SPCurve *curve;
-		ArtPoint b1[BEZIER_MAX_LENGTH], b2[BEZIER_MAX_LENGTH];
+		NRPointF b1[BEZIER_MAX_LENGTH], b2[BEZIER_MAX_LENGTH];
 		gint nb1, nb2;            /* number of blocks */
 
 #ifdef DYNA_DRAW_VERBOSE
@@ -884,7 +893,7 @@ fit_and_split_calligraphics (SPDynaDrawContext *dc, gboolean release)
 		nb2 = sp_bezier_fit_cubic_r (b2, dc->point2, dc->npoints,
 					     tolerance, BEZIER_MAX_DEPTH);
 		if (nb1 != -1 && nb2 != -1) {
-			ArtPoint *bp1, *bp2;
+			NRPointF *bp1, *bp2;
 			/* Fit and draw and reset state */
 #ifdef DYNA_DRAW_VERBOSE
 			g_print ("nb1:%d nb2:%d\n", nb1, nb2);
@@ -946,10 +955,10 @@ fit_and_split_calligraphics (SPDynaDrawContext *dc, gboolean release)
 			curve = sp_curve_copy (dc->currentcurve);
 			sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (cbp), curve);
 			sp_curve_unref (curve);
-			sp_canvas_bpath_set_fill (SP_CANVAS_BPATH (cbp), 0x000000ff, ART_WIND_RULE_ODDEVEN);
-			sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cbp), 0x00000000, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
+			sp_canvas_bpath_set_fill (SP_CANVAS_BPATH (cbp), 0x000000ff, SP_WIND_RULE_EVENODD);
+			sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cbp), 0x00000000, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 			/* fixme: Cannot we cascade it to root more clearly? */
-			gtk_signal_connect (GTK_OBJECT (cbp), "event", GTK_SIGNAL_FUNC (sp_desktop_root_handler), SP_EVENT_CONTEXT (dc)->desktop);
+			g_signal_connect (G_OBJECT (cbp), "event", G_CALLBACK (sp_desktop_root_handler), SP_EVENT_CONTEXT (dc)->desktop);
 
 			dc->segments = g_slist_prepend (dc->segments, cbp);
 		}
