@@ -13,31 +13,37 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
+
 #include "helper/sp-canvas.h"
 #include "xml/repr-private.h"
 #include "sp-cursor.h"
+
+#include "shortcuts.h"
+
 #include "desktop.h"
 #include "desktop-handles.h"
 #include "desktop-affine.h"
+#include "selection.h"
 #include "event-context.h"
 #include "event-broker.h"
 #include "sp-item.h"
 #include "zoom-context.h"
 #include "file.h"
 #include "interface.h"
+#include "helper/sp-intl.h"
 #include "selection-chemistry.h"
 #include "dialogs/desktop-properties.h"
 
-static void sp_event_context_class_init (SPEventContextClass * klass);
-static void sp_event_context_init (SPEventContext * event_context);
-static void sp_event_context_destroy (GtkObject * object);
+static void sp_event_context_class_init (SPEventContextClass *klass);
+static void sp_event_context_init (SPEventContext *event_context);
+static void sp_event_context_dispose (GObject *object);
 
 static void sp_event_context_private_setup (SPEventContext *ec);
 static gint sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent * event);
@@ -45,46 +51,37 @@ static gint sp_event_context_private_item_handler (SPEventContext * event_contex
 
 static void set_event_location (SPDesktop * desktop, GdkEvent * event);
 
-#if 0
-static void sp_event_grab_item_destroy (GtkObject * object, gpointer data);
-#endif
+static GObjectClass *parent_class;
 
-static GtkObjectClass * parent_class;
-
-GtkType
+GType
 sp_event_context_get_type (void)
 {
-	static GtkType event_context_type = 0;
-
-	if (!event_context_type) {
-
-		static const GtkTypeInfo event_context_info = {
-			"SPEventContext",
-			sizeof (SPEventContext),
+	static GType type = 0;
+	if (!type) {
+		GTypeInfo info = {
 			sizeof (SPEventContextClass),
-			(GtkClassInitFunc) sp_event_context_class_init,
-			(GtkObjectInitFunc) sp_event_context_init,
-			NULL,
-			NULL,
-			(GtkClassInitFunc) NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_event_context_class_init,
+			NULL, NULL,
+			sizeof (SPEventContext),
+			4,
+			(GInstanceInitFunc) sp_event_context_init,
 		};
-
-		event_context_type = gtk_type_unique (gtk_object_get_type (), &event_context_info);
+		type = g_type_register_static (G_TYPE_OBJECT, "SPEventContext", &info, 0);
 	}
-
-	return event_context_type;
+	return type;
 }
 
 static void
-sp_event_context_class_init (SPEventContextClass * klass)
+sp_event_context_class_init (SPEventContextClass *klass)
 {
-	GtkObjectClass * object_class;
+	GObjectClass *object_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 
-	parent_class = gtk_type_class (gtk_object_get_type ());
+	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->destroy = sp_event_context_destroy;
+	object_class->dispose = sp_event_context_dispose;
 
 	klass->setup = sp_event_context_private_setup;
 	klass->root_handler = sp_event_context_private_root_handler;
@@ -92,27 +89,25 @@ sp_event_context_class_init (SPEventContextClass * klass)
 }
 
 static void
-sp_event_context_init (SPEventContext * event_context)
+sp_event_context_init (SPEventContext *event_context)
 {
 	event_context->desktop = NULL;
 	event_context->cursor = NULL;
 }
 
 static void
-sp_event_context_destroy (GtkObject *object)
+sp_event_context_dispose (GObject *object)
 {
 	SPEventContext *ec;
 
 	ec = SP_EVENT_CONTEXT (object);
 
 	if (ec->cursor != NULL) {
-		gdk_cursor_destroy (ec->cursor);
+		gdk_cursor_unref (ec->cursor);
+		ec->cursor = NULL;
 	}
 
 	if (ec->desktop) {
-#if 0
-		gtk_signal_disconnect_by_data (GTK_OBJECT (ec->desktop), ec);
-#endif
 		ec->desktop = NULL;
 	}
 
@@ -122,15 +117,14 @@ sp_event_context_destroy (GtkObject *object)
 		ec->repr = NULL;
 	}
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
 sp_event_context_private_setup (SPEventContext *ec)
 {
-	GtkWidget * w;
-	GdkBitmap * bitmap, * mask;
+	GtkWidget *w;
+	GdkBitmap *bitmap, * mask;
 
 	w = GTK_WIDGET (SP_DT_CANVAS (ec->desktop));
 	if (w->window) {
@@ -150,10 +144,10 @@ sp_event_context_private_setup (SPEventContext *ec)
 }
 
 static gint
-sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent * event)
+sp_event_context_private_root_handler (SPEventContext *event_context, GdkEvent *event)
 {
-	static ArtPoint s;
-	static gboolean panning = FALSE;
+	static NRPointF s;
+	static unsigned int panning = 0;
 	gint ret;
 	SPDesktop * desktop;
 	ret = FALSE;
@@ -162,203 +156,184 @@ sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent
 
 	switch (event->type) {
 	case GDK_2BUTTON_PRESS:
-		sp_desktop_dialog ();
+		if (panning) {
+			panning = 0;
+			sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
+			ret = TRUE;
+		} else {
+			/* sp_desktop_dialog (); */
+		}
 		break;
 	case GDK_BUTTON_PRESS:
 		switch (event->button.button) {
 		case 2:
 			s.x = event->button.x;
 			s.y = event->button.y;
-			panning = TRUE;
+			panning = 2;
 			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
-					     GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
+					     GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK,
 					     NULL, event->button.time);
 
 			ret = TRUE;
 			break;
 		case 3:
-			/* fixme: */
-			sp_event_root_menu_popup (desktop, NULL, event);
+			if (event->button.state & GDK_SHIFT_MASK) {
+				s.x = event->button.x;
+				s.y = event->button.y;
+				panning = 3;
+				sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
+						     GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK,
+						     NULL, event->button.time);
+				ret = TRUE;
+			} else {
+				/* fixme: */
+				sp_event_root_menu_popup (desktop, NULL, event);
+			}
 			break;
 		default:
 			break;
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
-		if (panning && (event->motion.state & GDK_BUTTON2_MASK)) {
-			sp_desktop_scroll_world (event_context->desktop, event->motion.x - s.x, event->motion.y - s.y);
-			ret = TRUE;
+		if (panning) {
+			if (((panning == 2) && !(event->motion.state & GDK_BUTTON2_MASK)) ||
+			    ((panning == 3) && !(event->motion.state & GDK_BUTTON3_MASK))) {
+				/* Gdk seems to lose button release for us sometimes :-( */
+				panning = 0;
+				sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
+				ret = TRUE;
+			} else {
+				sp_desktop_scroll_relative_canvas (event_context->desktop, event->motion.x - s.x, event->motion.y - s.y);
+				ret = TRUE;
+			}
 		}
 		break;
 	case GDK_BUTTON_RELEASE:
-		if (event->button.button == 2) {
-			panning = FALSE;
+		if (panning == event->button.button) {
+			panning = 0;
 			sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
 			ret = TRUE;
 		}
 		break;
         case GDK_KEY_PRESS:
-		switch (event->key.keyval) {  
+		switch (event->key.keyval) {
+			unsigned int shortcut;
+		case GDK_F1:
+			/* Grab it away from Gtk */
+			shortcut = event->key.keyval;
+			if (event->key.state & GDK_SHIFT_MASK) shortcut |= SP_SHORTCUT_SHIFT_MASK;
+			if (event->key.state & GDK_CONTROL_MASK) shortcut |= SP_SHORTCUT_CONTROL_MASK;
+			if (event->key.state & GDK_MOD1_MASK) shortcut |= SP_SHORTCUT_ALT_MASK;
+			ret = sp_shortcut_run (shortcut);
 		case GDK_Tab: // disable tab/shift-tab which cycle widget focus
 		case GDK_ISO_Left_Tab: // they will get different functions
 			ret = TRUE;
 			break;
-		case GDK_F1: // F1 - select context 
-			sp_event_context_set_select(NULL);
-			ret = TRUE;
-			break;
-		case GDK_F2: // F2 - node edit context 
-			sp_event_context_set_node_edit(NULL);	    
-			ret = TRUE;
-			break;
-		case GDK_F3: // F3 - zoom context
-			sp_event_context_set_zoom(NULL);	    
-			ret = TRUE;
-			break;
-		case GDK_F4: // F4 - rect context
-			sp_event_context_set_rect(NULL);	    
-			ret = TRUE;
-			break;
-		case GDK_F5: // F5 - arc context
-			sp_event_context_set_arc(NULL);	    
-			ret = TRUE;
-			break;
-		case GDK_F6: // F6 - frehand line context
-			sp_event_context_set_freehand(NULL);	    
-			ret = TRUE;
-			break;
-		case GDK_F7: // F7 - text context
-			sp_event_context_set_text(NULL);	    
-			ret = TRUE;
-			break;
-		case GDK_equal: // = 
-		case GDK_plus: // + - zoom in
-			sp_zoom_in(NULL);
-			ret = TRUE;
-			break;
-		case GDK_minus: // - - zoom out
-			sp_zoom_out(NULL);
-			ret = TRUE;
-			break;
-		case GDK_0: // 0 - zoom entire page
-			sp_zoom_page(NULL);
-			ret = TRUE;
-			break;
-		case GDK_1: // 1 - zoom 1 to 1
-			sp_zoom_1_to_1(NULL);
-			ret = TRUE;
-			break;
-		case GDK_z: // Ctrl z - undo
-			if (event->key.state & GDK_CONTROL_MASK) {
-				sp_undo (NULL);
-				ret = TRUE;
-			}
-			break;
-		case GDK_r: // Ctrl r - redo
-			if (event->key.state & GDK_CONTROL_MASK) {
-				sp_redo (NULL);
-				ret = TRUE;
-			}
-			break;
-		case GDK_w: // Crtl w - close view
+		case GDK_W:
+		case GDK_w:
+			/* Close view */
 			if (event->key.state & GDK_CONTROL_MASK) {
 				sp_ui_close_view (NULL);
 				ret = TRUE;
 			}
 			break;
-		case GDK_n: // Crtl n - new document
-			if (event->key.state & GDK_CONTROL_MASK) {
-				ret = TRUE;
-				sp_file_new ();
-			}
-			break;
-		case GDK_N: // Ctrl N - new view
-			if (event->key.state & GDK_CONTROL_MASK) {
-				sp_ui_new_view (NULL);
-				ret = TRUE;
-			}
-			break;
-		case GDK_o: // Ctrl o - open file
-			if (event->key.state & GDK_CONTROL_MASK) {
-				sp_file_open_dialog (NULL, NULL);
-				ret = TRUE;
-			}
-			break;
-#if 0
-		case GDK_e: // Ctrl e - export file
-			if (event->key.state & GDK_CONTROL_MASK) {
-				sp_file_export (NULL);
-				ret = TRUE;
-			}
-			break;
-#endif
 		case GDK_i: // Ctrl i - import file
 			if (event->key.state & GDK_CONTROL_MASK) {
 				sp_file_import (NULL);
 				ret = TRUE;
 			}
 			break;
-		case GDK_p: // Crtl p - print document
+		case GDK_q: // Ctrl q - quit
 			if (event->key.state & GDK_CONTROL_MASK) {
+				sp_file_exit ();
 				ret = TRUE;
-				sp_file_print (NULL, NULL);
 			}
 			break;
-		case GDK_P: // Crtl P - print preview
+		case GDK_Left: // Ctrl Left 
 			if (event->key.state & GDK_CONTROL_MASK) {
+				sp_desktop_scroll_relative_canvas (event_context->desktop, 10, 0);
 				ret = TRUE;
-				sp_file_print_preview (NULL, NULL);
 			}
 			break;
-	  case GDK_s: // Crtl s - save file
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      ret = TRUE;
-	      sp_file_save (NULL);
-	    }
-	    break;
-	  case GDK_S: // Crtl S - save file as
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      ret = TRUE;
-	      sp_file_save_as (NULL);
-	    }
-	    break;
-	  case GDK_q: // Ctrl q - quit
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_file_exit ();
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_Left: // Ctrl Left 
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_desktop_scroll_world (event_context->desktop, 10, 0);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_Up: // Ctrl Up
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_desktop_scroll_world (event_context->desktop, 0, +10);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_Right: // Ctrl Right
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_desktop_scroll_world (event_context->desktop, -10, 0);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_Down: // Ctrl Down
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_desktop_scroll_world (event_context->desktop, 0, -10);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_space: // Space - root menu
-	    sp_event_root_menu_popup (desktop, NULL, event);
-	    ret= TRUE;
-	    break;
-          }
-          g_print ("What a funny key: %d \n", event->key.keyval);
-          break;
+		case GDK_Up: // Ctrl Up
+			if (event->key.state & GDK_CONTROL_MASK) {
+				sp_desktop_scroll_relative_canvas (event_context->desktop, 0, +10);
+				ret = TRUE;
+			}
+			break;
+		case GDK_Right: // Ctrl Right
+			if (event->key.state & GDK_CONTROL_MASK) {
+				sp_desktop_scroll_relative_canvas (event_context->desktop, -10, 0);
+				ret = TRUE;
+			}
+			break;
+		case GDK_Down: // Ctrl Down
+			if (event->key.state & GDK_CONTROL_MASK) {
+				sp_desktop_scroll_relative_canvas (event_context->desktop, 0, -10);
+				ret = TRUE;
+			}
+			break;
+		case GDK_space: // Space - root menu
+			sp_event_root_menu_popup (desktop, NULL, event);
+			ret= TRUE;
+			break;
+		case GDK_F10:
+			if (event->key.state & GDK_SHIFT_MASK) {
+				sp_event_root_menu_popup (desktop, NULL, event);
+				ret= TRUE;
+			}
+			break;
+		}
+#if 0
+		g_print ("What a funny key: %d \n", event->key.keyval);
+#endif
+		break;
+	case GDK_SCROLL:
+		/* ctrl + wheel, pan left--right */
+		if (event->scroll.state & GDK_CONTROL_MASK) {
+			switch (event->scroll.direction) {
+			case GDK_SCROLL_UP:
+				sp_desktop_scroll_relative_canvas (desktop, SP_MOUSEMOVE_STEP, 0);
+				break;
+			case GDK_SCROLL_DOWN:
+				sp_desktop_scroll_relative_canvas (desktop, -SP_MOUSEMOVE_STEP, 0);
+				break;
+			default:
+				break;
+			}
+			/* shift + wheel, zoom in--out */
+		} else if (event->scroll.state & GDK_SHIFT_MASK) {
+			NRRectF d;
+			switch (event->scroll.direction) {
+			case GDK_SCROLL_UP:
+				sp_desktop_get_display_area (desktop, &d);
+				sp_desktop_zoom_relative (desktop, (d.x0 + d.x1) / 2, (d.y0 + d.y1) / 2, SP_DESKTOP_ZOOM_INC);
+				break;
+			case GDK_SCROLL_DOWN:
+				sp_desktop_get_display_area (desktop, &d);
+				sp_desktop_zoom_relative (desktop, (d.x0 + d.x1) / 2, (d.y0 + d.y1) / 2, 1 / SP_DESKTOP_ZOOM_INC);
+				break;
+			default:
+				break;
+			}
+			/* no modifier, pan up--down (left--right on multiwheel mice?) */
+		} else {
+			switch (event->scroll.direction) {
+			case GDK_SCROLL_UP:
+				sp_desktop_scroll_relative_canvas (desktop, 0, SP_MOUSEMOVE_STEP);
+				break;
+			case GDK_SCROLL_DOWN:
+				sp_desktop_scroll_relative_canvas (desktop, 0, -SP_MOUSEMOVE_STEP);
+				break;
+			case GDK_SCROLL_LEFT:
+				sp_desktop_scroll_relative_canvas (desktop, SP_MOUSEMOVE_STEP, 0);
+				break;
+			case GDK_SCROLL_RIGHT:
+				sp_desktop_scroll_relative_canvas (desktop, -SP_MOUSEMOVE_STEP, 0);
+				break;
+			}
+		}
+		break;
 	default:
 		break;
 	}
@@ -369,42 +344,33 @@ sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent
 /* fixme: do context sensitive popup menu on items */
 
 static gint
-sp_event_context_private_item_handler (SPEventContext *ctx, SPItem *item, GdkEvent *event)
+sp_event_context_private_item_handler (SPEventContext *ec, SPItem *item, GdkEvent *event)
 {
+	int ret;
+
+	ret = FALSE;
+
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
-		switch (event->button.button) {
-		case 3:
-			/* fixme: */
-			sp_event_root_menu_popup (ctx->desktop, item, event);
-			return TRUE;
-		default:
-			break;
+		if ((event->button.button == 3) && !(event->button.state & GDK_SHIFT_MASK)) {
+			sp_event_root_menu_popup (ec->desktop, item, event);
+			ret = TRUE;
 		}
 		break;
 	default:
 		break;
 	}
 
-	return FALSE;
+	return ret;
 }
 
-#if 0
-static void
-sp_event_context_desktop_destroy (GtkObject *object, SPEventContext *ec)
-{
-	/* This is actually non-event, as desktop should keep the only ref of ec */
-	ec->desktop = NULL;
-}
-#endif
-
 static void
 sp_ec_repr_destroy (SPRepr *repr, gpointer data)
 {
 	g_warning ("Oops! Repr destroyed while event context still present");
 }
 
-static gboolean
+static unsigned int
 sp_ec_repr_change_attr (SPRepr *repr, const guchar *key, const guchar *oldval, const guchar *newval, gpointer data)
 {
 	SPEventContext *ec;
@@ -423,8 +389,8 @@ sp_ec_repr_attr_changed (SPRepr *repr, const guchar *key, const guchar *oldval,
 
 	ec = SP_EVENT_CONTEXT (data);
 
-	if (SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->set)
-		SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->set (ec, key, newval);
+	if (((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->set)
+		((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->set (ec, key, newval);
 }
 
 SPReprEventVector sp_ec_event_vector = {
@@ -442,29 +408,26 @@ SPReprEventVector sp_ec_event_vector = {
 };
 
 SPEventContext *
-sp_event_context_new (GtkType type, SPDesktop *desktop, SPRepr *repr)
+sp_event_context_new (GType type, SPDesktop *desktop, SPRepr *repr, unsigned int key)
 {
 	SPEventContext *ec;
 
-	g_return_val_if_fail (gtk_type_is_a (type, SP_TYPE_EVENT_CONTEXT), NULL);
+	g_return_val_if_fail (g_type_is_a (type, SP_TYPE_EVENT_CONTEXT), NULL);
 	g_return_val_if_fail (desktop != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DESKTOP (desktop), NULL);
 
-	ec = gtk_type_new (type);
+	ec = g_object_new (type, NULL);
 
 	ec->desktop = desktop;
-#if 0
-	gtk_signal_connect (GTK_OBJECT (desktop), "destroy", GTK_SIGNAL_FUNC (sp_event_context_desktop_destroy), ec);
-#endif
-
+	ec->key = key;
 	ec->repr = repr;
 	if (ec->repr) {
 		sp_repr_ref (ec->repr);
 		sp_repr_add_listener (ec->repr, &sp_ec_event_vector, ec);
 	}
 
-	if (SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->setup)
-		SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->setup (ec);
+	if (((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->setup)
+		((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->setup (ec);
 
 	return ec;
 }
@@ -475,8 +438,12 @@ sp_event_context_finish (SPEventContext *ec)
 	g_return_if_fail (ec != NULL);
 	g_return_if_fail (SP_IS_EVENT_CONTEXT (ec));
 
-	if (SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->finish)
-		SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->finish (ec);
+	if (ec->next) {
+		g_warning ("Finishing event context with active link\n");
+	}
+
+	if (((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->finish)
+		((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->finish (ec);
 }
 
 void
@@ -489,17 +456,37 @@ sp_event_context_read (SPEventContext *ec, const guchar *key)
 	if (ec->repr) {
 		const guchar *val;
 		val = sp_repr_attr (ec->repr, key);
-		if (SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->set)
-			SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->set (ec, key, val);
+		if (((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->set)
+			((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->set (ec, key, val);
 	}
 }
 
+void
+sp_event_context_activate (SPEventContext *ec)
+{
+	g_return_if_fail (ec != NULL);
+	g_return_if_fail (SP_IS_EVENT_CONTEXT (ec));
+
+	if (((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->activate)
+		((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->activate (ec);
+}
+
+void
+sp_event_context_desactivate (SPEventContext *ec)
+{
+	g_return_if_fail (ec != NULL);
+	g_return_if_fail (SP_IS_EVENT_CONTEXT (ec));
+
+	if (((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->activate)
+		((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->activate (ec);
+}
+
 gint
 sp_event_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 {
 	gint ret;
 
-	ret = (* SP_EVENT_CONTEXT_CLASS (event_context->object.klass)->root_handler) (event_context, event);
+	ret = ((SPEventContextClass *) G_OBJECT_GET_CLASS (event_context))->root_handler (event_context, event);
 
 	set_event_location (event_context->desktop, event);
 
@@ -511,7 +498,7 @@ sp_event_context_item_handler (SPEventContext * event_context, SPItem * item, Gd
 {
 	gint ret;
 
-	ret = (* SP_EVENT_CONTEXT_CLASS (event_context->object.klass)->item_handler) (event_context, item, event);
+	ret = ((SPEventContextClass *) G_OBJECT_GET_CLASS (event_context))->item_handler (event_context, item, event);
 
 	if (! ret) {
 		ret = sp_event_context_root_handler (event_context, event);
@@ -528,8 +515,8 @@ sp_event_context_config_widget (SPEventContext *ec)
 	g_return_val_if_fail (ec != NULL, NULL);
 	g_return_val_if_fail (SP_IS_EVENT_CONTEXT (ec), NULL);
 
-	if (SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->config_widget)
-		return SP_EVENT_CONTEXT_CLASS (((GtkObject *) ec)->klass)->config_widget (ec);
+	if (((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->config_widget)
+		return ((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->config_widget (ec);
 
 	return NULL;
 }
@@ -552,6 +539,10 @@ sp_event_root_menu_popup (SPDesktop *desktop, SPItem *item, GdkEvent *event)
 {
 	GtkWidget *menu;
 
+	/* fixme: This is not what I want but works for now (Lauris) */
+	if (event->type == GDK_KEY_PRESS) {
+		item = sp_selection_item (SP_DT_SELECTION (desktop));
+	}
 	menu = sp_ui_generic_menu (SP_VIEW (desktop), item);
 	gtk_widget_show (menu);
 
