@@ -15,21 +15,22 @@
  */
 
 #include "forward.h"
+#include "modules/sp-module.h"
 
 void sp_file_new (void);
 
-void sp_file_open (const guchar *uri);
-
+void sp_file_open (const unsigned char *uri, const unsigned char *key);
 void sp_file_open_dialog (gpointer object, gpointer data);
 
-void sp_file_save (GtkWidget *widget);
-void sp_file_save_document (SPDocument *document);
+void sp_file_save (gpointer object, gpointer data);
+void sp_file_save_as (gpointer object, gpointer data);
 
-void sp_file_save_as (GtkWidget * widget);
+void sp_file_save_document (SPDocument *document);
 
 void sp_file_import (GtkWidget * widget);
 
-void sp_file_print (gpointer object, gpointer data);
+void sp_file_print (void);
+void sp_file_print_direct (void);
 void sp_file_print_preview (gpointer object, gpointer data);
 
 #if 0
