@@ -31,6 +31,9 @@
  * Lauris
  */
 
+static SPGradient *sp_gradient_get_private_normalized (SPDocument *document, SPGradient *vector);
+static SPGradient *sp_gradient_get_radial_private_normalized (SPDocument *document, SPGradient *vector);
+
 static void sp_gradient_repr_set_link (SPRepr *repr, SPGradient *gr);
 static void sp_item_repr_set_style_gradient (SPRepr *repr, const guchar *property, SPGradient *gr);
 static guchar *sp_style_change_property (const guchar *sstr, const guchar *key, const guchar *value);
@@ -71,7 +74,8 @@ sp_gradient_ensure_vector_normalized (SPGradient *gr)
 		/* And do not forget to flatten original */
 		g_print ("GVECTORNORM: Gradient %s IS NOT in <defs>\n", SP_OBJECT_ID (gr));
 		/* Step 1 - flatten original EXCEPT vector */
-		sp_gradient_repr_flatten_attributes (gr, SP_OBJECT_REPR (gr), TRUE);
+		sp_object_invoke_write ((SPObject *) gr, ((SPObject *) gr)->repr,
+					SP_OBJECT_WRITE_SODIPODI | SP_OBJECT_WRITE_ALL);
 		g_print ("GVECTORNORM: Gradient %s attributes flattened\n", SP_OBJECT_ID (gr));
 		/* Step 2 - create new empty gradient and prepend it to <defs> */
 		repr = sp_repr_new ("linearGradient");
@@ -109,13 +113,14 @@ sp_gradient_ensure_vector_normalized (SPGradient *gr)
 		/* Nof break free hrefing */
 		if (gr->href) {
 			/* We are hrefing someone, so require flattening */
-			sp_gradient_repr_flatten_attributes (gr, SP_OBJECT_REPR (gr), TRUE);
+			sp_object_invoke_write ((SPObject *) gr, ((SPObject *) gr)->repr,
+						SP_OBJECT_WRITE_SODIPODI | SP_OBJECT_WRITE_ALL);
 			g_print ("GVECTORNORM: Gradient %s attributes flattened\n", SP_OBJECT_ID (gr));
 			sp_gradient_repr_set_link (SP_OBJECT_REPR (gr), NULL);
 		}
 		/* Now we can be sure we are in good condition */
 		/* Still have to ensure we are in vector position */
-		for (child = defs->children; child != NULL; child = child->next) {
+		for (child = ((SPObject *) defs)->children; child != NULL; child = child->next) {
 			if (SP_IS_GRADIENT (child)) {
 				SPGradient *gchild;
 				gchild = SP_GRADIENT (child);
@@ -182,6 +187,7 @@ sp_gradient_ensure_private_normalized (SPGradient *gr, SPGradient *vector)
 	    (gr->state != SP_GRADIENT_STATE_UNKNOWN) ||
 	    (SP_OBJECT_PARENT (gr) != SP_OBJECT (defs)) ||
 	    (SP_OBJECT_HREFCOUNT (gr) > 1)) {
+#if 0
 		SPRepr *repr;
 		/* We are either in some lonely place, or have multiple refs, or vector or whatever, so clone */
 		/* fixme: no 'linear' please */
@@ -197,6 +203,9 @@ sp_gradient_ensure_private_normalized (SPGradient *gr, SPGradient *vector)
 		/* fixme: Maybe add extra sanity check here */
 		gr->state = SP_GRADIENT_STATE_PRIVATE;
 		return gr;
+#else
+		return sp_gradient_get_private_normalized (SP_OBJECT_DOCUMENT (gr), vector);
+#endif
 	} else {
 		SPObject *sibling;
 		/* We still have to determine, whether we have to change our position */
@@ -248,6 +257,7 @@ sp_gradient_ensure_radial_private_normalized (SPGradient *gr, SPGradient *vector
 	    (gr->state != SP_GRADIENT_STATE_UNKNOWN) ||
 	    (SP_OBJECT_PARENT (gr) != SP_OBJECT (defs)) ||
 	    (SP_OBJECT_HREFCOUNT (gr) > 1)) {
+#if 0
 		SPRepr *repr;
 		/* We are either in some lonely place, or have multiple refs, or vector or whatever, so clone */
 		/* fixme: no 'linear' please */
@@ -263,6 +273,9 @@ sp_gradient_ensure_radial_private_normalized (SPGradient *gr, SPGradient *vector
 		/* fixme: Maybe add extra sanity check here */
 		gr->state = SP_GRADIENT_STATE_PRIVATE;
 		return gr;
+#else
+		return sp_gradient_get_radial_private_normalized (SP_OBJECT_DOCUMENT (gr), vector);
+#endif
 	} else {
 		SPObject *sibling;
 		/* We still have to determine, whether we have to change our position */
@@ -329,7 +342,7 @@ sp_gradient_get_private_normalized (SPDocument *document, SPGradient *vector)
 
 	defs = (SPDefs *) SP_DOCUMENT_DEFS (document);
 
-	for (child = defs->children; child != NULL; child = child->next) {
+	for (child = ((SPObject *) defs)->children; child != NULL; child = child->next) {
 		if (SP_IS_LINEARGRADIENT (child)) {
 			SPGradient *gr;
 			gr = SP_GRADIENT (child);
@@ -399,7 +412,7 @@ sp_gradient_get_radial_private_normalized (SPDocument *document, SPGradient *vec
 
 	defs = (SPDefs *) SP_DOCUMENT_DEFS (document);
 
-	for (child = defs->children; child != NULL; child = child->next) {
+	for (child = ((SPObject *) defs)->children; child != NULL; child = child->next) {
 		if (SP_IS_RADIALGRADIENT (child)) {
 			SPGradient *gr;
 			gr = SP_GRADIENT (child);
@@ -718,7 +731,7 @@ sp_document_default_gradient_vector (SPDocument *document)
 
 	defs = (SPDefs *) SP_DOCUMENT_DEFS (document);
 
-	for (child = defs->children; child != NULL; child = child->next) {
+	for (child = ((SPObject *) defs)->children; child != NULL; child = child->next) {
 		if (SP_IS_GRADIENT (child)) {
 			SPGradient *gr;
 			gr = SP_GRADIENT (child);
@@ -792,7 +805,7 @@ sp_object_ensure_fill_gradient_normalized (SPObject *object)
 	/* Fill has to be set */
 	g_return_if_fail (object->style->fill.set);
 	/* Fill has to be paintserver */
-	g_return_if_fail (object->style->fill.type = SP_PAINT_TYPE_PAINTSERVER);
+	g_return_if_fail (object->style->fill.type == SP_PAINT_TYPE_PAINTSERVER);
 	/* Has to be linear gradient */
 	g_return_if_fail (SP_IS_LINEARGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object)));
 
