@@ -13,7 +13,11 @@
  */
 
 #include <png.h>
+
+#include <libarikkei/arikkei-strlib.h>
+
 #include <glib.h>
+
 #include "png-write.h"
 
 /* This is an example of how to use libpng to read and write PNG files.
@@ -148,16 +152,18 @@ sp_png_write_rgba (const unsigned char *filename, const unsigned char *px, int w
 #endif
 
 	/* Optionally write comments into the image */
-	text_ptr[0].key = "Title";
-	text_ptr[0].text = "Made with Sodipodi";
+	text_ptr[0].key = "Software";
+	text_ptr[0].text = "Sodipodi";
 	text_ptr[0].compression = PNG_TEXT_COMPRESSION_NONE;
+#if 0
 	text_ptr[1].key = "Author";
 	text_ptr[1].text = "Unknown";
 	text_ptr[1].compression = PNG_TEXT_COMPRESSION_NONE;
 	text_ptr[2].key = "Description";
 	text_ptr[2].text = "a picture";
 	text_ptr[2].compression = PNG_TEXT_COMPRESSION_zTXt;
-	png_set_text(png_ptr, info_ptr, text_ptr, 3);
+#endif
+	png_set_text(png_ptr, info_ptr, text_ptr, 1);
 
 	/* other optional chunks like cHRM, bKGD, tRNS, tIME, oFFs, pHYs, */
 	/* note that if sRGB is present the cHRM chunk must be ignored
@@ -216,11 +222,19 @@ sp_png_write_rgba (const unsigned char *filename, const unsigned char *px, int w
 }
 #endif
 
+#ifdef WIN32
+#include <windows.h>
+#include <tchar.h>
+#endif
+
 int
 sp_png_write_rgba_striped (const unsigned char *filename, int width, int height,
 			   int (* get_rows) (const unsigned char **rows, int row, int num_rows, void *data),
 			   void *data)
 {
+#ifdef WIN32
+	TCHAR *tfilename;
+#endif
 	FILE *fp;
 	png_structp png_ptr;
 	png_infop info_ptr;
@@ -231,8 +245,17 @@ sp_png_write_rgba_striped (const unsigned char *filename, int width, int height,
 	g_return_val_if_fail (filename != NULL, FALSE);
 
 	/* open the file */
-
+#ifdef WIN32
+#ifdef _UNICODE
+	tfilename = arikkei_utf8_ucs2_strdup (filename);
+#else
+	tfilename = strdup (filename);
+#endif
+	fp = _tfopen (tfilename, TEXT("wb"));
+	free (tfilename);
+#else
 	fp = fopen (filename, "wb");
+#endif
 	g_return_val_if_fail (fp != NULL, FALSE);
 
 	/* Create and initialize the png_struct with the desired error handler
