@@ -7,11 +7,11 @@
  *
  */
 
-#include <gtk/gtkpacker.h>
+#include <gtk/gtkenums.h>
 #include "sp-canvas.h"
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define SP_TYPE_CTRL            (sp_ctrl_get_type ())
 #define SP_CTRL(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_CTRL, SPCtrl))
@@ -52,9 +52,9 @@ struct _SPCtrl {
 	guint32 fill_color;
 	guint32 stroke_color;
 
-	ArtIRect box;			/* NB! x1 & y1 are included */
-        gpointer cache;
-        GdkPixbuf * pixbuf;
+	NRRectL box; /* NB! x1 & y1 are included */
+	guchar *cache;
+	GdkPixbuf *pixbuf;
 };
 
 struct _SPCtrlClass {
@@ -68,6 +68,6 @@ GtkType sp_ctrl_get_type (void);
 
 void sp_ctrl_moveto (SPCtrl * ctrl, double x, double y);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
