@@ -14,10 +14,11 @@
  *
  */
 
+#include <glib.h>
 #include <libart_lgpl/art_rect.h>
 #include "sp-canvas.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define SP_TYPE_CTRLRECT (sp_ctrlrect_get_type ())
 #define SP_CTRLRECT(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_CTRLRECT, SPCtrlRect))
@@ -51,13 +52,13 @@ struct _SPCtrlRectClass {
 /* Standard Gtk function */
 GtkType sp_ctrlrect_get_type (void);
 
-void sp_ctrlrect_set_area (SPCtrlRect *rect, gint x0, gint y0, gint x1, gint y1);
-void sp_ctrlrect_set_color (SPCtrlRect *rect, guint32 border_color, gboolean has_fill, guint32 fill_color);
+void sp_ctrlrect_set_area (SPCtrlRect *rect, double x0, double y0, double x1, double y1);
+void sp_ctrlrect_set_color (SPCtrlRect *cr, unsigned int rgba, unsigned int fill, unsigned int fill_rgba);
 void sp_ctrlrect_set_shadow (SPCtrlRect *rect, gint shadow_size, guint32 shadow_color);
 
 /* Deprecated */
 void sp_ctrlrect_set_rect (SPCtrlRect * rect, ArtDRect * box);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
