@@ -19,6 +19,35 @@
 #include <libart_lgpl/art_rgb_svp.h>
 #include "sp-canvas-util.h"
 
+#if 0
+/*
+ * Grabs canvas item, but unlike the original method does not pass
+ * illegal key event mask to canvas, who passes it ahead to Gdk, but
+ * instead sets event mask in canvas struct by hand
+ */
+
+int
+sp_canvas_item_grab (GnomeCanvasItem *item, unsigned int event_mask, GdkCursor *cursor, guint32 etime)
+{
+	int ret;
+
+	g_return_val_if_fail (item != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_CANVAS_ITEM (item), -1);
+
+	ret = gnome_canvas_item_grab (item,
+				      event_mask & (~(GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK)),
+				      cursor,
+				      etime);
+			
+	/* fixme: Top hack (Lauris) */
+	/* fixme: If we add key masks to event mask, Gdk will abort (Lauris) */
+	/* fixme: But Canvas actualle does get key events, so all we need is routing these here */
+	item->canvas->grabbed_event_mask = event_mask;
+
+	return ret;
+}
+#endif
+
 void
 sp_canvas_update_bbox (SPCanvasItem *item, int x1, int y1, int x2, int y2)
 {
@@ -42,25 +71,34 @@ sp_canvas_item_reset_bounds (SPCanvasItem *item)
 void
 sp_canvas_buf_ensure_buf (SPCanvasBuf *buf)
 {
-	if (!buf->is_buf) {
-		unsigned int r, g, b;
+	unsigned int r, g, b;
+	if (!buf->pixblock.empty) return;
+	r = buf->bgcolor >> 16;
+	g = (buf->bgcolor >> 8) & 0xff;
+	b = buf->bgcolor & 0xff;
+	if ((r != g) || (r != b)) {
 		int x, y;
-		r = buf->bg_color >> 16;
-		g = (buf->bg_color >> 8) & 0xff;
-		b = buf->bg_color & 0xff;
-		for (y = buf->rect.y0; y < buf->rect.y1; y++) {
-			guchar *p;
-			p = buf->buf + (y - buf->rect.y0) * buf->buf_rowstride;
-			for (x = buf->rect.x0; x < buf->rect.x1; x++) {
+		for (y = buf->pixblock.area.y0; y < buf->pixblock.area.y1; y++) {
+			unsigned char *p;
+			p = NR_PIXBLOCK_PX (&buf->pixblock) + (y - buf->pixblock.area.y0) * buf->pixblock.rs;
+			for (x = buf->pixblock.area.x0; x < buf->pixblock.area.x1; x++) {
 				*p++ = r;
 				*p++ = g;
 				*p++ = b;
 			}
 		}
-		buf->is_buf = 1;
+	} else {
+		int y;
+		for (y = buf->pixblock.area.y0; y < buf->pixblock.area.y1; y++) {
+			unsigned char *p;
+			p = NR_PIXBLOCK_PX (&buf->pixblock) + (y - buf->pixblock.area.y0) * buf->pixblock.rs;
+			memset (p, r, 3 * (buf->pixblock.area.x1 - buf->pixblock.area.x0));
+		}
 	}
+	buf->pixblock.empty = 0;
 }
 
+#if 0
 void
 sp_canvas_clear_buffer (SPCanvasBuf *buf)
 {
@@ -88,7 +126,9 @@ sp_canvas_clear_buffer (SPCanvasBuf *buf)
 		}
 	}
 }
+#endif
 
+#if 0
 void
 sp_canvas_render_svp_translated (SPCanvasBuf * buf, ArtSVP * svp,
 	guint32 rgba, gint x, gint y)
@@ -143,7 +183,9 @@ sp_canvas_render_svp_translated (SPCanvasBuf * buf, ArtSVP * svp,
 				   NULL);
 	}
 }
+#endif
 
+#if 0
 void sp_canvas_item_i2p_affine (SPCanvasItem * item, double affine[])
 {
 	g_return_if_fail (item != NULL);
@@ -189,6 +231,7 @@ void sp_canvas_item_set_i2w_affine (SPCanvasItem * item, double i2w[])
 	art_affine_multiply (i2p, i2w, w2p);
 	sp_canvas_item_affine_absolute (item, i2p);
 }
+#endif
 
 void sp_canvas_item_move_to_z (SPCanvasItem * item, gint z)
 {
@@ -220,3 +263,81 @@ sp_canvas_item_compare_z (SPCanvasItem * a, SPCanvasItem * b)
 	return 0;
 }
 
+#define RGBA_R(v) ((v) >> 24)
+#define RGBA_G(v) (((v) >> 16) & 0xff)
+#define RGBA_B(v) (((v) >> 8) & 0xff)
+#define RGBA_A(v) ((v) & 0xff)
+#define COMPOSE(b,f,a) (((255 - (a)) * b + (f * a) + 127) / 255)
+
+void
+sp_render_hline (NRPixBlock *pb, gint y, gint xs, gint xe, guint32 rgba)
+{
+	if ((y >= pb->area.y0) && (y < pb->area.y1)) {
+		unsigned int r, g, b, a;
+		int x0, x1, x;
+		unsigned char *p;
+		r = RGBA_R (rgba);
+		g = RGBA_G (rgba);
+		b = RGBA_B (rgba);
+		a = RGBA_A (rgba);
+		x0 = MAX (pb->area.x0, xs);
+		x1 = MIN (pb->area.x1, xe + 1);
+		p = NR_PIXBLOCK_PX (pb) + (y - pb->area.y0) * pb->rs + (x0 - pb->area.x0) * 3;
+		for (x = x0; x < x1; x++) {
+			p[0] = COMPOSE (p[0], r, a);
+			p[1] = COMPOSE (p[1], g, a);
+			p[2] = COMPOSE (p[2], b, a);
+			p += 3;
+		}
+	}
+}
+
+void
+sp_render_vline (NRPixBlock *pb, gint x, gint ys, gint ye, guint32 rgba)
+{
+	if ((x >= pb->area.x0) && (x < pb->area.x1)) {
+		guint r, g, b, a;
+		gint y0, y1, y;
+		guchar *p;
+		r = RGBA_R (rgba);
+		g = RGBA_G (rgba);
+		b = RGBA_B (rgba);
+		a = RGBA_A (rgba);
+		y0 = MAX (pb->area.y0, ys);
+		y1 = MIN (pb->area.y1, ye + 1);
+		p = NR_PIXBLOCK_PX (pb) + (y0 - pb->area.y0) * pb->rs + (x - pb->area.x0) * 3;
+		for (y = y0; y < y1; y++) {
+			p[0] = COMPOSE (p[0], r, a);
+			p[1] = COMPOSE (p[1], g, a);
+			p[2] = COMPOSE (p[2], b, a);
+			p += pb->rs;
+		}
+	}
+}
+
+void
+sp_render_area (NRPixBlock *pb, gint xs, gint ys, gint xe, gint ye, guint32 rgba)
+{
+	guint r, g, b, a;
+	gint x0, x1, x;
+	gint y0, y1, y;
+	guchar *p;
+	r = RGBA_R (rgba);
+	g = RGBA_G (rgba);
+	b = RGBA_B (rgba);
+	a = RGBA_A (rgba);
+	x0 = MAX (pb->area.x0, xs);
+	x1 = MIN (pb->area.x1, xe + 1);
+	y0 = MAX (pb->area.y0, ys);
+	y1 = MIN (pb->area.y1, ye + 1);
+	for (y = y0; y < y1; y++) {
+		p = NR_PIXBLOCK_PX (pb) + (y - pb->area.y0) * pb->rs + (x0 - pb->area.x0) * 3;
+		for (x = x0; x < x1; x++) {
+			p[0] = COMPOSE (p[0], r, a);
+			p[1] = COMPOSE (p[1], g, a);
+			p[2] = COMPOSE (p[2], b, a);
+			p += 3;
+		}
+	}
+}
+
