@@ -106,8 +106,10 @@ sp_ctrlline_destroy (GtkObject *object)
 
 	ctrlline = SP_CTRLLINE (object);
 
-	if (ctrlline->svp)
+	if (ctrlline->svp) {
 		art_svp_free (ctrlline->svp);
+		ctrlline->svp = NULL;
+	}
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
@@ -122,8 +124,10 @@ sp_ctrlline_render (SPCanvasItem *item, SPCanvasBuf *buf)
 
 	if (ctrlline->svp) {
 		sp_canvas_buf_ensure_buf (buf);
-		art_rgb_svp_alpha (ctrlline->svp, buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1, ctrlline->rgba,
-				   buf->buf, buf->buf_rowstride,
+		art_rgb_svp_alpha (ctrlline->svp,
+				   buf->pixblock.area.x0, buf->pixblock.area.y0, buf->pixblock.area.x1, buf->pixblock.area.y1,
+				   ctrlline->rgba,
+				   NR_PIXBLOCK_PX (&buf->pixblock), buf->pixblock.rs,
 				   NULL);
 	}
 }
@@ -134,7 +138,6 @@ sp_ctrlline_update (SPCanvasItem *item, double *affine, unsigned int flags)
 	SPCtrlLine *cl;
 	ArtPoint p;
 	ArtVpath vpath[3];
-	ArtSVP *svp;
 	ArtDRect dbox;
 	ArtIRect ibox;
 
@@ -147,6 +150,11 @@ sp_ctrlline_update (SPCanvasItem *item, double *affine, unsigned int flags)
 
 	sp_canvas_item_reset_bounds (item);
 
+	if (cl->svp) {
+		art_svp_free (cl->svp);
+		cl->svp = NULL;
+	}
+
 	p.x = cl->s.x;
 	p.y = cl->s.y;
 	art_affine_point (&p, &p, affine);
@@ -165,8 +173,7 @@ sp_ctrlline_update (SPCanvasItem *item, double *affine, unsigned int flags)
 
 	vpath[2].code = ART_END;
 
-	svp = art_svp_vpath_stroke (vpath, ART_PATH_STROKE_CAP_BUTT, ART_PATH_STROKE_JOIN_MITER, 1, 4, 0.25);
-	cl->svp = svp;
+	cl->svp = art_svp_vpath_stroke (vpath, ART_PATH_STROKE_CAP_BUTT, ART_PATH_STROKE_JOIN_MITER, 1, 4, 0.25);
 
 	art_drect_svp (&dbox, cl->svp);
 	art_drect_to_irect (&ibox, &dbox);
