@@ -14,7 +14,7 @@
 
 #include "sp-canvas.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define SP_TYPE_CTRLLINE (sp_ctrlline_get_type ())
 #define SP_CTRLLINE(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_CTRLLINE, SPCtrlLine))
@@ -26,8 +26,8 @@ typedef struct _SPCtrlLineClass SPCtrlLineClass;
 GtkType sp_ctrlline_get_type (void);
 
 void sp_ctrlline_set_rgba32 (SPCtrlLine *cl, guint32 rgba);
-void sp_ctrlline_set_coords (SPCtrlLine *cl, gdouble x0, gdouble y0, gdouble x2, gdouble y1);
+void sp_ctrlline_set_coords (SPCtrlLine *cl, gdouble x0, gdouble y0, gdouble x1, gdouble y1);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
