@@ -10,11 +10,11 @@
  *
  */
 
+#include <glib.h>
 #include <gtk/gtkoptionmenu.h>
-#include <libgnome/gnome-defs.h>
 #include "units.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 /* Unit selector Widget */
 
@@ -27,25 +27,24 @@ BEGIN_GNOME_DECLS
 typedef struct _SPUnitSelector SPUnitSelector;
 typedef struct _SPUnitSelectorClass SPUnitSelectorClass;
 
-GtkType sp_unit_selector_get_type (void);
+GType sp_unit_selector_get_type (void);
 
 GtkWidget *sp_unit_selector_new (guint bases);
 
 const SPUnit *sp_unit_selector_get_unit (SPUnitSelector *selector);
 
 void sp_unit_selector_set_bases (SPUnitSelector *selector, guint bases);
+void sp_unit_selector_add_unit (SPUnitSelector *selector, const SPUnit *unit, int position);
+
 void sp_unit_selector_set_unit (SPUnitSelector *selector, const SPUnit *unit);
 void sp_unit_selector_add_adjustment (SPUnitSelector *selector, GtkAdjustment *adjustment);
 void sp_unit_selector_remove_adjustment (SPUnitSelector *selector, GtkAdjustment *adjustment);
 
 gboolean sp_unit_selector_update_test (SPUnitSelector *selector);
 
-END_GNOME_DECLS
+float sp_unit_selector_get_value_in_points (SPUnitSelector *selector, GtkAdjustment *adj);
+void sp_unit_selector_set_value_in_points (SPUnitSelector *selector, GtkAdjustment *adj, float value);
+
+G_END_DECLS
 
 #endif
-
-
-
-
-
-
