@@ -18,13 +18,17 @@
 
 void sp_create_window (SPViewWidget *vw, gboolean editable);
 
-void sp_ui_new_view (GtkWidget * widget);
-void sp_ui_new_view_preview (GtkWidget * widget);
-void sp_ui_close_view (GtkWidget * widget);
+void sp_ui_new_view (GtkWidget *widget);
+void sp_ui_new_view_preview (GtkWidget *widget);
+void sp_ui_close_view (GtkWidget *widget);
+
+unsigned int sp_ui_close_all (void);
 
 /* I am not sure, what is the right place for that (Lauris) */
 
 GtkWidget *sp_ui_main_menu (void);
 GtkWidget *sp_ui_generic_menu (SPView *v, SPItem *item);
 
+void sp_menu_append_recent_documents (GtkWidget *menu);
+
 #endif
