@@ -6,18 +6,18 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2002 Lauris Kaplinski
- *
- * Released under GNU GPL, read the file 'COPYING' for more information
+ * This code is in public domain
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <string.h>
 #include "nr-pixops.h"
 #include "nr-compose.h"
 
-#ifdef USE_MMX
+#ifdef WITH_MMX
 /* fixme: */
 int nr_have_mmx (void);
 void nr_mmx_R8G8B8A8_P_EMPTY_A8_RGBAP (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned char *c);
@@ -97,7 +97,7 @@ nr_R8G8B8A8_N_EMPTY_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, const u
 void
 nr_R8G8B8A8_P_EMPTY_R8G8B8A8_N (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned int alpha)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s;
@@ -121,7 +121,7 @@ nr_R8G8B8A8_P_EMPTY_R8G8B8A8_N (unsigned char *px, int w, int h, int rs, const u
 void
 nr_R8G8B8A8_P_EMPTY_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned int alpha)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s;
@@ -150,7 +150,7 @@ nr_R8G8B8A8_P_EMPTY_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, const u
 void
 nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_N (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned int alpha)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s;
@@ -187,7 +187,7 @@ nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_N (unsigned char *px, int w, int h, int rs, co
 void
 nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned int alpha)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s;
@@ -237,7 +237,7 @@ nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, co
 void
 nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_N (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned int alpha)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s;
@@ -271,7 +271,7 @@ nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_N (unsigned char *px, int w, int h, int rs, co
 void
 nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned int alpha)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s;
@@ -319,7 +319,7 @@ nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, co
 void
 nr_R8G8B8A8_N_EMPTY_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int x, y;
+	int x, y;
 
 	for (y = 0; y < h; y++) {
 		unsigned char *d, *s, *m;
@@ -344,7 +344,7 @@ nr_R8G8B8A8_N_EMPTY_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs, cons
 void
 nr_R8G8B8A8_N_EMPTY_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int x, y;
+	int x, y;
 
 	for (y = 0; y < h; y++) {
 		unsigned char *d, *s, *m;
@@ -375,7 +375,7 @@ nr_R8G8B8A8_N_EMPTY_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs, cons
 void
 nr_R8G8B8A8_P_EMPTY_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s, *m;
@@ -402,7 +402,7 @@ nr_R8G8B8A8_P_EMPTY_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs, cons
 void
 nr_R8G8B8A8_P_EMPTY_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s, *m;
@@ -434,7 +434,7 @@ nr_R8G8B8A8_P_EMPTY_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs, cons
 void
 nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s, *m;
@@ -474,7 +474,7 @@ nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs,
 void
 nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s, *m;
@@ -527,7 +527,7 @@ nr_R8G8B8A8_N_R8G8B8A8_N_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs,
 void
 nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s, *m;
@@ -564,7 +564,7 @@ nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs,
 void
 nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
 {
-	unsigned int r, c;
+	int r, c;
 
 	for (r = 0; r < h; r++) {
 		unsigned char *d, *s, *m;
@@ -612,11 +612,12 @@ nr_R8G8B8A8_P_R8G8B8A8_P_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs,
 
 /* FINAL DST MASK COLOR */
 
+#if 0
 void
 nr_R8G8B8_EMPTY_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const unsigned char *mpx, int mrs, unsigned long rgba)
 {
 	unsigned int r, g, b, a;
-	unsigned int x, y;
+	int x, y;
 
 	r = NR_RGBA32_R (rgba);
 	g = NR_RGBA32_G (rgba);
@@ -642,12 +643,13 @@ nr_R8G8B8_EMPTY_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const unsign
 		mpx += mrs;
 	}
 }
+#endif
 
 void
 nr_R8G8B8A8_N_EMPTY_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned long rgba)
 {
 	unsigned int r, g, b, a;
-	unsigned int x, y;
+	int x, y;
 
 	r = NR_RGBA32_R (rgba);
 	g = NR_RGBA32_G (rgba);
@@ -677,7 +679,7 @@ void
 nr_R8G8B8A8_P_EMPTY_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned long rgba)
 {
 	unsigned int r, g, b, a;
-	unsigned int x, y;
+	int x, y;
 
 	r = NR_RGBA32_R (rgba);
 	g = NR_RGBA32_G (rgba);
@@ -686,7 +688,7 @@ nr_R8G8B8A8_P_EMPTY_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const un
 
 	if (a == 0) return;
 
-#ifdef USE_MMX
+#ifdef WITH_MMX
 	if (NR_PIXOPS_MMX) {
 		unsigned char c[4];
 		c[0] = NR_PREMUL (r, a);
@@ -722,7 +724,7 @@ void
 nr_R8G8B8_R8G8B8_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const unsigned char *mpx, int mrs, unsigned long rgba)
 {
 	unsigned int r, g, b, a;
-	unsigned int x, y;
+	int x, y;
 
 	r = NR_RGBA32_R (rgba);
 	g = NR_RGBA32_G (rgba);
@@ -753,7 +755,7 @@ void
 nr_R8G8B8A8_N_R8G8B8A8_N_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned long rgba)
 {
 	unsigned int r, g, b, a;
-	unsigned int x, y;
+	int x, y;
 
 	r = NR_RGBA32_R (rgba);
 	g = NR_RGBA32_G (rgba);
@@ -798,7 +800,7 @@ void
 nr_R8G8B8A8_P_R8G8B8A8_P_A8_RGBA32 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, unsigned long rgba)
 {
 	unsigned int r, g, b, a;
-	unsigned int x, y;
+	int x, y;
 
 	if (!rgba & 0xff) return;
 
@@ -807,7 +809,7 @@ nr_R8G8B8A8_P_R8G8B8A8_P_A8_RGBA32 (unsigned char *px, int w, int h, int rs, con
 	b = NR_RGBA32_B (rgba);
 	a = NR_RGBA32_A (rgba);
 
-#ifdef USE_MMX
+#ifdef WITH_MMX
 	if (NR_PIXOPS_MMX) {
 		unsigned char c[4];
 		c[0] = NR_PREMUL (r, a);
@@ -859,7 +861,7 @@ nr_R8G8B8_R8G8B8_R8G8B8A8_P (unsigned char *px, int w, int h, int rs, const unsi
 
 	if (alpha == 0) return;
 
-#ifdef USE_MMX
+#ifdef WITH_MMX
 	if (NR_PIXOPS_MMX) {
 		/* WARNING: MMX composer REQUIRES w > 0 and h > 0 */
 		nr_mmx_R8G8B8_R8G8B8_R8G8B8A8_P (px, w, h, rs, spx, srs, alpha);
@@ -962,4 +964,64 @@ nr_R8G8B8_R8G8B8_R8G8B8A8_N (unsigned char *px, int w, int h, int rs, const unsi
 	}
 }
 
+void
+nr_R8G8B8_R8G8B8_R8G8B8A8_P_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
+{
+	int x, y;
+
+	for (y = 0; y < h; y++) {
+		unsigned char *d, *s, *m;
+		d = (unsigned char *) px;
+		s = (unsigned char *) spx;
+		m = (unsigned char *) mpx;
+		for (x = 0; x < w; x++) {
+			unsigned int a;
+			a = NR_PREMUL (s[3], m[0]);
+			if (a != 0) {
+				unsigned int r, g, b;
+				r = NR_PREMUL (s[0], m[0]);
+				d[0] = NR_COMPOSEP11 (r, a, d[0]);
+				g = NR_PREMUL (s[1], m[0]);
+				d[1] = NR_COMPOSEP11 (g, a, d[1]);
+				b = NR_PREMUL (s[2], m[0]);
+				d[2] = NR_COMPOSEP11 (b, a, d[2]);
+			}
+			d += 3;
+			s += 4;
+			m += 1;
+		}
+		px += rs;
+		spx += srs;
+		mpx += mrs;
+	}
+}
+
+void
+nr_R8G8B8_R8G8B8_R8G8B8A8_N_A8 (unsigned char *px, int w, int h, int rs, const unsigned char *spx, int srs, const unsigned char *mpx, int mrs)
+{
+	int x, y;
+
+	for (y = 0; y < h; y++) {
+		unsigned char *d, *s, *m;
+		d = (unsigned char *) px;
+		s = (unsigned char *) spx;
+		m = (unsigned char *) mpx;
+		for (x = 0; x < w; x++) {
+			unsigned int a;
+			a = NR_PREMUL (s[3], m[0]);
+			if (a != 0) {
+				d[0] = NR_COMPOSEP11 (s[0], a, d[0]);
+				d[1] = NR_COMPOSEP11 (s[1], a, d[1]);
+				d[2] = NR_COMPOSEP11 (s[2], a, d[2]);
+			}
+			d += 3;
+			s += 4;
+			m += 1;
+		}
+		px += rs;
+		spx += srs;
+		mpx += mrs;
+	}
+}
+
 
