@@ -3,19 +3,20 @@
 /*
  * Node editing context
  *
- * Author:
+ * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2002 Lauris Kaplinski
- *
- * Released under GNU GPL, read the file 'COPYING' for more information
+ * This code is in public domain, except stamping code,
+ * which is Copyright (C) Masatake Yamato 2002
  */
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
+#include "macros.h"
 #include "xml/repr.h"
 #include "svg/svg.h"
 #include "helper/sp-canvas-util.h"
+#include "object-edit.h"
 #include "sp-path.h"
 #include "rubberband.h"
 #include "desktop.h"
@@ -32,7 +33,7 @@
 
 static void sp_node_context_class_init (SPNodeContextClass * klass);
 static void sp_node_context_init (SPNodeContext * node_context);
-static void sp_node_context_destroy (GtkObject * object);
+static void sp_node_context_dispose (GObject *object);
 
 static void sp_node_context_setup (SPEventContext *ec);
 static gint sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event);
@@ -44,42 +45,37 @@ static gboolean sp_node_context_stamp (SPNodeContext * node_context);
 static SPEventContextClass * parent_class;
 GdkCursor * CursorNodeMouseover = NULL, * CursorNodeDragging = NULL;
 
-GtkType
+GType
 sp_node_context_get_type (void)
 {
-	static GtkType node_context_type = 0;
-
-	if (!node_context_type) {
-
-		static const GtkTypeInfo node_context_info = {
-			"SPNodeContext",
-			sizeof (SPNodeContext),
+	static GType type = 0;
+	if (!type) {
+		GTypeInfo info = {
 			sizeof (SPNodeContextClass),
-			(GtkClassInitFunc) sp_node_context_class_init,
-			(GtkObjectInitFunc) sp_node_context_init,
-			NULL,
-			NULL,
-			(GtkClassInitFunc) NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_node_context_class_init,
+			NULL, NULL,
+			sizeof (SPNodeContext),
+			4,
+			(GInstanceInitFunc) sp_node_context_init,
 		};
-
-		node_context_type = gtk_type_unique (sp_event_context_get_type (), &node_context_info);
+		type = g_type_register_static (SP_TYPE_EVENT_CONTEXT, "SPNodeContext", &info, 0);
 	}
-
-	return node_context_type;
+	return type;
 }
 
 static void
 sp_node_context_class_init (SPNodeContextClass * klass)
 {
-	GtkObjectClass * object_class;
+	GObjectClass *object_class;
 	SPEventContextClass * event_context_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 	event_context_class = (SPEventContextClass *) klass;
 
-	parent_class = gtk_type_class (sp_event_context_get_type ());
+	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->destroy = sp_node_context_destroy;
+	object_class->dispose = sp_node_context_dispose;
 
 	event_context_class->setup = sp_node_context_setup;
 	event_context_class->root_handler = sp_node_context_root_handler;
@@ -103,17 +99,27 @@ sp_node_context_init (SPNodeContext * node_context)
 }
 
 static void
-sp_node_context_destroy (GtkObject * object)
+sp_node_context_dispose (GObject *object)
 {
 	SPNodeContext * nc;
 
 	nc = SP_NODE_CONTEXT (object);
 
-	if (nc->nodepath) sp_nodepath_destroy (nc->nodepath);
-	if (nc->knot_holder) sp_knot_holder_destroy (nc->knot_holder);
+	if (nc->nodepath) {
+		sp_nodepath_destroy (nc->nodepath);
+		nc->nodepath = NULL;
+	}
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (nc->knot_holder) {
+		sp_knot_holder_destroy (nc->knot_holder);
+		nc->knot_holder = NULL;
+	}
+
+	if (SP_EVENT_CONTEXT_DESKTOP (nc)) {
+		sp_signal_disconnect_by_data (SP_DT_SELECTION (SP_EVENT_CONTEXT_DESKTOP (nc)), nc);
+	}
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -124,12 +130,10 @@ sp_node_context_setup (SPEventContext *ec)
 
 	nc = SP_NODE_CONTEXT (ec);
 
-	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (ec);
+	if (((SPEventContextClass *) parent_class)->setup)
+		((SPEventContextClass *) parent_class)->setup (ec);
 
-	gtk_signal_connect_while_alive (GTK_OBJECT (SP_DT_SELECTION (ec->desktop)), "changed",
-					GTK_SIGNAL_FUNC (sp_node_context_selection_changed), nc,
-					GTK_OBJECT (nc));
+	g_signal_connect (G_OBJECT (SP_DT_SELECTION (ec->desktop)), "changed", G_CALLBACK (sp_node_context_selection_changed), nc);
 
 	item = sp_selection_item (SP_DT_SELECTION (ec->desktop));
 
@@ -171,8 +175,8 @@ sp_node_context_item_handler (SPEventContext * event_context, SPItem * item, Gdk
 	}
 
 	if (!ret) {
-		if (SP_EVENT_CONTEXT_CLASS (parent_class)->item_handler)
-			ret = SP_EVENT_CONTEXT_CLASS (parent_class)->item_handler (event_context, item, event);
+		if (((SPEventContextClass *) parent_class)->item_handler)
+			ret = ((SPEventContextClass *) parent_class)->item_handler (event_context, item, event);
 	}
 
 	return ret;
@@ -184,7 +188,7 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 	SPDesktop * desktop;
 	SPNodeContext * nc;
 	NRPointF p;
-	ArtDRect b;
+	NRRectD b;
 	gint ret;
 
 	ret = FALSE;
@@ -269,8 +273,7 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 			default:
 				break;
 			}
-		}
-		if (!(event->key.state & GDK_SHIFT_MASK) && !(event->key.state & GDK_CONTROL_MASK)) {
+		} else if (!(event->key.state & GDK_SHIFT_MASK) && !(event->key.state & GDK_CONTROL_MASK)) {
 			switch (event->key.keyval) {
 			case GDK_space:
 				if ((event->key.state & GDK_BUTTON1_MASK)
@@ -278,21 +281,27 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 					ret = sp_node_context_stamp(nc);
 				}
 				break;
+			case GDK_Delete:
+			case GDK_KP_Delete:
+				/* Catch delete so action will not be called */
+				/* Otherwise the whole object gets deleted */
+				node_key (event);
+				ret = TRUE;
+				break;
 			default:
 				ret = node_key (event);
 				break;
 			}
 		}
 
-
 		break;
 	default:
 		break;
 	}
 
 	if (!ret) {
-		if (SP_EVENT_CONTEXT_CLASS (parent_class)->root_handler)
-			ret = SP_EVENT_CONTEXT_CLASS (parent_class)->root_handler (event_context, event);
+		if (((SPEventContextClass *) parent_class)->root_handler)
+			ret = ((SPEventContextClass *) parent_class)->root_handler (event_context, event);
 	}
 
 	return ret;
@@ -327,7 +336,6 @@ sp_node_context_stamp (SPNodeContext * nc)
 {
 	SPItem * original_item, * copy_item;
 	SPRepr * original_repr, * copy_repr;
-	gdouble * new_affine;
 	gchar tstr[80];
 	SPEventContext * ec;
 	
@@ -339,11 +347,11 @@ sp_node_context_stamp (SPNodeContext * nc)
 		copy_item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (ec->desktop), 
 							     copy_repr);
 		
-		new_affine = original_item->affine;
-		if (sp_svg_write_affine (tstr, 79, new_affine)) 
+		if (sp_svg_transform_write (tstr, 80, &original_item->transform)) {
 			sp_repr_set_attr (copy_repr, "transform", tstr);
-		else
+		} else {
 			sp_repr_set_attr (copy_repr, "transform", NULL);
+		}
 		
 		sp_repr_unref (copy_repr);
 		sp_document_done (SP_DT_DOCUMENT (ec->desktop));
