@@ -1,4 +1,4 @@
-#define __NODEPATH_C__
+#define __SP_NODEPATH_C__
 
 /*
  * Path handler in node edit mode
@@ -6,15 +6,11 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2002 Authors
- *
- * Released under GNU GPL, read the file 'COPYING' for more information
+ * This code is in public domain
  */
 
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
-#include <libart_lgpl/art_affine.h>
-#include <gtk/gtksignal.h>
 #include "svg/svg.h"
 #include "helper/sp-canvas-util.h"
 #include "helper/sp-ctrlline.h"
@@ -63,7 +59,7 @@ static gchar * create_typestr (SPNodePath * np);
 static void sp_node_ensure_ctrls (SPPathNode * node);
 
 void sp_nodepath_node_select (SPPathNode * node, gboolean incremental);
-void sp_nodepath_select_rect (SPNodePath * nodepath, ArtDRect * b, gboolean incremental);
+void sp_nodepath_select_rect (SPNodePath * nodepath, NRRectD *b, gboolean incremental);
 
 static void sp_node_set_selected (SPPathNode * node, gboolean selected);
 
@@ -77,12 +73,12 @@ static void sp_node_adjust_knots (SPPathNode * node);
 static void node_clicked (SPKnot * knot, guint state, gpointer data);
 static void node_grabbed (SPKnot * knot, guint state, gpointer data);
 static void node_ungrabbed (SPKnot * knot, guint state, gpointer data);
-static gboolean node_request (SPKnot * knot, ArtPoint * p, guint state, gpointer data);
+static gboolean node_request (SPKnot * knot, NRPointF *p, guint state, gpointer data);
 static void node_ctrl_clicked (SPKnot * knot, guint state, gpointer data);
 static void node_ctrl_grabbed (SPKnot * knot, guint state, gpointer data);
 static void node_ctrl_ungrabbed (SPKnot * knot, guint state, gpointer data);
-static gboolean node_ctrl_request (SPKnot * knot, ArtPoint * p, guint state, gpointer data);
-static void node_ctrl_moved (SPKnot * knot, ArtPoint * p, guint state, gpointer data);
+static gboolean node_ctrl_request (SPKnot * knot, NRPointF *p, guint state, gpointer data);
+static void node_ctrl_moved (SPKnot * knot, NRPointF *p, guint state, gpointer data);
 
 /* Constructors and destrouctos */
 
@@ -91,7 +87,7 @@ static void sp_nodepath_subpath_destroy (SPNodeSubPath * subpath);
 static void sp_nodepath_subpath_close (SPNodeSubPath * sp);
 static void sp_nodepath_subpath_open (SPNodeSubPath * sp, SPPathNode * n);
 static SPPathNode * sp_nodepath_node_new (SPNodeSubPath * sp, SPPathNode * next, SPPathNodeType type, ArtPathcode code,
-					  ArtPoint * ppos, ArtPoint * pos, ArtPoint * npos);
+					  NRPointF *ppos, NRPointF *pos, NRPointF *npos);
 static void sp_nodepath_node_destroy (SPPathNode * node);
 
 /* Helpers */
@@ -102,7 +98,6 @@ static ArtPathcode sp_node_path_code_from_side (SPPathNode * node, SPPathNodeSid
 
 // active_node indicates mouseover node
 static SPPathNode * active_node = NULL;
-extern GdkCursor * CursorNodeMouseover, * CursorNodeDragging;
 
 /* Creation from object */
 
@@ -116,11 +111,11 @@ sp_nodepath_new (SPDesktop * desktop, SPItem * item)
 	const gchar * nodetypes;
 	gchar * typestr;
 	gint length;
+	NRMatrixF i2d;
 
 	if (!SP_IS_PATH (item)) return NULL;
 	path = SP_PATH (item);
-	if (!path->independent) return NULL;
-	curve = sp_path_normalized_bpath (path);
+	curve = sp_shape_get_curve (SP_SHAPE (path));
 	g_return_val_if_fail (curve != NULL, NULL);
 
 	bpath = sp_curve_first_bpath (curve);
@@ -134,8 +129,9 @@ sp_nodepath_new (SPDesktop * desktop, SPItem * item)
 	np->path = path;
 	np->subpaths = NULL;
 	np->selected = NULL;
-	sp_item_i2d_affine (SP_ITEM (path), np->i2d);
-	art_affine_invert (np->d2i, np->i2d);
+	sp_item_i2d_affine (SP_ITEM (path), &i2d);
+	nr_matrix_d_from_f (&np->i2d, &i2d);
+	nr_matrix_d_invert (&np->d2i, &np->i2d);
 
 	/* Now the bitchy part */
 
@@ -156,7 +152,7 @@ subpath_from_bpath (SPNodePath * np, ArtBpath * b, const gchar * t)
 {
 	SPNodeSubPath * sp;
 	SPPathNode * n;
-	ArtPoint ppos, pos, npos;
+	NRPointF ppos, pos, npos;
 	gboolean closed;
 
 	g_assert ((b->code == ART_MOVETO) || (b->code == ART_MOVETO_OPEN));
@@ -164,13 +160,11 @@ subpath_from_bpath (SPNodePath * np, ArtBpath * b, const gchar * t)
 	sp = sp_nodepath_subpath_new (np);
 	closed = (b->code == ART_MOVETO);
 
-	pos.x = b->x3;
-	pos.y = b->y3;
-	art_affine_point (&pos, &pos, np->i2d);
-	if ((b + 1)->code == ART_CURVETO) {
-		npos.x = (b + 1)->x1;
-		npos.y = (b + 1)->y1;
-		art_affine_point (&npos, &npos, np->i2d);
+	pos.x = NR_MATRIX_DF_TRANSFORM_X (&np->i2d, b->x3, b->y3);
+	pos.y = NR_MATRIX_DF_TRANSFORM_Y (&np->i2d, b->x3, b->y3);
+	if (b[1].code == ART_CURVETO) {
+		npos.x = NR_MATRIX_DF_TRANSFORM_X (&np->i2d, b[1].x1, b[1].y1);
+		npos.y = NR_MATRIX_DF_TRANSFORM_Y (&np->i2d, b[1].x1, b[1].y1);
 	} else {
 		npos = pos;
 	}
@@ -182,20 +176,17 @@ subpath_from_bpath (SPNodePath * np, ArtBpath * b, const gchar * t)
 	t++;
 
 	while ((b->code == ART_CURVETO) || (b->code == ART_LINETO)) {
-		pos.x = b->x3;
-		pos.y = b->y3;
-		art_affine_point (&pos, &pos, np->i2d);
+		pos.x = NR_MATRIX_DF_TRANSFORM_X (&np->i2d, b->x3, b->y3);
+		pos.y = NR_MATRIX_DF_TRANSFORM_Y (&np->i2d, b->x3, b->y3);
 		if (b->code == ART_CURVETO) {
-			ppos.x = b->x2;
-			ppos.y = b->y2;
-			art_affine_point (&ppos, &ppos, np->i2d);
+			ppos.x = NR_MATRIX_DF_TRANSFORM_X (&np->i2d, b->x2, b->y2);
+			ppos.y = NR_MATRIX_DF_TRANSFORM_Y (&np->i2d, b->x2, b->y2);
 		} else {
 			ppos = pos;
 		}
-		if ((b + 1)->code == ART_CURVETO) {
-			npos.x = (b + 1)->x1;
-			npos.y = (b + 1)->y1;
-			art_affine_point (&npos, &npos, np->i2d);
+		if (b[1].code == ART_CURVETO) {
+			npos.x = NR_MATRIX_DF_TRANSFORM_X (&np->i2d, b[1].x1, b[1].y1);
+			npos.y = NR_MATRIX_DF_TRANSFORM_Y (&np->i2d, b[1].x1, b[1].y1);
 		} else {
 			npos = pos;
 		}
@@ -259,8 +250,7 @@ update_object (SPNodePath * np)
 
 	curve = create_curve (np);
 
-	sp_path_clear (np->path);
-	sp_path_add_bpath (np->path, curve, TRUE, NULL);
+	sp_shape_set_curve (SP_SHAPE (np->path), curve, TRUE);
 
 	sp_curve_unref (curve);
 }
@@ -326,7 +316,7 @@ create_curve (SPNodePath * np)
 {
 	SPCurve * curve;
 	GSList * spl;
-	ArtPoint p1, p2, p3;
+	NRPointF p1, p2, p3;
 
 	curve = sp_curve_new ();
 
@@ -334,26 +324,22 @@ create_curve (SPNodePath * np)
 		SPNodeSubPath * sp;
 		SPPathNode * n;
 		sp = (SPNodeSubPath *) spl->data;
-		p3.x = sp->first->pos.x;
-		p3.y = sp->first->pos.y;
-		art_affine_point (&p3, &p3, np->d2i);
+		p3.x = NR_MATRIX_DF_TRANSFORM_X (&np->d2i, sp->first->pos.x, sp->first->pos.y);
+		p3.y = NR_MATRIX_DF_TRANSFORM_Y (&np->d2i, sp->first->pos.x, sp->first->pos.y);
 		sp_curve_moveto (curve, p3.x, p3.y);
 		n = sp->first->n.other;
 		while (n) {
-			p3.x = n->pos.x;
-			p3.y = n->pos.y;
-			art_affine_point (&p3, &p3, np->d2i);
+			p3.x = NR_MATRIX_DF_TRANSFORM_X (&np->d2i, n->pos.x, n->pos.y);
+			p3.y = NR_MATRIX_DF_TRANSFORM_Y (&np->d2i, n->pos.x, n->pos.y);
 			switch (n->code) {
 			case ART_LINETO:
-				sp_curve_lineto (curve, p3.x ,p3.y);
+				sp_curve_lineto (curve, p3.x, p3.y);
 				break;
 			case ART_CURVETO:
-				p1.x = n->p.other->n.pos.x;
-				p1.y = n->p.other->n.pos.y;
-				art_affine_point (&p1, &p1, np->d2i);
-				p2.x = n->p.pos.x;
-				p2.y = n->p.pos.y;
-				art_affine_point (&p2, &p2, np->d2i);
+				p1.x = NR_MATRIX_DF_TRANSFORM_X (&np->d2i, n->p.other->n.pos.x, n->p.other->n.pos.y);
+				p1.y = NR_MATRIX_DF_TRANSFORM_Y (&np->d2i, n->p.other->n.pos.x, n->p.other->n.pos.y);
+				p2.x = NR_MATRIX_DF_TRANSFORM_X (&np->d2i, n->p.pos.x, n->p.pos.y);
+				p2.y = NR_MATRIX_DF_TRANSFORM_Y (&np->d2i, n->p.pos.x, n->p.pos.y);
 				sp_curve_curveto (curve, p1.x, p1.y, p2.x, p2.y, p3.x, p3.y);
 				break;
 			default:
@@ -652,9 +638,9 @@ sp_nodepath_set_node_type (SPPathNode * node, SPPathNodeType type)
 	node->type = type;
 
 	if (node->type == SP_PATHNODE_CUSP) {
-		gtk_object_set (GTK_OBJECT (node->knot), "shape", SP_KNOT_SHAPE_DIAMOND, "size", 9, NULL);
+		g_object_set (G_OBJECT (node->knot), "shape", SP_KNOT_SHAPE_DIAMOND, "size", 9, NULL);
 	} else {
-		gtk_object_set (GTK_OBJECT (node->knot), "shape", SP_KNOT_SHAPE_SQUARE, "size", 7, NULL);
+		g_object_set (G_OBJECT (node->knot), "shape", SP_KNOT_SHAPE_SQUARE, "size", 7, NULL);
 	}
 
 	sp_node_adjust_knots (node);
@@ -712,7 +698,7 @@ sp_nodepath_selected_nodes_move (SPNodePath * nodepath, gdouble dx, gdouble dy)
 
 	for (l = nodepath->selected; l != NULL; l = l->next) {
 		SPPathNode * n;
-		ArtPoint p;
+		NRPointF p;
 		n = (SPPathNode *) l->data;
 		p.x = n->pos.x + dx;
 		p.y = n->pos.y + dy;
@@ -743,7 +729,7 @@ sp_node_ensure_knot (SPPathNode * node, gint which, gboolean show_knot)
 	SPNodePath * nodepath;
 	SPPathNodeSide * side;
 	ArtPathcode code;
-	ArtPoint p;
+	NRPointF p;
 
 	g_assert (node != NULL);
 
@@ -777,7 +763,7 @@ void
 sp_node_ensure_ctrls (SPPathNode * node)
 {
 	SPNodePath * nodepath;
-	ArtPoint p;
+	NRPointF p;
 	gboolean show_knots;
 
 	g_assert (node != NULL);
@@ -890,7 +876,7 @@ sp_node_selected_join (void)
 	SPNodePath * nodepath;
 	SPNodeSubPath * sa, * sb;
 	SPPathNode * a, * b, * n;
-	ArtPoint p, c;
+	NRPointF p, c;
 	ArtPathcode code;
 
 	nodepath = sp_nodepath_current ();
@@ -930,7 +916,7 @@ sp_node_selected_join (void)
 	sb = b->subpath;
 
 	if (a == sa->first) {
-		SPNodeSubPath * t;
+		SPNodeSubPath *t;
 		p = sa->first->n.pos;
 		code = sa->first->n.other->code;
 		t = sp_nodepath_subpath_new (sa->nodepath);
@@ -975,6 +961,99 @@ sp_node_selected_join (void)
 	update_repr (nodepath);
 }
 
+void
+sp_node_selected_join_segment (void)
+{
+	SPNodePath * nodepath;
+	SPNodeSubPath * sa, * sb;
+	SPPathNode * a, * b, * n;
+	NRPointF p;
+	ArtPathcode code;
+
+	nodepath = sp_nodepath_current ();
+	if (!nodepath) return;
+	if (g_slist_length (nodepath->selected) != 2) return;
+
+	a = (SPPathNode *) nodepath->selected->data;
+	b = (SPPathNode *) nodepath->selected->next->data;
+
+	g_assert (a != b);
+
+	if ((a->subpath->closed) || (b->subpath->closed)) return;
+	if (a->p.other && a->n.other) return;
+	g_assert (a->p.other || a->n.other);
+	if (b->p.other && b->n.other) return;
+	g_assert (b->p.other || b->n.other);
+	/* a and b are endpoints */
+
+	if (a->subpath == b->subpath) {
+		SPNodeSubPath * sp;
+
+		sp = a->subpath;
+
+                /*similar to sp_nodepath_subpath_close (sp), without the node destruction*/
+	        sp->closed = TRUE;
+
+	        sp->first->p.other = sp->last;
+	        sp->last->n.other = sp->first;
+	        sp->last->n.pos = sp->first->n.pos;
+
+		sp_nodepath_ensure_ctrls (sp->nodepath);
+
+		update_repr (nodepath);
+
+		return;
+	}
+
+	/* a and b are separate subpaths */
+	sa = a->subpath;
+	sb = b->subpath;
+
+
+	if (a == sa->first) {
+		SPNodeSubPath *t;
+		p = sa->first->pos;
+		code = sa->first->n.other->code;
+		t = sp_nodepath_subpath_new (sa->nodepath);
+		n = sa->last;
+		sp_nodepath_node_new (t, NULL, SP_PATHNODE_CUSP, ART_MOVETO, &n->n.pos, &n->pos, &n->p.pos);
+		for (n = n->p.other; n != NULL; n = n->p.other) {
+			sp_nodepath_node_new (t, NULL, n->type, n->n.other->code, &n->n.pos, &n->pos, &n->p.pos);
+		}
+		sp_nodepath_subpath_destroy (sa);
+		sa = t;
+	} else if (a == sa->last) {
+		p = sa->last->pos;
+		code = sa->last->code;
+	} else {
+		code = ART_END;
+		g_assert_not_reached ();
+	}
+
+	if (b == sb->first) {
+		n = sb->first;
+		sp_nodepath_node_new (sa, NULL, SP_PATHNODE_CUSP, code, &p, &n->pos, &n->n.pos);
+		for (n = n->n.other; n != NULL; n = n->n.other) {
+			sp_nodepath_node_new (sa, NULL, n->type, n->code, &n->p.pos, &n->pos, &n->n.pos);
+		}
+	} else if (b == sb->last) {
+		n = sb->last;
+		sp_nodepath_node_new (sa, NULL, SP_PATHNODE_CUSP, code, &p, &n->pos, &n->p.pos);
+		for (n = n->p.other; n != NULL; n = n->p.other) {
+ 			sp_nodepath_node_new (sa, NULL, n->type, n->n.other->code, &n->n.pos, &n->pos, &n->p.pos);
+		}
+	} else {
+		g_assert_not_reached ();
+	}
+	/* and now destroy sb */
+
+	sp_nodepath_subpath_destroy (sb);
+
+	sp_nodepath_ensure_ctrls (sa->nodepath);
+
+	update_repr (nodepath);
+}
+
 void
 sp_node_selected_delete (void)
 {
@@ -1040,19 +1119,19 @@ sp_node_set_selected (SPPathNode * node, gboolean selected)
 	node->selected = selected;
 
 	if (selected) {
-		gtk_object_set (GTK_OBJECT (node->knot),
-			"fill", NODE_FILL_SEL,
-			"fill_mouseover", NODE_FILL_SEL_HI,
-			"stroke", NODE_STROKE_SEL,
-			"stroke_mouseover", NODE_STROKE_SEL_HI,
-			NULL);
+		g_object_set (G_OBJECT (node->knot),
+			      "fill", NODE_FILL_SEL,
+			      "fill_mouseover", NODE_FILL_SEL_HI,
+			      "stroke", NODE_STROKE_SEL,
+			      "stroke_mouseover", NODE_STROKE_SEL_HI,
+			      NULL);
 	} else {
-		gtk_object_set (GTK_OBJECT (node->knot),
-			"fill", NODE_FILL,
-			"fill_mouseover", NODE_FILL_HI,
-			"stroke", NODE_STROKE,
-			"stroke_mouseover", NODE_STROKE_HI,
-			NULL);
+		g_object_set (G_OBJECT (node->knot),
+			      "fill", NODE_FILL,
+			      "fill_mouseover", NODE_FILL_HI,
+			      "stroke", NODE_STROKE,
+			      "stroke_mouseover", NODE_STROKE_HI,
+			      NULL);
 	}
 
 	sp_node_ensure_ctrls (node);
@@ -1087,7 +1166,7 @@ sp_nodepath_node_select (SPPathNode * node, gboolean incremental)
 }
 
 void
-sp_nodepath_select_rect (SPNodePath * nodepath, ArtDRect * b, gboolean incremental)
+sp_nodepath_select_rect (SPNodePath *nodepath, NRRectD *b, gboolean incremental)
 {
 	SPNodeSubPath * subpath;
 	SPPathNode * node;
@@ -1392,7 +1471,7 @@ node_ungrabbed (SPKnot * knot, guint state, gpointer data)
 }
 
 static gboolean
-node_request (SPKnot * knot, ArtPoint * p, guint state, gpointer data)
+node_request (SPKnot *knot, NRPointF *p, guint state, gpointer data)
 {
 	SPPathNode * n;
 
@@ -1438,7 +1517,7 @@ node_ctrl_ungrabbed (SPKnot * knot, guint state, gpointer data)
 }
 
 static gboolean
-node_ctrl_request (SPKnot * knot, ArtPoint * p, guint state, gpointer data)
+node_ctrl_request (SPKnot * knot, NRPointF *p, guint state, gpointer data)
 {
 	SPPathNode * n;
 	SPPathNodeSide * me, * opposite;
@@ -1490,7 +1569,7 @@ node_ctrl_request (SPKnot * knot, ArtPoint * p, guint state, gpointer data)
 }
 
 static void
-node_ctrl_moved (SPKnot * knot, ArtPoint * p, guint state, gpointer data)
+node_ctrl_moved (SPKnot *knot, NRPointF *p, guint state, gpointer data)
 {
 	SPPathNode * n;
 	SPPathNodeSide * me;
@@ -1623,8 +1702,8 @@ sp_nodepath_subpath_open (SPNodeSubPath * sp, SPPathNode * n)
 }
 
 SPPathNode *
-sp_nodepath_node_new (SPNodeSubPath * sp, SPPathNode * next, SPPathNodeType type, ArtPathcode code,
-		      ArtPoint * ppos, ArtPoint * pos, ArtPoint * npos)
+sp_nodepath_node_new (SPNodeSubPath *sp, SPPathNode *next, SPPathNodeType type, ArtPathcode code,
+		      NRPointF *ppos, NRPointF *pos, NRPointF *npos)
 {
 	SPPathNode * n, * prev;
 
@@ -1670,57 +1749,50 @@ sp_nodepath_node_new (SPNodeSubPath * sp, SPPathNode * next, SPPathNodeType type
 
 	n->knot = sp_knot_new (sp->nodepath->desktop);
 	sp_knot_set_position (n->knot, pos, 0);
-	gtk_object_set (GTK_OBJECT (n->knot),
-			"anchor", GTK_ANCHOR_CENTER,
-			"fill", NODE_FILL,
-			"fill_mouseover", NODE_FILL_HI,
-			"stroke", NODE_STROKE,
-			"stroke_mouseover", NODE_STROKE_HI,
-			"cursor_mouseover", CursorNodeMouseover,
-			"cursor_dragging", CursorNodeDragging,
-			NULL);
+	g_object_set (G_OBJECT (n->knot),
+		      "anchor", GTK_ANCHOR_CENTER,
+		      "fill", NODE_FILL,
+		      "fill_mouseover", NODE_FILL_HI,
+		      "stroke", NODE_STROKE,
+		      "stroke_mouseover", NODE_STROKE_HI,
+#if 0
+		      "cursor_mouseover", CursorNodeMouseover,
+		      "cursor_dragging", CursorNodeDragging,
+#endif
+		      NULL);
 	if (n->type == SP_PATHNODE_CUSP) {
-		gtk_object_set (GTK_OBJECT (n->knot), "shape", SP_KNOT_SHAPE_DIAMOND, "size", 9, NULL);
+		g_object_set (G_OBJECT (n->knot), "shape", SP_KNOT_SHAPE_DIAMOND, "size", 9, NULL);
 	} else {
-		gtk_object_set (GTK_OBJECT (n->knot), "shape", SP_KNOT_SHAPE_SQUARE, "size", 7, NULL);
+		g_object_set (G_OBJECT (n->knot), "shape", SP_KNOT_SHAPE_SQUARE, "size", 7, NULL);
 	}
-	gtk_signal_connect (GTK_OBJECT (n->knot), "event",
-			    GTK_SIGNAL_FUNC (node_event), n);
-	gtk_signal_connect (GTK_OBJECT (n->knot), "clicked",
-			    GTK_SIGNAL_FUNC (node_clicked), n);
-	gtk_signal_connect (GTK_OBJECT (n->knot), "grabbed",
-			    GTK_SIGNAL_FUNC (node_grabbed), n);
-	gtk_signal_connect (GTK_OBJECT (n->knot), "ungrabbed",
-			    GTK_SIGNAL_FUNC (node_ungrabbed), n);
-	gtk_signal_connect (GTK_OBJECT (n->knot), "request",
-			    GTK_SIGNAL_FUNC (node_request), n);
+	g_signal_connect (G_OBJECT (n->knot), "event", G_CALLBACK (node_event), n);
+	g_signal_connect (G_OBJECT (n->knot), "clicked", G_CALLBACK (node_clicked), n);
+	g_signal_connect (G_OBJECT (n->knot), "grabbed", G_CALLBACK (node_grabbed), n);
+	g_signal_connect (G_OBJECT (n->knot), "ungrabbed", G_CALLBACK (node_ungrabbed), n);
+	g_signal_connect (G_OBJECT (n->knot), "request", G_CALLBACK (node_request), n);
 	sp_knot_show (n->knot);
 
 	n->p.knot = sp_knot_new (sp->nodepath->desktop);
 	sp_knot_set_position (n->p.knot, ppos, 0);
-	gtk_object_set (GTK_OBJECT (n->p.knot),
-			"shape", SP_KNOT_SHAPE_CIRCLE,
-			"size", 7,
-			"anchor", GTK_ANCHOR_CENTER,
-			"fill", KNOT_FILL,
-			"fill_mouseover", KNOT_FILL_HI,
-			"stroke", KNOT_STROKE,
-			"stroke_mouseover", KNOT_STROKE_HI,
-			"cursor_mouseover", CursorNodeMouseover,
-			"cursor_dragging", CursorNodeDragging,
-			NULL);
-	gtk_signal_connect (GTK_OBJECT (n->p.knot), "clicked",
-			    GTK_SIGNAL_FUNC (node_ctrl_clicked), n);
-	gtk_signal_connect (GTK_OBJECT (n->p.knot), "grabbed",
-			    GTK_SIGNAL_FUNC (node_ctrl_grabbed), n);
-	gtk_signal_connect (GTK_OBJECT (n->p.knot), "ungrabbed",
-			    GTK_SIGNAL_FUNC (node_ctrl_ungrabbed), n);
-	gtk_signal_connect (GTK_OBJECT (n->p.knot), "request",
-			    GTK_SIGNAL_FUNC (node_ctrl_request), n);
-	gtk_signal_connect (GTK_OBJECT (n->p.knot), "moved",
-			    GTK_SIGNAL_FUNC (node_ctrl_moved), n);
-	gtk_signal_connect (GTK_OBJECT (n->p.knot), "event",
-			    GTK_SIGNAL_FUNC (node_ctrl_event), n);
+	g_object_set (G_OBJECT (n->p.knot),
+		      "shape", SP_KNOT_SHAPE_CIRCLE,
+		      "size", 7,
+		      "anchor", GTK_ANCHOR_CENTER,
+		      "fill", KNOT_FILL,
+		      "fill_mouseover", KNOT_FILL_HI,
+		      "stroke", KNOT_STROKE,
+		      "stroke_mouseover", KNOT_STROKE_HI,
+#if 0
+		      "cursor_mouseover", CursorNodeMouseover,
+		      "cursor_dragging", CursorNodeDragging,
+#endif
+		      NULL);
+	g_signal_connect (G_OBJECT (n->p.knot), "clicked", G_CALLBACK (node_ctrl_clicked), n);
+	g_signal_connect (G_OBJECT (n->p.knot), "grabbed", G_CALLBACK (node_ctrl_grabbed), n);
+	g_signal_connect (G_OBJECT (n->p.knot), "ungrabbed", G_CALLBACK (node_ctrl_ungrabbed), n);
+	g_signal_connect (G_OBJECT (n->p.knot), "request", G_CALLBACK (node_ctrl_request), n);
+	g_signal_connect (G_OBJECT (n->p.knot), "moved", G_CALLBACK (node_ctrl_moved), n);
+	g_signal_connect (G_OBJECT (n->p.knot), "event", G_CALLBACK (node_ctrl_event), n);
 	
 	sp_knot_hide (n->p.knot);
 	n->p.line = sp_canvas_item_new (SP_DT_CONTROLS (n->subpath->nodepath->desktop),
@@ -1729,29 +1801,25 @@ sp_nodepath_node_new (SPNodeSubPath * sp, SPPathNode * next, SPPathNodeType type
 
 	n->n.knot = sp_knot_new (sp->nodepath->desktop);
 	sp_knot_set_position (n->n.knot, npos, 0);
-	gtk_object_set (GTK_OBJECT (n->n.knot),
-			"shape", SP_KNOT_SHAPE_CIRCLE,
-			"size", 7,
-			"anchor", GTK_ANCHOR_CENTER,
-			"fill", KNOT_FILL,
-			"fill_mouseover", KNOT_FILL_HI,
-			"stroke", KNOT_STROKE,
-			"stroke_mouseover", KNOT_STROKE_HI,
-			"cursor_mouseover", CursorNodeMouseover,
-			"cursor_dragging", CursorNodeDragging,
-			NULL);
-	gtk_signal_connect (GTK_OBJECT (n->n.knot), "clicked",
-			    GTK_SIGNAL_FUNC (node_ctrl_clicked), n);
-	gtk_signal_connect (GTK_OBJECT (n->n.knot), "grabbed",
-			    GTK_SIGNAL_FUNC (node_ctrl_grabbed), n);
-	gtk_signal_connect (GTK_OBJECT (n->n.knot), "ungrabbed",
-			    GTK_SIGNAL_FUNC (node_ctrl_ungrabbed), n);
-	gtk_signal_connect (GTK_OBJECT (n->n.knot), "request",
-			    GTK_SIGNAL_FUNC (node_ctrl_request), n);
-	gtk_signal_connect (GTK_OBJECT (n->n.knot), "moved",
-			    GTK_SIGNAL_FUNC (node_ctrl_moved), n);
-	gtk_signal_connect (GTK_OBJECT (n->n.knot), "event",
-			    GTK_SIGNAL_FUNC (node_ctrl_event), n);
+	g_object_set (G_OBJECT (n->n.knot),
+		      "shape", SP_KNOT_SHAPE_CIRCLE,
+		      "size", 7,
+		      "anchor", GTK_ANCHOR_CENTER,
+		      "fill", KNOT_FILL,
+		      "fill_mouseover", KNOT_FILL_HI,
+		      "stroke", KNOT_STROKE,
+		      "stroke_mouseover", KNOT_STROKE_HI,
+#if 0
+		      "cursor_mouseover", CursorNodeMouseover,
+		      "cursor_dragging", CursorNodeDragging,
+#endif
+		      NULL);
+	g_signal_connect (G_OBJECT (n->n.knot), "clicked", G_CALLBACK (node_ctrl_clicked), n);
+	g_signal_connect (G_OBJECT (n->n.knot), "grabbed", G_CALLBACK (node_ctrl_grabbed), n);
+	g_signal_connect (G_OBJECT (n->n.knot), "ungrabbed", G_CALLBACK (node_ctrl_ungrabbed), n);
+	g_signal_connect (G_OBJECT (n->n.knot), "request", G_CALLBACK (node_ctrl_request), n);
+	g_signal_connect (G_OBJECT (n->n.knot), "moved", G_CALLBACK (node_ctrl_moved), n);
+	g_signal_connect (G_OBJECT (n->n.knot), "event", G_CALLBACK (node_ctrl_event), n);
 	sp_knot_hide (n->n.knot);
 	n->n.line = sp_canvas_item_new (SP_DT_CONTROLS (n->subpath->nodepath->desktop),
 					       SP_TYPE_CTRLLINE, NULL);
@@ -1787,10 +1855,11 @@ sp_nodepath_node_destroy (SPPathNode * node)
 	sp_knot_hide (node->p.knot);
 	sp_knot_hide (node->n.knot);
 	*/
-	gtk_object_destroy (GTK_OBJECT (node->knot));
-	gtk_object_destroy (GTK_OBJECT (node->p.knot));
+	g_object_unref (G_OBJECT (node->knot));
+	g_object_unref (G_OBJECT (node->p.knot));
+	g_object_unref (G_OBJECT (node->n.knot));
+
 	gtk_object_destroy (GTK_OBJECT (node->p.line));
-	gtk_object_destroy (GTK_OBJECT (node->n.knot));
 	gtk_object_destroy (GTK_OBJECT (node->n.line));
 	
 	if (sp->nodes) {
