@@ -49,7 +49,6 @@ sp_selected_path_combine (void)
 	SPPath * path;
 	SPCurve * c;
 	ArtBpath * abp;
-	gdouble i2root[6];
 	gchar * d, * str, * style;
 
 	sp_selected_path_to_curves0 (FALSE, 0);
@@ -64,8 +63,7 @@ sp_selected_path_combine (void)
 
 	for (l = il; l != NULL; l = l->next) {
 		item = (SPItem *) l->data;
-		if (!SP_IS_PATH (item)) return;
-		if (!((SPPath *) item)->independent) return;
+		if (!SP_IS_SHAPE (item)) return;
 	}
 
 	il = g_slist_copy (il);
@@ -74,10 +72,13 @@ sp_selected_path_combine (void)
 	style = g_strdup (sp_repr_attr ((SP_OBJECT (il->data))->repr, "style"));
 
 	for (l = il; l != NULL; l = l->next) {
+		NRMatrixF i2root;
+		NRMatrixD i2rootd;
 		path = (SPPath *) l->data;
-		c = sp_path_normalized_bpath (path);
-		sp_item_i2root_affine (SP_ITEM (path), i2root);
-		abp = art_bpath_affine_transform (c->bpath, i2root);
+		c = sp_shape_get_curve (SP_SHAPE (path));
+		sp_item_i2root_affine (SP_ITEM (path), &i2root);
+		nr_matrix_d_from_f (&i2rootd, &i2root);
+		abp = art_bpath_affine_transform (c->bpath, NR_MATRIX_D_TO_DOUBLE (&i2rootd));
 		sp_curve_unref (c);
 		str = sp_svg_write_path (abp);
 		art_free (abp);
@@ -109,7 +110,8 @@ sp_selected_path_break_apart (void)
 	SPPath * path;
 	SPCurve * curve;
 	ArtBpath * abp;
-	double i2root[6];
+	NRMatrixF i2root;
+	NRMatrixD d;
 	gchar * style, * str;
 	GSList * list, * l;
 	SPDesktop * desktop;
@@ -125,15 +127,15 @@ sp_selected_path_break_apart (void)
 	if (!SP_IS_PATH (item)) return;
 
 	path = SP_PATH (item);
-	if (!sp_path_independent (path)) return;
 
-	curve = sp_path_normalized_bpath (path);
+	curve = sp_shape_get_curve (SP_SHAPE (path));
 	if (curve == NULL) return;
 
-	sp_item_i2root_affine (SP_ITEM (path), i2root);
+	sp_item_i2root_affine (SP_ITEM (path), &i2root);
+	nr_matrix_d_from_f (&d, &i2root);
 	style = g_strdup (sp_repr_attr (SP_OBJECT (item)->repr, "style"));
 
-	abp = art_bpath_affine_transform (curve->bpath, i2root);
+	abp = art_bpath_affine_transform (curve->bpath, NR_MATRIX_D_TO_DOUBLE (&d));
 
 	sp_curve_unref (curve);
 	sp_repr_unparent (SP_OBJECT_REPR (item));
@@ -214,15 +216,15 @@ sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy)
 	if (!item)
 		return NULL;
 		
-	if (SP_IS_PATH (item) && !SP_PATH (item)->independent) 
-		curve = sp_path_normalized_bpath (SP_PATH (item));
-	else if (SP_IS_TEXT (item))
+	if (SP_IS_SHAPE (item)) {
+		curve = sp_shape_get_curve (SP_SHAPE (item));
+	} else if (SP_IS_TEXT (item)) {
 		curve = sp_text_normalized_bpath (SP_TEXT (item));
-	else
+	} else {
 		curve = NULL;
+	}
 	
-	if (!curve)
-		return NULL;
+	if (!curve) return NULL;
 	
 	repr = sp_repr_new ("path");
 	/* Transformation */
@@ -254,7 +256,7 @@ sp_path_cleanup (SPPath *path)
 	style = SP_OBJECT_STYLE (path);
 	if (style->fill.type == SP_PAINT_TYPE_NONE) return;
 
-	curve = sp_path_normalized_bpath (path);
+	curve = sp_shape_get_curve (SP_SHAPE (path));
 	if (!curve) return;
 	c = sp_curve_split (curve);
 	sp_curve_unref (curve);
