@@ -7,9 +7,7 @@
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2002 Lauris Kaplinski
- *
- * Released under GNU GPL, read the file 'COPYING' for more information
+ * This code is in public domain
  */
 
 #include <libnr/nr-types.h>
@@ -18,7 +16,6 @@
 
 unsigned int sp_print_bind (SPPrintContext *ctx, const NRMatrixF *transform, float opacity);
 unsigned int sp_print_release (SPPrintContext *ctx);
-
 unsigned int sp_print_fill (SPPrintContext *ctx, const NRBPath *bpath, const NRMatrixF *ctm, const SPStyle *style,
 			    const NRRectF *pbox, const NRRectF *dbox, const NRRectF *bbox);
 unsigned int sp_print_stroke (SPPrintContext *ctx, const NRBPath *bpath, const NRMatrixF *transform, const SPStyle *style,
@@ -31,7 +28,7 @@ unsigned int sp_print_image_R8G8B8A8_N (SPPrintContext *ctx,
 
 /* UI */
 void sp_print_preview_document (SPDocument *doc);
-void sp_print_document (SPDocument *doc);
+void sp_print_document (SPDocument *doc, unsigned int direct);
 void sp_print_document_to_file (SPDocument *doc, const unsigned char *filename);
 
 #endif
