@@ -11,19 +11,15 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <math.h>
 #include <string.h>
 #include <glib.h>
-#include <libart_lgpl/art_affine.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkradiobutton.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtktogglebutton.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
+#include <gtk/gtk.h>
 #include "helper/sp-canvas-util.h"
 #include "rubberband.h"
 #include "sodipodi-private.h"
@@ -35,17 +31,21 @@
 #include "pixmaps/cursor-select-m.xpm"
 #include "pixmaps/cursor-select-d.xpm"
 #include "pixmaps/handles.xpm"
+#include "helper/sp-intl.h"
+
 #include "select-context.h"
 #include "selection-chemistry.h"
 #include "path-chemistry.h"
 #include "desktop.h"
+#include "widgets/spw-utilities.h"
 #include "dialogs/object-properties.h"
 #include "sp-metrics.h"
+#include "sp-item.h"
 #include "desktop-snap.h"
 
 static void sp_select_context_class_init (SPSelectContextClass * klass);
 static void sp_select_context_init (SPSelectContext * select_context);
-static void sp_select_context_destroy (GtkObject * object);
+static void sp_select_context_dispose (GObject *object);
 
 static void sp_select_context_setup (SPEventContext *ec);
 static void sp_select_context_set (SPEventContext *ec, const guchar *key, const guchar *val);
@@ -58,45 +58,41 @@ static void sp_selection_moveto (SPSelTrans * seltrans, double x, double y, guin
 
 static SPEventContextClass * parent_class;
 
-GdkCursor * CursorSelectMouseover = NULL, * CursorSelectDragging = NULL;
+static GdkCursor *CursorSelectMouseover = NULL;
+static GdkCursor *CursorSelectDragging = NULL;
 GdkPixbuf * handles[13];
 
 GtkType
 sp_select_context_get_type (void)
 {
-	static GtkType select_context_type = 0;
-
-	if (!select_context_type) {
-
-		static const GtkTypeInfo select_context_info = {
-			"SPSelectContext",
-			sizeof (SPSelectContext),
+	static GType type = 0;
+	if (!type) {
+		GTypeInfo info = {
 			sizeof (SPSelectContextClass),
-			(GtkClassInitFunc) sp_select_context_class_init,
-			(GtkObjectInitFunc) sp_select_context_init,
-			NULL,
-			NULL,
-			(GtkClassInitFunc) NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_select_context_class_init,
+			NULL, NULL,
+			sizeof (SPSelectContext),
+			4,
+			(GInstanceInitFunc) sp_select_context_init,
 		};
-
-		select_context_type = gtk_type_unique (sp_event_context_get_type (), &select_context_info);
+		type = g_type_register_static (SP_TYPE_EVENT_CONTEXT, "SPSelectContext", &info, 0);
 	}
-
-	return select_context_type;
+	return type;
 }
 
 static void
 sp_select_context_class_init (SPSelectContextClass * klass)
 {
-	GtkObjectClass * object_class;
+	GObjectClass *object_class;
 	SPEventContextClass * event_context_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 	event_context_class = (SPEventContextClass *) klass;
 
-	parent_class = gtk_type_class (sp_event_context_get_type ());
+	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->destroy = sp_select_context_destroy;
+	object_class->dispose = sp_select_context_dispose;
 
 	event_context_class->setup = sp_select_context_setup;
 	event_context_class->set = sp_select_context_set;
@@ -133,21 +129,20 @@ sp_select_context_init (SPSelectContext * sc)
 }
 
 static void
-sp_select_context_destroy (GtkObject * object)
+sp_select_context_dispose (GObject *object)
 {
 	SPSelectContext *sc;
 
 	sc = SP_SELECT_CONTEXT (object);
 
 	if (sc->grabbed) {
-		sp_canvas_item_ungrab (sc->grabbed, gdk_time_get ());
+		sp_canvas_item_ungrab (sc->grabbed, GDK_CURRENT_TIME);
 		sc->grabbed = NULL;
 	}
 
 	sp_sel_trans_shutdown (&sc->seltrans);
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -157,8 +152,8 @@ sp_select_context_setup (SPEventContext *ec)
 
 	select_context = SP_SELECT_CONTEXT (ec);
 
-	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (ec);
+	if (((SPEventContextClass *) parent_class)->setup)
+		((SPEventContextClass *) parent_class)->setup (ec);
 
 	sp_sel_trans_init (&select_context->seltrans, ec->desktop);
 
@@ -206,7 +201,9 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 
 	switch (event->type) {
 	case GDK_2BUTTON_PRESS:
-		ret = TRUE;
+		if (event->button.button == 1) {
+			ret = TRUE;
+		}
 		break;
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
@@ -243,6 +240,11 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 		break;
 	case GDK_BUTTON_RELEASE:
 		if (event->button.button == 1) {
+			if (sc->grabbed) {
+				/* Better release grab here to make debugging easier */
+				sp_canvas_item_ungrab (sc->grabbed, event->button.time);
+				sc->grabbed = NULL;
+			}
 			if (sc->moved) {
 				// item has been moved
 				sp_sel_trans_ungrab (seltrans);
@@ -273,10 +275,6 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 			}
 			sc->dragging = FALSE;
 			sc->item = NULL;
-			if (sc->grabbed) {
-				sp_canvas_item_ungrab (sc->grabbed, event->button.time);
-				sc->grabbed = NULL;
-			}
 			ret = TRUE;
 		} 
 		break;
@@ -300,8 +298,8 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 	}
 
 	if (!ret) {
-		if (SP_EVENT_CONTEXT_CLASS (parent_class)->item_handler)
-			ret = SP_EVENT_CONTEXT_CLASS (parent_class)->item_handler (event_context, item, event);
+		if (((SPEventContextClass *) parent_class)->item_handler)
+			ret = ((SPEventContextClass *) parent_class)->item_handler (event_context, item, event);
 	}
 
 	return ret;
@@ -317,7 +315,7 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 	SPItem *item;
 	gint ret = FALSE;
 	NRPointF p;
-	ArtDRect b;
+	NRRectD b;
 	GSList *l;
 
 	desktop = event_context->desktop;
@@ -434,74 +432,6 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 		break;
 	case GDK_KEY_PRESS: // keybindings for select context
           switch (event->key.keyval) {  
-          case GDK_x: // Ctrl x - cut
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selection_cut(NULL);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_c:  // Ctrl c - copy
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selection_copy(NULL);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_v: // Ctrl v - paste
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selection_paste(NULL);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_g: // Ctrl g - group
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selection_group(NULL);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_G: // Ctrl G - ungroup
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selection_ungroup(NULL);
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_k: // Ctrl k - combine (shift - break appart)
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selected_path_combine();
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_K: // Ctrl K - break appart
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selected_path_break_apart();
-	      ret = TRUE;
-	    }
-	    break;
-	  case GDK_Home: // Home - raise selection to top
-	    sp_selection_raise_to_top (NULL);
-            ret = TRUE;
-            break;
-	  case GDK_End: // End - lower selection to bottom
-	    sp_selection_lower_to_bottom (NULL);
-            ret = TRUE;
-            break;
-	  case GDK_Page_Up: // PageUp - raise selection one layer
-	    sp_selection_raise (NULL);
-            ret = TRUE;
-            break;
-	  case GDK_Page_Down: // PageDown - lower selection one layer
-	    sp_selection_lower (NULL);
-            ret = TRUE;
-            break;
-	  case GDK_Delete: // Del - delete selection
-	    sp_selection_delete (NULL);
-	    ret = TRUE;
-	    break;
-	  case GDK_d: // Ctrl d - duplicate selection
-	    if (event->key.state & GDK_CONTROL_MASK) {
-	      sp_selection_duplicate (NULL);
-	      ret = TRUE;
-	    }
-	    break;
 	  case GDK_Left: // Left - move selection left
 	    if (event->key.state != GDK_CONTROL_MASK) {
 	      if (event->key.state == GDK_SHIFT_MASK) sp_selection_move_screen (-1,0);
@@ -552,8 +482,8 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 	}
 	
 	if (!ret) {
-	  if (SP_EVENT_CONTEXT_CLASS (parent_class)->root_handler)
-	    ret = SP_EVENT_CONTEXT_CLASS (parent_class)->root_handler (event_context, event);
+		if (((SPEventContextClass *) parent_class)->root_handler)
+			ret = ((SPEventContextClass *) parent_class)->root_handler (event_context, event);
 	}
 	
 	return ret;
@@ -563,8 +493,8 @@ static void
 sp_selection_moveto (SPSelTrans * seltrans, double x, double y, guint state)
 {
 	double dx, dy;
-	double move[6];
-	ArtPoint p, norm = {0,0};
+	NRMatrixD move;
+	NRPointF p, norm = {0,0};
 	GString * xs, * ys;
 	gchar status[80];
 	SPDesktop * desktop;
@@ -576,22 +506,23 @@ sp_selection_moveto (SPSelTrans * seltrans, double x, double y, guint state)
 	dy = y - p.y;
 
 	if (state & GDK_MOD1_MASK) {
-	  dx /= 10;
-	  dy /= 10;
+		dx /= 10;
+		dy /= 10;
 	}
 
-	dx = sp_desktop_horizontal_snap_list (desktop, seltrans->snappoints, dx);
-	dy = sp_desktop_vertical_snap_list (desktop, seltrans->snappoints, dy);
+	dx = sp_desktop_horizontal_snap_list (desktop, seltrans->spp, seltrans->spp_length, dx);
+	dy = sp_desktop_vertical_snap_list (desktop, seltrans->spp, seltrans->spp_length, dy);
 
 	if (state & GDK_CONTROL_MASK) {
-		if (fabs (dx) > fabs (dy)) 
-		  dy = 0.0;
-		else 
-		  dx = 0.0;
+		if (fabs (dx) > fabs (dy)) {
+			dy = 0.0;
+		} else {
+			dx = 0.0;
+		}
 	}
 
-	art_affine_translate (move, dx, dy);
-	sp_sel_trans_transform (seltrans, move, &norm);
+	nr_matrix_d_set_translate (&move, dx, dy);
+	sp_sel_trans_transform (seltrans, &move, &norm);
 
 	// status text
 	xs = SP_PT_TO_METRIC_STRING (dx, SP_DEFAULT_METRIC);
@@ -636,48 +567,54 @@ sp_select_context_config_widget (SPEventContext *ec)
 	gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
 
 	f = gtk_frame_new (_("Visual transformation"));
-	gtk_widget_show (f);
 	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
 
 	fb = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (fb);
 	gtk_container_add (GTK_CONTAINER (f), fb);
 
 	b = gtk_radio_button_new_with_label (NULL, _("Show content"));
-	gtk_widget_show (b);
 	gtk_object_set_data (GTK_OBJECT (b), "value", "content");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.show == SP_SELTRANS_SHOW_CONTENT);
 	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
 	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_show_toggled), sc);
 
 	b = gtk_radio_button_new_with_label (gtk_radio_button_group (GTK_RADIO_BUTTON (b)), _("Show outline"));
-	gtk_widget_show (b);
 	gtk_object_set_data (GTK_OBJECT (b), "value", "outline");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.show == SP_SELTRANS_SHOW_OUTLINE);
 	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
 	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_show_toggled), sc);
 
 	f = gtk_frame_new (_("Object transformation"));
-	gtk_widget_show (f);
 	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
 
 	fb = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (fb);
 	gtk_container_add (GTK_CONTAINER (f), fb);
 
 	b = gtk_radio_button_new_with_label (NULL, _("Optimize"));
-	gtk_widget_show (b);
 	gtk_object_set_data (GTK_OBJECT (b), "value", "optimize");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.transform == SP_SELTRANS_TRANSFORM_OPTIMIZE);
 	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
 	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_transform_toggled), sc);
 
 	b = gtk_radio_button_new_with_label (gtk_radio_button_group (GTK_RADIO_BUTTON (b)), _("Preserve"));
-	gtk_widget_show (b);
 	gtk_object_set_data (GTK_OBJECT (b), "value", "keep");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.transform == SP_SELTRANS_TRANSFORM_KEEP);
 	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
 	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_transform_toggled), sc);
 
+	f = gtk_frame_new (_("[Tab] Cycling"));
+	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+	fb = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (f), fb);
+
+	b = sp_config_check_button_new (_("Cycle only among visible items"),
+					"tools.select.cycling", "scope", "visible", "global");
+	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
+	b = sp_config_check_button_new (_("Center view to selected item"),
+					"tools.select.cycling", "focus", "follow", "preserve");
+	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
+
+	gtk_widget_show_all (vb);
+
 	return vb;
 }
