@@ -12,10 +12,11 @@
  */
 
 #include <math.h>
-#include <gdk/gdkkeysyms.h>
-#include <libart_lgpl/art_affine.h>
-#include <gtk/gtksignal.h>
 
+#include <libnr/nr-matrix.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtksignal.h>
+#include "macros.h"
 #include "svg/svg.h"
 #include "sodipodi-private.h"
 #include "sodipodi.h"
@@ -40,14 +41,13 @@ static void sp_show_handles (SPSelTrans * seltrans, SPKnot * knot[], const SPSel
 
 static void sp_sel_trans_handle_grab (SPKnot * knot, guint state, gpointer data);
 static void sp_sel_trans_handle_ungrab (SPKnot * knot, guint state, gpointer data);
-static void sp_sel_trans_handle_new_event (SPKnot * knot, ArtPoint * position, guint32 state, gpointer data);
-static gboolean sp_sel_trans_handle_request (SPKnot * knot, ArtPoint * p, guint state, gboolean * data);
+static void sp_sel_trans_handle_new_event (SPKnot * knot, NRPointF *position, guint32 state, gpointer data);
+static gboolean sp_sel_trans_handle_request (SPKnot * knot, NRPointF *p, guint state, gboolean * data);
 
 static void sp_sel_trans_sel_changed (SPSelection * selection, gpointer data);
 static void sp_sel_trans_sel_modified (SPSelection * selection, guint flags, gpointer data);
 
-extern GdkCursor * CursorSelectMouseover, * CursorSelectDragging;
-extern GdkPixbuf * handles[];
+extern GdkPixbuf *handles[];
 
 static gboolean
 sp_seltrans_handle_event (SPKnot *knot, GdkEvent *event, gpointer data)
@@ -69,7 +69,6 @@ sp_seltrans_handle_event (SPKnot *knot, GdkEvent *event, gpointer data)
 		}
 		break;
 	default:
-		g_print ("Event type %d occured\n", event->type);
 		break;
 	}
 
@@ -91,6 +90,12 @@ sp_sel_trans_init (SPSelTrans * seltrans, SPDesktop * desktop)
 	seltrans->show = SP_SELTRANS_SHOW_CONTENT;
 	seltrans->transform = SP_SELTRANS_TRANSFORM_OPTIMIZE;
 
+	seltrans->items = NULL;
+	seltrans->transforms = NULL;
+	seltrans->nitems = 0;
+
+	seltrans->spp = nr_new (NRPointF, SP_SELTRANS_SPP_SIZE);
+
 	seltrans->grabbed = FALSE;
 	seltrans->show_handles = TRUE;
 	for (i = 0; i < 8; i++) seltrans->shandle[i] = NULL;
@@ -105,8 +110,8 @@ sp_sel_trans_init (SPSelTrans * seltrans, SPDesktop * desktop)
 	sp_sel_trans_update_handles (seltrans);
 
 	seltrans->selection = SP_DT_SELECTION (desktop);
-	gtk_signal_connect (GTK_OBJECT (seltrans->selection), "changed", GTK_SIGNAL_FUNC (sp_sel_trans_sel_changed), seltrans);
-	gtk_signal_connect (GTK_OBJECT (seltrans->selection), "modified", GTK_SIGNAL_FUNC (sp_sel_trans_sel_modified), seltrans);
+	g_signal_connect (G_OBJECT (seltrans->selection), "changed", G_CALLBACK (sp_sel_trans_sel_changed), seltrans);
+	g_signal_connect (G_OBJECT (seltrans->selection), "modified", G_CALLBACK (sp_sel_trans_sel_modified), seltrans);
 
 	seltrans->norm = sp_canvas_item_new (SP_DT_CONTROLS (desktop),
 		SP_TYPE_CTRL,
@@ -154,16 +159,16 @@ sp_sel_trans_shutdown (SPSelTrans *seltrans)
 
 	for (i = 0; i < 8; i++) {
 		if (seltrans->shandle[i]) {
-			gtk_object_unref (GTK_OBJECT (seltrans->shandle[i]));
+			g_object_unref (G_OBJECT (seltrans->shandle[i]));
 			seltrans->shandle[i] = NULL;
 		}
 		if (seltrans->rhandle[i]) {
-			gtk_object_unref (GTK_OBJECT (seltrans->rhandle[i]));
+			g_object_unref (G_OBJECT (seltrans->rhandle[i]));
 			seltrans->rhandle[i] = NULL;
 		}
 	}
 	if (seltrans->chandle) {
-		gtk_object_unref (GTK_OBJECT (seltrans->chandle));
+		g_object_unref (G_OBJECT (seltrans->chandle));
 		seltrans->chandle = NULL;
 	}
 
@@ -193,8 +198,17 @@ sp_sel_trans_shutdown (SPSelTrans *seltrans)
 	}
 
 	if (seltrans->selection) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (seltrans->selection), seltrans);
+		sp_signal_disconnect_by_data (seltrans->selection, seltrans);
 	}
+
+	nr_free (seltrans->spp);
+
+	if (seltrans->items) {
+		int i;
+		for (i = 0; i < seltrans->nitems; i++) sp_object_unref (SP_OBJECT (seltrans->items[i]), NULL);
+		nr_free (seltrans->items);
+	}
+	if (seltrans->transforms) nr_free (seltrans->transforms);
 }
 
 void
@@ -227,7 +241,9 @@ sp_sel_trans_set_center (SPSelTrans * seltrans, gdouble x, gdouble y)
 void
 sp_sel_trans_grab (SPSelTrans * seltrans, NRPointF *p, gdouble x, gdouble y, gboolean show_handles)
 {
-	SPSelection * selection;
+	SPSelection *selection;
+	const GSList *l;
+	int n;
 
 	selection = SP_DT_SELECTION (seltrans->desktop);
 
@@ -238,16 +254,27 @@ sp_sel_trans_grab (SPSelTrans * seltrans, NRPointF *p, gdouble x, gdouble y, gbo
 	sp_sel_trans_update_volatile_state (seltrans);
 
 	seltrans->changed = FALSE;
-	seltrans->sel_changed = FALSE;
 
 	if (seltrans->empty) return;
 
-	art_affine_identity (seltrans->current);
+	l = sp_selection_item_list (selection);
+	seltrans->nitems = g_slist_length ((GSList *) l);
+	seltrans->items = nr_new (SPItem *, seltrans->nitems);
+	seltrans->transforms = nr_new (NRMatrixF, seltrans->nitems);
+	n = 0;
+	while (l) {
+		seltrans->items[n] = (SPItem *) sp_object_ref (SP_OBJECT (l->data), NULL);
+		sp_item_i2d_affine (seltrans->items[n], &seltrans->transforms[n]);
+		l = l->next;
+		n += 1;
+	}
+
+	nr_matrix_d_set_identity (&seltrans->current);
 
 	seltrans->point.x = p->x;
 	seltrans->point.y = p->y;
 
-	seltrans->snappoints = sp_selection_snappoints (selection);
+	seltrans->spp_length = sp_selection_snappoints (selection, seltrans->spp, SP_SELTRANS_SPP_SIZE);
 
 	seltrans->opposit.x = seltrans->box.x0 + (1 - x) * fabs (seltrans->box.x1 - seltrans->box.x0);
 	seltrans->opposit.y = seltrans->box.y0 + (1 - y) * fabs (seltrans->box.y1 - seltrans->box.y0);
@@ -270,58 +297,45 @@ sp_sel_trans_grab (SPSelTrans * seltrans, NRPointF *p, gdouble x, gdouble y, gbo
 }
 
 void
-sp_sel_trans_transform (SPSelTrans * seltrans, gdouble affine[], ArtPoint * norm)
+sp_sel_trans_transform (SPSelTrans * seltrans, NRMatrixD *affine, NRPointF *norm)
 {
-	SPItem * item;
-	const GSList * l;
-	gdouble current_[6], i2d[6], i2current[6], i2dnew[6], n2p[6], p2n[6];
-	ArtPoint p1, p2, p3, p4;
-	gint i;
+	NRMatrixD n2p, p2n;
 
 	g_return_if_fail (seltrans->grabbed);
 	g_return_if_fail (!seltrans->empty);
 
-	art_affine_invert (current_, seltrans->current);
-	art_affine_translate (n2p, norm->x, norm->y);
-	art_affine_invert (p2n, n2p);
-	art_affine_multiply (affine, p2n, affine);
-	art_affine_multiply (affine, affine, n2p);
+	nr_matrix_d_set_translate (&n2p, norm->x, norm->y);
+	nr_matrix_d_set_translate (&p2n, -norm->x, -norm->y);
+	nr_matrix_multiply_ddd (affine, &p2n, affine);
+	nr_matrix_multiply_ddd (affine, affine, &n2p);
 
 	if (seltrans->show == SP_SELTRANS_SHOW_CONTENT) {
 	        // update the content
-
-         	/* We accept empty lists here, as something may well remove items */
-	        /* and seltrans does not update, if frozen */
-	        l = sp_selection_item_list (SP_DT_SELECTION (seltrans->desktop));
-	        while (l != NULL) {
-		        item = SP_ITEM (l->data);
-			sp_item_i2d_affine (item, i2d);
-			art_affine_multiply (i2current, i2d, current_);
-			art_affine_multiply (i2dnew, i2current, affine);
-			sp_item_set_i2d_affine (item, i2dnew);
-			l = l->next;
+		int i;
+		for (i = 0; i < seltrans->nitems; i++) {
+			NRMatrixF i2dnew;
+			nr_matrix_multiply_ffd (&i2dnew, &seltrans->transforms[i], affine);
+			sp_item_set_i2d_affine (seltrans->items[i], &i2dnew);
 		}
 	} else {
-        	// update the outline
-                p1.x = seltrans->box.x0;
-		p1.y = seltrans->box.y0;
-		p2.x = seltrans->box.x0;
-		p2.y = seltrans->box.y1;
-		p3.x = seltrans->box.x1;
-		p3.y = seltrans->box.y1;
-		p4.x = seltrans->box.x1;
-		p4.y = seltrans->box.y0;
-		art_affine_point (&p1, &p1, affine);
-		art_affine_point (&p2, &p2, affine);
-		art_affine_point (&p3, &p3, affine);
-		art_affine_point (&p4, &p4, affine);
-		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l1), p1.x, p1.y, p2.x, p2.y);
-		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l2), p2.x, p2.y, p3.x, p3.y);
-		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l3), p3.x, p3.y, p4.x, p4.y);
-		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l4), p4.x, p4.y, p1.x, p1.y);
+		NRPointF p[4];
+        	/* update the outline */
+		p[0].x = NR_MATRIX_DF_TRANSFORM_X (affine, seltrans->box.x0, seltrans->box.y0);
+		p[0].y = NR_MATRIX_DF_TRANSFORM_Y (affine, seltrans->box.x0, seltrans->box.y0);
+		p[1].x = NR_MATRIX_DF_TRANSFORM_X (affine, seltrans->box.x1, seltrans->box.y0);
+		p[1].y = NR_MATRIX_DF_TRANSFORM_Y (affine, seltrans->box.x1, seltrans->box.y0);
+		p[2].x = NR_MATRIX_DF_TRANSFORM_X (affine, seltrans->box.x1, seltrans->box.y1);
+		p[2].y = NR_MATRIX_DF_TRANSFORM_Y (affine, seltrans->box.x1, seltrans->box.y1);
+		p[3].x = NR_MATRIX_DF_TRANSFORM_X (affine, seltrans->box.x0, seltrans->box.y1);
+		p[3].y = NR_MATRIX_DF_TRANSFORM_Y (affine, seltrans->box.x0, seltrans->box.y1);
+		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l1), p[0].x, p[0].y, p[1].x, p[1].y);
+		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l2), p[1].x, p[1].y, p[2].x, p[2].y);
+		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l3), p[2].x, p[2].y, p[3].x, p[3].y);
+		sp_ctrlline_set_coords (SP_CTRLLINE (seltrans->l4), p[3].x, p[3].y, p[0].x, p[0].y);
 	}
 
-	for (i = 0; i < 6; i++) seltrans->current[i] = affine[i];
+	
+	seltrans->current = *affine;
 
 	seltrans->changed = TRUE;
 
@@ -333,33 +347,32 @@ sp_sel_trans_ungrab (SPSelTrans * seltrans)
 {
 	SPItem * item;
 	const GSList * l;
-	gchar tstr[80];
-	ArtPoint p;
-	gdouble i2d[6], i2dnew[6];
+	NRPointD p;
+	unsigned int updh;
 
 	g_return_if_fail (seltrans->grabbed);
 
+	updh = TRUE;
 	if (!seltrans->empty && seltrans->changed) {
 		l = sp_selection_item_list (SP_DT_SELECTION (seltrans->desktop));
-
-		tstr[79] = '\0';
-
 		while (l != NULL) {
 			item = SP_ITEM (l->data);
 			/* fixme: We do not have to set it here (Lauris) */
 			if (seltrans->show == SP_SELTRANS_SHOW_OUTLINE) {
-				sp_item_i2d_affine (item, i2d);
-				art_affine_multiply (i2dnew, i2d, seltrans->current);
-				sp_item_set_i2d_affine (item, i2dnew);
+				NRMatrixF i2d, i2dnew;
+				sp_item_i2d_affine (item, &i2d);
+				nr_matrix_multiply_ffd (&i2dnew, &i2d, &seltrans->current);
+				sp_item_set_i2d_affine (item, &i2dnew);
 			}
 			if (seltrans->transform == SP_SELTRANS_TRANSFORM_OPTIMIZE) {
-				sp_item_write_transform (item, SP_OBJECT_REPR (item), item->affine);
+				sp_item_write_transform (item, SP_OBJECT_REPR (item), &item->transform);
 				/* because item/repr affines may be out of sync, invoke reread */
 				/* fixme: We should test equality to avoid unnecessary rereads */
 				/* fixme: This probably is not needed (Lauris) */
-				sp_object_invoke_read_attr (SP_OBJECT (item), "transform");
+				/* sp_object_read_attr (SP_OBJECT (item), "transform"); */
 			} else {
-				if (sp_svg_write_affine (tstr, 79, item->affine)) {
+				unsigned char tstr[80];
+				if (sp_svg_transform_write (tstr, 80, &item->transform)) {
 					sp_repr_set_attr (SP_OBJECT (item)->repr, "transform", tstr);
 				} else {
 					sp_repr_set_attr (SP_OBJECT (item)->repr, "transform", NULL);
@@ -367,31 +380,31 @@ sp_sel_trans_ungrab (SPSelTrans * seltrans)
 			}
 			l = l->next;
 		}
-		p.x = seltrans->center.x;
-		p.y = seltrans->center.y;
-		art_affine_point (&p, &p, seltrans->current);
-		seltrans->center.x = p.x;
-		seltrans->center.y = p.y;
+		p = seltrans->center;
+		seltrans->center.x = NR_MATRIX_DF_TRANSFORM_X (&seltrans->current, p.x, p.y);
+		seltrans->center.y = NR_MATRIX_DF_TRANSFORM_Y (&seltrans->current, p.x, p.y);
 		
 		sp_document_done (SP_DT_DOCUMENT (seltrans->desktop));
 		sp_selection_changed (SP_DT_SELECTION (seltrans->desktop));
+
+		updh = FALSE;
 	}
 
+	if (seltrans->items) {
+		int i;
+		for (i = 0; i < seltrans->nitems; i++) sp_object_unref (SP_OBJECT (seltrans->items[i]), NULL);
+		nr_free (seltrans->items);
+		seltrans->items = NULL;
+	}
+	if (seltrans->transforms) {
+		nr_free (seltrans->transforms);
+		seltrans->transforms = NULL;
+	}
+	seltrans->nitems = 0;
+
 	seltrans->grabbed = FALSE;
 	seltrans->show_handles = TRUE;
 	
-	/* free snappoints */
-	while (seltrans->snappoints) {
-		g_free (seltrans->snappoints->data);
-		seltrans->snappoints = g_slist_remove (seltrans->snappoints, seltrans->snappoints->data);
-	}
-
-#if 0
-	if (seltrans->sel_changed) {
-		seltrans->state = SP_SEL_TRANS_SCALE;
-	}
-#endif
-
 	sp_canvas_item_hide (seltrans->norm);
 	sp_canvas_item_hide (seltrans->grip);
 
@@ -403,7 +416,7 @@ sp_sel_trans_ungrab (SPSelTrans * seltrans)
 	}
 
 	sp_sel_trans_update_volatile_state (seltrans);
-	sp_sel_trans_update_handles (seltrans);
+	if (updh) sp_sel_trans_update_handles (seltrans);
 	if (seltrans->stamp_cache) {
 		g_slist_free(seltrans->stamp_cache);
 		seltrans->stamp_cache = NULL;
@@ -426,26 +439,25 @@ sp_sel_trans_stamp (SPSelTrans * seltrans)
 	/* stamping mode */
 	SPItem * original_item, * copy_item;
 	SPRepr * original_repr, * copy_repr;
-	GSList *l;
+	GSList * l;
 
 	gchar tstr[80];
-	gdouble i2d[6], i2dnew[6];
-	
-	gdouble * new_affine;
+	NRMatrixF i2d, i2dnew;
+	NRMatrixF *new_affine;
 
 	tstr[79] = '\0';
 	
 	if (!seltrans->empty) {
-		
-		if (seltrans->stamp_cache) {
-			l = seltrans->stamp_cache;
-		} else {
-			/* Build cache */
-			l  = (GSList *) sp_selection_item_list (SP_DT_SELECTION (seltrans->desktop));
-			l  = g_slist_copy (l);
-			l  = g_slist_sort (l, (GCompareFunc) sp_object_compare_position);
-			seltrans->stamp_cache = l;
-		}
+
+                if (seltrans->stamp_cache) {
+                        l = seltrans->stamp_cache;
+                } else {
+                        /* Build cache */
+                        l  = (GSList *) sp_selection_item_list (SP_DT_SELECTION (seltrans->desktop));
+                        l  = g_slist_copy (l);
+                        l  = g_slist_sort (l, (GCompareFunc) sp_object_compare_position);
+                        seltrans->stamp_cache = l;
+                }
 
 		while (l) {
 			original_item = SP_ITEM(l->data);
@@ -455,17 +467,19 @@ sp_sel_trans_stamp (SPSelTrans * seltrans)
 								     copy_repr);
 			
 			if (seltrans->show == SP_SELTRANS_SHOW_OUTLINE) {
-				sp_item_i2d_affine (original_item, i2d);
-				art_affine_multiply (i2dnew, i2d, seltrans->current);
-				sp_item_set_i2d_affine (copy_item, i2dnew);
-				new_affine = copy_item->affine;
-			} else 
-				new_affine = original_item->affine;
+				sp_item_i2d_affine (original_item, &i2d);
+				nr_matrix_multiply_ffd (&i2dnew, &i2d, &seltrans->current);
+				sp_item_set_i2d_affine (copy_item, &i2dnew);
+				new_affine = &copy_item->transform;
+			} else {
+				new_affine = &original_item->transform;
+			}
 
-			if (sp_svg_write_affine (tstr, 79, new_affine)) 
+			if (sp_svg_transform_write (tstr, 80, new_affine)) {
 				sp_repr_set_attr (copy_repr, "transform", tstr);
-			else
+			} else {
 				sp_repr_set_attr (copy_repr, "transform", NULL);
+			}
 			sp_repr_unref (copy_repr);
 			l = l->next;
 		}
@@ -473,8 +487,8 @@ sp_sel_trans_stamp (SPSelTrans * seltrans)
 	}
 }
 
-ArtPoint *
-sp_sel_trans_point_desktop (SPSelTrans * seltrans, ArtPoint * p)
+NRPointF *
+sp_sel_trans_point_desktop (SPSelTrans *seltrans, NRPointF *p)
 {
 	g_return_val_if_fail (p != NULL, NULL);
 
@@ -484,8 +498,8 @@ sp_sel_trans_point_desktop (SPSelTrans * seltrans, ArtPoint * p)
 	return p;
 }
 
-ArtPoint *
-sp_sel_trans_origin_desktop (SPSelTrans * seltrans, ArtPoint * p)
+NRPointF *
+sp_sel_trans_origin_desktop (SPSelTrans *seltrans, NRPointF *p)
 {
 	g_return_val_if_fail (p != NULL, NULL);
 
@@ -498,6 +512,8 @@ sp_sel_trans_origin_desktop (SPSelTrans * seltrans, ArtPoint * p)
 static void
 sp_sel_trans_update_handles (SPSelTrans * seltrans)
 {
+	NRPointF p;
+
 	g_return_if_fail (seltrans != NULL);
 
       	if ((!seltrans->show_handles) || (seltrans->empty)) {
@@ -520,40 +536,45 @@ sp_sel_trans_update_handles (SPSelTrans * seltrans)
 	// center handle
 	if (seltrans->chandle == NULL) {
 	  seltrans->chandle = sp_knot_new (seltrans->desktop);
-	  gtk_object_set (GTK_OBJECT (seltrans->chandle),
-			  "anchor", handle_center.anchor, 
-			  "shape", SP_CTRL_SHAPE_BITMAP,
-			  "size", 13,
-			  "mode", SP_CTRL_MODE_COLOR,
-					"fill", 0x00000000,
-					"fill_mouseover", 0x0000007f,
-					"stroke", 0x000000ff,
-					"stroke_mouseover", 0x000000ff,
+	  g_object_set (G_OBJECT (seltrans->chandle),
+			"anchor", handle_center.anchor, 
+			"shape", SP_CTRL_SHAPE_BITMAP,
+			"size", 13,
+			"mode", SP_CTRL_MODE_COLOR,
+			"fill", 0x00000000,
+			"fill_mouseover", 0x0000007f,
+			"stroke", 0x000000ff,
+			"stroke_mouseover", 0x000000ff,
 			  //"fill", 0xff40ffa0,
 			  //"fill_mouseover", 0x40ffffa0,
 			  //"stroke", 0xFFb0b0ff,
 			  //"stroke_mouseover", 0xffffffFF,
-			  "pixbuf", handles[handle_center.control],
-			  "cursor_mouseover", CursorSelectMouseover,
-			  "cursor_dragging", CursorSelectDragging,
-			  NULL);
-	  gtk_signal_connect (GTK_OBJECT (seltrans->chandle), "request",
-			      GTK_SIGNAL_FUNC (sp_sel_trans_handle_request), (gpointer) &handle_center);
-	  gtk_signal_connect (GTK_OBJECT (seltrans->chandle), "moved",
-	  		      GTK_SIGNAL_FUNC (sp_sel_trans_handle_new_event), (gpointer) &handle_center);
-	  gtk_signal_connect (GTK_OBJECT (seltrans->chandle), "grabbed",
-			      GTK_SIGNAL_FUNC (sp_sel_trans_handle_grab), (gpointer) &handle_center);
-	  gtk_signal_connect (GTK_OBJECT (seltrans->chandle), "ungrabbed",
-			      GTK_SIGNAL_FUNC (sp_sel_trans_handle_ungrab), (gpointer) &handle_center);
+			"pixbuf", handles[handle_center.control],
+#if 0
+			"cursor_mouseover", CursorSelectMouseover,
+			"cursor_dragging", CursorSelectDragging,
+#endif
+			NULL);
+	  g_signal_connect (G_OBJECT (seltrans->chandle), "request",
+			    G_CALLBACK (sp_sel_trans_handle_request), (gpointer) &handle_center);
+	  g_signal_connect (G_OBJECT (seltrans->chandle), "moved",
+			    G_CALLBACK (sp_sel_trans_handle_new_event), (gpointer) &handle_center);
+	  g_signal_connect (G_OBJECT (seltrans->chandle), "grabbed",
+			    G_CALLBACK (sp_sel_trans_handle_grab), (gpointer) &handle_center);
+	  g_signal_connect (G_OBJECT (seltrans->chandle), "ungrabbed",
+			    G_CALLBACK (sp_sel_trans_handle_ungrab), (gpointer) &handle_center);
 	}
 	sp_knot_show (seltrans->chandle);
-	sp_knot_set_position (seltrans->chandle, &seltrans->center, 0);
+	p.x = seltrans->center.x;
+	p.y = seltrans->center.y;
+	sp_knot_set_position (seltrans->chandle, &p, 0);
 }
 
 static void
 sp_sel_trans_update_volatile_state (SPSelTrans * seltrans)
 {
-	SPSelection * selection;
+	SPSelection *selection;
+	NRRectF b;
 
 	g_return_if_fail (seltrans != NULL);
 
@@ -563,14 +584,18 @@ sp_sel_trans_update_volatile_state (SPSelTrans * seltrans)
 
 	if (seltrans->empty) return;
 
-	sp_selection_bbox (SP_DT_SELECTION (seltrans->desktop), &seltrans->box);
+	sp_selection_bbox (SP_DT_SELECTION (seltrans->desktop), &b);
+	seltrans->box.x0 = b.x0;
+	seltrans->box.y0 = b.y0;
+	seltrans->box.x1 = b.x1;
+	seltrans->box.y1 = b.y1;
 
 	if ((seltrans->box.x0 > seltrans->box.x1) || (seltrans->box.y0 > seltrans->box.y1)) {
 		seltrans->empty = TRUE;
 		return;
 	}
 
-	art_affine_identity (seltrans->current);
+	nr_matrix_d_set_identity (&seltrans->current);
 }
 
 static void
@@ -588,13 +613,13 @@ sp_remove_handles (SPKnot * knot[], gint num)
 static void
 sp_show_handles (SPSelTrans * seltrans, SPKnot * knot[], const SPSelTransHandle handle[], gint num)
 {
-	ArtPoint p;
+	NRPointF p;
 	gint i;
 
 	for (i = 0; i < num; i++) {
 		if (knot[i] == NULL) {
 			knot[i] = sp_knot_new (seltrans->desktop);
-			gtk_object_set (GTK_OBJECT (knot[i]),
+			g_object_set (G_OBJECT (knot[i]),
 					"anchor", handle[i].anchor, 
 					"shape", SP_CTRL_SHAPE_BITMAP,
 					"size", 13,
@@ -608,19 +633,21 @@ sp_show_handles (SPSelTrans * seltrans, SPKnot * knot[], const SPSelTransHandle
 					//"stroke", 0xFFFFFFff,
 					//"stroke_mouseover", 0xb0b0b0FF,
 					"pixbuf", handles[handle[i].control],
+#if 0
 					"cursor_mouseover", CursorSelectMouseover,
 					"cursor_dragging", CursorSelectDragging,
+#endif
 					NULL);
 
-			gtk_signal_connect (GTK_OBJECT (knot[i]), "request",
-				GTK_SIGNAL_FUNC (sp_sel_trans_handle_request), (gpointer) &handle[i]);
- 			gtk_signal_connect (GTK_OBJECT (knot[i]), "moved",
-				GTK_SIGNAL_FUNC (sp_sel_trans_handle_new_event), (gpointer) &handle[i]);
-			gtk_signal_connect (GTK_OBJECT (knot[i]), "grabbed",
-				GTK_SIGNAL_FUNC (sp_sel_trans_handle_grab), (gpointer) &handle[i]);
-			gtk_signal_connect (GTK_OBJECT (knot[i]), "ungrabbed",
-				GTK_SIGNAL_FUNC (sp_sel_trans_handle_ungrab), (gpointer) &handle[i]);
-			gtk_signal_connect (GTK_OBJECT (knot[i]), "event", GTK_SIGNAL_FUNC (sp_seltrans_handle_event), (gpointer) &handle[i]);
+			g_signal_connect (G_OBJECT (knot[i]), "request",
+					  G_CALLBACK (sp_sel_trans_handle_request), (gpointer) &handle[i]);
+ 			g_signal_connect (G_OBJECT (knot[i]), "moved",
+					  G_CALLBACK (sp_sel_trans_handle_new_event), (gpointer) &handle[i]);
+			g_signal_connect (G_OBJECT (knot[i]), "grabbed",
+					  G_CALLBACK (sp_sel_trans_handle_grab), (gpointer) &handle[i]);
+			g_signal_connect (G_OBJECT (knot[i]), "ungrabbed",
+					  G_CALLBACK (sp_sel_trans_handle_ungrab), (gpointer) &handle[i]);
+			g_signal_connect (G_OBJECT (knot[i]), "event", G_CALLBACK (sp_seltrans_handle_event), (gpointer) &handle[i]);
 		}
 		sp_knot_show (knot[i]);
 
@@ -642,7 +669,7 @@ sp_sel_trans_handle_grab (SPKnot * knot, guint state, gpointer data)
 	SPDesktop * desktop;
 	SPSelTrans * seltrans;
 	SPSelTransHandle * handle;
-	ArtPoint p;
+	NRPointF p;
 	NRPointF pf;
 
 	desktop = knot->desktop;
@@ -651,14 +678,14 @@ sp_sel_trans_handle_grab (SPKnot * knot, guint state, gpointer data)
 
 	switch (handle->anchor) {
 	case GTK_ANCHOR_CENTER:
-	  gtk_object_set (GTK_OBJECT (seltrans->grip),
+	  g_object_set (G_OBJECT (seltrans->grip),
 			  "shape", SP_CTRL_SHAPE_BITMAP,
 			  "size", 13.0,
 			  NULL);
 	  sp_canvas_item_show (seltrans->grip);
 	  break;
 	default:
-	  gtk_object_set (GTK_OBJECT (seltrans->grip),
+	  g_object_set (G_OBJECT (seltrans->grip),
 			  "shape", SP_CTRL_SHAPE_CROSS,
 			  "size", 7.0,
 			  NULL);
@@ -690,7 +717,7 @@ sp_sel_trans_handle_ungrab (SPKnot * knot, guint state, gpointer data)
 }
 
 static void
-sp_sel_trans_handle_new_event (SPKnot * knot, ArtPoint * position, guint state, gpointer data)
+sp_sel_trans_handle_new_event (SPKnot * knot, NRPointF *position, guint state, gpointer data)
 {
 	SPDesktop * desktop;
 	SPSelTrans * seltrans;
@@ -709,12 +736,12 @@ sp_sel_trans_handle_new_event (SPKnot * knot, ArtPoint * position, guint state,
 /* fixme: Highly experimental test :) */
 
 static gboolean
-sp_sel_trans_handle_request (SPKnot * knot, ArtPoint * position, guint state, gboolean * data)
+sp_sel_trans_handle_request (SPKnot * knot, NRPointF *position, guint state, gboolean * data)
 {
 	SPDesktop * desktop;
 	SPSelTrans * seltrans;
 	SPSelTransHandle * handle;
-	ArtPoint s, point;
+	NRPointF point;
 
 	if (!SP_KNOT_IS_GRABBED (knot)) return TRUE;
 
@@ -726,23 +753,24 @@ sp_sel_trans_handle_request (SPKnot * knot, ArtPoint * position, guint state, gb
 	sp_view_set_position (SP_VIEW (desktop), position->x, position->y);
 
 	if (state & GDK_MOD1_MASK) {
-	  sp_sel_trans_point_desktop (seltrans, &point);
-	  position->x = point.x + (position->x - point.x) / 10;
-	  position->y = point.y + (position->y - point.y) / 10;
+		sp_sel_trans_point_desktop (seltrans, &point);
+		position->x = point.x + (position->x - point.x) / 10;
+		position->y = point.y + (position->y - point.y) / 10;
+	}
+
+	if (!(state & GDK_SHIFT_MASK) == !(seltrans->state == SP_SELTRANS_STATE_ROTATE)) {
+		seltrans->origin = seltrans->opposit;
+	} else {
+		seltrans->origin = seltrans->center;
 	}
-	if (state & GDK_SHIFT_MASK) seltrans->origin = seltrans->center;
-	else seltrans->origin = seltrans->opposit;
 
 	if (handle->request (seltrans, handle, position, state)) {
-	  sp_knot_set_position (knot, position, state);
-	  s.x = rint (position->x);
-	  s.y = rint (position->y);	  
-	  sp_ctrl_moveto (SP_CTRL (seltrans->grip), s.x, s.y);
-	  sp_ctrl_moveto (SP_CTRL (seltrans->norm), seltrans->origin.x, seltrans->origin.y);
+		sp_knot_set_position (knot, position, state);
+		sp_ctrl_moveto (SP_CTRL (seltrans->grip), position->x, position->y);
+		sp_ctrl_moveto (SP_CTRL (seltrans->norm), seltrans->origin.x, seltrans->origin.y);
 	}
 	
 	return TRUE;
-
 }
 
 static void
@@ -752,9 +780,7 @@ sp_sel_trans_sel_changed (SPSelection * selection, gpointer data)
 
 	seltrans = (SPSelTrans *) data;
 
-	if (seltrans->grabbed) {
-		seltrans->sel_changed = TRUE;
-	} else {
+	if (!seltrans->grabbed) {
 		sp_sel_trans_update_volatile_state (seltrans);
 		seltrans->center.x = (seltrans->box.x0 + seltrans->box.x1) / 2;
 		seltrans->center.y = (seltrans->box.y0 + seltrans->box.y1) / 2;
@@ -770,9 +796,7 @@ sp_sel_trans_sel_modified (SPSelection *selection, guint flags, gpointer data)
 
 	seltrans = (SPSelTrans *) data;
 
-	if (seltrans->grabbed) {
-		seltrans->sel_changed = TRUE;
-	} else {
+	if (!seltrans->grabbed) {
 		sp_sel_trans_update_volatile_state (seltrans);
 		seltrans->center.x = (seltrans->box.x0 + seltrans->box.x1) / 2;
 		seltrans->center.y = (seltrans->box.y0 + seltrans->box.y1) / 2;
@@ -786,9 +810,9 @@ sp_sel_trans_sel_modified (SPSelection *selection, guint flags, gpointer data)
  */
 
 gboolean
-sp_sel_trans_scale_request (SPSelTrans *seltrans, SPSelTransHandle *handle, ArtPoint *p, guint state)
+sp_sel_trans_scale_request (SPSelTrans *seltrans, SPSelTransHandle *handle, NRPointF *p, guint state)
 {
-	ArtPoint norm, point;
+	NRPointF norm, point;
 	gdouble sx, sy;
 	gchar status[80];
 	SPDesktop *desktop;
@@ -821,12 +845,12 @@ sp_sel_trans_scale_request (SPSelTrans *seltrans, SPSelTransHandle *handle, ArtP
 		if (!xd || !yd) return FALSE;
 	        if (fabs (sy) > fabs (sx)) sy = fabs (sx) * sy / fabs (sy);
 		if (fabs (sx) > fabs (sy)) sx = fabs (sy) * sx / fabs (sx);
-		r = sp_desktop_vector_snap_list (desktop, seltrans->snappoints, &norm, sx, sy);
+		r = sp_desktop_vector_snap_list (desktop, seltrans->spp, seltrans->spp_length, &norm, sx, sy);
 		sx = fabs (r) * sx / fabs (sx);
 		sy = fabs (r) * sy / fabs (sy);
 	} else {
-		if (xd) sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->snappoints, &norm, sx);
-		if (yd) sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->snappoints, &norm, sy);
+		if (xd) sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->spp, seltrans->spp_length, &norm, sx);
+		if (yd) sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->spp, seltrans->spp_length, &norm, sy);
 	}
 
 	p->x = (point.x - norm.x) * sx + norm.x;
@@ -840,11 +864,11 @@ sp_sel_trans_scale_request (SPSelTrans *seltrans, SPSelTransHandle *handle, ArtP
 }
 
 gboolean
-sp_sel_trans_stretch_request (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_stretch_request (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF *p, guint state)
 {
 	double sx = 1.0, sy = 1.0, ratio;
 	gchar status[80];	
-	ArtPoint norm, point;
+	NRPointF norm, point;
 	SPDesktop * desktop;
 
 	desktop = seltrans->desktop;
@@ -861,11 +885,11 @@ sp_sel_trans_stretch_request (SPSelTrans * seltrans, SPSelTransHandle * handle,
 		if (state & GDK_CONTROL_MASK) {
 			if (fabs (sy) > 1) sy = sy / fabs (sy);
 			sx = fabs (sy);
-			ratio = sp_desktop_vector_snap_list (desktop, seltrans->snappoints, &norm, sx, sy);
+			ratio = sp_desktop_vector_snap_list (desktop, seltrans->spp, seltrans->spp_length, &norm, sx, sy);
 			sy = (fabs (ratio) < 1) ? fabs (ratio) * sy / fabs (sy) : sy / fabs (sy);
 			sx = fabs (sy);
 		} else {
-			sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->snappoints, &norm, sy); 
+			sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->spp, seltrans->spp_length, &norm, sy); 
 		}
 		break;
 	case GDK_LEFT_SIDE:
@@ -876,11 +900,11 @@ sp_sel_trans_stretch_request (SPSelTrans * seltrans, SPSelTransHandle * handle,
 		if (state & GDK_CONTROL_MASK) {
 			if (fabs (sx) > 1) sx = sx / fabs (sx);
 			sy = fabs (sx);
-			ratio = sp_desktop_vector_snap_list (desktop, seltrans->snappoints, &norm, sx, sy);
+			ratio = sp_desktop_vector_snap_list (desktop, seltrans->spp, seltrans->spp_length, &norm, sx, sy);
 			sx = (fabs (sx) < 1) ? fabs (ratio) * sx / fabs (sx) : sx / fabs (sx);
 			sy = fabs (ratio);
 		} else {
-			sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->snappoints, &norm, sx);
+			sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->spp, seltrans->spp_length, &norm, sx);
 		}
 		break;
 	default:
@@ -898,11 +922,11 @@ sp_sel_trans_stretch_request (SPSelTrans * seltrans, SPSelTransHandle * handle,
 }
 
 gboolean
-sp_sel_trans_skew_request (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_skew_request (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF *p, guint state)
 {
         double skew[6], sx = 1.0, sy = 1.0;
         gchar status[80];
-	ArtPoint norm, point;
+	NRPointF norm, point;
 	SPDesktop * desktop;
 
 	desktop = seltrans->desktop;
@@ -920,11 +944,11 @@ sp_sel_trans_skew_request (SPSelTrans * seltrans, SPSelTransHandle * handle, Art
 	case GDK_SB_V_DOUBLE_ARROW:
 	  if (fabs (point.x - norm.x) < 1e-15) return FALSE;
 	  skew[1] = (p->y - point.y) / (point.x - norm.x);
-	  skew[1] = sp_desktop_vertical_snap_list_skew (desktop, seltrans->snappoints, &norm, skew[1]);
+	  skew[1] = sp_desktop_vertical_snap_list_skew (desktop, seltrans->spp, seltrans->spp_length, &norm, skew[1]);
 	  p->y = (point.x - norm.x) * skew[1] + point.y; 
 	  sx = (p->x - norm.x) / (point.x - norm.x);
 	  if (state & GDK_CONTROL_MASK) {
-	    sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->snappoints, &norm, sx);
+	    sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->spp, seltrans->spp_length, &norm, sx);
 	  } else {
 	    if (fabs (sx) < 1e-15) sx = 1e-15;
 	    if (fabs (sx) < 1) sx = fabs (sx) / sx;
@@ -938,12 +962,12 @@ sp_sel_trans_skew_request (SPSelTrans * seltrans, SPSelTransHandle * handle, Art
 	case GDK_SB_H_DOUBLE_ARROW:
 	  if (fabs (point.y - norm.y) < 1e-15) return FALSE;
 	  skew[2] = (p->x - point.x) / (point.y - norm.y);
-	  skew[2] = sp_desktop_horizontal_snap_list_skew (desktop, seltrans->snappoints, &norm, skew[2]);
+	  skew[2] = sp_desktop_horizontal_snap_list_skew (desktop, seltrans->spp, seltrans->spp_length, &norm, skew[2]);
 	  p->x = (point.y - norm.y) * skew[2] + point.x; 
 	  sy = (p->y - norm.y) / (point.y - norm.y);
 	  
 	  if (state & GDK_CONTROL_MASK) {
-	    sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->snappoints, &norm, sy);
+	    sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->spp, seltrans->spp_length, &norm, sy);
 	  } else {
 	    if (fabs (sy) < 1e-15) sy = 1e-15;
 	    if (fabs (sy) < 1) sy = fabs (sy) / sy;
@@ -965,10 +989,13 @@ sp_sel_trans_skew_request (SPSelTrans * seltrans, SPSelTransHandle * handle, Art
 }
 
 gboolean
-sp_sel_trans_rotate_request (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_rotate_request (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF *p, guint state)
 {
-	ArtPoint norm, point, q1, q2;
-	gdouble h1, h2, r1[6], r2[6], rotate[6], n2p[6], p2n[6], dx1, dx2, dy1, dy2, sg, angle;
+	NRPointF norm, point, q1, q2;
+	double h1, h2;
+	NRMatrixD r1, r2, n2p, p2n;
+	NRMatrixF rotate;
+	double dx1, dx2, dy1, dy2, sg, angle;
 	SPDesktop * desktop;
 	gchar status[80];
 
@@ -990,19 +1017,22 @@ sp_sel_trans_rotate_request (SPSelTrans * seltrans, SPSelTransHandle * handle, A
 	if (fabs (h2) < 1e-15) return FALSE;
 	q2.x = (dx2) / h2;
 	q2.y = (dy2) / h2;
-	r1[0] = q1.x;  r1[1] = -q1.y;  r1[2] =  q1.y;  r1[3] = q1.x;  r1[4] = 0;  r1[5] = 0;
-	r2[0] = q2.x;  r2[1] =  q2.y;  r2[2] = -q2.y;  r2[3] = q2.x;  r2[4] = 0;  r2[5] = 0;
-	art_affine_translate (n2p, norm.x, norm.y);
-	art_affine_invert (p2n, n2p);
-	art_affine_multiply (rotate, p2n, r1);
-	art_affine_multiply (rotate, rotate, r2);
-	art_affine_multiply (rotate, rotate, n2p);
+	r1.c[0] = q1.x;  r1.c[1] = -q1.y;  r1.c[2] =  q1.y;  r1.c[3] = q1.x;  r1.c[4] = 0;  r1.c[5] = 0;
+	r2.c[0] = q2.x;  r2.c[1] =  q2.y;  r2.c[2] = -q2.y;  r2.c[3] = q2.x;  r2.c[4] = 0;  r2.c[5] = 0;
+	nr_matrix_d_set_translate (&n2p, norm.x, norm.y);
+	nr_matrix_d_invert (&p2n, &n2p);
+	nr_matrix_multiply_fdd (&rotate, &p2n, &r1);
+	nr_matrix_multiply_ffd (&rotate, &rotate, &r2);
+	nr_matrix_multiply_ffd (&rotate, &rotate, &n2p);
 
-	// snap
-	sp_desktop_circular_snap_list (desktop, seltrans->snappoints, &norm, rotate);
-	art_affine_point (p, &point, rotate);
+#if 0
+	/* Snap */
+	sp_desktop_circular_snap_list (desktop, seltrans->spp, seltrans->spp_length, &norm, &rotate);
+#endif
+	p->x = NR_MATRIX_DF_TRANSFORM_X (&rotate, point.x, point.y);
+	p->y = NR_MATRIX_DF_TRANSFORM_Y (&rotate, point.x, point.y);
 
-	// status text
+	/* status text */
 	dx2 = p->x    - norm.x;
 	dy2 = p->y    - norm.y;
 	h2 = hypot (dx2, dy2);
@@ -1018,12 +1048,12 @@ sp_sel_trans_rotate_request (SPSelTrans * seltrans, SPSelTransHandle * handle, A
 }
 
 gboolean
-sp_sel_trans_center_request (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_center_request (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF *p, guint state)
 {
 	gchar status[80];
 	GString * xs, * ys;
 	SPDesktop * desktop;
-	ArtPoint point;
+	NRPointF point;
 	
 	desktop  = seltrans->desktop;
 	sp_desktop_free_snap (desktop, p);
@@ -1058,10 +1088,11 @@ sp_sel_trans_center_request (SPSelTrans * seltrans, SPSelTransHandle * handle, A
  */
 
 void
-sp_sel_trans_stretch (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_stretch (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF *p, guint state)
 {
-	double stretch[6], sx = 1.0, sy = 1.0;
-	ArtPoint norm, point;
+	NRMatrixD stretch;
+	double sx = 1.0, sy = 1.0;
+	NRPointF norm, point;
 
 	sp_sel_trans_point_desktop (seltrans, &point);
 	sp_sel_trans_origin_desktop (seltrans, &norm);
@@ -1093,15 +1124,16 @@ sp_sel_trans_stretch (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint
 
 	if (fabs (sx) < 1e-15) sx = 1e-15; 
 	if (fabs (sy) < 1e-15) sy = 1e-15;
-	art_affine_scale (stretch, sx, sy);
-	sp_sel_trans_transform (seltrans, stretch, &norm);
+	nr_matrix_d_set_scale (&stretch, sx, sy);
+	sp_sel_trans_transform (seltrans, &stretch, &norm);
 }
 
 void
-sp_sel_trans_scale (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state) 
+sp_sel_trans_scale (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF * p, guint state) 
 {
-	double scale[6], sx, sy;
-	ArtPoint norm, point;
+	NRMatrixD scale;
+	double sx, sy;
+	NRPointF norm, point;
 
 	sp_sel_trans_point_desktop (seltrans, &point);
 	sp_sel_trans_origin_desktop (seltrans, &norm);
@@ -1121,51 +1153,52 @@ sp_sel_trans_scale (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint *
 	if (fabs (sx) < 1e-9) sx = 1e-9; 
 	if (fabs (sy) < 1e-9) sy = 1e-9;
 
-	art_affine_scale (scale, sx, sy);
-	sp_sel_trans_transform (seltrans, scale, &norm);
+	nr_matrix_d_set_scale (&scale, sx, sy);
+	sp_sel_trans_transform (seltrans, &scale, &norm);
 }
 
 void
-sp_sel_trans_skew (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_skew (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF *p, guint state)
 {
-        double skew[6];
-	ArtPoint norm, point;
+        NRMatrixD skew;
+	NRPointF norm, point;
 
 	sp_sel_trans_point_desktop (seltrans, &point);
 	sp_sel_trans_origin_desktop (seltrans, &norm);
   
-	skew[4]=0;
-	skew[5]=0;
+	skew.c[4]=0;
+	skew.c[5]=0;
 	
 	switch (handle->cursor) {
 	case GDK_SB_H_DOUBLE_ARROW:
-	  if (fabs (point.y - norm.y) < 1e-15) return;
-	  skew[2] = (p->x - point.x) / (point.y - norm.y);
-	  skew[3] = (p->y - norm.y) / (point.y - norm.y);
-	  skew[1] = 0;
-	  skew[0] = 1;
-	  break;
+		if (fabs (point.y - norm.y) < 1e-15) return;
+		skew.c[2] = (p->x - point.x) / (point.y - norm.y);
+		skew.c[3] = (p->y - norm.y) / (point.y - norm.y);
+		skew.c[1] = 0;
+		skew.c[0] = 1;
+		break;
 	case GDK_SB_V_DOUBLE_ARROW:
-	  if (fabs (point.x - norm.x) < 1e-15) return;
-	  skew[1] = (p->y - point.y) / (point.x - norm.x);
-	  skew[0] = (p->x - norm.x) / (point.x - norm.x);
-	  skew[2] = 0;
-	  skew[3] = 1;
-	  break;
+		if (fabs (point.x - norm.x) < 1e-15) return;
+		skew.c[1] = (p->y - point.y) / (point.x - norm.x);
+		skew.c[0] = (p->x - norm.x) / (point.x - norm.x);
+		skew.c[2] = 0;
+		skew.c[3] = 1;
+		break;
 	default:
 		break;
 	}
 
-	if (fabs (skew[0]) < 1e-15) skew[0] = 1e-15;
-	if (fabs (skew[3]) < 1e-15) skew[3] = 1e-15;
-	sp_sel_trans_transform (seltrans, skew, &norm);
+	if (fabs (skew.c[0]) < 1e-15) skew.c[0] = 1e-15;
+	if (fabs (skew.c[3]) < 1e-15) skew.c[3] = 1e-15;
+	sp_sel_trans_transform (seltrans, &skew, &norm);
 }
 
 void
-sp_sel_trans_rotate (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_rotate (SPSelTrans *seltrans, SPSelTransHandle *handle, NRPointF *p, guint state)
 {
-        ArtPoint q1, q2, point, norm;
-	gdouble rotate[6], r1[6], r2[6], h1, h2;
+        NRPointF q1, q2, point, norm;
+	NRMatrixD rotate, r1, r2;
+	double h1, h2;
 
 	sp_sel_trans_point_desktop (seltrans, &point);
 	sp_sel_trans_origin_desktop (seltrans, &norm);
@@ -1178,15 +1211,15 @@ sp_sel_trans_rotate (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint
 	if (fabs (h2) < 1e-15) return;
 	q2.x = (p->x - norm.x) / h2;
 	q2.y = (p->y - norm.y) / h2;
-	r1[0] = q1.x;  r1[1] = -q1.y;  r1[2] =  q1.y;  r1[3] = q1.x;  r1[4] = 0;  r1[5] = 0;
-	r2[0] = q2.x;  r2[1] =  q2.y;  r2[2] = -q2.y;  r2[3] = q2.x;  r2[4] = 0;  r2[5] = 0;
+	r1.c[0] = q1.x;  r1.c[1] = -q1.y;  r1.c[2] =  q1.y;  r1.c[3] = q1.x;  r1.c[4] = 0;  r1.c[5] = 0;
+	r2.c[0] = q2.x;  r2.c[1] =  q2.y;  r2.c[2] = -q2.y;  r2.c[3] = q2.x;  r2.c[4] = 0;  r2.c[5] = 0;
 
-	art_affine_multiply (rotate, r1, r2);
-	sp_sel_trans_transform (seltrans, rotate, &norm);
+	nr_matrix_multiply_ddd (&rotate, &r1, &r2);
+	sp_sel_trans_transform (seltrans, &rotate, &norm);
 }
 
 void
-sp_sel_trans_center (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_center (SPSelTrans * seltrans, SPSelTransHandle * handle, NRPointF *p, guint state)
 {
 	sp_sel_trans_set_center (seltrans, p->x, p->y);
 }
