@@ -13,12 +13,20 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#define SP_TYPE_SODIPODI (sodipodi_get_type ())
+#define SP_SODIPODI(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_SODIPODI, Sodipodi))
+#define SP_SODIPODI_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_SODIPODI, SodipodiClass))
+#define SP_IS_SODIPODI(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_SODIPODI))
+#define SP_IS_SODIPODI_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_SODIPODI))
+
 #include "sodipodi.h"
 
-Sodipodi * sodipodi_new ();
+GType sodipodi_get_type (void);
 
-#define sodipodi_ref() gtk_object_ref (GTK_OBJECT (SODIPODI))
-#define sodipodi_unref() gtk_object_unref (GTK_OBJECT (SODIPODI))
+Sodipodi *sodipodi_new ();
+
+void sodipodi_ref (void);
+void sodipodi_unref (void);
 
 /*
  * These are meant solely for desktop, document etc. implementations
@@ -34,6 +42,8 @@ void sodipodi_activate_desktop (SPDesktop * desktop);
 void sodipodi_add_document (SPDocument *document);
 void sodipodi_remove_document (SPDocument *document);
 
+void sodipodi_set_color (SPColor *color, float opacity);
+
 #endif
 
 
