@@ -12,16 +12,18 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#define noSP_ANCHOR_VERBOSE
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmenuitem.h>
-#include <libgnome/gnome-url.h>
+
+#include "helper/sp-intl.h"
 #include "helper/sp-canvas.h"
 #include "dialogs/object-attributes.h"
+#include "attributes.h"
 #include "sp-anchor.h"
 
 /* fixme: This is insane, and should be removed */
@@ -32,33 +34,29 @@ static void sp_anchor_init (SPAnchor *anchor);
 
 static void sp_anchor_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_anchor_release (SPObject *object);
-static void sp_anchor_read_attr (SPObject * object, const gchar * attr);
+static void sp_anchor_set (SPObject *object, unsigned int key, const unsigned char *value);
 static SPRepr *sp_anchor_write (SPObject *object, SPRepr *repr, guint flags);
 
 static gchar *sp_anchor_description (SPItem *item);
 static gint sp_anchor_event (SPItem *item, SPEvent *event);
-static void sp_anchor_menu (SPItem *item, SPDesktop *desktop, GtkMenu *menu);
-
-static void sp_anchor_link_properties (GtkMenuItem *menuitem, SPAnchor *anchor);
-static void sp_anchor_link_follow (GtkMenuItem *menuitem, SPAnchor *anchor);
-static void sp_anchor_link_remove (GtkMenuItem *menuitem, SPAnchor *anchor);
 
 static SPGroupClass *parent_class;
 
-GtkType
+GType
 sp_anchor_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"SPAnchor",
-			sizeof (SPAnchor),
+		GTypeInfo info = {
 			sizeof (SPAnchorClass),
-			(GtkClassInitFunc) sp_anchor_class_init,
-			(GtkObjectInitFunc) sp_anchor_init,
-			NULL, NULL, NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_anchor_class_init,
+			NULL, NULL,
+			sizeof (SPAnchor),
+			16,
+			(GInstanceInitFunc) sp_anchor_init,
 		};
-		type = gtk_type_unique (SP_TYPE_GROUP, &info);
+		type = g_type_register_static (SP_TYPE_GROUP, "SPAnchor", &info, 0);
 	}
 	return type;
 }
@@ -66,24 +64,23 @@ sp_anchor_get_type (void)
 static void
 sp_anchor_class_init (SPAnchorClass *klass)
 {
-	GtkObjectClass *gtk_object_class;
+	GObjectClass *gobject_class;
 	SPObjectClass *sp_object_class;
 	SPItemClass *item_class;
 
-	gtk_object_class = GTK_OBJECT_CLASS (klass);
+	gobject_class = (GObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 	item_class = (SPItemClass *) klass;
 
-	parent_class = gtk_type_class (SP_TYPE_GROUP);
+	parent_class = g_type_class_ref (SP_TYPE_GROUP);
 
 	sp_object_class->build = sp_anchor_build;
 	sp_object_class->release = sp_anchor_release;
-	sp_object_class->read_attr = sp_anchor_read_attr;
+	sp_object_class->set = sp_anchor_set;
 	sp_object_class->write = sp_anchor_write;
 
 	item_class->description = sp_anchor_description;
 	item_class->event = sp_anchor_event;
-	item_class->menu = sp_anchor_menu;
 }
 
 static void
@@ -101,14 +98,14 @@ static void sp_anchor_build (SPObject *object, SPDocument * document, SPRepr * r
 	if (((SPObjectClass *) (parent_class))->build)
 		((SPObjectClass *) (parent_class))->build (object, document, repr);
 
-	sp_anchor_read_attr (object, "xlink:type");
-	sp_anchor_read_attr (object, "xlink:role");
-	sp_anchor_read_attr (object, "xlink:arcrole");
-	sp_anchor_read_attr (object, "xlink:title");
-	sp_anchor_read_attr (object, "xlink:show");
-	sp_anchor_read_attr (object, "xlink:actuate");
-	sp_anchor_read_attr (object, "xlink:href");
-	sp_anchor_read_attr (object, "target");
+	sp_object_read_attr (object, "xlink:type");
+	sp_object_read_attr (object, "xlink:role");
+	sp_object_read_attr (object, "xlink:arcrole");
+	sp_object_read_attr (object, "xlink:title");
+	sp_object_read_attr (object, "xlink:show");
+	sp_object_read_attr (object, "xlink:actuate");
+	sp_object_read_attr (object, "xlink:href");
+	sp_object_read_attr (object, "target");
 }
 
 static void
@@ -128,27 +125,31 @@ sp_anchor_release (SPObject *object)
 }
 
 static void
-sp_anchor_read_attr (SPObject *object, const gchar *key)
+sp_anchor_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
-	SPAnchor * anchor;
+	SPAnchor *anchor;
 
 	anchor = SP_ANCHOR (object);
 
-	if (!strcmp (key, "xlink:href")) {
+	switch (key) {
+	case SP_ATTR_XLINK_HREF:
 		if (anchor->href) g_free (anchor->href);
-		anchor->href = g_strdup (sp_repr_attr (SP_OBJECT_REPR (object), key));
+		anchor->href = g_strdup (value);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "xlink:type") ||
-		   !strcmp (key, "xlink:role") ||
-		   !strcmp (key, "xlink:arcrole") ||
-		   !strcmp (key, "xlink:title") ||
-		   !strcmp (key, "xlink:show") ||
-		   !strcmp (key, "xlink:actuate") ||
-		   !strcmp (key, "target")) {
+		break;
+	case SP_ATTR_XLINK_TYPE:
+	case SP_ATTR_XLINK_ROLE:
+	case SP_ATTR_XLINK_ARCROLE:
+	case SP_ATTR_XLINK_TITLE:
+	case SP_ATTR_XLINK_SHOW:
+	case SP_ATTR_XLINK_ACTUATE:
+	case SP_ATTR_TARGET:
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else {
-		if (((SPObjectClass *) (parent_class))->read_attr)
-			((SPObjectClass *) (parent_class))->read_attr (object, key);
+		break;
+	default:
+		if (((SPObjectClass *) (parent_class))->set)
+			((SPObjectClass *) (parent_class))->set (object, key, value);
+		break;
 	}
 }
 
@@ -192,7 +193,7 @@ sp_anchor_description (SPItem *item)
 
 	anchor = SP_ANCHOR (item);
 
-	snprintf (c, 128, _("Link to %s"), anchor->href);
+	g_snprintf (c, 128, _("Link to %s"), anchor->href);
 
 	return g_strdup (c);
 }
@@ -213,7 +214,9 @@ sp_anchor_event (SPItem *item, SPEvent *event)
 	case SP_EVENT_ACTIVATE:
 		if (anchor->href) {
 			g_print ("Activated xlink:href=\"%s\"\n", anchor->href);
-			gnome_url_show (anchor->href);
+#if 0
+			gnome_url_show (anchor->href, NULL);
+#endif
 			return TRUE;
 		}
 		break;
@@ -240,80 +243,3 @@ sp_anchor_event (SPItem *item, SPEvent *event)
 	return FALSE;
 }
 
-/* Generate context menu item section */
-
-static void
-sp_anchor_menu (SPItem *item, SPDesktop *desktop, GtkMenu *menu)
-{
-	GtkWidget *i, *m, *w;
-
-	if (SP_ITEM_CLASS (parent_class)->menu)
-		(* SP_ITEM_CLASS (parent_class)->menu) (item, desktop, menu);
-
-	/* Create toplevel menuitem */
-	i = gtk_menu_item_new_with_label (_("Link"));
-	m = gtk_menu_new ();
-	/* Link dialog */
-	w = gtk_menu_item_new_with_label (_("Link Properties"));
-	gtk_object_set_data (GTK_OBJECT (w), "desktop", desktop);
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC (sp_anchor_link_properties), item);
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-	/* Separator */
-	w = gtk_menu_item_new ();
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-	/* Select item */
-	w = gtk_menu_item_new_with_label (_("Follow link"));
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC (sp_anchor_link_follow), item);
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-	/* Reset transformations */
-	w = gtk_menu_item_new_with_label (_("Remove link"));
-	gtk_object_set_data (GTK_OBJECT (w), "desktop", desktop);
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC (sp_anchor_link_remove), item);
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-	/* Show menu */
-	gtk_widget_show (m);
-
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), m);
-
-	gtk_menu_append (menu, i);
-	gtk_widget_show (i);
-}
-
-static void
-sp_anchor_link_properties (GtkMenuItem *menuitem, SPAnchor *anchor)
-{
-	sp_object_attributes_dialog (SP_OBJECT (anchor), "SPAnchor");
-}
-
-static void
-sp_anchor_link_follow (GtkMenuItem *menuitem, SPAnchor *anchor)
-{
-	g_return_if_fail (anchor != NULL);
-	g_return_if_fail (SP_IS_ANCHOR (anchor));
-
-	if (anchor->href) {
-		gnome_url_show (anchor->href);
-	}
-}
-
-static void
-sp_anchor_link_remove (GtkMenuItem *menuitem, SPAnchor *anchor)
-{
-	GSList *children;
-
-	g_return_if_fail (anchor != NULL);
-	g_return_if_fail (SP_IS_ANCHOR (anchor));
-
-	children = NULL;
-	sp_item_group_ungroup (SP_GROUP (anchor), &children);
-
-#if 0
-	sp_selection_set_item_list (SP_DT_SELECTION (desktop), children);
-#endif
-	g_slist_free (children);
-}
-
