@@ -13,10 +13,19 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifndef _USE_MATH_DEFINES
+#define _USE_MATH_DEFINES
+#endif
 #include <math.h>
+
 #include <string.h>
-#include "helper/art-utils.h"
+
 #include "svg/svg.h"
+#include "attributes.h"
 #include "style.h"
 /* For versioning */
 #include "sp-root.h"
@@ -27,14 +36,18 @@
 
 #define noELLIPSE_VERBOSE
 
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
 #define SP_2PI (2 * M_PI)
 
 #if 1
 /* Hmmm... shouldn't this also qualify */
-/* Wheter it is faster or not, well, nobody knows */
-#define sp_round(v,m) (((v) < 0.0) ? (ceil ((v) / (m) - 0.5)) * (m) : (floor ((v) / (m) + 0.5)))
+/* Whether it is faster or not, well, nobody knows */
+#define sp_round(v,m) (((v) < 0.0) ? ((ceil ((v) / (m) - 0.5)) * (m)) : ((floor ((v) / (m) + 0.5)) * (m)))
 #else
-/* we does not use C99 round(3) function yet */
+/* we do not use C99 round(3) function yet */
 static double sp_round (double x, double y)
 {
 	double remain;
@@ -55,31 +68,31 @@ static double sp_round (double x, double y)
 static void sp_genericellipse_class_init (SPGenericEllipseClass *klass);
 static void sp_genericellipse_init (SPGenericEllipse *ellipse);
 
-static void sp_genericellipse_build (SPObject *object, SPDocument *document, SPRepr *repr);
-static void sp_genericellipse_read_attr (SPObject * object, const gchar * attr);
+static void sp_genericellipse_update (SPObject *object, SPCtx *ctx, guint flags);
 
-static GSList *sp_genericellipse_snappoints (SPItem *item, GSList *points);
+static int sp_genericellipse_snappoints (SPItem *item, NRPointF *p, int size);
 
-static void sp_genericellipse_glue_set_shape (SPShape *shape);
-static void sp_genericellipse_set_shape (SPGenericEllipse *ellipse);
-static void sp_genericellipse_normalize (SPGenericEllipse *ellipse);
+static void sp_genericellipse_set_shape (SPShape *shape);
 
 static SPShapeClass *ge_parent_class;
 
-GtkType
+GType
 sp_genericellipse_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"SPGenericEllipse",
-			sizeof (SPGenericEllipse),
+		GTypeInfo info = {
 			sizeof (SPGenericEllipseClass),
-			(GtkClassInitFunc) sp_genericellipse_class_init,
-			(GtkObjectInitFunc) sp_genericellipse_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_genericellipse_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPGenericEllipse),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_genericellipse_init,
 		};
-		type = gtk_type_unique (sp_shape_get_type (), &info);
+		type = g_type_register_static (SP_TYPE_SHAPE, "SPGenericEllipse", &info, 0);
 	}
 	return type;
 }
@@ -87,31 +100,28 @@ sp_genericellipse_get_type (void)
 static void
 sp_genericellipse_class_init (SPGenericEllipseClass *klass)
 {
-	GtkObjectClass *gtk_object_class;
+	GObjectClass *gobject_class;
 	SPObjectClass * sp_object_class;
 	SPItemClass *item_class;
 	SPShapeClass *shape_class;
 
-	gtk_object_class = (GtkObjectClass *) klass;
+	gobject_class = (GObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 	item_class = (SPItemClass *) klass;
 	shape_class = (SPShapeClass *) klass;
 
-	ge_parent_class = gtk_type_class (sp_shape_get_type ());
+	ge_parent_class = g_type_class_ref (SP_TYPE_SHAPE);
 
-	sp_object_class->build = sp_genericellipse_build;
-	sp_object_class->read_attr = sp_genericellipse_read_attr;
+	sp_object_class->update = sp_genericellipse_update;
 
 	item_class->snappoints = sp_genericellipse_snappoints;
 
-	shape_class->set_shape = sp_genericellipse_glue_set_shape;
+	shape_class->set_shape = sp_genericellipse_set_shape;
 }
 
 static void
 sp_genericellipse_init (SPGenericEllipse *ellipse)
 {
-	ellipse->shape.path.independent = FALSE;
-
 	sp_svg_length_unset (&ellipse->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 	sp_svg_length_unset (&ellipse->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
 	sp_svg_length_unset (&ellipse->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
@@ -123,85 +133,39 @@ sp_genericellipse_init (SPGenericEllipse *ellipse)
 }
 
 static void
-sp_genericellipse_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_genericellipse_update (SPObject *object, SPCtx *ctx, guint flags)
 {
-	if (((SPObjectClass *) ge_parent_class)->build)
-		(* ((SPObjectClass *) ge_parent_class)->build) (object, document, repr);
-}
-
-static void
-sp_genericellipse_read_attr (SPObject * object, const gchar * attr)
-{
-	SPGenericEllipse *ellipse;
-	const gchar *str;
-
-	ellipse = SP_GENERICELLIPSE (object);
-
-#ifdef ELLIPSE_VERBOSE
-	g_print ("sp_ellipse_read_attr: attr %s\n", attr);
-#endif
-
-	str = sp_repr_attr (object->repr, attr);
-
-	if (((SPObjectClass *) ge_parent_class)->read_attr)
-		(* ((SPObjectClass *) ge_parent_class)->read_attr) (object, attr);
-}
-
-static void sp_genericellipse_glue_set_shape (SPShape *shape)
-{
-	SPGenericEllipse *ge;
-
-	ge = SP_GENERICELLIPSE (shape);
-
-	sp_genericellipse_set_shape (ge);
-}
-
-static void
-sp_genericellipse_update_length (SPSVGLength *length, gdouble em, gdouble ex, gdouble scale)
-{
-	if (length->unit == SP_SVG_UNIT_EM) {
-		length->computed = length->value * em;
-	} else if (length->unit == SP_SVG_UNIT_EX) {
-		length->computed = length->value * ex;
-	} else if (length->unit == SP_SVG_UNIT_PERCENT) {
-		length->computed = length->value * scale;
+	if (flags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG)) {
+		SPGenericEllipse *ellipse;
+		SPStyle *style;
+		double d;
+		ellipse = (SPGenericEllipse *) object;
+		style = object->style;
+		d = 1.0 / NR_MATRIX_DF_EXPANSION (&((SPItemCtx *) ctx)->i2vp);
+		sp_svg_length_update (&ellipse->cx, style->font_size.computed, style->font_size.computed * 0.5, d);
+		sp_svg_length_update (&ellipse->cy, style->font_size.computed, style->font_size.computed * 0.5, d);
+		sp_svg_length_update (&ellipse->rx, style->font_size.computed, style->font_size.computed * 0.5, d);
+		sp_svg_length_update (&ellipse->ry, style->font_size.computed, style->font_size.computed * 0.5, d);
+		sp_shape_set_shape ((SPShape *) object);
 	}
+
+	if (((SPObjectClass *) ge_parent_class)->update)
+		((SPObjectClass *) ge_parent_class)->update (object, ctx, flags);
 }
 
-static void
-sp_genericellipse_compute_values (SPGenericEllipse *ellipse)
-{
-	SPStyle *style;
-	gdouble i2vp[6], vp2i[6];
-	gdouble aw, ah;
-	gdouble d;
+#define noC1 0.552
 
-	style = SP_OBJECT_STYLE (ellipse);
-
-	/* fixme: It is somewhat dangerous, yes (Lauris) */
-	/* fixme: And it is terribly slow too (Lauris) */
-	/* fixme: In general we want to keep viewport scales around */
-	sp_item_i2vp_affine (SP_ITEM (ellipse), i2vp);
-	art_affine_invert (vp2i, i2vp);
-	aw = sp_distance_d_matrix_d_transform (1.0, vp2i);
-	ah = sp_distance_d_matrix_d_transform (1.0, vp2i);
-	/* sqrt ((actual_width) ** 2 + (actual_height) ** 2)) / sqrt (2) */
-	d = sqrt (aw * aw + ah * ah) * M_SQRT1_2;
-
-	sp_genericellipse_update_length (&ellipse->cx, style->font_size.computed, style->font_size.computed * 0.5, d);
-	sp_genericellipse_update_length (&ellipse->cy, style->font_size.computed, style->font_size.computed * 0.5, d);
-	sp_genericellipse_update_length (&ellipse->rx, style->font_size.computed, style->font_size.computed * 0.5, d);
-	sp_genericellipse_update_length (&ellipse->ry, style->font_size.computed, style->font_size.computed * 0.5, d);
-}
-
-#define C1 0.552
+#define C1 (4.0/3.0*(sqrt(2.0)-1.0))
 
 /* fixme: Think (Lauris) */
 
-static void sp_genericellipse_set_shape (SPGenericEllipse *ellipse)
+#include <libart_lgpl/art_misc.h>
+#include <libart_lgpl/art_bpath.h>
+
+static void sp_genericellipse_set_shape (SPShape *shape)
 {
-	SPPath * path;
-	SPCurve * c;
+	SPGenericEllipse *ellipse;
+	SPCurve *c;
 	ArtBpath bpath[16], * abp;
 
 	double cx, cy, rx, ry, s, e;
@@ -211,13 +175,7 @@ static void sp_genericellipse_set_shape (SPGenericEllipse *ellipse)
 	gint slice = FALSE;
 	gint i;
 
-	path = SP_PATH (ellipse);
-
-	sp_path_clear (SP_PATH (ellipse));
-
-	/* fixme: Maybe track, whether we have em,ex,% (Lauris) */
-	/* fixme: Alternately we can use ::modified to keep everything up-to-date (Lauris) */
-	sp_genericellipse_compute_values (ellipse);
+	ellipse = (SPGenericEllipse *) shape;
 
 	if ((ellipse->rx.computed < 1e-18) || (ellipse->ry.computed < 1e-18)) return;
 	if (fabs (ellipse->end - ellipse->start) < 1e-9) return;
@@ -237,7 +195,7 @@ static void sp_genericellipse_set_shape (SPGenericEllipse *ellipse)
 		slice = TRUE;
 	}
 
-	art_affine_scale (affine, rx, ry);
+	nr_matrix_d_set_scale (NR_MATRIX_D_FROM_DOUBLE (affine), rx, ry);
 	affine[4] = ellipse->cx.computed;
 	affine[5] = ellipse->cy.computed;
 
@@ -297,35 +255,34 @@ g_print ("step %d s %f e %f coords %f %f %f %f %f %f\n",
 	abp = art_bpath_affine_transform (bpath, affine);
 
 	c = sp_curve_new_from_bpath (abp);
-	sp_path_add_bpath (SP_PATH (ellipse), c, TRUE, NULL);
+	sp_shape_set_curve_insync ((SPShape *) ellipse, c, TRUE);
 	sp_curve_unref (c);
-
-	sp_object_request_modified (SP_OBJECT (ellipse), SP_OBJECT_MODIFIED_FLAG);
 }
 
-static GSList * 
-sp_genericellipse_snappoints (SPItem *item, GSList *points)
+static int
+sp_genericellipse_snappoints (SPItem *item, NRPointF *p, int size)
 {
 	SPGenericEllipse *ge;
-	ArtPoint *p;
-	gdouble affine[6];
+	NRMatrixF i2d;
+	int pos;
 
 	ge = SP_GENERICELLIPSE (item);
 
 	/* we use corners of item and center of ellipse */
-	if (SP_ITEM_CLASS (ge_parent_class)->snappoints)
-		points = (SP_ITEM_CLASS (ge_parent_class)->snappoints) (item, points);
+	pos = 0;
+	if (((SPItemClass *) ge_parent_class)->snappoints)
+		pos = ((SPItemClass *) ge_parent_class)->snappoints (item, p, size);
 
-	p = g_new (ArtPoint,1);
-	p->x = ge->cx.computed;
-	p->y = ge->cy.computed;
-	sp_item_i2d_affine (item, affine);
-	art_affine_point (p, p, affine);
-	points = g_slist_append (points, p);
-	return points;
+	if (pos < size) {
+		sp_item_i2d_affine (item, &i2d);
+		p[pos].x = NR_MATRIX_DF_TRANSFORM_X (&i2d, ge->cx.computed, ge->cy.computed);
+		p[pos].y = NR_MATRIX_DF_TRANSFORM_Y (&i2d, ge->cx.computed, ge->cy.computed);
+	}
+
+	return pos;
 }
 
-static void
+void
 sp_genericellipse_normalize (SPGenericEllipse *ellipse)
 {
 	double diff;
@@ -342,27 +299,6 @@ sp_genericellipse_normalize (SPGenericEllipse *ellipse)
 	/* Now we keep: 0 <= start < end <= 2*PI */
 }
 
-/*
- * return values:
- *   1  : inside
- *   0  : on the curves
- *   -1 : outside
- */
-static gint
-sp_genericellipse_side (SPGenericEllipse *ellipse, const ArtPoint *p)
-{
-	gdouble dx, dy;
-	gdouble s;
-
-	dx = p->x - ellipse->cx.computed;
-	dy = p->y - ellipse->cy.computed;
-
-	s = dx * dx / (ellipse->rx.computed * ellipse->rx.computed) + dy * dy / (ellipse->ry.computed * ellipse->ry.computed);
-	if (s < 1.0) return 1;
-	if (s > 1.0) return -1;
-	return 0;
-}
-
 /* SVG <ellipse> element */
 
 static void sp_ellipse_class_init (SPEllipseClass *class);
@@ -370,25 +306,28 @@ static void sp_ellipse_init (SPEllipse *ellipse);
 
 static void sp_ellipse_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static SPRepr *sp_ellipse_write (SPObject *object, SPRepr *repr, guint flags);
-static void sp_ellipse_read_attr (SPObject * object, const gchar * attr);
+static void sp_ellipse_set (SPObject *object, unsigned int key, const unsigned char *value);
 static gchar * sp_ellipse_description (SPItem * item);
 
 static SPGenericEllipseClass *ellipse_parent_class;
 
-GtkType
+GType
 sp_ellipse_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"SPEllipse",
-			sizeof (SPEllipse),
+		GTypeInfo info = {
 			sizeof (SPEllipseClass),
-			(GtkClassInitFunc) sp_ellipse_class_init,
-			(GtkObjectInitFunc) sp_ellipse_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_ellipse_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPEllipse),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_ellipse_init,
 		};
-		type = gtk_type_unique (SP_TYPE_GENERICELLIPSE, &info);
+		type = g_type_register_static (SP_TYPE_GENERICELLIPSE, "SPEllipse", &info, 0);
 	}
 	return type;
 }
@@ -396,19 +335,19 @@ sp_ellipse_get_type (void)
 static void
 sp_ellipse_class_init (SPEllipseClass *class)
 {
-	GtkObjectClass * gtk_object_class;
+	GObjectClass * gobject_class;
 	SPObjectClass * sp_object_class;
 	SPItemClass *item_class;
 
-	gtk_object_class = (GtkObjectClass *) class;
+	gobject_class = (GObjectClass *) class;
 	sp_object_class = (SPObjectClass *) class;
 	item_class = (SPItemClass *) class;
 
-	ellipse_parent_class = gtk_type_class (SP_TYPE_GENERICELLIPSE);
+	ellipse_parent_class = g_type_class_ref (SP_TYPE_GENERICELLIPSE);
 
 	sp_object_class->build = sp_ellipse_build;
 	sp_object_class->write = sp_ellipse_write;
-	sp_object_class->read_attr = sp_ellipse_read_attr;
+	sp_object_class->set = sp_ellipse_set;
 
 	item_class->description = sp_ellipse_description;
 }
@@ -425,10 +364,10 @@ sp_ellipse_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	if (((SPObjectClass *) ellipse_parent_class)->build)
 		(* ((SPObjectClass *) ellipse_parent_class)->build) (object, document, repr);
 
-	sp_ellipse_read_attr (object, "cx");
-	sp_ellipse_read_attr (object, "cy");
-	sp_ellipse_read_attr (object, "rx");
-	sp_ellipse_read_attr (object, "ry");
+	sp_object_read_attr (object, "cx");
+	sp_object_read_attr (object, "cy");
+	sp_object_read_attr (object, "rx");
+	sp_object_read_attr (object, "ry");
 }
 
 static SPRepr *
@@ -454,53 +393,41 @@ sp_ellipse_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 static void
-sp_ellipse_read_attr (SPObject * object, const gchar * attr)
+sp_ellipse_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
 	SPGenericEllipse *ellipse;
-	const gchar *str;
-	gulong unit;
 
 	ellipse = SP_GENERICELLIPSE (object);
 
-#ifdef ELLIPSE_VERBOSE
-	g_print ("sp_ellipse_read_attr: attr %s\n", attr);
-#endif
-
-	str = sp_repr_attr (object->repr, attr);
-
-	if (!strcmp (attr, "cx")) {
-		if (!sp_svg_length_read (str, &ellipse->cx)) {
-			ellipse->cx.set = FALSE;
-			ellipse->cx.computed = 0.0;
+	switch (key) {
+	case SP_ATTR_CX:
+		if (!sp_svg_length_read (value, &ellipse->cx)) {
+			sp_svg_length_unset (&ellipse->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else if (!strcmp (attr, "cy")) {
-		if (!sp_svg_length_read (str, &ellipse->cy)) {
-			ellipse->cy.set = FALSE;
-			ellipse->cy.computed = 0.0;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_CY:
+		if (!sp_svg_length_read (value, &ellipse->cy)) {
+			sp_svg_length_unset (&ellipse->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else if (!strcmp (attr, "rx")) {
-		if (sp_svg_length_read_lff (str, &unit, &ellipse->rx.value, &ellipse->rx.computed) && (ellipse->rx.value > 0.0)) {
-			ellipse->rx.set = TRUE;
-			ellipse->rx.unit = unit;
-		} else {
-			ellipse->rx.set = FALSE;
-			ellipse->rx.computed = 0.0;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_RX:
+		if (!sp_svg_length_read (value, &ellipse->rx) || (ellipse->rx.value <= 0.0)) {
+			sp_svg_length_unset (&ellipse->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else if (!strcmp (attr, "ry")) {
-		if (sp_svg_length_read_lff (str, &unit, &ellipse->ry.value, &ellipse->ry.computed) && (ellipse->ry.value > 0.0)) {
-			ellipse->ry.set = TRUE;
-			ellipse->ry.unit = unit;
-		} else {
-			ellipse->ry.set = FALSE;
-			ellipse->ry.computed = 0.0;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_RY:
+		if (!sp_svg_length_read (value, &ellipse->ry) || (ellipse->ry.value <= 0.0)) {
+			sp_svg_length_unset (&ellipse->ry, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else {
-		if (((SPObjectClass *) ellipse_parent_class)->read_attr)
-			(* ((SPObjectClass *) ellipse_parent_class)->read_attr) (object, attr);
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	default:
+		if (((SPObjectClass *) ellipse_parent_class)->set)
+			((SPObjectClass *) ellipse_parent_class)->set (object, key, value);
+		break;
 	}
 }
 
@@ -512,7 +439,7 @@ sp_ellipse_description (SPItem * item)
 
 
 void
-sp_ellipse_set (SPEllipse *ellipse, gdouble x, gdouble y, gdouble rx, gdouble ry)
+sp_ellipse_position_set (SPEllipse *ellipse, gdouble x, gdouble y, gdouble rx, gdouble ry)
 {
 	SPGenericEllipse *ge;
 
@@ -526,7 +453,7 @@ sp_ellipse_set (SPEllipse *ellipse, gdouble x, gdouble y, gdouble rx, gdouble ry
 	ge->rx.computed = rx;
 	ge->ry.computed = ry;
 
-	sp_genericellipse_set_shape (ge);
+	sp_object_request_update ((SPObject *) ge, SP_OBJECT_MODIFIED_FLAG);
 }
 
 /* SVG <circle> element */
@@ -536,25 +463,28 @@ static void sp_circle_init (SPCircle *circle);
 
 static void sp_circle_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static SPRepr *sp_circle_write (SPObject *object, SPRepr *repr, guint flags);
-static void sp_circle_read_attr (SPObject * object, const gchar * attr);
+static void sp_circle_set (SPObject *object, unsigned int key, const unsigned char *value);
 static gchar * sp_circle_description (SPItem * item);
 
 static SPGenericEllipseClass *circle_parent_class;
 
-GtkType
+GType
 sp_circle_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"SPCircle",
-			sizeof (SPCircle),
+		GTypeInfo info = {
 			sizeof (SPCircleClass),
-			(GtkClassInitFunc) sp_circle_class_init,
-			(GtkObjectInitFunc) sp_circle_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_circle_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPCircle),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_circle_init,
 		};
-		type = gtk_type_unique (SP_TYPE_GENERICELLIPSE, &info);
+		type = g_type_register_static (SP_TYPE_GENERICELLIPSE, "SPCircle", &info, 0);
 	}
 	return type;
 }
@@ -562,19 +492,19 @@ sp_circle_get_type (void)
 static void
 sp_circle_class_init (SPCircleClass *class)
 {
-	GtkObjectClass * gtk_object_class;
+	GObjectClass * gobject_class;
 	SPObjectClass * sp_object_class;
 	SPItemClass *item_class;
 
-	gtk_object_class = (GtkObjectClass *) class;
+	gobject_class = (GObjectClass *) class;
 	sp_object_class = (SPObjectClass *) class;
 	item_class = (SPItemClass *) class;
 
-	circle_parent_class = gtk_type_class (SP_TYPE_GENERICELLIPSE);
+	circle_parent_class = g_type_class_ref (SP_TYPE_GENERICELLIPSE);
 
 	sp_object_class->build = sp_circle_build;
 	sp_object_class->write = sp_circle_write;
-	sp_object_class->read_attr = sp_circle_read_attr;
+	sp_object_class->set = sp_circle_set;
 
 	item_class->description = sp_circle_description;
 }
@@ -591,9 +521,9 @@ sp_circle_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	if (((SPObjectClass *) circle_parent_class)->build)
 		(* ((SPObjectClass *) circle_parent_class)->build) (object, document, repr);
 
-	sp_circle_read_attr (object, "cx");
-	sp_circle_read_attr (object, "cy");
-	sp_circle_read_attr (object, "r");
+	sp_object_read_attr (object, "cx");
+	sp_object_read_attr (object, "cy");
+	sp_object_read_attr (object, "r");
 }
 
 static SPRepr *
@@ -618,45 +548,36 @@ sp_circle_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 static void
-sp_circle_read_attr (SPObject *object, const gchar * attr)
+sp_circle_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
-	SPGenericEllipse *circle;
-	const gchar *str;
-	gulong unit;
+	SPGenericEllipse *ge;
 
-	circle = SP_GENERICELLIPSE (object);
+	ge = SP_GENERICELLIPSE (object);
 
-#ifdef CIRCLE_VERBOSE
-	g_print ("sp_circle_read_attr: attr %s\n", attr);
-#endif
-
-	str = sp_repr_attr (object->repr, attr);
-
-	if (!strcmp (attr, "cx")) {
-		if (!sp_svg_length_read (str, &circle->cx)) {
-			circle->cx.set = FALSE;
-			circle->cx.computed = 0.0;
+	switch (key) {
+	case SP_ATTR_CX:
+		if (!sp_svg_length_read (value, &ge->cx)) {
+			sp_svg_length_unset (&ge->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (circle);
-	} else if (!strcmp (attr, "cy")) {
-		if (!sp_svg_length_read (str, &circle->cy)) {
-			circle->cy.set = FALSE;
-			circle->cy.computed = 0.0;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_CY:
+		if (!sp_svg_length_read (value, &ge->cy)) {
+			sp_svg_length_unset (&ge->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (circle);
-	} if (!strcmp (attr, "r")) {
-		if (sp_svg_length_read_lff (str, &unit, &circle->rx.value, &circle->rx.computed) && (circle->rx.value > 0.0)) {
-			circle->rx.set = TRUE;
-			circle->rx.unit = unit;
-		} else {
-			circle->rx.set = FALSE;
-			circle->rx.computed = 0.0;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_R:
+		if (!sp_svg_length_read (value, &ge->rx) || (ge->rx.value <= 0.0)) {
+			sp_svg_length_unset (&ge->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		circle->ry.computed = circle->rx.computed;
-		sp_genericellipse_set_shape (circle);
-	} else {
-		if (((SPObjectClass *) circle_parent_class)->read_attr)
-			(* ((SPObjectClass *) circle_parent_class)->read_attr) (object, attr);
+		ge->ry = ge->rx;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	default:
+		if (((SPObjectClass *) circle_parent_class)->set)
+			((SPObjectClass *) circle_parent_class)->set (object, key, value);
+		break;
 	}
 }
 
@@ -673,28 +594,30 @@ static void sp_arc_init (SPArc *arc);
 
 static void sp_arc_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static SPRepr *sp_arc_write (SPObject *object, SPRepr *repr, guint flags);
-static void sp_arc_read_attr (SPObject *object, const gchar *attr);
+static void sp_arc_set (SPObject *object, unsigned int key, const unsigned char *value);
 static void sp_arc_modified (SPObject *object, guint flags);
 
 static gchar * sp_arc_description (SPItem * item);
-static SPKnotHolder *sp_arc_knot_holder (SPItem * item, SPDesktop *desktop);
 
 static SPGenericEllipseClass *arc_parent_class;
 
-GtkType
+GType
 sp_arc_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"SPArc",
-			sizeof (SPArc),
+		GTypeInfo info = {
 			sizeof (SPArcClass),
-			(GtkClassInitFunc) sp_arc_class_init,
-			(GtkObjectInitFunc) sp_arc_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_arc_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPArc),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_arc_init,
 		};
-		type = gtk_type_unique (SP_TYPE_GENERICELLIPSE, &info);
+		type = g_type_register_static (SP_TYPE_GENERICELLIPSE, "SPArc", &info, 0);
 	}
 	return type;
 }
@@ -702,23 +625,22 @@ sp_arc_get_type (void)
 static void
 sp_arc_class_init (SPArcClass *class)
 {
-	GtkObjectClass * gtk_object_class;
+	GObjectClass * gobject_class;
 	SPObjectClass * sp_object_class;
 	SPItemClass *item_class;
 
-	gtk_object_class = (GtkObjectClass *) class;
+	gobject_class = (GObjectClass *) class;
 	sp_object_class = (SPObjectClass *) class;
 	item_class = (SPItemClass *) class;
 
-	arc_parent_class = gtk_type_class (SP_TYPE_GENERICELLIPSE);
+	arc_parent_class = g_type_class_ref (SP_TYPE_GENERICELLIPSE);
 
 	sp_object_class->build = sp_arc_build;
 	sp_object_class->write = sp_arc_write;
-	sp_object_class->read_attr = sp_arc_read_attr;
+	sp_object_class->set = sp_arc_set;
 	sp_object_class->modified = sp_arc_modified;
 
 	item_class->description = sp_arc_description;
-	item_class->knot_holder = sp_arc_knot_holder;
 }
 
 static void
@@ -755,21 +677,21 @@ sp_arc_build (SPObject *object, SPDocument *document, SPRepr *repr)
 
 	if (version < 25) {
 		/* Old spec violating arc attributes */
-		sp_arc_read_attr (object, "cx");
-		sp_arc_read_attr (object, "cy");
-		sp_arc_read_attr (object, "rx");
-		sp_arc_read_attr (object, "ry");
+		sp_object_read_attr (object, "cx");
+		sp_object_read_attr (object, "cy");
+		sp_object_read_attr (object, "rx");
+		sp_object_read_attr (object, "ry");
 	} else {
 		/* New attributes */
-		sp_arc_read_attr (object, "sodipodi:cx");
-		sp_arc_read_attr (object, "sodipodi:cy");
-		sp_arc_read_attr (object, "sodipodi:rx");
-		sp_arc_read_attr (object, "sodipodi:ry");
+		sp_object_read_attr (object, "sodipodi:cx");
+		sp_object_read_attr (object, "sodipodi:cy");
+		sp_object_read_attr (object, "sodipodi:rx");
+		sp_object_read_attr (object, "sodipodi:ry");
 	}
 
-	sp_arc_read_attr (object, "sodipodi:start");
-	sp_arc_read_attr (object, "sodipodi:end");
-	sp_arc_read_attr (object, "sodipodi:open");
+	sp_object_read_attr (object, "sodipodi:start");
+	sp_object_read_attr (object, "sodipodi:end");
+	sp_object_read_attr (object, "sodipodi:open");
 
 	if (version < 25) {
 		/* fixme: I am 99.9% sure we can do this here safely, but check nevertheless (Lauris) */
@@ -789,15 +711,17 @@ sp_arc_build (SPObject *object, SPDocument *document, SPRepr *repr)
  * See SVG 1.0 Specification W3C Recommendation
  * ``F.6 Ellptical arc implementation notes'' for more detail.
  */
+
+#define ARC_BUFSIZE 1024
+
 static gboolean
 sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 {
-#define ARC_BUFSIZE 256
 	SPGenericEllipse *ge;
-	ArtPoint p1, p2;
+	NRPointF p1, p2;
 	gint fa, fs;
 	gdouble  dt;
-	gchar c[ARC_BUFSIZE];
+	unsigned char c[ARC_BUFSIZE];
 
 	ge = SP_GENERICELLIPSE (arc);
 
@@ -806,17 +730,59 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 
 	dt = fmod (ge->end - ge->start, SP_2PI);
 	if (fabs (dt) < 1e-6) {
-		ArtPoint ph;
+		NRPointF ph;
+		int pos = 0;
 		sp_arc_get_xy (arc, (ge->start + ge->end) / 2.0, &ph);
-		g_snprintf (c, ARC_BUFSIZE, "M %f %f A %f %f 0 %d %d %f,%f A %g %g 0 %d %d %g %g L %f %f z",
-			    p1.x, p1.y,
-			    ge->rx.computed, ge->ry.computed,
-			    1, (dt > 0),
-			    ph.x, ph.y,
-			    ge->rx.computed, ge->ry.computed,
-			    1, (dt > 0),
-			    p2.x, p2.y,
-			    ge->cx.computed, ge->cy.computed);
+		c[pos++] = 'M';
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, p1.x, 6, 0, FALSE);
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, p1.y, 6, 0, FALSE);
+		c[pos++] = ' ';
+		c[pos++] = 'A';
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ge->rx.computed, 6, 0, FALSE);
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ge->ry.computed, 6, 0, FALSE);
+		c[pos++] = ' ';
+		c[pos++] = '0';
+		c[pos++] = ' ';
+		c[pos++] = '1';
+		c[pos++] = ' ';
+		c[pos++] = (dt > 0) ? '1' : 0;
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ph.x, 6, 0, FALSE);
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ph.y, 6, 0, FALSE);
+		c[pos++] = ' ';
+		c[pos++] = 'A';
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ge->rx.computed, 6, 0, FALSE);
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ge->ry.computed, 6, 0, FALSE);
+		c[pos++] = ' ';
+		c[pos++] = '0';
+		c[pos++] = ' ';
+		c[pos++] = '1';
+		c[pos++] = ' ';
+		c[pos++] = (dt > 0) ? '1' : 0;
+		c[pos++] = ' ';
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, p2.x, 6, 0, FALSE);
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, p2.y, 6, 0, FALSE);
+
+#if 0
+		c[pos++] = ' ';
+		c[pos++] = 'L';
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ge->cx.computed, 6, 0, FALSE);
+		c[pos++] = ' ';
+		pos += sp_svg_number_write_d (c + pos, ge->cy.computed, 6, 0, FALSE);
+#endif
+		c[pos++] = ' ';
+		c[pos++] = 'z';
+		c[pos++] = '\0';
 	} else {
 		fa = (fabs (dt) > M_PI) ? 1 : 0;
 		fs = (dt > 0) ? 1 : 0;
@@ -824,21 +790,66 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 		g_print ("start:%g end:%g fa=%d fs=%d\n", ge->start, ge->end, fa, fs);
 #endif
 		if (ge->closed) {
-			g_snprintf (c, ARC_BUFSIZE, "M %f,%f A %f,%f 0 %d %d %f,%f L %f,%f z",
-				    p1.x, p1.y,
-				    ge->rx.computed, ge->ry.computed,
-				    fa, fs,
-				    p2.x, p2.y,
-				    ge->cx.computed, ge->cy.computed);
+			int pos = 0;
+			c[pos++] = 'M';
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p1.x, 6, 0, FALSE);
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p1.y, 6, 0, FALSE);
+			
+			c[pos++] = ' ';
+			c[pos++] = 'A';
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, ge->rx.computed, 6, 0, FALSE);
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, ge->ry.computed, 6, 0, FALSE);
+			c[pos++] = ' ';
+			c[pos++] = '0';
+			c[pos++] = ' ';
+			c[pos++] = (fabs (dt) > M_PI) ? 1 : 0;
+			c[pos++] = ' ';
+			c[pos++] = (dt > 0) ? '1' : 0;
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p2.x, 6, 0, FALSE);
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p2.y, 6, 0, FALSE);
+
+			c[pos++] = ' ';
+			c[pos++] = 'L';
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, ge->cx.computed, 6, 0, FALSE);
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, ge->cy.computed, 6, 0, FALSE);
+			c[pos++] = 'z';
+			c[pos++] = '\0';
 		} else {
-			g_snprintf (c, ARC_BUFSIZE, "M %f,%f A %f,%f 0 %d %d %f,%f",
-				    p1.x, p1.y,
-				    ge->rx.computed, ge->ry.computed,
-				    fa, fs, p2.x, p2.y);
+			int pos = 0;
+			c[pos++] = 'M';
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p1.x, 6, 0, FALSE);
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p1.y, 6, 0, FALSE);
+			
+			c[pos++] = ' ';
+			c[pos++] = 'A';
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, ge->rx.computed, 6, 0, FALSE);
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, ge->ry.computed, 6, 0, FALSE);
+			c[pos++] = ' ';
+			c[pos++] = '0';
+			c[pos++] = ' ';
+			c[pos++] = (fabs (dt) > M_PI) ? 1 : 0;
+			c[pos++] = ' ';
+			c[pos++] = (dt > 0) ? '1' : 0;
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p2.x, 6, 0, FALSE);
+			c[pos++] = ' ';
+			pos += sp_svg_number_write_d (c + pos, p2.y, 6, 0, FALSE);
+			c[pos++] = '\0';
 		}
 	}
 
-
 	return sp_repr_set_attr (repr, "d", c);
 }
 
@@ -892,71 +903,73 @@ sp_arc_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 static void
-sp_arc_read_attr (SPObject *object, const gchar *attr)
+sp_arc_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
-	SPGenericEllipse *ellipse;
-	const gchar *str;
+	SPGenericEllipse *ge;
 	guint version;
-	gulong unit;
-	double n;
 
-	ellipse = SP_GENERICELLIPSE (object);
+	ge = SP_GENERICELLIPSE (object);
 	
-#ifdef ARC_VERBOSE
-	g_print ("sp_arc_read_attr: attr %s\n", attr);
-#endif
-
-	str = sp_repr_attr (object->repr, attr);
-
 	version = sp_arc_find_version (object);
 
-	if (((version < 25) && !strcmp (attr, "cx")) || ((version >= 25) && !strcmp (attr, "sodipodi:cx"))) {
-		if (!sp_svg_length_read (str, &ellipse->cx)) {
-			ellipse->cx.set = FALSE;
-			ellipse->cx.computed = 0.0;
+	if (version < 25) {
+		switch (key) {
+		case SP_ATTR_CX:
+			key = SP_ATTR_SODIPODI_CX;
+			break;
+		case SP_ATTR_CY:
+			key = SP_ATTR_SODIPODI_CY;
+			break;
+		case SP_ATTR_RX:
+			key = SP_ATTR_SODIPODI_RX;
+			break;
+		case SP_ATTR_RY:
+			key = SP_ATTR_SODIPODI_RY;
+			break;
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else if (((version < 25) && !strcmp (attr, "cy")) || ((version >= 25) && !strcmp (attr, "sodipodi:cy"))) {
-		if (!sp_svg_length_read (str, &ellipse->cy)) {
-			ellipse->cy.set = FALSE;
-			ellipse->cy.computed = 0.0;
+	}
+
+	switch (key) {
+	case SP_ATTR_SODIPODI_CX:
+		if (!sp_svg_length_read (value, &ge->cx)) {
+			sp_svg_length_unset (&ge->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else if (((version < 25) && !strcmp (attr, "rx")) || ((version >= 25) && !strcmp (attr, "sodipodi:rx"))) {
-		if (sp_svg_length_read_lff (str, &unit, &ellipse->rx.value, &ellipse->rx.computed) && (ellipse->rx.value > 0.0)) {
-			ellipse->rx.set = TRUE;
-			ellipse->rx.unit = unit;
-		} else {
-			ellipse->rx.set = FALSE;
-			ellipse->rx.computed = 0.0;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_SODIPODI_CY:
+		if (!sp_svg_length_read (value, &ge->cy)) {
+			sp_svg_length_unset (&ge->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else if (((version < 25) && !strcmp (attr, "ry")) || ((version >= 25) && !strcmp (attr, "sodipodi:ry"))) {
-		if (sp_svg_length_read_lff (str, &unit, &ellipse->ry.value, &ellipse->ry.computed) && (ellipse->ry.value > 0.0)) {
-			ellipse->ry.set = TRUE;
-			ellipse->ry.unit = unit;
-		} else {
-			ellipse->ry.set = FALSE;
-			ellipse->ry.computed = 0.0;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_SODIPODI_RX:
+		if (!sp_svg_length_read (value, &ge->rx) || (ge->rx.computed <= 0.0)) {
+			sp_svg_length_unset (&ge->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_genericellipse_set_shape (ellipse);
-	} else if (!strcmp (attr, "sodipodi:start")) {
-		n = sp_repr_get_double_attribute (object->repr, attr, ellipse->start);
-		ellipse->start = n;
-		sp_genericellipse_set_shape (ellipse);
-	} else if (!strcmp (attr, "sodipodi:end")) {
-		n = sp_repr_get_double_attribute (object->repr, attr, ellipse->end);
-		ellipse->end = n;
-		sp_genericellipse_set_shape (ellipse);
-	} else if (!strcmp (attr, "sodipodi:open")) {
-		if (sp_repr_attr_is_set (object->repr, attr))
-			ellipse->closed = FALSE;
-		else
-			ellipse->closed = TRUE;
-		sp_genericellipse_set_shape (ellipse);
-	} else {
-		if (((SPObjectClass *) arc_parent_class)->read_attr)
-			((SPObjectClass *) arc_parent_class)->read_attr (object, attr);
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_SODIPODI_RY:
+		if (!sp_svg_length_read (value, &ge->ry) || (ge->ry.computed <= 0.0)) {
+			sp_svg_length_unset (&ge->ry, SP_SVG_UNIT_NONE, 0.0, 0.0);
+		}
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_SODIPODI_START:
+		sp_svg_number_read_d (value, &ge->start);
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_SODIPODI_END:
+		sp_svg_number_read_d (value, &ge->end);
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_SODIPODI_OPEN:
+		ge->closed = (!value);
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	default:
+		if (((SPObjectClass *) arc_parent_class)->set)
+			((SPObjectClass *) arc_parent_class)->set (object, key, value);
+		break;
 	}
 }
 
@@ -978,7 +991,7 @@ sp_arc_description (SPItem * item)
 }
 
 void
-sp_arc_set (SPArc *arc, gdouble x, gdouble y, gdouble rx, gdouble ry)
+sp_arc_position_set (SPArc *arc, gdouble x, gdouble y, gdouble rx, gdouble ry)
 {
 	SPGenericEllipse *ge;
 
@@ -992,97 +1005,11 @@ sp_arc_set (SPArc *arc, gdouble x, gdouble y, gdouble rx, gdouble ry)
 	ge->rx.computed = rx;
 	ge->ry.computed = ry;
 
-	sp_genericellipse_set_shape (ge);
-}
-
-static void
-sp_arc_start_set (SPItem *item, const ArtPoint *p, guint state)
-{
-	SPGenericEllipse *ge;
-	SPArc *arc;
-	gdouble dx, dy;
-
-	ge = SP_GENERICELLIPSE (item);
-	arc = SP_ARC(item);
-
-	ge->closed = (sp_genericellipse_side (ge, p) == -1) ? TRUE : FALSE;
-
-	dx = p->x - ge->cx.computed;
-	dy = p->y - ge->cy.computed;
-
-	ge->start = atan2 (dy / ge->ry.computed, dx / ge->rx.computed);
-	if (state & GDK_CONTROL_MASK) {
-		ge->start = sp_round(ge->start, M_PI_4);
-	}
-	sp_genericellipse_normalize (ge);
-}
-
-static void
-sp_arc_start_get (SPItem *item, ArtPoint *p)
-{
-	SPGenericEllipse *ge;
-	SPArc *arc;
-
-	ge = SP_GENERICELLIPSE (item);
-	arc = SP_ARC (item);
-
-	sp_arc_get_xy (arc, ge->start, p);
-}
-
-static void
-sp_arc_end_set (SPItem *item, const ArtPoint *p, guint state)
-{
-	SPGenericEllipse *ge;
-	SPArc *arc;
-	gdouble dx, dy;
-
-	ge = SP_GENERICELLIPSE (item);
-	arc = SP_ARC(item);
-
-	ge->closed = (sp_genericellipse_side (ge, p) == -1) ? TRUE : FALSE;
-
-	dx = p->x - ge->cx.computed;
-	dy = p->y - ge->cy.computed;
-	ge->end = atan2 (dy / ge->ry.computed, dx / ge->rx.computed);
-	if (state & GDK_CONTROL_MASK) {
-		ge->end = sp_round(ge->end, M_PI_4);
-	}
-	sp_genericellipse_normalize (ge);
-}
-
-static void
-sp_arc_end_get (SPItem *item, ArtPoint *p)
-{
-	SPGenericEllipse *ge;
-	SPArc *arc;
-
-	ge = SP_GENERICELLIPSE (item);
-	arc = SP_ARC (item);
-
-	sp_arc_get_xy (arc, ge->end, p);
-}
-
-static SPKnotHolder *
-sp_arc_knot_holder (SPItem *item, SPDesktop *desktop)
-{
-	SPArc *arc;
-	SPKnotHolder *knot_holder;
-
-	arc = SP_ARC (item);
-	knot_holder = sp_knot_holder_new (desktop, item, NULL);
-	
-	sp_knot_holder_add (knot_holder,
-			    sp_arc_start_set,
-			    sp_arc_start_get);
-	sp_knot_holder_add (knot_holder,
-			    sp_arc_end_set,
-			    sp_arc_end_get);
-	
-	return knot_holder;
+	sp_object_request_update ((SPObject *) arc, SP_OBJECT_MODIFIED_FLAG);
 }
 
 void
-sp_arc_get_xy (SPArc *arc, gdouble arg, ArtPoint *p)
+sp_arc_get_xy (SPArc *arc, gdouble arg, NRPointF *p)
 {
 	SPGenericEllipse *ge;
 
