@@ -12,26 +12,26 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
 #include <libnr/nr-matrix.h>
-#include <glib.h>
-#include <libart_lgpl/art_affine.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmenuitem.h>
-#include <gdk-pixbuf/gdk-pixbuf-loader.h>
+
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include "display/nr-arena-image.h"
 #include "svg/svg.h"
+#include "attributes.h"
 #include "print.h"
 #include "style.h"
 #include "brokenimage.xpm"
 #include "document.h"
 #include "dialogs/object-attributes.h"
 #include "sp-image.h"
+#include "helper/sp-intl.h"
 
 /*
  * SPImage
@@ -42,18 +42,17 @@ static void sp_image_init (SPImage * image);
 
 static void sp_image_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_image_release (SPObject * object);
-static void sp_image_read_attr (SPObject * object, const gchar * key);
+static void sp_image_set (SPObject *object, unsigned int key, const unsigned char *value);
+static void sp_image_update (SPObject *object, SPCtx *ctx, unsigned int flags);
 static SPRepr *sp_image_write (SPObject *object, SPRepr *repr, guint flags);
 
-static void sp_image_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform);
+static void sp_image_bbox (SPItem *item, NRRectF *bbox, const NRMatrixD *transform, unsigned int flags);
 static void sp_image_print (SPItem * item, SPPrintContext *ctx);
 static gchar * sp_image_description (SPItem * item);
-static GSList * sp_image_snappoints (SPItem * item, GSList * points);
-static NRArenaItem *sp_image_show (SPItem *item, NRArena *arena);
-static void sp_image_write_transform (SPItem *item, SPRepr *repr, gdouble *transform);
-static void sp_image_menu (SPItem *item, SPDesktop *desktop, GtkMenu *menu);
+static int sp_image_snappoints (SPItem *item, NRPointF *p, int size);
+static NRArenaItem *sp_image_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
+static void sp_image_write_transform (SPItem *item, SPRepr *repr, NRMatrixF *transform);
 
-static void sp_image_image_properties (GtkMenuItem *menuitem, SPAnchor *anchor);
 #ifdef ENABLE_AUTOTRACE
 static void sp_image_autotrace (GtkMenuItem *menuitem, SPAnchor *anchor);
 static void autotrace_dialog(SPImage * img);
@@ -67,20 +66,23 @@ static GdkPixbuf * sp_image_repr_read_b64 (const gchar * uri_data);
 
 static SPItemClass *parent_class;
 
-GtkType
+GType
 sp_image_get_type (void)
 {
-	static GtkType image_type = 0;
+	static GType image_type = 0;
 	if (!image_type) {
-		GtkTypeInfo image_info = {
-			"SPImage",
-			sizeof (SPImage),
+		GTypeInfo image_info = {
 			sizeof (SPImageClass),
-			(GtkClassInitFunc) sp_image_class_init,
-			(GtkObjectInitFunc) sp_image_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_image_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPImage),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_image_init,
 		};
-		image_type = gtk_type_unique (sp_item_get_type (), &image_info);
+		image_type = g_type_register_static (sp_item_get_type (), "SPImage", &image_info, 0);
 	}
 	return image_type;
 }
@@ -88,19 +90,20 @@ sp_image_get_type (void)
 static void
 sp_image_class_init (SPImageClass * klass)
 {
-	GtkObjectClass * gtk_object_class;
+	GObjectClass * gobject_class;
 	SPObjectClass * sp_object_class;
 	SPItemClass * item_class;
 
-	gtk_object_class = (GtkObjectClass *) klass;
+	gobject_class = (GObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 	item_class = (SPItemClass *) klass;
 
-	parent_class = gtk_type_class (sp_item_get_type ());
+	parent_class = g_type_class_ref (sp_item_get_type ());
 
 	sp_object_class->build = sp_image_build;
 	sp_object_class->release = sp_image_release;
-	sp_object_class->read_attr = sp_image_read_attr;
+	sp_object_class->set = sp_image_set;
+	sp_object_class->update = sp_image_update;
 	sp_object_class->write = sp_image_write;
 
 	item_class->bbox = sp_image_bbox;
@@ -109,7 +112,6 @@ sp_image_class_init (SPImageClass * klass)
 	item_class->show = sp_image_show;
 	item_class->snappoints = sp_image_snappoints;
 	item_class->write_transform = sp_image_write_transform;
-	item_class->menu = sp_image_menu;
 }
 
 static void
@@ -119,10 +121,6 @@ sp_image_init (SPImage *image)
 	sp_svg_length_unset (&image->y, SP_SVG_UNIT_NONE, 0.0, 0.0);
 	sp_svg_length_unset (&image->width, SP_SVG_UNIT_NONE, 0.0, 0.0);
 	sp_svg_length_unset (&image->height, SP_SVG_UNIT_NONE, 0.0, 0.0);
-
-	image->href = NULL;
-
-	image->pixbuf = NULL;
 }
 
 static void
@@ -131,11 +129,11 @@ sp_image_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	if (((SPObjectClass *) parent_class)->build)
 		((SPObjectClass *) parent_class)->build (object, document, repr);
 
-	sp_image_read_attr (object, "xlink:href");
-	sp_image_read_attr (object, "x");
-	sp_image_read_attr (object, "y");
-	sp_image_read_attr (object, "width");
-	sp_image_read_attr (object, "height");
+	sp_object_read_attr (object, "xlink:href");
+	sp_object_read_attr (object, "x");
+	sp_object_read_attr (object, "y");
+	sp_object_read_attr (object, "width");
+	sp_object_read_attr (object, "height");
 
 	/* Register */
 	sp_document_add_resource (document, "image", object);
@@ -168,37 +166,21 @@ sp_image_release (SPObject *object)
 }
 
 static void
-sp_image_read_attr (SPObject * object, const gchar *key)
+sp_image_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
 	SPImage *image;
-	const guchar *str;
 	gulong unit;
 
 	image = SP_IMAGE (object);
 
-	str = sp_repr_attr (SP_OBJECT_REPR (object), key);
-
-	if (!strcmp (key, "xlink:href")) {
-		if (image->href) {
-			g_free (image->href);
-			image->href = NULL;
-		}
-		if (image->pixbuf) {
-			gdk_pixbuf_unref (image->pixbuf);
-			image->pixbuf = NULL;
-		}
-		if (str) {
-			GdkPixbuf *pixbuf;
-			image->href = g_strdup (str);
-			pixbuf = sp_image_repr_read_image (object->repr);
-			if (pixbuf) {
-				pixbuf = sp_image_pixbuf_force_rgba (pixbuf);
-				image->pixbuf = pixbuf;
-			}
-		}
-		sp_image_update_canvas_image (image);
-	} else if (!strcmp (key, "x")) {
-		if (sp_svg_length_read_lff (str, &unit, &image->x.value, &image->x.computed) &&
+	switch (key) {
+	case SP_ATTR_XLINK_HREF:
+		if (image->href) g_free (image->href);
+		image->href = (value) ? g_strdup (value) : NULL;
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_IMAGE_HREF_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_X:
+		if (sp_svg_length_read_lff (value, &unit, &image->x.value, &image->x.computed) &&
 		    /* fixme: These are probably valid, but require special treatment (Lauris) */
 		    (unit != SP_SVG_UNIT_EM) &&
 		    (unit != SP_SVG_UNIT_EX) &&
@@ -206,15 +188,12 @@ sp_image_read_attr (SPObject * object, const gchar *key)
 			image->x.set = TRUE;
 			image->x.unit = unit;
 		} else {
-			image->x.set = FALSE;
-			image->x.unit = SP_SVG_UNIT_NONE;
-			image->x.computed = 0.0;
+			sp_svg_length_unset (&image->x, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		/* fixme: Do async (Lauris) */
-		sp_image_update_canvas_image (image);
-	} else if (!strcmp (key, "y")) {
-		if (sp_svg_length_read_lff (str, &unit, &image->y.value, &image->y.computed) &&
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_Y:
+		if (sp_svg_length_read_lff (value, &unit, &image->y.value, &image->y.computed) &&
 		    /* fixme: These are probably valid, but require special treatment (Lauris) */
 		    (unit != SP_SVG_UNIT_EM) &&
 		    (unit != SP_SVG_UNIT_EX) &&
@@ -222,15 +201,12 @@ sp_image_read_attr (SPObject * object, const gchar *key)
 			image->y.set = TRUE;
 			image->y.unit = unit;
 		} else {
-			image->y.set = FALSE;
-			image->y.unit = SP_SVG_UNIT_NONE;
-			image->y.computed = 0.0;
+			sp_svg_length_unset (&image->y, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		/* fixme: Do async (Lauris) */
-		sp_image_update_canvas_image (image);
-	} else if (!strcmp (key, "width")) {
-		if (sp_svg_length_read_lff (str, &unit, &image->width.value, &image->width.computed) &&
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_WIDTH:
+		if (sp_svg_length_read_lff (value, &unit, &image->width.value, &image->width.computed) &&
 		    /* fixme: These are probably valid, but require special treatment (Lauris) */
 		    (unit != SP_SVG_UNIT_EM) &&
 		    (unit != SP_SVG_UNIT_EX) &&
@@ -238,15 +214,12 @@ sp_image_read_attr (SPObject * object, const gchar *key)
 			image->width.set = TRUE;
 			image->width.unit = unit;
 		} else {
-			image->width.set = FALSE;
-			image->width.unit = SP_SVG_UNIT_NONE;
-			image->width.computed = 0.0;
+			sp_svg_length_unset (&image->width, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		/* fixme: Do async (Lauris) */
-		sp_image_update_canvas_image (image);
-	} else if (!strcmp (key, "height")) {
-		if (sp_svg_length_read_lff (str, &unit, &image->height.value, &image->height.computed) &&
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_HEIGHT:
+		if (sp_svg_length_read_lff (value, &unit, &image->height.value, &image->height.computed) &&
 		    /* fixme: These are probably valid, but require special treatment (Lauris) */
 		    (unit != SP_SVG_UNIT_EM) &&
 		    (unit != SP_SVG_UNIT_EX) &&
@@ -254,18 +227,45 @@ sp_image_read_attr (SPObject * object, const gchar *key)
 			image->height.set = TRUE;
 			image->height.unit = unit;
 		} else {
-			image->height.set = FALSE;
-			image->height.unit = SP_SVG_UNIT_NONE;
-			image->height.computed = 0.0;
+			sp_svg_length_unset (&image->height, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		/* fixme: Do async (Lauris) */
-		sp_image_update_canvas_image (image);
-	} else if (((SPObjectClass *) (parent_class))->read_attr) {
-		(* ((SPObjectClass *) (parent_class))->read_attr) (object, key);
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	default:
+		if (((SPObjectClass *) (parent_class))->set)
+			((SPObjectClass *) (parent_class))->set (object, key, value);
+		break;
 	}
 }
 
+static void
+sp_image_update (SPObject *object, SPCtx *ctx, unsigned int flags)
+{
+	SPImage *image;
+
+	image = (SPImage *) object;
+
+	if (((SPObjectClass *) (parent_class))->update)
+		((SPObjectClass *) (parent_class))->update (object, ctx, flags);
+
+	if (flags & SP_IMAGE_HREF_MODIFIED_FLAG) {
+		if (image->pixbuf) {
+			gdk_pixbuf_unref (image->pixbuf);
+			image->pixbuf = NULL;
+		}
+		if (image->href) {
+			GdkPixbuf *pixbuf;
+			pixbuf = sp_image_repr_read_image (object->repr);
+			if (pixbuf) {
+				pixbuf = sp_image_pixbuf_force_rgba (pixbuf);
+				image->pixbuf = pixbuf;
+			}
+		}
+	}
+
+	sp_image_update_canvas_image ((SPImage *) object);
+}
+
 static SPRepr *
 sp_image_write (SPObject *object, SPRepr *repr, guint flags)
 {
@@ -291,28 +291,51 @@ sp_image_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 static void
-sp_image_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
+sp_image_bbox (SPItem *item, NRRectF *bbox, const NRMatrixD *transform, unsigned int flags)
 {
 	SPImage *image;
 
 	image = SP_IMAGE (item);
 
 	if ((image->width.computed > 0.0) && (image->height.computed > 0.0)) {
-		ArtDRect dim;
+		float x0, y0, x1, y1;
+		float x, y;
 
-		dim.x0 = image->x.computed;
-		dim.y0 = image->y.computed;
-		dim.x1 = dim.x0 + image->width.computed;
-		dim.y1 = dim.y0 + image->height.computed;
+		x0 = image->x.computed;
+		y0 = image->y.computed;
+		x1 = x0 + image->width.computed;
+		y1 = y0 + image->height.computed;
 
-		art_drect_affine_transform (bbox, &dim, transform);
+		x = NR_MATRIX_DF_TRANSFORM_X (transform, x0, y0);
+		y = NR_MATRIX_DF_TRANSFORM_Y (transform, x0, y0);
+		bbox->x0 = MIN (bbox->x0, x);
+		bbox->y0 = MIN (bbox->y0, y);
+		bbox->x1 = MAX (bbox->x1, x);
+		bbox->y1 = MAX (bbox->y1, y);
+		x = NR_MATRIX_DF_TRANSFORM_X (transform, x1, y0);
+		y = NR_MATRIX_DF_TRANSFORM_Y (transform, x1, y0);
+		bbox->x0 = MIN (bbox->x0, x);
+		bbox->y0 = MIN (bbox->y0, y);
+		bbox->x1 = MAX (bbox->x1, x);
+		bbox->y1 = MAX (bbox->y1, y);
+		x = NR_MATRIX_DF_TRANSFORM_X (transform, x1, y1);
+		y = NR_MATRIX_DF_TRANSFORM_Y (transform, x1, y1);
+		bbox->x0 = MIN (bbox->x0, x);
+		bbox->y0 = MIN (bbox->y0, y);
+		bbox->x1 = MAX (bbox->x1, x);
+		bbox->y1 = MAX (bbox->y1, y);
+		x = NR_MATRIX_DF_TRANSFORM_X (transform, x0, y1);
+		y = NR_MATRIX_DF_TRANSFORM_Y (transform, x0, y1);
+		bbox->x0 = MIN (bbox->x0, x);
+		bbox->y0 = MIN (bbox->y0, y);
+		bbox->x1 = MAX (bbox->x1, x);
+		bbox->y1 = MAX (bbox->y1, y);
 	}
 }
 
 static void
 sp_image_print (SPItem *item, SPPrintContext *ctx)
 {
-#if 1
 	SPImage *image;
 	NRMatrixF tp, ti, s, t;
 	unsigned char *px;
@@ -337,53 +360,6 @@ sp_image_print (SPItem *item, SPPrintContext *ctx)
 	nr_matrix_multiply_fff (&t, &ti, &t);
 
 	sp_print_image_R8G8B8A8_N (ctx, px, w, h, rs, &t, SP_OBJECT_STYLE (item));
-#else
-	SPObject *object;
-	SPImage *image;
-	guchar *pixels;
-	gint width, height, rowstride;
-
-	object = SP_OBJECT (item);
-	image = SP_IMAGE (item);
-
-	if (!image->pixbuf) return;
-	if ((image->width.computed <= 0.0) || (image->height.computed <= 0.0)) return;
-
-	pixels = gdk_pixbuf_get_pixels (image->pixbuf);
-	width = gdk_pixbuf_get_width (image->pixbuf);
-	height = gdk_pixbuf_get_height (image->pixbuf);
-	rowstride = gdk_pixbuf_get_rowstride (image->pixbuf);
-
-	gnome_print_gsave (gpc);
-
-	gnome_print_translate (gpc, image->x.computed, image->y.computed);
-	gnome_print_scale (gpc, image->width.computed, -image->height.computed);
-	gnome_print_translate (gpc, 0.0, -1.0);
-
-	if (object->style->opacity.value != SP_SCALE24_MAX) {
-		guchar *px, *d, *s;
-		gint x, y;
-		guint32 alpha;
-		alpha = (guint32) floor (SP_SCALE24_TO_FLOAT (object->style->opacity.value) * 255.9999);
-		px = g_new (guchar, width * height * 4);
-		for (y = 0; y < height; y++) {
-			s = pixels + y * rowstride;
-			d = px + y * width * 4;
-			memcpy (d, s, width * 4);
-			for (x = 0; x < width; x++) {
-				d[3] = (s[3] * alpha) / 255;
-				s += 4;
-				d += 4;
-			}
-		}
-		gnome_print_rgbaimage (gpc, px, width, height, width * 4);
-		g_free (px);
-	} else {
-		gnome_print_rgbaimage (gpc, pixels, width, height, rowstride);
-	}
-
-	gnome_print_grestore (gpc);
-#endif
 }
 
 static gchar *
@@ -404,7 +380,7 @@ sp_image_description (SPItem * item)
 }
 
 static NRArenaItem *
-sp_image_show (SPItem *item, NRArena *arena)
+sp_image_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags)
 {
 	SPImage * image;
 	NRArenaItem *ai;
@@ -413,7 +389,15 @@ sp_image_show (SPItem *item, NRArena *arena)
 
 	ai = nr_arena_item_new (arena, NR_TYPE_ARENA_IMAGE);
 
-	nr_arena_image_set_pixbuf (NR_ARENA_IMAGE (ai), image->pixbuf);
+	if (image->pixbuf) {
+		nr_arena_image_set_pixels (NR_ARENA_IMAGE (ai),
+					   gdk_pixbuf_get_pixels (image->pixbuf),
+					   gdk_pixbuf_get_width (image->pixbuf),
+					   gdk_pixbuf_get_height (image->pixbuf),
+					   gdk_pixbuf_get_rowstride (image->pixbuf));
+	} else {
+		nr_arena_image_set_pixels (NR_ARENA_IMAGE (ai), NULL, 0, 0, 0);
+	}
 	nr_arena_image_set_geometry (NR_ARENA_IMAGE (ai), image->x.computed, image->y.computed, image->width.computed, image->height.computed);
 
 	return ai;
@@ -437,31 +421,36 @@ sp_image_repr_read_image (SPRepr * repr)
 	filename = sp_repr_attr (repr, "xlink:href");
 	if (filename == NULL) filename = sp_repr_attr (repr, "href"); /* FIXME */
 	if (filename != NULL) {
-		if (strncmp (filename,"data:",5) == 0) {
+		if (strncmp (filename,"file:",5) == 0) {
+			fullname = g_filename_from_uri(filename, NULL, NULL);
+			if (fullname) {
+				pixbuf = gdk_pixbuf_new_from_file (fullname, NULL);
+				g_free (fullname);
+				if (pixbuf != NULL) return pixbuf;
+			}
+		} else if (strncmp (filename,"data:",5) == 0) {
 			/* data URI - embedded image */
 			filename += 5;
 			pixbuf = sp_image_repr_read_dataURI (filename);
 			if (pixbuf != NULL) return pixbuf;
-		}
-		else if (!g_path_is_absolute (filename)) {
+		} else if (!g_path_is_absolute (filename)) {
 			/* try to load from relative pos */
 			docbase = sp_repr_attr (sp_repr_document_root (sp_repr_document (repr)), "sodipodi:docbase");
-			if (docbase != NULL) {
-				fullname = g_strconcat (docbase, filename, NULL);
-				pixbuf = gdk_pixbuf_new_from_file (fullname);
-				g_free (fullname);
-				if (pixbuf != NULL) return pixbuf;
-			}
+			if (!docbase) docbase = "./";
+			fullname = g_strconcat (docbase, filename, NULL);
+			pixbuf = gdk_pixbuf_new_from_file (fullname, NULL);
+			g_free (fullname);
+			if (pixbuf != NULL) return pixbuf;
 		} else {
 			/* try absolute filename */
-			pixbuf = gdk_pixbuf_new_from_file (filename);
+			pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 			if (pixbuf != NULL) return pixbuf;
 		}
 	}
 	/* at last try to load from sp absolute path name */
 	filename = sp_repr_attr (repr, "sodipodi:absref");
 	if (filename != NULL) {
-		pixbuf = gdk_pixbuf_new_from_file (filename);
+		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 		if (pixbuf != NULL) return pixbuf;
 	}
 	/* Nope: We do not find any valid pixmap file :-( */
@@ -508,48 +497,58 @@ sp_image_update_canvas_image (SPImage *image)
 	}
 
 	for (v = item->display; v != NULL; v = v->next) {
-		nr_arena_image_set_pixbuf (NR_ARENA_IMAGE (v->arenaitem), image->pixbuf);
+		nr_arena_image_set_pixels (NR_ARENA_IMAGE (v->arenaitem),
+					   gdk_pixbuf_get_pixels (image->pixbuf),
+					   gdk_pixbuf_get_width (image->pixbuf),
+					   gdk_pixbuf_get_height (image->pixbuf),
+					   gdk_pixbuf_get_rowstride (image->pixbuf));
 		nr_arena_image_set_geometry (NR_ARENA_IMAGE (v->arenaitem),
 					     image->x.computed, image->y.computed,
 					     image->width.computed, image->height.computed);
 	}
 }
 
-static GSList * 
-sp_image_snappoints (SPItem * item, GSList * points)
+static int
+sp_image_snappoints (SPItem *item, NRPointF *p, int size)
 {
-	ArtPoint *p, p1, p2, p3, p4;
-	gdouble affine[6];
 	SPImage *image;
+	NRMatrixF i2d;
+	float x0, y0, x1, y1;
+	int i;
 
 	image = SP_IMAGE (item);
 
-	sp_item_i2d_affine (item, affine);
+	sp_item_i2d_affine (item, &i2d);
 
 	/* we use corners of image only */
-	p1.x = image->x.computed;
-	p1.y = image->y.computed;
-	p2.x = p1.x + image->width.computed;
-	p2.y = p1.y;
-	p3.x = p1.x;
-	p3.y = p1.y + image->height.computed;
-	p4.x = p2.x;
-	p4.y = p3.y;
+	x0 = image->x.computed;
+	y0 = image->y.computed;
+	x1 = x0 + image->width.computed;
+	y1 = y0 + image->height.computed;
 
-	p = g_new (ArtPoint,1);
-	art_affine_point (p, &p1, affine);
-	points = g_slist_append (points, p);
-	p = g_new (ArtPoint,1);
-	art_affine_point (p, &p2, affine);
-	points = g_slist_append (points, p);
-	p = g_new (ArtPoint,1);
-	art_affine_point (p, &p3, affine);
-	points = g_slist_append (points, p);
-	p = g_new (ArtPoint,1);
-	art_affine_point (p, &p4, affine);
-	points = g_slist_append (points, p);
+	i = 0;
+	if (i < size) {
+		p[i].x = NR_MATRIX_DF_TRANSFORM_X (&i2d, x0, y0);
+		p[i].y = NR_MATRIX_DF_TRANSFORM_Y (&i2d, x0, y0);
+		i += 1;
+	}
+	if (i < size) {
+		p[i].x = NR_MATRIX_DF_TRANSFORM_X (&i2d, x1, y0);
+		p[i].y = NR_MATRIX_DF_TRANSFORM_Y (&i2d, x1, y0);
+		i += 1;
+	}
+	if (i < size) {
+		p[i].x = NR_MATRIX_DF_TRANSFORM_X (&i2d, x1, y1);
+		p[i].y = NR_MATRIX_DF_TRANSFORM_Y (&i2d, x1, y1);
+		i += 1;
+	}
+	if (i < size) {
+		p[i].x = NR_MATRIX_DF_TRANSFORM_X (&i2d, x0, y1);
+		p[i].y = NR_MATRIX_DF_TRANSFORM_Y (&i2d, x0, y1);
+		i += 1;
+	}
 
-	return points;
+	return i;
 }
 
 /*
@@ -558,97 +557,46 @@ sp_image_snappoints (SPItem * item, GSList * points)
  */
 
 static void
-sp_image_write_transform (SPItem *item, SPRepr *repr, gdouble *transform)
+sp_image_write_transform (SPItem *item, SPRepr *repr, NRMatrixF *t)
 {
 	SPImage *image;
-	gdouble rev[6];
+	NRMatrixF rev;
 	gdouble px, py, sw, sh;
-	guchar t[80];
 
 	image = SP_IMAGE (item);
 
 	/* Calculate text start in parent coords */
-	px = transform[0] * image->x.computed + transform[2] * image->y.computed + transform[4];
-	py = transform[1] * image->x.computed + transform[3] * image->y.computed + transform[5];
+	px = NR_MATRIX_DF_TRANSFORM_X (t, image->x.computed, image->y.computed);
+	py = NR_MATRIX_DF_TRANSFORM_Y (t, image->x.computed, image->y.computed);
 
 	/* Clear translation */
-	transform[4] = 0.0;
-	transform[5] = 0.0;
+	t->c[4] = 0.0;
+	t->c[5] = 0.0;
 
 	/* Scalers */
-	sw = sqrt (transform[0] * transform[0] + transform[1] * transform[1]);
-	sh = sqrt (transform[2] * transform[2] + transform[3] * transform[3]);
+	sw = sqrt (t->c[0] * t->c[0] + t->c[1] * t->c[1]);
+	sh = sqrt (t->c[2] * t->c[2] + t->c[3] * t->c[3]);
 	if (sw > 1e-9) {
-		transform[0] = transform[0] / sw;
-		transform[1] = transform[1] / sw;
+		t->c[0] = t->c[0] / sw;
+		t->c[1] = t->c[1] / sw;
 	} else {
-		transform[0] = 1.0;
-		transform[1] = 0.0;
+		t->c[0] = 1.0;
+		t->c[1] = 0.0;
 	}
 	if (sh > 1e-9) {
-		transform[2] = transform[2] / sh;
-		transform[3] = transform[3] / sh;
+		t->c[2] = t->c[2] / sh;
+		t->c[3] = t->c[3] / sh;
 	} else {
-		transform[2] = 0.0;
-		transform[3] = 1.0;
+		t->c[2] = 0.0;
+		t->c[3] = 1.0;
 	}
 	sp_repr_set_double_attribute (repr, "width", image->width.computed * sw);
 	sp_repr_set_double_attribute (repr, "height", image->height.computed * sh);
 
 	/* Find start in item coords */
-	art_affine_invert (rev, transform);
-	sp_repr_set_double_attribute (repr, "x", px * rev[0] + py * rev[2]);
-	sp_repr_set_double_attribute (repr, "y", px * rev[1] + py * rev[3]);
-
-	if (sp_svg_write_affine (t, 80, transform)) {
-		sp_repr_set_attr (SP_OBJECT_REPR (item), "transform", t);
-	} else {
-		sp_repr_set_attr (SP_OBJECT_REPR (item), "transform", NULL);
-	}
-}
-
-/* Generate context menu item section */
-
-static void
-sp_image_menu (SPItem *item, SPDesktop *desktop, GtkMenu *menu)
-{
-	GtkWidget *i, *m, *w;
-
-	if (SP_ITEM_CLASS (parent_class)->menu)
-		(* SP_ITEM_CLASS (parent_class)->menu) (item, desktop, menu);
-
-	/* Create toplevel menuitem */
-	i = gtk_menu_item_new_with_label (_("Image"));
-	m = gtk_menu_new ();
-	/* Link dialog */
-	w = gtk_menu_item_new_with_label (_("Image Properties"));
-	gtk_object_set_data (GTK_OBJECT (w), "desktop", desktop);
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC (sp_image_image_properties), item);
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-
-#ifdef ENABLE_AUTOTRACE
-	/* Autotrace dialog */
-	w = gtk_menu_item_new_with_label (_("Trace"));
-	gtk_object_set_data (GTK_OBJECT (w), "desktop", desktop);
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC (sp_image_autotrace), item);
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-#endif /* Def: ENABLE_AUTOTRACE */
-
-	/* Show menu */
-	gtk_widget_show (m);
-
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), m);
-
-	gtk_menu_append (menu, i);
-	gtk_widget_show (i);
-}
-
-static void
-sp_image_image_properties (GtkMenuItem *menuitem, SPAnchor *anchor)
-{
-	sp_object_attributes_dialog (SP_OBJECT (anchor), "SPImage");
+	nr_matrix_f_invert (&rev, t);
+	sp_repr_set_double_attribute (repr, "x", px * rev.c[0] + py * rev.c[2]);
+	sp_repr_set_double_attribute (repr, "y", px * rev.c[1] + py * rev.c[3]);
 }
 
 #ifdef ENABLE_AUTOTRACE
@@ -785,13 +733,13 @@ sp_image_repr_read_b64 (const gchar * uri_data)
 			}
 		}
 
-		if (!gdk_pixbuf_loader_write (loader, (const guchar *) bd, (size_t) l)) {
+		if (!gdk_pixbuf_loader_write (loader, (const guchar *) bd, (size_t) l, NULL)) {
 			failed = 1;
 			break;
 		}
 	}
 
-	gdk_pixbuf_loader_close (loader);
+	gdk_pixbuf_loader_close (loader, NULL);
 
 	if (!failed) pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 
@@ -815,9 +763,6 @@ sp_image_repr_read_b64 (const gchar * uri_data)
 /* g_strdup_printf */
 #include <glib.h>
 
-/* GnomeMessageBox */
-#include <libgnomeui/libgnomeui.h>
-
 static at_bitmap_type * gdk_pixbuf_to_at_bitmap (GdkPixbuf * pixbuf);
 static void load_trace_result(FrontlineDialog * fl_dialog, gpointer user_data);
 static void load_splines(at_splines_type * splines);
@@ -976,11 +921,13 @@ handle_msg(at_string msg, at_msg_type msg_type, at_address client_data)
 	if (msg_type == AT_MSG_FATAL) {
 		long_msg = g_strdup_printf(_("Error to write %s: %s"), 
 					   target, msg);	
-		dialog = gnome_message_box_new(long_msg,
-					       GNOME_MESSAGE_BOX_ERROR,
-					       _("Ok"), NULL);
-		gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
-		gnome_dialog_run(GNOME_DIALOG(dialog));
+		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 long_msg,
+						 NULL);
+		gtk_dialog_run(GTK_DIALOG(dialog));
+		gtk_widget_destroy(dialog);
 		g_free(long_msg);
 	}
 	else {
