@@ -12,64 +12,61 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
-#include <glib.h>
-#include <libart_lgpl/art_affine.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmenuitem.h>
+
 #include "display/nr-arena-group.h"
 #include "xml/repr-private.h"
 #include "sp-object-repr.h"
 #include "svg/svg.h"
 #include "document.h"
 #include "style.h"
-#include "desktop.h"
-#include "desktop-handles.h"
-#include "selection.h"
+
 #include "sp-root.h"
 #include "sp-item-group.h"
+#include "helper/sp-intl.h"
 
 static void sp_group_class_init (SPGroupClass *klass);
 static void sp_group_init (SPGroup *group);
 
 static void sp_group_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_group_release (SPObject *object);
-static void sp_group_read_attr (SPObject * object, const gchar * attr);
 static void sp_group_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
 static void sp_group_remove_child (SPObject * object, SPRepr * child);
 static void sp_group_order_changed (SPObject * object, SPRepr * child, SPRepr * old, SPRepr * new);
+static void sp_group_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_group_modified (SPObject *object, guint flags);
-static gint sp_group_sequence (SPObject *object, gint seq);
+static unsigned int sp_group_sequence (SPObject *object, SPObject *target, unsigned int *seq);
 static SPRepr *sp_group_write (SPObject *object, SPRepr *repr, guint flags);
 
-static void sp_group_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform);
+static void sp_group_bbox (SPItem *item, NRRectF *bbox, const NRMatrixD *transform, unsigned int flags);
 static void sp_group_print (SPItem * item, SPPrintContext *ctx);
 static gchar * sp_group_description (SPItem * item);
-static NRArenaItem *sp_group_show (SPItem *item, NRArena *arena);
-static void sp_group_hide (SPItem * item, NRArena *arena);
-static void sp_group_menu (SPItem *item, SPDesktop *desktop, GtkMenu *menu);
-
-static void sp_item_group_ungroup_activate (GtkMenuItem *menuitem, SPGroup *group);
+static NRArenaItem *sp_group_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
+static void sp_group_hide (SPItem * item, unsigned int key);
 
 static SPItemClass * parent_class;
 
-GtkType
+GType
 sp_group_get_type (void)
 {
-	static GtkType group_type = 0;
+	static GType group_type = 0;
 	if (!group_type) {
-		GtkTypeInfo group_info = {
-			"SPGroup",
-			sizeof (SPGroup),
+		GTypeInfo group_info = {
 			sizeof (SPGroupClass),
-			(GtkClassInitFunc) sp_group_class_init,
-			(GtkObjectInitFunc) sp_group_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_group_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPGroup),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_group_init,
 		};
-		group_type = gtk_type_unique (sp_item_get_type (), &group_info);
+		group_type = g_type_register_static (SP_TYPE_ITEM, "SPGroup", &group_info, 0);
 	}
 	return group_type;
 }
@@ -77,22 +74,22 @@ sp_group_get_type (void)
 static void
 sp_group_class_init (SPGroupClass *klass)
 {
-	GtkObjectClass * gtk_object_class;
+	GObjectClass * object_class;
 	SPObjectClass * sp_object_class;
 	SPItemClass * item_class;
 
-	gtk_object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 	item_class = (SPItemClass *) klass;
 
-	parent_class = gtk_type_class (sp_item_get_type ());
+	parent_class = g_type_class_ref (SP_TYPE_ITEM);
 
 	sp_object_class->build = sp_group_build;
 	sp_object_class->release = sp_group_release;
-	sp_object_class->read_attr = sp_group_read_attr;
 	sp_object_class->child_added = sp_group_child_added;
 	sp_object_class->remove_child = sp_group_remove_child;
 	sp_object_class->order_changed = sp_group_order_changed;
+	sp_object_class->update = sp_group_update;
 	sp_object_class->modified = sp_group_modified;
 	sp_object_class->sequence = sp_group_sequence;
 	sp_object_class->write = sp_group_write;
@@ -102,14 +99,12 @@ sp_group_class_init (SPGroupClass *klass)
 	item_class->description = sp_group_description;
 	item_class->show = sp_group_show;
 	item_class->hide = sp_group_hide;
-	item_class->menu = sp_group_menu;
 }
 
 static void
 sp_group_init (SPGroup *group)
 {
-	group->children = NULL;
-	group->transparent = FALSE;
+	/* Nothing here */
 }
 
 static void sp_group_build (SPObject *object, SPDocument * document, SPRepr * repr)
@@ -125,11 +120,15 @@ static void sp_group_build (SPObject *object, SPDocument * document, SPRepr * re
 
 	last = NULL;
 	for (rchild = repr->children; rchild != NULL; rchild = rchild->next) {
-		GtkType type;
+		GType type;
 		SPObject * child;
 		type = sp_repr_type_lookup (rchild);
-		child = gtk_type_new (type);
-		last ? last->next : group->children = sp_object_attach_reref (object, child, NULL);
+		child = g_object_new (type, 0);
+		if (last) {
+			last->next = sp_object_attach_reref (object, child, NULL);
+		} else {
+			object->children = sp_object_attach_reref (object, child, NULL);
+		}
 		sp_object_invoke_build (child, document, rchild, SP_OBJECT_IS_CLONED (object));
 		last = child;
 	}
@@ -142,31 +141,20 @@ sp_group_release (SPObject *object)
 
 	group = SP_GROUP (object);
 
-	while (group->children) {
-		group->children = sp_object_detach_unref (object, group->children);
+	while (object->children) {
+		object->children = sp_object_detach_unref (object, object->children);
 	}
 
 	if (((SPObjectClass *) parent_class)->release)
 		((SPObjectClass *) parent_class)->release (object);
 }
 
-static void
-sp_group_read_attr (SPObject * object, const gchar * attr)
-{
-	SPGroup * group;
-
-	group = SP_GROUP (object);
-
-	if (((SPObjectClass *) (parent_class))->read_attr)
-		((SPObjectClass *) (parent_class))->read_attr (object, attr);
-}
-
 static void
 sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 {
 	SPGroup *group;
 	SPItem *item;
-	GtkType type;
+	GType type;
 	SPObject *ochild, *prev;
 	gint position;
 
@@ -180,7 +168,7 @@ sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	prev = NULL;
 	position = 0;
 	if (ref != NULL) {
-		prev = group->children;
+		prev = object->children;
 		while (prev && (prev->repr != ref)) {
 			if (SP_IS_ITEM (prev)) position += 1;
 			prev = prev->next;
@@ -189,11 +177,11 @@ sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	}
 
 	type = sp_repr_type_lookup (child);
-	ochild = gtk_type_new (type);
+	ochild = g_object_new (type, 0);
 	if (prev) {
 		prev->next = sp_object_attach_reref (object, ochild, prev->next);
 	} else {
-		group->children = sp_object_attach_reref (object, ochild, group->children);
+		object->children = sp_object_attach_reref (object, ochild, object->children);
 	}
 
 	sp_object_invoke_build (ochild, object->document, child, SP_OBJECT_IS_CLONED (object));
@@ -202,16 +190,20 @@ sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 		SPItemView *v;
 		NRArenaItem *ac;
 		for (v = item->display; v != NULL; v = v->next) {
-			ac = sp_item_show (SP_ITEM (ochild), v->arena);
+			ac = sp_item_invoke_show (SP_ITEM (ochild), NR_ARENA_ITEM_ARENA (v->arenaitem), v->key, v->flags);
 			if (ac) {
 				nr_arena_item_add_child (v->arenaitem, ac, NULL);
 				nr_arena_item_set_order (ac, position);
-				gtk_object_unref (GTK_OBJECT(ac));
+				nr_arena_item_unref (ac);
 			}
 		}
 	}
+
+	sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 }
 
+/* fixme: hide (Lauris) */
+
 static void
 sp_group_remove_child (SPObject * object, SPRepr * child)
 {
@@ -224,21 +216,18 @@ sp_group_remove_child (SPObject * object, SPRepr * child)
 		(* ((SPObjectClass *) (parent_class))->remove_child) (object, child);
 
 	prev = NULL;
-	ochild = group->children;
+	ochild = object->children;
 	while (ochild->repr != child) {
 		prev = ochild;
 		ochild = ochild->next;
 	}
 
 	if (prev) {
-		prev->next = ochild->next;
+		prev->next = sp_object_detach_unref (object, ochild);
 	} else {
-		group->children = ochild->next;
+		object->children = sp_object_detach_unref (object, ochild);
 	}
-
-	ochild->parent = NULL;
-	ochild->next = NULL;
-	gtk_object_unref (GTK_OBJECT (ochild));
+	sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 }
 
 static void
@@ -258,7 +247,7 @@ sp_group_order_changed (SPObject *object, SPRepr *child, SPRepr *old, SPRepr *ne
 
 	/* Scan children list */
 	childpos = 0;
-	for (o = group->children; !childobj || (old && !oldobj) || (new && !newobj); o = o->next) {
+	for (o = object->children; !childobj || (old && !oldobj) || (new && !newobj); o = o->next) {
 		if (o->repr == child) {
 			childobj = o;
 		} else {
@@ -274,19 +263,17 @@ sp_group_order_changed (SPObject *object, SPRepr *child, SPRepr *old, SPRepr *ne
 		}
 	}
 
-	g_print ("oldpos %d newpos %d\n", oldpos, newpos);
-
 	if (oldobj) {
 		oldobj->next = childobj->next;
 	} else {
-		group->children = childobj->next;
+		object->children = childobj->next;
 	}
 	if (newobj) {
 		childobj->next = newobj->next;
 		newobj->next = childobj;
 	} else {
-		childobj->next = group->children;
-		group->children = childobj;
+		childobj->next = object->children;
+		object->children = childobj;
 	}
 
 	if (SP_IS_ITEM (childobj)) {
@@ -295,6 +282,50 @@ sp_group_order_changed (SPObject *object, SPRepr *child, SPRepr *old, SPRepr *ne
 			nr_arena_item_set_order (v->arenaitem, newpos);
 		}
 	}
+
+	sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+}
+
+static void
+sp_group_update (SPObject *object, SPCtx *ctx, unsigned int flags)
+{
+	SPGroup *group;
+	SPObject *child;
+	SPItemCtx *ictx, cctx;
+	GSList *l;
+
+	group = SP_GROUP (object);
+	ictx = (SPItemCtx *) ctx;
+	cctx = *ictx;
+
+	if (((SPObjectClass *) (parent_class))->update)
+		((SPObjectClass *) (parent_class))->update (object, ctx, flags);
+
+	if (flags & SP_OBJECT_MODIFIED_FLAG) flags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
+	flags &= SP_OBJECT_MODIFIED_CASCADE;
+
+	l = NULL;
+	for (child = object->children; child != NULL; child = child->next) {
+		g_object_ref (G_OBJECT (child));
+		l = g_slist_prepend (l, child);
+	}
+	l = g_slist_reverse (l);
+	while (l) {
+		child = SP_OBJECT (l->data);
+		l = g_slist_remove (l, child);
+		if (flags || (child->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
+			if (SP_IS_ITEM (child)) {
+				SPItem *chi;
+				chi = SP_ITEM (child);
+				nr_matrix_multiply_dfd (&cctx.i2doc, &chi->transform, &ictx->i2doc);
+				nr_matrix_multiply_dfd (&cctx.i2vp, &chi->transform, &ictx->i2vp);
+				sp_object_invoke_update (child, (SPCtx *) &cctx, flags);
+			} else {
+				sp_object_invoke_update (child, ctx, flags);
+			}
+		}
+		g_object_unref (G_OBJECT (child));
+	}
 }
 
 static void
@@ -310,36 +341,35 @@ sp_group_modified (SPObject *object, guint flags)
 	flags &= SP_OBJECT_MODIFIED_CASCADE;
 
 	l = NULL;
-	for (child = group->children; child != NULL; child = child->next) {
-		gtk_object_ref (GTK_OBJECT (child));
+	for (child = object->children; child != NULL; child = child->next) {
+		g_object_ref (G_OBJECT (child));
 		l = g_slist_prepend (l, child);
 	}
 	l = g_slist_reverse (l);
 	while (l) {
 		child = SP_OBJECT (l->data);
 		l = g_slist_remove (l, child);
-		if (flags || (GTK_OBJECT_FLAGS (child) & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
-			sp_object_modified (child, flags);
+		if (flags || (child->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
+			sp_object_invoke_modified (child, flags);
 		}
-		gtk_object_unref (GTK_OBJECT (child));
+		g_object_unref (G_OBJECT (child));
 	}
 }
 
-static gint
-sp_group_sequence (SPObject *object, gint seq)
+static unsigned int
+sp_group_sequence (SPObject *object, SPObject *target, unsigned int *seq)
 {
 	SPGroup *group;
 	SPObject *child;
 
-	group = SP_GROUP (object);
+	group = (SPGroup *) object;
 
-	seq += 1;
-
-	for (child = group->children; child != NULL; child = child->next) {
-		seq = sp_object_sequence (child, seq);
+	for (child = object->children; child != NULL; child = child->next) {
+		*seq += 1;
+		if (sp_object_invoke_sequence (child, target, seq)) return TRUE;
 	}
 
-	return seq;
+	return FALSE;
 }
 
 static SPRepr *
@@ -355,7 +385,7 @@ sp_group_write (SPObject *object, SPRepr *repr, guint flags)
 		GSList *l;
 		if (!repr) repr = sp_repr_new ("g");
 		l = NULL;
-		for (child = group->children; child != NULL; child = child->next) {
+		for (child = object->children; child != NULL; child = child->next) {
 			crepr = sp_object_invoke_write (child, NULL, flags);
 			if (crepr) l = g_slist_prepend (l, crepr);
 		}
@@ -365,7 +395,7 @@ sp_group_write (SPObject *object, SPRepr *repr, guint flags)
 			l = g_slist_remove (l, l->data);
 		}
 	} else {
-		for (child = group->children; child != NULL; child = child->next) {
+		for (child = object->children; child != NULL; child = child->next) {
 			sp_object_invoke_write (child, SP_OBJECT_REPR (child), flags);
 		}
 	}
@@ -377,25 +407,20 @@ sp_group_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 static void
-sp_group_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
+sp_group_bbox (SPItem *item, NRRectF *bbox, const NRMatrixD *transform, unsigned int flags)
 {
 	SPGroup * group;
 	SPItem * child;
-	ArtDRect child_bbox;
 	SPObject * o;
 
 	group = SP_GROUP (item);
 
-	for (o = group->children; o != NULL; o = o->next) {
+	for (o = ((SPObject *) item)->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
-			gdouble a[6];
+			NRMatrixD ct;
 			child = SP_ITEM (o);
-			art_affine_multiply (a, child->affine, transform);
-			sp_item_invoke_bbox (child, &child_bbox, a);
-			bbox->x0 = MIN (bbox->x0, child_bbox.x0);
-			bbox->y0 = MIN (bbox->y0, child_bbox.y0);
-			bbox->x1 = MAX (bbox->x1, child_bbox.x1);
-			bbox->y1 = MAX (bbox->y1, child_bbox.y1);
+			nr_matrix_multiply_dfd (&ct, &child->transform, transform);
+			sp_item_invoke_bbox_full (child, bbox, &ct, flags, FALSE);
 		}
 	}
 }
@@ -409,7 +434,7 @@ sp_group_print (SPItem * item, SPPrintContext *ctx)
 
 	group = SP_GROUP (item);
 
-	for (o = group->children; o != NULL; o = o->next) {
+	for (o = ((SPObject *) item)->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
 			child = SP_ITEM (o);
 			sp_item_invoke_print (SP_ITEM (o), ctx);
@@ -427,15 +452,15 @@ static gchar * sp_group_description (SPItem * item)
 	group = SP_GROUP (item);
 
 	len = 0;
-	for (o = group->children; o != NULL; o = o->next) len += 1;
+	for (o = ((SPObject *) item)->children; o != NULL; o = o->next) len += 1;
 
-	snprintf (c, 128, _("Group of %d objects"), len);
+	g_snprintf (c, 128, _("Group of %d objects"), len);
 
 	return g_strdup (c);
 }
 
 static NRArenaItem *
-sp_group_show (SPItem *item, NRArena *arena)
+sp_group_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags)
 {
 	SPGroup *group;
 	NRArenaItem *ai, *ac, *ar;
@@ -448,14 +473,14 @@ sp_group_show (SPItem *item, NRArena *arena)
 	nr_arena_group_set_transparent (NR_ARENA_GROUP (ai), group->transparent);
 
 	ar = NULL;
-	for (o = group->children; o != NULL; o = o->next) {
+	for (o = ((SPObject *) item)->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
 			child = SP_ITEM (o);
-			ac = sp_item_show (child, arena);
+			ac = sp_item_invoke_show (child, arena, key, flags);
 			if (ac) {
 				nr_arena_item_add_child (ai, ac, ar);
 				ar = ac;
-				gtk_object_unref (GTK_OBJECT(ac));
+				nr_arena_item_unref (ac);
 			}
 		}
 	}
@@ -464,7 +489,7 @@ sp_group_show (SPItem *item, NRArena *arena)
 }
 
 static void
-sp_group_hide (SPItem *item, NRArena *arena)
+sp_group_hide (SPItem *item, unsigned int key)
 {
 	SPGroup * group;
 	SPItem * child;
@@ -472,73 +497,15 @@ sp_group_hide (SPItem *item, NRArena *arena)
 
 	group = (SPGroup *) item;
 
-	for (o = group->children; o != NULL; o = o->next) {
+	for (o = ((SPObject *) item)->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
 			child = SP_ITEM (o);
-			sp_item_hide (child, arena);
+			sp_item_invoke_hide (child, key);
 		}
 	}
 
-	if (SP_ITEM_CLASS (parent_class)->hide)
-		(* SP_ITEM_CLASS (parent_class)->hide) (item, arena);
-}
-
-static void
-sp_group_menu (SPItem *item, SPDesktop *desktop, GtkMenu * menu)
-{
-	GtkWidget * i, * m, * w;
-
-	if (SP_ITEM_CLASS (parent_class)->menu)
-		(* SP_ITEM_CLASS (parent_class)->menu) (item, desktop, menu);
-
-	i = gtk_menu_item_new_with_label (_("Group"));
-	m = gtk_menu_new ();
-
-	/* Group dialog */
-	w = gtk_menu_item_new_with_label (_("Group Properties"));
-	gtk_object_set_data (GTK_OBJECT (w), "desktop", desktop);
-#if 0
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC (sp_item_properties), item);
-#endif
-	gtk_widget_set_sensitive (w, FALSE);
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-	/* Separator */
-	w = gtk_menu_item_new ();
-	gtk_widget_show (w);
-	gtk_menu_append (GTK_MENU (m), w);
-	/* "Ungroup" */
-	w = gtk_menu_item_new_with_label (_("Ungroup"));
-	gtk_object_set_data (GTK_OBJECT (w), "desktop", desktop);
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC (sp_item_group_ungroup_activate), item);
-	gtk_widget_show (w);
-
-	gtk_menu_append (GTK_MENU (m), w);
-	gtk_widget_show (m);
-
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), m);
-
-	gtk_menu_append (menu, i);
-	gtk_widget_show (i);
-}
-
-static void
-sp_item_group_ungroup_activate (GtkMenuItem *menuitem, SPGroup *group)
-{
-	SPDesktop *desktop;
-	GSList *children;
-
-	g_assert (SP_IS_GROUP (group));
-
-	desktop = gtk_object_get_data (GTK_OBJECT (menuitem), "desktop");
-	g_return_if_fail (desktop != NULL);
-	g_return_if_fail (SP_IS_DESKTOP (desktop));
-
-	children = NULL;
-	sp_item_group_ungroup (group, &children);
-
-	sp_selection_set_item_list (SP_DT_SELECTION (desktop), children);
-	g_slist_free (children);
+	if (((SPItemClass *) parent_class)->hide)
+		((SPItemClass *) parent_class)->hide (item, key);
 }
 
 void
@@ -567,19 +534,19 @@ sp_item_group_ungroup (SPGroup *group, GSList **children)
 	/* Step 1 - generate lists of children objects */
 	items = NULL;
 	objects = NULL;
-	for (child = group->children; child != NULL; child = child->next) {
+	for (child = ((SPObject *) group)->children; child != NULL; child = child->next) {
 		SPRepr *nrepr;
 		nrepr = sp_repr_duplicate (SP_OBJECT_REPR (child));
 		if (SP_IS_ITEM (child)) {
 			SPItem *citem;
-			NRMatrixD ctrans;
+			NRMatrixF ctrans;
 			gchar affinestr[80];
 			guchar *ss;
 
 			citem = SP_ITEM (child);
 
-			nr_matrix_multiply_ddd (&ctrans, (NRMatrixD *) citem->affine, (NRMatrixD *) gitem->affine);
-			if (sp_svg_write_affine (affinestr, 79, (double *) &ctrans)) {
+			nr_matrix_multiply_fff (&ctrans, &citem->transform, &gitem->transform);
+			if (sp_svg_transform_write (affinestr, 79, &ctrans)) {
 				sp_repr_set_attr (nrepr, "transform", affinestr);
 			} else {
 				sp_repr_set_attr (nrepr, "transform", NULL);
@@ -601,9 +568,9 @@ sp_item_group_ungroup (SPGroup *group, GSList **children)
 	objects = g_slist_reverse (objects);
 
 	/* Step 2 - clear group */
-	while (group->children) {
+	while (((SPObject *) group)->children) {
 		/* Now it is time to remove original */
-		sp_repr_remove_child (grepr, SP_OBJECT_REPR (group->children));
+		sp_repr_remove_child (grepr, SP_OBJECT_REPR (((SPObject *) group)->children));
 	}
 
 	/* Step 3 - add nonitems */
@@ -644,9 +611,19 @@ sp_item_group_item_list (SPGroup * group)
 
 	s = NULL;
 
-	for (o = group->children; o != NULL; o = o->next) {
+	for (o = ((SPObject *) group)->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) s = g_slist_prepend (s, o);
 	}
 
 	return g_slist_reverse (s);
 }
+
+SPObject *
+sp_item_group_get_child_by_name (SPGroup *group, SPObject *ref, const unsigned char *name)
+{
+	SPObject *child;
+	child = (ref) ? ref->next : ((SPObject *) group)->children;
+	while (child && strcmp (sp_repr_name (child->repr), name)) child = child->next;
+	return child;
+}
+
