@@ -15,28 +15,39 @@
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
+
+#include <libarikkei/arikkei-strlib.h>
+
 #include "helper/canvas-grid.h"
 #include "svg/svg.h"
+#include "attributes.h"
 #include "document.h"
 #include "desktop.h"
 #include "desktop-events.h"
 #include "desktop-handles.h"
 #include "sp-guide.h"
+#include "sp-item-group.h"
 #include "sp-namedview.h"
 
+#ifdef WIN32
+#define strcasecmp _stricmp
+#endif
+
 #define PTPERMM (72.0 / 25.4)
 
 #define DEFAULTTOLERANCE 5.0
 #define DEFAULTGRIDCOLOR 0x3f3fff2f
 #define DEFAULTGUIDECOLOR 0x0000ff7f
 #define DEFAULTGUIDEHICOLOR 0xff00007f
+#define DEFAULTBORDERCOLOR 0x000000ff
+#define DEFAULTPAGECOLOR 0xffffff00
 
 static void sp_namedview_class_init (SPNamedViewClass * klass);
 static void sp_namedview_init (SPNamedView * namedview);
 
 static void sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_namedview_release (SPObject *object);
-static void sp_namedview_read_attr (SPObject * object, const gchar * key);
+static void sp_namedview_set (SPObject *object, unsigned int key, const unsigned char *value);
 static void sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
 static void sp_namedview_remove_child (SPObject *object, SPRepr *child);
 static SPRepr *sp_namedview_write (SPObject *object, SPRepr *repr, guint flags);
@@ -50,20 +61,23 @@ static gboolean sp_nv_read_opacity (const guchar *str, guint32 *color);
 
 static SPObjectGroupClass * parent_class;
 
-GtkType
+GType
 sp_namedview_get_type (void)
 {
-	static GtkType namedview_type = 0;
+	static GType namedview_type = 0;
 	if (!namedview_type) {
-		GtkTypeInfo namedview_info = {
-			"SPNamedView",
-			sizeof (SPNamedView),
+		GTypeInfo namedview_info = {
 			sizeof (SPNamedViewClass),
-			(GtkClassInitFunc) sp_namedview_class_init,
-			(GtkObjectInitFunc) sp_namedview_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_namedview_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPNamedView),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_namedview_init,
 		};
-		namedview_type = gtk_type_unique (sp_objectgroup_get_type (), &namedview_info);
+		namedview_type = g_type_register_static (SP_TYPE_OBJECTGROUP, "SPNamedView", &namedview_info, 0);
 	}
 	return namedview_type;
 }
@@ -71,17 +85,17 @@ sp_namedview_get_type (void)
 static void
 sp_namedview_class_init (SPNamedViewClass * klass)
 {
-	GtkObjectClass * gtk_object_class;
+	GObjectClass * gobject_class;
 	SPObjectClass * sp_object_class;
 
-	gtk_object_class = (GtkObjectClass *) klass;
+	gobject_class = (GObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 
-	parent_class = gtk_type_class (sp_objectgroup_get_type ());
+	parent_class = g_type_class_ref (SP_TYPE_OBJECTGROUP);
 
 	sp_object_class->build = sp_namedview_build;
 	sp_object_class->release = sp_namedview_release;
-	sp_object_class->read_attr = sp_namedview_read_attr;
+	sp_object_class->set = sp_namedview_set;
 	sp_object_class->child_added = sp_namedview_child_added;
 	sp_object_class->remove_child = sp_namedview_remove_child;
 	sp_object_class->write = sp_namedview_write;
@@ -115,28 +129,32 @@ sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	if (((SPObjectClass *) (parent_class))->build)
 		(* ((SPObjectClass *) (parent_class))->build) (object, document, repr);
 
-	sp_namedview_read_attr (object, "viewonly");
-	sp_namedview_read_attr (object, "showgrid");
-	sp_namedview_read_attr (object, "snaptogrid");
-	sp_namedview_read_attr (object, "showguides");
-	sp_namedview_read_attr (object, "snaptoguides");
-	sp_namedview_read_attr (object, "gridtolerance");
-	sp_namedview_read_attr (object, "guidetolerance");
-	sp_namedview_read_attr (object, "gridoriginx");
-	sp_namedview_read_attr (object, "gridoriginy");
-	sp_namedview_read_attr (object, "gridspacingx");
-	sp_namedview_read_attr (object, "gridspacingy");
-	sp_namedview_read_attr (object, "gridcolor");
-	sp_namedview_read_attr (object, "gridopacity");
-	sp_namedview_read_attr (object, "guidecolor");
-	sp_namedview_read_attr (object, "guideopacity");
-	sp_namedview_read_attr (object, "guidehicolor");
-	sp_namedview_read_attr (object, "guidehiopacity");
-	sp_namedview_read_attr (object, "showborder");
+	sp_object_read_attr (object, "viewonly");
+	sp_object_read_attr (object, "showgrid");
+	sp_object_read_attr (object, "snaptogrid");
+	sp_object_read_attr (object, "showguides");
+	sp_object_read_attr (object, "snaptoguides");
+	sp_object_read_attr (object, "gridtolerance");
+	sp_object_read_attr (object, "guidetolerance");
+	sp_object_read_attr (object, "gridoriginx");
+	sp_object_read_attr (object, "gridoriginy");
+	sp_object_read_attr (object, "gridspacingx");
+	sp_object_read_attr (object, "gridspacingy");
+	sp_object_read_attr (object, "gridcolor");
+	sp_object_read_attr (object, "gridopacity");
+	sp_object_read_attr (object, "guidecolor");
+	sp_object_read_attr (object, "guideopacity");
+	sp_object_read_attr (object, "guidehicolor");
+	sp_object_read_attr (object, "guidehiopacity");
+	sp_object_read_attr (object, "showborder");
+	sp_object_read_attr (object, "borderlayer");
+	sp_object_read_attr (object, "bordercolor");
+	sp_object_read_attr (object, "borderopacity");
+	sp_object_read_attr (object, "pagecolor");
 
 	/* Construct guideline list */
 
-	for (o = og->children; o != NULL; o = o->next) {
+	for (o = object->children; o != NULL; o = o->next) {
 		if (SP_IS_GUIDE (o)) {
 			SPGuide * g;
 			g = SP_GUIDE (o);
@@ -145,7 +163,7 @@ sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 			} else {
 				nv->vguides = g_slist_prepend (nv->vguides, g);
 			}
-			gtk_object_set (GTK_OBJECT (g), "color", nv->guidecolor, "hicolor", nv->guidehicolor, NULL);
+			g_object_set (G_OBJECT (g), "color", nv->guidecolor, "hicolor", nv->guidehicolor, NULL);
 		}
 	}
 }
@@ -179,10 +197,9 @@ sp_namedview_release (SPObject * object)
 }
 
 static void
-sp_namedview_read_attr (SPObject * object, const gchar * key)
+sp_namedview_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
 	SPNamedView *nv;
-	const gchar *astr;
 	const SPUnit *pt = NULL;
 	const SPUnit *px = NULL;
 	const SPUnit *mm = NULL;
@@ -194,136 +211,178 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 	if (!px) px = sp_unit_get_by_abbreviation ("px");
 	if (!mm) mm = sp_unit_get_by_abbreviation ("mm");
 
-	astr = sp_repr_attr (object->repr, key);
-
-	if (!strcmp (key, "viewonly")) {
-		nv->editable = (astr == NULL);
+	switch (key) {
+	case SP_ATTR_VIEWONLY:
+		nv->editable = (!value);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "showgrid")) {
-		nv->showgrid = sp_str_to_bool (astr);
+		break;
+	case SP_ATTR_SHOWGRID:
+		nv->showgrid = sp_str_to_bool (value);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "snaptogrid")) {
-		nv->snaptogrid = sp_str_to_bool (astr);
+		break;
+	case SP_ATTR_SNAPTOGRID:
+		nv->snaptogrid = sp_str_to_bool (value);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "showguides")) {
-		nv->showguides = sp_str_to_bool (astr);
+		break;
+	case SP_ATTR_SHOWGUIDES:
+		nv->showguides = sp_str_to_bool (value);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "snaptoguides")) {
-		nv->snaptoguides = sp_str_to_bool (astr);
+		break;
+	case SP_ATTR_SNAPTOGUIDES:
+		nv->snaptoguides = sp_str_to_bool (value);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "gridtolerance")) {
+		break;
+	case SP_ATTR_GRIDTOLERANCE:
 		nv->gridtoleranceunit = px;
 		nv->gridtolerance = DEFAULTTOLERANCE;
-		if (astr) {
-			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->gridtolerance, &nv->gridtoleranceunit);
+		if (value) {
+			sp_nv_read_length (value, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->gridtolerance, &nv->gridtoleranceunit);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "guidetolerance")) {
+		break;
+	case SP_ATTR_GUIDETOLERANCE:
 		nv->guidetoleranceunit = px;
 		nv->guidetolerance = DEFAULTTOLERANCE;
-		if (astr) {
-			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->guidetolerance, &nv->guidetoleranceunit);
+		if (value) {
+			sp_nv_read_length (value, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->guidetolerance, &nv->guidetoleranceunit);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (strcmp (key, "gridoriginx") == 0) {
+		break;
+	case SP_ATTR_GRIDORIGINX:
 		nv->gridunit = mm;
 		nv->gridoriginx = 0.0;
-		if (astr) {
-			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE, &nv->gridoriginx, &nv->gridunit);
+		if (value) {
+			sp_nv_read_length (value, SP_UNIT_ABSOLUTE, &nv->gridoriginx, &nv->gridunit);
 		}
 		sp_convert_distance (&nv->gridoriginx, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "gridoriginy")) {
+		break;
+	case SP_ATTR_GRIDORIGINY:
 		nv->gridunit = mm;
 		nv->gridoriginy = 0.0;
-		if (astr) {
-			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE, &nv->gridoriginy, &nv->gridunit);
+		if (value) {
+			sp_nv_read_length (value, SP_UNIT_ABSOLUTE, &nv->gridoriginy, &nv->gridunit);
 		}
 		sp_convert_distance (&nv->gridoriginy, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "gridspacingx")) {
+		break;
+	case SP_ATTR_GRIDSPACINGX:
 		nv->gridunit = mm;
 		nv->gridspacingx = 5.0;
-		if (astr) {
-			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE, &nv->gridspacingx, &nv->gridunit);
+		if (value) {
+			sp_nv_read_length (value, SP_UNIT_ABSOLUTE, &nv->gridspacingx, &nv->gridunit);
 		}
 		sp_convert_distance (&nv->gridspacingx, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "gridspacingy")) {
+		break;
+	case SP_ATTR_GRIDSPACINGY:
 		nv->gridunit = mm;
 		nv->gridspacingy = 5.0;
-		if (astr) {
-			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE, &nv->gridspacingy, &nv->gridunit);
+		if (value) {
+			sp_nv_read_length (value, SP_UNIT_ABSOLUTE, &nv->gridspacingy, &nv->gridunit);
 		}
 		sp_convert_distance (&nv->gridspacingy, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "gridcolor")) {
+		break;
+	case SP_ATTR_GRIDCOLOR:
 		nv->gridcolor = (nv->gridcolor & 0xff) | (DEFAULTGRIDCOLOR & 0xffffff00);
-		if (astr) {
-			nv->gridcolor = (nv->gridcolor & 0xff) | sp_svg_read_color (astr, nv->gridcolor);
+		if (value) {
+			nv->gridcolor = (nv->gridcolor & 0xff) | sp_svg_read_color (value, nv->gridcolor);
 		}
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "gridopacity")) {
+		break;
+	case SP_ATTR_GRIDOPACITY:
 		nv->gridcolor = (nv->gridcolor & 0xffffff00) | (DEFAULTGRIDCOLOR & 0xff);
-		sp_nv_read_opacity (astr, &nv->gridcolor);
+		sp_nv_read_opacity (value, &nv->gridcolor);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "guidecolor")) {
+		break;
+	case SP_ATTR_GUIDECOLOR:
 		nv->guidecolor = (nv->guidecolor & 0xff) | (DEFAULTGUIDECOLOR & 0xffffff00);
-		if (astr) {
-			nv->guidecolor = (nv->guidecolor & 0xff) | sp_svg_read_color (astr, nv->guidecolor);
+		if (value) {
+			nv->guidecolor = (nv->guidecolor & 0xff) | sp_svg_read_color (value, nv->guidecolor);
 		}
 		for (l = nv->hguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "color", nv->guidecolor, NULL);
+			g_object_set (G_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
 		for (l = nv->vguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "color", nv->guidecolor, NULL);
+			g_object_set (G_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "guideopacity")) {
+		break;
+	case SP_ATTR_GUIDEOPACITY:
 		nv->guidecolor = (nv->guidecolor & 0xffffff00) | (DEFAULTGUIDECOLOR & 0xff);
-		sp_nv_read_opacity (astr, &nv->guidecolor);
+		sp_nv_read_opacity (value, &nv->guidecolor);
 		for (l = nv->hguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "color", nv->guidecolor, NULL);
+			g_object_set (G_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
 		for (l = nv->vguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "color", nv->guidecolor, NULL);
+			g_object_set (G_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "guidehicolor")) {
+		break;
+	case SP_ATTR_GUIDEHICOLOR:
 		nv->guidehicolor = (nv->guidehicolor & 0xff) | (DEFAULTGUIDEHICOLOR & 0xffffff00);
-		if (astr) {
-			nv->guidehicolor = (nv->guidehicolor & 0xff) | sp_svg_read_color (astr, nv->guidehicolor);
+		if (value) {
+			nv->guidehicolor = (nv->guidehicolor & 0xff) | sp_svg_read_color (value, nv->guidehicolor);
 		}
 		for (l = nv->hguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
+			g_object_set (G_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
 		for (l = nv->vguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
+			g_object_set (G_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "guidehiopacity")) {
+		break;
+	case SP_ATTR_GUIDEHIOPACITY:
 		nv->guidehicolor = (nv->guidehicolor & 0xffffff00) | (DEFAULTGUIDEHICOLOR & 0xff);
-		sp_nv_read_opacity (astr, &nv->guidehicolor);
+		sp_nv_read_opacity (value, &nv->guidehicolor);
 		for (l = nv->hguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
+			g_object_set (G_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
 		for (l = nv->vguides; l != NULL; l = l->next) {
-			gtk_object_set (GTK_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
+			g_object_set (G_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "showborder")) {
-		nv->showborder = (astr) ? sp_str_to_bool (astr) : TRUE;
+		break;
+	case SP_ATTR_SHOWBORDER:
+		nv->showborder = (value) ? sp_str_to_bool (value) : TRUE;
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else {
-		if (((SPObjectClass *) (parent_class))->read_attr)
-			(* ((SPObjectClass *) (parent_class))->read_attr) (object, key);
+		break;
+	case SP_ATTR_BORDERLAYER:
+		nv->borderlayer = SP_BORDER_LAYER_BOTTOM;
+		if (value && !strcasecmp (value, "top")) nv->borderlayer = SP_BORDER_LAYER_TOP;
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_BORDERCOLOR:
+		nv->bordercolor = (nv->bordercolor & 0xff) | (DEFAULTBORDERCOLOR & 0xffffff00);
+		if (value) {
+			nv->bordercolor = (nv->bordercolor & 0xff) | sp_svg_read_color (value, nv->bordercolor);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_BORDEROPACITY:
+		nv->bordercolor = (nv->bordercolor & 0xffffff00) | (DEFAULTBORDERCOLOR & 0xff);
+		sp_nv_read_opacity (value, &nv->bordercolor);
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_PAGECOLOR:
+		nv->pagecolor = (nv->pagecolor & 0xff) | (DEFAULTPAGECOLOR & 0xffffff00);
+		if (value) {
+			nv->pagecolor = (nv->pagecolor & 0xff) | sp_svg_read_color (value, nv->pagecolor);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		break;
+	default:
+		if (((SPObjectClass *) (parent_class))->set)
+			((SPObjectClass *) (parent_class))->set (object, key, value);
+		break;
 	}
 }
 
@@ -352,7 +411,7 @@ sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
 		} else {
 			nv->vguides = g_slist_prepend (nv->vguides, g);
 		}
-		gtk_object_set (GTK_OBJECT (g), "color", nv->guidecolor, "hicolor", nv->guidehicolor, NULL);
+		g_object_set (G_OBJECT (g), "color", nv->guidecolor, "hicolor", nv->guidehicolor, NULL);
 		if (nv->editable) {
 			for (l = nv->views; l != NULL; l = l->next) {
 				sp_guide_show (g, SP_DESKTOP (l->data)->guides, sp_dt_guide_event);
@@ -557,9 +616,9 @@ static gboolean
 sp_str_to_bool (const guchar *str)
 {
 	if (str) {
-		if (!strcasecmp (str, "true") ||
-		    !strcasecmp (str, "yes") ||
-		    !strcasecmp (str, "y") ||
+		if (!g_strcasecmp (str, "true") ||
+		    !g_strcasecmp (str, "yes") ||
+		    !g_strcasecmp (str, "y") ||
 		    (atoi (str) != 0)) return TRUE;
 	}
 
@@ -572,12 +631,14 @@ static gboolean
 sp_nv_read_length (const guchar *str, guint base, gdouble *val, const SPUnit **unit)
 {
 	gdouble v;
-	gchar *u;
+	const unsigned char *u;
+	int len;
 
 	if (!str) return FALSE;
 
-	v = strtod (str, &u);
-	if (!u) return FALSE;
+	len = arikkei_strtod_exp (str, 1024, &v);
+	if (len < 1) return FALSE;
+	u = str + len;
 	while (isspace (*u)) u += 1;
 
 	if (!*u) {
@@ -633,15 +694,35 @@ static gboolean
 sp_nv_read_opacity (const guchar *str, guint32 *color)
 {
 	gdouble v;
-	gchar *u;
+	int len;
 
 	if (!str) return FALSE;
 
-	v = strtod (str, &u);
-	if (!u) return FALSE;
+	len = arikkei_strtod_exp (str, 1024, &v);
+	if (!len) return FALSE;
 	v = CLAMP (v, 0.0, 1.0);
 
 	*color = (*color & 0xffffff00) | (guint32) floor (v * 255.9999);
 
 	return TRUE;
 }
+
+SPNamedView *
+sp_document_namedview (SPDocument *document, const gchar *id)
+{
+	SPObject *nv;
+
+	g_return_val_if_fail (document != NULL, NULL);
+	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
+
+	nv = sp_item_group_get_child_by_name ((SPGroup *) document->root, NULL, "sodipodi:namedview");
+	g_assert (nv != NULL);
+
+	if (id == NULL) return (SPNamedView *) nv;
+
+	while (nv && strcmp (nv->id, id)) {
+		nv = sp_item_group_get_child_by_name ((SPGroup *) document->root, nv, "sodipodi:namedview");
+	}
+
+	return (SPNamedView *) nv;
+}
