@@ -1,5 +1,4 @@
 #define __SP_OBJECT_C__
-
 /*
  * Abstract base class for all nodes
  *
@@ -14,32 +13,37 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <gtk/gtksignal.h>
+
+#include "helper/sp-marshal.h"
 #include "xml/repr-private.h"
+#include "attributes.h"
 #include "document.h"
 #include "style.h"
 #include "sp-object-repr.h"
+
 #include "sp-object.h"
 
+#define noSP_OBJECT_DEBUG
+#define noSP_OBJECT_DEBUG_CASCADE
+
 static void sp_object_class_init (SPObjectClass * klass);
 static void sp_object_init (SPObject * object);
-static void sp_object_destroy (GtkObject * object);
+static void sp_object_finalize (GObject * object);
 
 static void sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr);
-static void sp_object_release (SPObject *object);
 
-static void sp_object_read_attr (SPObject * object, const gchar * key);
+static void sp_object_private_set (SPObject *object, unsigned int key, const unsigned char *value);
 static SPRepr *sp_object_private_write (SPObject *object, SPRepr *repr, guint flags);
 
 /* Real handlers of repr signals */
 
-static gboolean sp_object_repr_change_attr (SPRepr *repr, const guchar *key, const guchar *oldval, const guchar *newval, gpointer data);
+static unsigned int sp_object_repr_change_attr (SPRepr *repr, const guchar *key, const guchar *oldval, const guchar *newval, gpointer data);
 static void sp_object_repr_attr_changed (SPRepr *repr, const guchar *key, const guchar *oldval, const guchar *newval, gpointer data);
 
 static void sp_object_repr_content_changed (SPRepr *repr, const guchar *oldcontent, const guchar *newcontent, gpointer data);
 
 static void sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data);
-static gboolean sp_object_repr_remove_child (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data);
+static unsigned int sp_object_repr_remove_child (SPRepr *repr, SPRepr *child, SPRepr *ref, void *data);
 
 static void sp_object_repr_order_changed (SPRepr *repr, SPRepr *child, SPRepr *old, SPRepr *new, gpointer data);
 
@@ -61,88 +65,78 @@ SPReprEventVector object_event_vector = {
 	sp_object_repr_order_changed
 };
 
-static GtkObjectClass *parent_class;
+static GObjectClass *parent_class;
 static guint object_signals[LAST_SIGNAL] = {0};
 
-GtkType
+GType
 sp_object_get_type (void)
 {
-	static GtkType object_type = 0;
-	if (!object_type) {
-		GtkTypeInfo object_info = {
-			"SPObject",
-			sizeof (SPObject),
+	static GType type = 0;
+	if (!type) {
+		GTypeInfo info = {
 			sizeof (SPObjectClass),
-			(GtkClassInitFunc) sp_object_class_init,
-			(GtkObjectInitFunc) sp_object_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			NULL, NULL,
+			(GClassInitFunc) sp_object_class_init,
+			NULL, NULL,
+			sizeof (SPObject),
+			16,
+			(GInstanceInitFunc) sp_object_init,
 		};
-		object_type = gtk_type_unique (gtk_object_get_type (), &object_info);
+		type = g_type_register_static (G_TYPE_OBJECT, "SPObject", &info, 0);
 	}
-	return object_type;
+	return type;
 }
 
 static void
 sp_object_class_init (SPObjectClass * klass)
 {
-	GtkObjectClass * gtk_object_class;
+	GObjectClass * object_class;
 
-	gtk_object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 
-	parent_class = gtk_type_class (gtk_object_get_type ());
+	parent_class = g_type_class_ref (G_TYPE_OBJECT);
 
-	object_signals[RELEASE] =  gtk_signal_new ("release",
-						   GTK_RUN_FIRST,
-						   gtk_object_class->type,
-						   GTK_SIGNAL_OFFSET (SPObjectClass, release),
-						   gtk_marshal_NONE__NONE,
-						   GTK_TYPE_NONE, 0);
-	object_signals[MODIFIED] = gtk_signal_new ("modified",
-						   GTK_RUN_FIRST,
-						   gtk_object_class->type,
-						   GTK_SIGNAL_OFFSET (SPObjectClass, modified),
-						   gtk_marshal_NONE__UINT,
-						   GTK_TYPE_NONE, 1, GTK_TYPE_UINT);
-	gtk_object_class_add_signals (gtk_object_class, object_signals, LAST_SIGNAL);
+	object_signals[RELEASE] =  g_signal_new ("release",
+						 G_TYPE_FROM_CLASS (klass),
+						 G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+						 G_STRUCT_OFFSET (SPObjectClass, release),
+						 NULL, NULL,
+						 sp_marshal_VOID__VOID,
+						 G_TYPE_NONE, 0);
+	object_signals[MODIFIED] = g_signal_new ("modified",
+                                                 G_TYPE_FROM_CLASS (klass),
+                                                 G_SIGNAL_RUN_FIRST,
+                                                 G_STRUCT_OFFSET (SPObjectClass, modified),
+                                                 NULL, NULL,
+                                                 sp_marshal_NONE__UINT,
+                                                 G_TYPE_NONE, 1, G_TYPE_UINT);
 
-	gtk_object_class->destroy = sp_object_destroy;
+	object_class->finalize = sp_object_finalize;
 
 	klass->build = sp_object_build;
-	klass->release = sp_object_release;
-	klass->read_attr = sp_object_read_attr;
+
+	klass->set = sp_object_private_set;
 	klass->write = sp_object_private_write;
 }
 
 static void
 sp_object_init (SPObject * object)
 {
-	object->hrefcount = 0;
-	object->document = NULL;
-	object->parent = object->next = NULL;
-	object->repr = NULL;
-	object->id = NULL;
-	object->style = NULL;
-	object->title = NULL;
-	object->description = NULL;
+#ifdef SP_OBJECT_DEBUG
+	g_print("sp_object_init: id=%x, typename=%s\n",
+		object, g_type_name_from_instance((GTypeInstance*)object));
+#endif
 }
 
 static void
-sp_object_destroy (GtkObject * object)
+sp_object_finalize (GObject * object)
 {
 	SPObject *spobject;
 
 	spobject = (SPObject *) object;
 
-	/* fixme: This is here temporarily */
-	/* fixme: I f everything is ported to ::release, we can remove destroy at all */
-	if (spobject->document) {
-		sp_object_invoke_release (spobject);
-	}
-
-	if (((GtkObjectClass *) (parent_class))->destroy)
-		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
+	if (((GObjectClass *) (parent_class))->finalize)
+		(* ((GObjectClass *) (parent_class))->finalize) (object);
 }
 
 /*
@@ -159,7 +153,7 @@ sp_object_ref (SPObject *object, SPObject *owner)
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 	g_return_val_if_fail (!owner || SP_IS_OBJECT (owner), NULL);
 
-	gtk_object_ref (GTK_OBJECT (object));
+	g_object_ref (G_OBJECT (object));
 
 	return object;
 }
@@ -171,7 +165,7 @@ sp_object_unref (SPObject *object, SPObject *owner)
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 	g_return_val_if_fail (!owner || SP_IS_OBJECT (owner), NULL);
 
-	gtk_object_unref (GTK_OBJECT (object));
+	g_object_unref (G_OBJECT (object));
 
 	return NULL;
 }
@@ -215,7 +209,7 @@ sp_object_attach_reref (SPObject *parent, SPObject *object, SPObject *next)
 	g_return_val_if_fail (!object->next, NULL);
 
 	sp_object_ref (object, parent);
-	gtk_object_unref (GTK_OBJECT (object));
+	g_object_unref (G_OBJECT (object));
 	object->parent = parent;
 	object->next = next;
 
@@ -272,21 +266,19 @@ static void
 sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
 	/* Nothing specific here */
-}
+#ifdef SP_OBJECT_DEBUG
+	g_print("sp_object_build: id=%x, typename=%s\n", object, g_type_name_from_instance((GTypeInstance*)object));
+#endif
 
-static void
-sp_object_release (SPObject * object)
-{
-	/* Nothing specific here */
+	sp_object_read_attr (object, "xml:space");
 }
 
 void
-sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr, gboolean cloned)
+sp_object_invoke_build (SPObject *object, SPDocument *document, SPRepr *repr, unsigned int cloned)
 {
-	const gchar * id;
-	gchar * realid;
-	gboolean ret;
-
+#ifdef SP_OBJECT_DEBUG
+	g_print("sp_object_invoke_build: id=%x, typename=%s\n", object, g_type_name_from_instance((GTypeInstance*)object));
+#endif
 	g_assert (object != NULL);
 	g_assert (SP_IS_OBJECT (object));
 	g_assert (document != NULL);
@@ -302,18 +294,24 @@ sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr,
 	object->document = document;
 	object->repr = repr;
 	sp_repr_ref (repr);
-	if (cloned) GTK_OBJECT_SET_FLAGS (object, SP_OBJECT_CLONED_FLAG);
+	object->cloned = cloned;
 
 	/* If we are not cloned, force unique id */
 	if (!SP_OBJECT_IS_CLONED (object)) {
-		id = sp_repr_attr (repr, "id");
+		const gchar * id;
+		gchar * realid;
+
+		id = sp_repr_attr (object->repr, "id");
 		realid = sp_object_get_unique_id (object, id);
 		g_assert (realid != NULL);
-		sp_document_def_id (document, realid, object);
+
+		sp_document_def_id (object->document, realid, object);
 		object->id = realid;
+
 		/* Redefine ID, if required */
 		if ((id == NULL) || (strcmp (id, realid) != 0)) {
-			ret = sp_repr_set_attr (repr, "id", realid);
+			int ret;
+			ret = sp_repr_set_attr (object->repr, "id", realid);
 			if (!ret) {
 				g_error ("Cannot change id %s -> %s - probably there is stale ref", id, realid);
 			}
@@ -324,11 +322,10 @@ sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr,
 
 	/* Invoke derived methods, if any */
 
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->build)
-		(*((SPObjectClass *)(((GtkObject *) object)->klass))->build) (object, document, repr);
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->build)
+		(*((SPObjectClass *) G_OBJECT_GET_CLASS(object))->build) (object, document, repr);
 
 	/* Signalling (should be connected AFTER processing derived methods */
-
 	sp_repr_add_listener (repr, &object_event_vector, object);
 }
 
@@ -344,7 +341,7 @@ sp_object_invoke_release (SPObject *object)
 	g_assert (object->document);
 	g_assert (object->repr);
 
-	gtk_signal_emit (GTK_OBJECT (object), object_signals[RELEASE]);
+	g_signal_emit (G_OBJECT (object), object_signals[RELEASE], 0);
 
 	/* href holders HAVE to release it in signal handler */
 	g_assert (object->hrefcount == 0);
@@ -376,23 +373,19 @@ sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer d
 
 	object = SP_OBJECT (data);
 
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->child_added)
-		(*((SPObjectClass *)(((GtkObject *) object)->klass))->child_added) (object, child, ref);
-
-	sp_document_child_added (object->document, object, child, ref);
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->child_added)
+		(*((SPObjectClass *)G_OBJECT_GET_CLASS(object))->child_added) (object, child, ref);
 }
 
-static gboolean
+static unsigned int
 sp_object_repr_remove_child (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
 {
 	SPObject * object;
 
 	object = SP_OBJECT (data);
 
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->remove_child)
-		(* ((SPObjectClass *)(((GtkObject *) object)->klass))->remove_child) (object, child);
-
-	sp_document_child_removed (object->document, object, child, ref);
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->remove_child)
+		(* ((SPObjectClass *)G_OBJECT_GET_CLASS(object))->remove_child) (object, child);
 
 	return TRUE;
 }
@@ -406,41 +399,66 @@ sp_object_repr_order_changed (SPRepr * repr, SPRepr * child, SPRepr * old, SPRep
 
 	object = SP_OBJECT (data);
 
-	if (((SPObjectClass *) (((GtkObject *) object)->klass))->order_changed)
-		(* ((SPObjectClass *)(((GtkObject *) object)->klass))->order_changed) (object, child, old, new);
-
-	sp_document_order_changed (object->document, object, child, old, new);
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->order_changed)
+		(* ((SPObjectClass *)G_OBJECT_GET_CLASS(object))->order_changed) (object, child, old, new);
 }
 
 static void
-sp_object_read_attr (SPObject * object, const gchar * key)
+sp_object_private_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
-	const gchar * id;
-
 	g_assert (SP_IS_DOCUMENT (object->document));
 	/* fixme: rething that cloning issue */
 	g_assert (SP_OBJECT_IS_CLONED (object) || object->id != NULL);
-	g_assert (key != NULL);
+	g_assert (key != SP_ATTR_INVALID);
 
-	if (strcmp (key, "id") == 0) {
+	switch (key) {
+	case SP_ATTR_ID:
 		if (!SP_OBJECT_IS_CLONED (object)) {
-			id = sp_repr_attr (object->repr, "id");
-			g_assert (id != NULL);
-			if (strcmp (id, object->id) == 0) return;
-			g_assert (!sp_document_lookup_id (object->document, id));
+			g_assert (value != NULL);
+			g_assert (strcmp (value, object->id));
+			g_assert (!sp_document_lookup_id (object->document, value));
 			sp_document_undef_id (object->document, object->id);
 			g_free (object->id);
-			object->id = g_strdup (id);
+			object->id = g_strdup (value);
 			sp_document_def_id (object->document, object->id, object);
 		} else {
 			g_warning ("ID of cloned object changed, so document is out of sync");
 		}
+		break;
+	case SP_ATTR_XML_SPACE:
+		if (value && !strcmp (value, "preserve")) {
+			object->xml_space.value = SP_XML_SPACE_PRESERVE;
+			object->xml_space.set = TRUE;
+		} else if (value && !strcmp (value, "default")) {
+			object->xml_space.value = SP_XML_SPACE_DEFAULT;
+			object->xml_space.set = TRUE;
+		} else if (object->parent) {
+			SPObject *parent;
+			parent = object->parent;
+			object->xml_space.value = parent->xml_space.value;
+		}
+		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+		break;
+	default:
+		break;
 	}
 }
 
 void
-sp_object_invoke_read_attr (SPObject * object, const gchar * key)
+sp_object_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
+	g_assert (object != NULL);
+	g_assert (SP_IS_OBJECT (object));
+
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS (object))->set)
+		((SPObjectClass *) G_OBJECT_GET_CLASS(object))->set (object, key, value);
+}
+
+void
+sp_object_read_attr (SPObject *object, const gchar *key)
+{
+	unsigned int keyid;
+
 	g_assert (object != NULL);
 	g_assert (SP_IS_OBJECT (object));
 	g_assert (key != NULL);
@@ -450,11 +468,15 @@ sp_object_invoke_read_attr (SPObject * object, const gchar * key)
 	/* fixme: rething that cloning issue */
 	g_assert (SP_OBJECT_IS_CLONED (object) || object->id != NULL);
 
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->read_attr)
-		(*((SPObjectClass *)(((GtkObject *) object)->klass))->read_attr) (object, key);
+	keyid = sp_attribute_lookup (key);
+	if (keyid != SP_ATTR_INVALID) {
+		const unsigned char *value;
+		value = sp_repr_attr (object->repr, key);
+		sp_object_set (object, keyid, value);
+	}
 }
 
-static gboolean
+static unsigned int
 sp_object_repr_change_attr (SPRepr *repr, const guchar *key, const guchar *oldval, const guchar *newval, gpointer data)
 {
 	SPObject * object;
@@ -463,6 +485,7 @@ sp_object_repr_change_attr (SPRepr *repr, const guchar *key, const guchar *oldva
 	object = SP_OBJECT (data);
 
 	if (strcmp (key, "id") == 0) {
+		if (!newval) return FALSE;
 		defid = sp_document_lookup_id (object->document, newval);
 		if (defid == object) return TRUE;
 		if (defid) return FALSE;
@@ -478,9 +501,7 @@ sp_object_repr_attr_changed (SPRepr *repr, const guchar *key, const guchar *oldv
 
 	object = SP_OBJECT (data);
 
-	sp_object_invoke_read_attr (object, key);
-
-	sp_document_attr_changed (object->document, object, key, oldval, newval);
+	sp_object_read_attr (object, key);
 }
 
 static void
@@ -490,10 +511,8 @@ sp_object_repr_content_changed (SPRepr *repr, const guchar *oldcontent, const gu
 
 	object = SP_OBJECT (data);
 
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->read_content)
-		(*((SPObjectClass *)(((GtkObject *) object)->klass))->read_content) (object);
-
-	sp_document_content_changed (object->document, object, oldcontent, newcontent);
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->read_content)
+		(*((SPObjectClass *) G_OBJECT_GET_CLASS(object))->read_content) (object);
 }
 
 void
@@ -507,11 +526,6 @@ sp_object_invoke_forall (SPObject *object, SPObjectMethod func, gpointer data)
 
 	func (object, data);
 
-#if 0
-
-	if (((SPObjectClass *) (((GtkObject *) object)->klass))->forall)
-		((SPObjectClass *) (((GtkObject *) object)->klass))->forall (object, func, data);
-#else
 	repr = SP_OBJECT_REPR (object);
 	for (child = repr->children; child != NULL; child = child->next) {
 		const unsigned char *id;
@@ -520,7 +534,20 @@ sp_object_invoke_forall (SPObject *object, SPObjectMethod func, gpointer data)
 		cho = sp_document_lookup_id (SP_OBJECT_DOCUMENT (object), id);
 		if (cho) sp_object_invoke_forall (cho, func, data);
 	}
-#endif
+}
+
+static const gchar*
+sp_xml_get_space_string(unsigned int space)
+{
+	switch(space)
+	{
+	case SP_XML_SPACE_DEFAULT:
+		return "default";
+	case SP_XML_SPACE_PRESERVE:
+		return "preserve";
+	default:
+		return NULL;
+	}
 }
 
 static SPRepr *
@@ -530,8 +557,14 @@ sp_object_private_write (SPObject *object, SPRepr *repr, guint flags)
 		repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
 	} else {
 		sp_repr_set_attr (repr, "id", object->id);
-	}
 
+		if (object->xml_space.set) {
+			const char *xml_space;
+			xml_space = sp_xml_get_space_string(object->xml_space.value);
+			sp_repr_set_attr (repr, "xml:space", xml_space);
+		}
+	}
+	
 	return repr;
 }
 
@@ -541,13 +574,13 @@ sp_object_invoke_write (SPObject *object, SPRepr *repr, guint flags)
 	g_return_val_if_fail (object != NULL, NULL);
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 
-	if (((SPObjectClass *) (((GtkObject *) object)->klass))->write) {
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->write) {
 		if (!(flags & SP_OBJECT_WRITE_BUILD) && !repr) {
 			repr = SP_OBJECT_REPR (object);
 		}
-		return ((SPObjectClass *) (((GtkObject *) object)->klass))->write (object, repr, flags);
+		return ((SPObjectClass *) G_OBJECT_GET_CLASS(object))->write (object, repr, flags);
 	} else {
-		g_warning ("Class %s does not implement ::write", gtk_type_name (GTK_OBJECT_TYPE (object)));
+		g_warning ("Class %s does not implement ::write", G_OBJECT_TYPE_NAME (object));
 		if (!repr) {
 			if (flags & SP_OBJECT_WRITE_BUILD) {
 				repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
@@ -563,9 +596,9 @@ sp_object_invoke_write (SPObject *object, SPRepr *repr, guint flags)
 /* Modification */
 
 void
-sp_object_request_modified (SPObject *object, guint flags)
+sp_object_request_update (SPObject *object, unsigned int flags)
 {
-	gboolean propagate;
+	unsigned int propagate;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (SP_IS_OBJECT (object));
@@ -575,10 +608,96 @@ sp_object_request_modified (SPObject *object, guint flags)
 
 	/* Check for propagate before we set any flags */
 	/* Propagate means, that object is not passed through by modification request cascade yet */
-	propagate = (!(SP_OBJECT_FLAGS (object) & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
+	propagate = (!(object->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
 	/* Just set object flags safe even if some have been set before */
-	SP_OBJECT_SET_FLAGS (object, flags);
+	object->uflags |= flags;
+
+	if (propagate) {
+		if (object->parent) {
+			sp_object_request_update (object->parent, SP_OBJECT_CHILD_MODIFIED_FLAG);
+		} else {
+			sp_document_request_modified (object->document);
+		}
+	}
+}
+
+void
+sp_object_invoke_update (SPObject *object, SPCtx *ctx, unsigned int flags)
+{
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (SP_IS_OBJECT (object));
+	g_return_if_fail (!(flags & ~SP_OBJECT_MODIFIED_CASCADE));
+
+#ifdef SP_OBJECT_DEBUG_CASCADE
+	g_print("Update %s:%s %x %x\n", g_type_name_from_instance ((GTypeInstance *) object), SP_OBJECT_ID (object), flags, object->flags);
+#endif
+
+	/* Get object flags */
+	flags |= object->uflags;
+	/* Copy flags to modified cascade for later processing */
+	object->mflags |= object->uflags;
+	/* We have to clear flags here to allow rescheduling update */
+	object->uflags = 0;
+
+	/* Merge style if we have good reasons to think that parent style is changed */
+	/* I am not sure, whether we should check only propagated flag */
+	/* We are currently assuming, that style parsing is done immediately */
+	/* I think this is correct (Lauris) */
+	if ((flags & SP_OBJECT_STYLE_MODIFIED_FLAG) && (flags & SP_OBJECT_PARENT_MODIFIED_FLAG)) {
+		if (object->style && object->parent) {
+			sp_style_merge_from_parent (object->style, object->parent->style);
+		}
+		/* attribute */
+		sp_object_read_attr (object, "xml:space");
+	}
+
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS (object))->update)
+		((SPObjectClass *) G_OBJECT_GET_CLASS (object))->update (object, ctx, flags);
+}
+
+void
+sp_object_invoke_children_update (SPObject *object, SPCtx *ctx, unsigned int flags)
+{
+	SPObject *child;
+	GSList *l;
+
+	if (flags & SP_OBJECT_MODIFIED_FLAG) flags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
+	flags &= SP_OBJECT_MODIFIED_CASCADE;
+
+	l = NULL;
+	for (child = object->children; child != NULL; child = child->next) {
+		g_object_ref ((GObject *) child);
+		l = g_slist_prepend (l, child);
+	}
+	l = g_slist_reverse (l);
+	while (l) {
+		child = (SPObject *) l->data;
+		l = g_slist_remove (l, child);
+		if (flags || (child->uflags & SP_OBJECT_MODIFIED_FLAG)) {
+			sp_object_invoke_update (child, ctx, flags);
+		}
+		g_object_unref ((GObject *) child);
+	}
+}
+
+void
+sp_object_request_modified (SPObject *object, unsigned int flags)
+{
+	unsigned int propagate;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (SP_IS_OBJECT (object));
+	g_return_if_fail (!(flags & SP_OBJECT_PARENT_MODIFIED_FLAG));
+	g_return_if_fail ((flags & SP_OBJECT_MODIFIED_FLAG) || (flags & SP_OBJECT_CHILD_MODIFIED_FLAG));
+	g_return_if_fail (!((flags & SP_OBJECT_MODIFIED_FLAG) && (flags & SP_OBJECT_CHILD_MODIFIED_FLAG)));
+
+	/* Check for propagate before we set any flags */
+	/* Propagate means, that object is not passed through by modification request cascade yet */
+	propagate = (!(object->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
+
+	/* Just set object flags safe even if some have been set before */
+	object->mflags |= flags;
 
 	if (propagate) {
 		if (object->parent) {
@@ -590,98 +709,144 @@ sp_object_request_modified (SPObject *object, guint flags)
 }
 
 void
-sp_object_modified (SPObject *object, guint flags)
+sp_object_invoke_modified (SPObject *object, unsigned int flags)
 {
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (SP_IS_OBJECT (object));
 	g_return_if_fail (!(flags & ~SP_OBJECT_MODIFIED_CASCADE));
 
-	flags |= (GTK_OBJECT_FLAGS (object) & SP_OBJECT_MODIFIED_STATE);
+#ifdef SP_OBJECT_DEBUG_CASCADE
+	g_print("Modified %s:%s %x %x\n", g_type_name_from_instance ((GTypeInstance *) object), SP_OBJECT_ID (object), flags, object->flags);
+#endif
 
-	g_return_if_fail (flags != 0);
+	/* Get object flags */
+	flags |= object->mflags;
+	/* We have to clear flags here to allow rescheduling modified */
+	object->mflags = 0;
 
-	/* Merge style if we have good reasons to think that parent style is changed */
-	/* I am not sure, whether we should check only propagated flag */
-	/* We are currently assuming, that style parsing is done immediately */
-	/* I think this is correct (Lauris) */
-	if (flags & SP_OBJECT_STYLE_MODIFIED_FLAG) {
-		if (object->style && object->parent) {
-			sp_style_merge_from_parent (object->style, object->parent->style);
-		}
-	}
-
-	gtk_object_ref (GTK_OBJECT (object));
-	gtk_signal_emit (GTK_OBJECT (object), object_signals[MODIFIED], flags);
-	gtk_object_unref (GTK_OBJECT (object));
+	g_object_ref (G_OBJECT (object));
+	g_signal_emit (G_OBJECT (object), object_signals[MODIFIED], 0, flags);
+	g_object_unref (G_OBJECT (object));
 
+#if 0
 	/* If style is modified, invoke style_modified virtual method */
 	/* It is pure convenience, and should be used with caution */
 	/* The cascade is created solely by modified method plus appropriate flag */
 	/* Also, it merely signals, that actual style object has changed */
 	if (flags & SP_OBJECT_STYLE_MODIFIED_FLAG) {
-		if (((SPObjectClass *)(((GtkObject *) object)->klass))->style_modified)
-			(*((SPObjectClass *)(((GtkObject *) object)->klass))->style_modified) (object, flags);
+		if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->style_modified)
+			(*((SPObjectClass *) G_OBJECT_GET_CLASS(object))->style_modified) (object, flags);
 	}
-
-	/*
-	 * fixme: I am not sure - it was before class method, but moved it here
-	 */
-
-	SP_OBJECT_UNSET_FLAGS (object, SP_OBJECT_MODIFIED_STATE);
+#endif
 }
 
-/* Sequence */
-gint
-sp_object_sequence (SPObject *object, gint seq)
+void
+sp_object_invoke_children_modified (SPObject *object, unsigned int flags)
 {
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->sequence)
-		return (*((SPObjectClass *)(((GtkObject *) object)->klass))->sequence) (object, seq);
+	SPObject *child;
+	GSList *l;
 
-	return seq + 1;
+	if (flags & SP_OBJECT_MODIFIED_FLAG) flags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
+	flags &= SP_OBJECT_MODIFIED_CASCADE;
+
+	l = NULL;
+	for (child = object->children; child != NULL; child = child->next) {
+		g_object_ref ((GObject *) child);
+		l = g_slist_prepend (l, child);
+	}
+	l = g_slist_reverse (l);
+	while (l) {
+		child = (SPObject *) l->data;
+		l = g_slist_remove (l, child);
+		if (flags || (child->mflags & SP_OBJECT_MODIFIED_FLAG)) {
+			sp_object_invoke_modified (child, flags);
+		}
+		g_object_unref ((GObject *) child);
+	}
 }
 
-const guchar *
-sp_object_getAttribute (SPObject *object, const guchar *key, SPException *ex)
-{
-	g_return_val_if_fail (object != NULL, NULL);
-	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
-	g_return_val_if_fail (key != NULL, NULL);
-	g_return_val_if_fail (*key != '\0', NULL);
+/* Calculate sequence number of target */
 
+unsigned int
+sp_object_invoke_sequence (SPObject *object, SPObject *target, unsigned int *seq)
+{
+	if (object == target) return TRUE;
+
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS (object))->sequence)
+		return (*((SPObjectClass *) G_OBJECT_GET_CLASS(object))->sequence) (object, target, seq);
+
+	return FALSE;
+}
+
+/*
+ * Get and set descriptive parameters
+ *
+ * These are inefficent, so they are not intended to be used interactively
+ */
+
+const unsigned char *
+sp_object_title_get (SPObject *object)
+{
+	return NULL;
+}
+
+const unsigned char *
+sp_object_description_get (SPObject *object)
+{
+	return NULL;
+}
+
+unsigned int
+sp_object_title_set (SPObject *object, const unsigned char *title)
+{
+	return FALSE;
+}
+
+unsigned int
+sp_object_description_set (SPObject *object, const unsigned char *desc)
+{
+	return FALSE;
+}
+
+const unsigned char *
+sp_object_tagName_get (const SPObject *object, SPException *ex)
+{
 	/* If exception is not clear, return */
 	if (!SP_EXCEPTION_IS_OK (ex)) return NULL;
 
+	/* fixme: Exception if object is NULL? */
+	return sp_repr_name (object->repr);
+}
+
+const unsigned char *
+sp_object_getAttribute (const SPObject *object, const unsigned char *key, SPException *ex)
+{
+	/* If exception is not clear, return */
+	if (!SP_EXCEPTION_IS_OK (ex)) return NULL;
+
+	/* fixme: Exception if object is NULL? */
 	return (const guchar *) sp_repr_attr (object->repr, key);
 }
 
 void
-sp_object_setAttribute (SPObject *object, const guchar *key, const guchar *value, SPException *ex)
+sp_object_setAttribute (SPObject *object, const unsigned char *key, const unsigned char *value, SPException *ex)
 {
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
-	g_return_if_fail (key != NULL);
-	g_return_if_fail (*key != '\0');
-	g_return_if_fail (value != NULL);
-
 	/* If exception is not clear, return */
 	g_return_if_fail (SP_EXCEPTION_IS_OK (ex));
 
+	/* fixme: Exception if object is NULL? */
 	if (!sp_repr_set_attr (object->repr, key, value)) {
 		ex->code = SP_NO_MODIFICATION_ALLOWED_ERR;
 	}
 }
 
 void
-sp_object_removeAttribute (SPObject *object, const guchar *key, SPException *ex)
+sp_object_removeAttribute (SPObject *object, const unsigned char *key, SPException *ex)
 {
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
-	g_return_if_fail (key != NULL);
-	g_return_if_fail (*key != '\0');
-
 	/* If exception is not clear, return */
 	g_return_if_fail (SP_EXCEPTION_IS_OK (ex));
 
+	/* fixme: Exception if object is NULL? */
 	if (!sp_repr_set_attr (object->repr, key, NULL)) {
 		ex->code = SP_NO_MODIFICATION_ALLOWED_ERR;
 	}
@@ -694,6 +859,7 @@ sp_object_get_unique_id (SPObject * object, const gchar * id)
 {
 	static gint count = 0;
 	const gchar * name;
+	const gchar * local;
 	gchar * realid;
 	gchar * b;
 	gint len;
@@ -705,6 +871,12 @@ sp_object_get_unique_id (SPObject * object, const gchar * id)
 
 	name = sp_repr_name (object->repr);
 	g_assert (name != NULL);
+
+	local = strchr (name, ':');
+	if (local) {
+		name = local + 1;
+	}
+
 	len = strlen (name) + 17;
 	b = alloca (len);
 	g_assert (b != NULL);
@@ -763,4 +935,3 @@ sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *d
 
 	return def;
 }
-
