@@ -14,12 +14,16 @@
 #include <string.h>
 #include <libnr/nr-rect.h>
 #include <libnr/nr-matrix.h>
+
+#include <libarikkei/arikkei-strlib.h>
+
 #include <gtk/gtksignal.h>
-#include "helper/nr-plain-stuff.h"
+#include "macros.h"
 #include "xml/repr-private.h"
 #include "svg/svg.h"
 #include "display/nr-arena.h"
 #include "display/nr-arena-group.h"
+#include "attributes.h"
 #include "document.h"
 #include "sp-object-repr.h"
 #include "sp-item.h"
@@ -40,6 +44,7 @@ struct _SPPatPainter {
 	NRMatrixF pcs2px;
 
 	NRArena *arena;
+	unsigned int dkey;
 	NRArenaItem *root;
 };
 
@@ -48,33 +53,38 @@ static void sp_pattern_init (SPPattern *gr);
 
 static void sp_pattern_build (SPObject *object, SPDocument *document, SPRepr *repr);
 static void sp_pattern_release (SPObject *object);
-static void sp_pattern_read_attr (SPObject *object, const gchar *key);
+static void sp_pattern_set (SPObject *object, unsigned int key, const unsigned char *value);
 static void sp_pattern_child_added (SPObject *object, SPRepr *child, SPRepr *ref);
 static void sp_pattern_remove_child (SPObject *object, SPRepr *child);
-static void sp_pattern_modified (SPObject *object, guint flags);
+static void sp_pattern_update (SPObject *object, SPCtx *ctx, unsigned int flags);
+static void sp_pattern_modified (SPObject *object, unsigned int flags);
+static unsigned int sp_pattern_sequence (SPObject *object, SPObject *target, unsigned int *seq);
 
-static void sp_pattern_href_release (SPObject *href, SPPattern *pattern);
+static void sp_pattern_href_destroy (SPObject *href, SPPattern *pattern);
 static void sp_pattern_href_modified (SPObject *href, guint flags, SPPattern *pattern);
 
-static SPPainter *sp_pattern_painter_new (SPPaintServer *ps, const gdouble *affine, const ArtDRect *bbox);
+static SPPainter *sp_pattern_painter_new (SPPaintServer *ps, const gdouble *affine, const NRRectF *bbox);
 static void sp_pattern_painter_free (SPPaintServer *ps, SPPainter *painter);
 
 static SPPaintServerClass * pattern_parent_class;
 
-GtkType
+GType
 sp_pattern_get_type (void)
 {
-	static GtkType pattern_type = 0;
+	static GType pattern_type = 0;
 	if (!pattern_type) {
-		GtkTypeInfo pattern_info = {
-			"SPPattern",
-			sizeof (SPPattern),
+		GTypeInfo pattern_info = {
 			sizeof (SPPatternClass),
-			(GtkClassInitFunc) sp_pattern_class_init,
-			(GtkObjectInitFunc) sp_pattern_init,
-			NULL, NULL, NULL
+			NULL,	/* base_init */
+			NULL,	/* base_finalize */
+			(GClassInitFunc) sp_pattern_class_init,
+			NULL,	/* class_finalize */
+			NULL,	/* class_data */
+			sizeof (SPPattern),
+			16,	/* n_preallocs */
+			(GInstanceInitFunc) sp_pattern_init,
 		};
-		pattern_type = gtk_type_unique (SP_TYPE_PAINT_SERVER, &pattern_info);
+		pattern_type = g_type_register_static (SP_TYPE_PAINT_SERVER, "SPPattern", &pattern_info, 0);
 	}
 	return pattern_type;
 }
@@ -82,22 +92,22 @@ sp_pattern_get_type (void)
 static void
 sp_pattern_class_init (SPPatternClass *klass)
 {
-	GtkObjectClass *gtk_object_class;
 	SPObjectClass *sp_object_class;
 	SPPaintServerClass *ps_class;
 
-	gtk_object_class = (GtkObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 	ps_class = (SPPaintServerClass *) klass;
 
-	pattern_parent_class = gtk_type_class (SP_TYPE_PAINT_SERVER);
+	pattern_parent_class = g_type_class_ref (SP_TYPE_PAINT_SERVER);
 
 	sp_object_class->build = sp_pattern_build;
 	sp_object_class->release = sp_pattern_release;
-	sp_object_class->read_attr = sp_pattern_read_attr;
+	sp_object_class->set = sp_pattern_set;
 	sp_object_class->child_added = sp_pattern_child_added;
 	sp_object_class->remove_child = sp_pattern_remove_child;
+	sp_object_class->update = sp_pattern_update;
 	sp_object_class->modified = sp_pattern_modified;
+	sp_object_class->sequence = sp_pattern_sequence;
 
 	ps_class->painter_new = sp_pattern_painter_new;
 	ps_class->painter_free = sp_pattern_painter_free;
@@ -129,27 +139,31 @@ sp_pattern_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	if (((SPObjectClass *) pattern_parent_class)->build)
 		(* ((SPObjectClass *) pattern_parent_class)->build) (object, document, repr);
 
+	sp_object_read_attr (object, "patternUnits");
+	sp_object_read_attr (object, "patternContentUnits");
+	sp_object_read_attr (object, "patternTransform");
+	sp_object_read_attr (object, "x");
+	sp_object_read_attr (object, "y");
+	sp_object_read_attr (object, "width");
+	sp_object_read_attr (object, "height");
+	sp_object_read_attr (object, "viewBox");
+	sp_object_read_attr (object, "xlink:href");
+
 	last = NULL;
 	for (rchild = repr->children; rchild != NULL; rchild = rchild->next) {
-		GtkType type;
+		GType type;
 		SPObject * child;
 		type = sp_repr_type_lookup (rchild);
-		child = gtk_type_new (type);
-		last ? last->next : pat->children = sp_object_attach_reref (object, child, NULL);
+		child = g_object_new (type, NULL);
+		if (last) {
+			last->next = sp_object_attach_reref (object, child, NULL);
+		} else {
+			object->children = sp_object_attach_reref (object, child, NULL);
+		}
 		sp_object_invoke_build (child, document, rchild, SP_OBJECT_IS_CLONED (object));
 		last = child;
 	}
 
-	sp_pattern_read_attr (object, "patternUnits");
-	sp_pattern_read_attr (object, "patternContentUnits");
-	sp_pattern_read_attr (object, "patternTransform");
-	sp_pattern_read_attr (object, "x");
-	sp_pattern_read_attr (object, "y");
-	sp_pattern_read_attr (object, "width");
-	sp_pattern_read_attr (object, "height");
-	sp_pattern_read_attr (object, "viewBox");
-	sp_pattern_read_attr (object, "xlink:href");
-
 	/* Register ourselves */
 	sp_document_add_resource (document, "pattern", object);
 }
@@ -167,7 +181,7 @@ sp_pattern_release (SPObject *object)
 	}
 
 	if (pat->href) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (pat->href), pat);
+		sp_signal_disconnect_by_data (pat->href, pat);
 		sp_object_hunref (SP_OBJECT (pat->href), object);
 	}
 
@@ -176,19 +190,17 @@ sp_pattern_release (SPObject *object)
 }
 
 static void
-sp_pattern_read_attr (SPObject *object, const gchar *key)
+sp_pattern_set (SPObject *object, unsigned int key, const unsigned char *value)
 {
 	SPPattern *pat;
-	const guchar *val;
 
 	pat = SP_PATTERN (object);
 
-	val = sp_repr_attr (object->repr, key);
-
 	/* fixme: We should unset properties, if val == NULL */
-	if (!strcmp (key, "patternUnits")) {
-		if (val) {
-			if (!strcmp (val, "userSpaceOnUse")) {
+	switch (key) {
+	case SP_ATTR_PATTERNUNITS:
+		if (value) {
+			if (!strcmp (value, "userSpaceOnUse")) {
 				pat->patternUnits = SP_PATTERN_UNITS_USERSPACEONUSE;
 			} else {
 				pat->patternUnits = SP_PATTERN_UNITS_OBJECTBOUNDINGBOX;
@@ -198,9 +210,10 @@ sp_pattern_read_attr (SPObject *object, const gchar *key)
 			pat->patternUnits_set = FALSE;
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "patternContentUnits")) {
-		if (val) {
-			if (!strcmp (val, "userSpaceOnUse")) {
+		break;
+	case SP_ATTR_PATTERNCONTENTUNITS:
+		if (value) {
+			if (!strcmp (value, "userSpaceOnUse")) {
 				pat->patternContentUnits = SP_PATTERN_UNITS_USERSPACEONUSE;
 			} else {
 				pat->patternContentUnits = SP_PATTERN_UNITS_OBJECTBOUNDINGBOX;
@@ -210,49 +223,58 @@ sp_pattern_read_attr (SPObject *object, const gchar *key)
 			pat->patternContentUnits_set = FALSE;
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "patternTransform")) {
-		nr_matrix_d_set_identity (&pat->patternTransform);
-		if (val) {
-			sp_svg_read_affine (pat->patternTransform.c, val);
+		break;
+	case SP_ATTR_PATTERNTRANSFORM: {
+		NRMatrixF t;
+		if (value && sp_svg_transform_read (value, &t)) {
+			int i;
+			for (i = 0; i < 6; i++) pat->patternTransform.c[i] = t.c[i];
 			pat->patternTransform_set = TRUE;
 		} else {
+			nr_matrix_d_set_identity (&pat->patternTransform);
 			pat->patternTransform_set = FALSE;
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "x")) {
-		if (!sp_svg_length_read (val, &pat->x)) {
+		break;
+	}
+	case SP_ATTR_X:
+		if (!sp_svg_length_read (value, &pat->x)) {
 			sp_svg_length_unset (&pat->x, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "y")) {
-		if (!sp_svg_length_read (val, &pat->y)) {
+		break;
+	case SP_ATTR_Y:
+		if (!sp_svg_length_read (value, &pat->y)) {
 			sp_svg_length_unset (&pat->y, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "width")) {
-		if (!sp_svg_length_read (val, &pat->width)) {
+		break;
+	case SP_ATTR_WIDTH:
+		if (!sp_svg_length_read (value, &pat->width)) {
 			sp_svg_length_unset (&pat->width, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "height")) {
-		if (!sp_svg_length_read (val, &pat->height)) {
+		break;
+	case SP_ATTR_HEIGHT:
+		if (!sp_svg_length_read (value, &pat->height)) {
 			sp_svg_length_unset (&pat->height, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "viewBox")) {
+		break;
+	case SP_ATTR_VIEWBOX: {
 		/* fixme: Think (Lauris) */
 		double x, y, width, height;
 		char *eptr;
 
-		if (val) {
-			eptr = (gchar *) val;
-			x = strtod (eptr, &eptr);
+		if (value) {
+			eptr = (gchar *) value;
+			eptr += arikkei_strtod_exp (eptr, 1024, &x);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			y = strtod (eptr, &eptr);
+			eptr += arikkei_strtod_exp (eptr, 1024, &y);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			width = strtod (eptr, &eptr);
+			eptr += arikkei_strtod_exp (eptr, 1024, &width);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			height = strtod (eptr, &eptr);
+			eptr += arikkei_strtod_exp (eptr, 1024, &height);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
 			if ((width > 0) && (height > 0)) {
 				pat->viewBox.x0 = x;
@@ -267,25 +289,28 @@ sp_pattern_read_attr (SPObject *object, const gchar *key)
 			pat->viewBox_set = FALSE;
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
-	} else if (!strcmp (key, "xlink:href")) {
+		break;
+	}
+	case SP_ATTR_XLINK_HREF:
 		if (pat->href) {
-			gtk_signal_disconnect_by_data (GTK_OBJECT (pat->href), pat);
+			sp_signal_disconnect_by_data (pat->href, pat);
 			pat->href = (SPPattern *) sp_object_hunref (SP_OBJECT (pat->href), object);
 		}
-		if (val && *val == '#') {
+		if (value && *value == '#') {
 			SPObject *href;
-			href = sp_document_lookup_id (object->document, val + 1);
+			href = sp_document_lookup_id (object->document, value + 1);
 			if (SP_IS_PATTERN (href)) {
 				pat->href = (SPPattern *) sp_object_href (href, object);
-				gtk_signal_connect (GTK_OBJECT (href), "release", GTK_SIGNAL_FUNC (sp_pattern_href_release), pat);
-				gtk_signal_connect (GTK_OBJECT (href), "modified", GTK_SIGNAL_FUNC (sp_pattern_href_modified), pat);
+				g_signal_connect (G_OBJECT (href), "destroy", G_CALLBACK (sp_pattern_href_destroy), pat);
+				g_signal_connect (G_OBJECT (href), "modified", G_CALLBACK (sp_pattern_href_modified), pat);
 			}
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
-	} else {
-		if (((SPObjectClass *) pattern_parent_class)->read_attr)
-			(* ((SPObjectClass *) pattern_parent_class)->read_attr) (object, key);
+		break;
+	default:
+		if (((SPObjectClass *) pattern_parent_class)->set)
+			((SPObjectClass *) pattern_parent_class)->set (object, key, value);
+		break;
 	}
 }
 
@@ -293,7 +318,7 @@ static void
 sp_pattern_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 {
 	SPPattern *pat;
-	GtkType type;
+	GType type;
 	SPObject *ochild, *prev;
 	gint position;
 
@@ -306,7 +331,7 @@ sp_pattern_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	prev = NULL;
 	position = 0;
 	if (ref != NULL) {
-		prev = pat->children;
+		prev = object->children;
 		while (prev && (prev->repr != ref)) {
 			if (SP_IS_ITEM (prev)) position += 1;
 			prev = prev->next;
@@ -315,11 +340,11 @@ sp_pattern_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	}
 
 	type = sp_repr_type_lookup (child);
-	ochild = gtk_type_new (type);
+	ochild = g_object_new (type, NULL);
 	if (prev) {
 		prev->next = sp_object_attach_reref (object, ochild, prev->next);
 	} else {
-		pat->children = sp_object_attach_reref (object, ochild, pat->children);
+		object->children = sp_object_attach_reref (object, ochild, object->children);
 	}
 
 	sp_object_invoke_build (ochild, object->document, child, SP_OBJECT_IS_CLONED (object));
@@ -333,7 +358,7 @@ sp_pattern_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 			SPPatPainter *pp;
 			NRArenaItem *ai;
 			pp = (SPPatPainter *) p;
-			ai = sp_item_show (SP_ITEM (ochild), pp->arena);
+			ai = sp_item_invoke_show (SP_ITEM (ochild), pp->arena, pp->dkey, SP_ITEM_REFERENCE_FLAGS);
 			if (ai) {
 				nr_arena_item_add_child (pp->root, ai, NULL);
 				nr_arena_item_set_order (ai, position);
@@ -355,7 +380,7 @@ sp_pattern_remove_child (SPObject *object, SPRepr *child)
 		(* ((SPObjectClass *) (pattern_parent_class))->remove_child) (object, child);
 
 	prev = NULL;
-	ochild = pat->children;
+	ochild = object->children;
 	while (ochild->repr != child) {
 		prev = ochild;
 		ochild = ochild->next;
@@ -364,12 +389,42 @@ sp_pattern_remove_child (SPObject *object, SPRepr *child)
 	if (prev) {
 		prev->next = sp_object_detach_unref (object, prev->next);
 	} else {
-		pat->children = sp_object_detach_unref (object, pat->children);
+		object->children = sp_object_detach_unref (object, object->children);
 	}
 }
 
 /* fixme: We need ::order_changed handler too (Lauris) */
 
+/* fixme: Transformation and stuff (Lauris) */
+
+static void
+sp_pattern_update (SPObject *object, SPCtx *ctx, unsigned int flags)
+{
+	SPPattern *pat;
+	SPObject *child;
+	GSList *l;
+
+	pat = SP_PATTERN (object);
+
+	if (flags & SP_OBJECT_MODIFIED_FLAG) flags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
+	flags &= SP_OBJECT_MODIFIED_CASCADE;
+
+	l = NULL;
+	for (child = object->children; child != NULL; child = child->next) {
+		sp_object_ref (child, NULL);
+		l = g_slist_prepend (l, child);
+	}
+	l = g_slist_reverse (l);
+	while (l) {
+		child = SP_OBJECT (l->data);
+		l = g_slist_remove (l, child);
+		if (flags || (child->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
+			sp_object_invoke_update (child, ctx, flags);
+		}
+		sp_object_unref (child, NULL);
+	}
+}
+
 static void
 sp_pattern_modified (SPObject *object, guint flags)
 {
@@ -383,23 +438,39 @@ sp_pattern_modified (SPObject *object, guint flags)
 	flags &= SP_OBJECT_MODIFIED_CASCADE;
 
 	l = NULL;
-	for (child = pat->children; child != NULL; child = child->next) {
-		gtk_object_ref (GTK_OBJECT (child));
+	for (child = object->children; child != NULL; child = child->next) {
+		sp_object_ref (child, NULL);
 		l = g_slist_prepend (l, child);
 	}
 	l = g_slist_reverse (l);
 	while (l) {
 		child = SP_OBJECT (l->data);
 		l = g_slist_remove (l, child);
-		if (flags || (GTK_OBJECT_FLAGS (child) & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
-			sp_object_modified (child, flags);
+		if (flags || (child->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
+			sp_object_invoke_modified (child, flags);
 		}
-		gtk_object_unref (GTK_OBJECT (child));
+		sp_object_unref (child, NULL);
 	}
 }
 
+static unsigned int
+sp_pattern_sequence (SPObject *object, SPObject *target, unsigned int *seq)
+{
+	SPPattern *pat;
+	SPObject *child;
+
+	pat = (SPPattern *) object;
+
+	for (child = object->children; child != NULL; child = child->next) {
+		*seq += 1;
+		if (sp_object_invoke_sequence (child, target, seq)) return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
-sp_pattern_href_release (SPObject *href, SPPattern *pattern)
+sp_pattern_href_destroy (SPObject *href, SPPattern *pattern)
 {
 	pattern->href = (SPPattern *) sp_object_hunref (href, pattern);
 	sp_object_request_modified (SP_OBJECT (pattern), SP_OBJECT_MODIFIED_FLAG);
@@ -413,10 +484,10 @@ sp_pattern_href_modified (SPObject *href, guint flags, SPPattern *pattern)
 
 /* Painter */
 
-static void sp_pat_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint height, gint rowstride);
+static void sp_pat_fill (SPPainter *painter, NRPixBlock *pb);
 
 static SPPainter *
-sp_pattern_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtDRect *bbox)
+sp_pattern_painter_new (SPPaintServer *ps, const gdouble *ctm, const NRRectF *bbox)
 {
 	SPPattern *pat;
 	SPPatPainter *pp;
@@ -521,28 +592,25 @@ sp_pattern_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtDRect *b
 	/* fixme: Create arena */
 	/* fixme: Actually we need some kind of constructor function */
 	/* fixme: But to do that, we need actual arena implementaion */
-	pp->arena = gtk_type_new (NR_TYPE_ARENA);
+	pp->arena = (NRArena *) nr_object_new (NR_TYPE_ARENA);
+
+	pp->dkey = sp_item_display_key_new (1);
 
 	/* fixme: Create group */
 	pp->root = nr_arena_item_new (pp->arena, NR_TYPE_ARENA_GROUP);
 
 	/* fixme: Show items */
 	/* fixme: Among other thing we want to traverse href here */
-	for (child = pat->children; child != NULL; child = child->next) {
+	for (child = ((SPObject *) pat)->children; child != NULL; child = child->next) {
 		if (SP_IS_ITEM (child)) {
 			NRArenaItem *cai;
-			cai = sp_item_show (SP_ITEM (child), pp->arena);
+			cai = sp_item_invoke_show (SP_ITEM (child), pp->arena, pp->dkey, SP_ITEM_REFERENCE_FLAGS);
 			nr_arena_item_append_child (pp->root, cai);
 			nr_arena_item_unref (cai);
 		}
 	}
 
-	gc.affine[0] = pp->pcs2px.c[0];
-	gc.affine[1] = pp->pcs2px.c[1];
-	gc.affine[2] = pp->pcs2px.c[2];
-	gc.affine[3] = pp->pcs2px.c[3];
-	gc.affine[4] = pp->pcs2px.c[4];
-	gc.affine[5] = pp->pcs2px.c[5];
+	nr_matrix_d_from_f (&gc.transform, &pp->pcs2px);
 
 	nr_arena_item_invoke_update (pp->root, NULL, &gc, NR_ARENA_ITEM_STATE_ALL, NR_ARENA_ITEM_STATE_ALL);
 
@@ -560,9 +628,9 @@ sp_pattern_painter_free (SPPaintServer *ps, SPPainter *painter)
 	pat = pp->pat;
 
 	/* fixme: Among other thing we want to traverse href here */
-	for (child = pat->children; child != NULL; child = child->next) {
+	for (child = ((SPObject *) pat)->children; child != NULL; child = child->next) {
 		if (SP_IS_ITEM (child)) {
-			sp_item_hide (SP_ITEM (child), pp->arena);
+			sp_item_invoke_hide (SP_ITEM (child), pp->dkey);
 		}
 	}
 
@@ -571,18 +639,16 @@ sp_pattern_painter_free (SPPaintServer *ps, SPPainter *painter)
 	}
 
 	if (pp->arena) {
-		gtk_object_unref (GTK_OBJECT (pp->arena));
+		nr_object_unref ((NRObject *) pp->arena);
 	}
 
 	g_free (pp);
 }
 
 static void
-sp_pat_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint height, gint rowstride)
+sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 {
 	SPPatPainter *pp;
-	/* fixme: (Lauris) */
-	NRBuffer b;
 	NRRectF ba, psa;
 	NRRectL area;
 	float x, y;
@@ -592,24 +658,13 @@ sp_pat_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint
 	if (pp->pat->width.computed < NR_EPSILON_F) return;
 	if (pp->pat->height.computed < NR_EPSILON_F) return;
 
-	/* Set up buffer */
-	/* fixme: (Lauris) */
-	b.size = NR_SIZE_BIG;
-	b.mode = NR_IMAGE_R8G8B8A8;
-	b.empty = FALSE;
-	b.premul = FALSE;
-	b.w = width;
-	b.h = height;
-	b.rs = rowstride;
-	b.px = px;
-
 	/* Find buffer area in gradient space */
 	/* fixme: This is suboptimal (Lauris) */
 
-	ba.x0 = x0;
-	ba.y0 = y0;
-	ba.x1 = x0 + width;
-	ba.y1 = y0 + height;
+	ba.x0 = pb->area.x0;
+	ba.y0 = pb->area.y0;
+	ba.x1 = pb->area.x1;
+	ba.y1 = pb->area.y1;
 	nr_rect_f_matrix_f_transform (&psa, &ba, &pp->px2ps);
 
 	psa.x0 = floor ((psa.x0 - pp->pat->x.computed) / pp->pat->width.computed);
@@ -619,19 +674,26 @@ sp_pat_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint
 
 	for (y = psa.y0; y < psa.y1; y++) {
 		for (x = psa.x0; x < psa.x1; x++) {
+			NRPixBlock ppb;
 			float psx, psy;
 
 			psx = x * pp->pat->width.computed;
 			psy = y * pp->pat->height.computed;
 
-			area.x0 = x0 - (pp->ps2px.c[0] * psx + pp->ps2px.c[2] * psy);
-			area.y0 = y0 - (pp->ps2px.c[1] * psx + pp->ps2px.c[3] * psy);
-			area.x1 = area.x0 + width;
-			area.y1 = area.y0 + height;
+			area.x0 = pb->area.x0 - (pp->ps2px.c[0] * psx + pp->ps2px.c[2] * psy);
+			area.y0 = pb->area.y0 - (pp->ps2px.c[1] * psx + pp->ps2px.c[3] * psy);
+			area.x1 = area.x0 + pb->area.x1 - pb->area.x0;
+			area.y1 = area.y0 + pb->area.y1 - pb->area.y0;
 
 			/* We do not update here anymore */
 
-			nr_arena_item_invoke_render (pp->root, &area, &b);
+			/* Set up buffer */
+			/* fixme: (Lauris) */
+			nr_pixblock_setup_extern (&ppb, pb->mode, area.x0, area.y0, area.x1, area.y1, NR_PIXBLOCK_PX (pb), pb->rs, FALSE, FALSE);
+
+			nr_arena_item_invoke_render (pp->root, &area, &ppb, 0);
+
+			nr_pixblock_release (&ppb);
 		}
 	}
 }
