@@ -13,27 +13,29 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <libgnome/gnome-defs.h>
+#include "testing.h"
 
-BEGIN_GNOME_DECLS
+#include <glib.h>
+
+G_BEGIN_DECLS
 
 #define SP_TYPE_TEXT (sp_text_get_type ())
-#define SP_TEXT(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_TEXT, SPText))
-#define SP_TEXT_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_TEXT, SPTextClass))
-#define SP_IS_TEXT(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_TEXT))
-#define SP_IS_TEXT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_TEXT))
+#define SP_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_TEXT, SPText))
+#define SP_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_TEXT, SPTextClass))
+#define SP_IS_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_TEXT))
+#define SP_IS_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_TEXT))
 
 #define SP_TYPE_TSPAN (sp_tspan_get_type ())
-#define SP_TSPAN(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_TSPAN, SPTSpan))
-#define SP_TSPAN_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_TSPAN, SPTSpanClass))
-#define SP_IS_TSPAN(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_TSPAN))
-#define SP_IS_TSPAN_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_TSPAN))
+#define SP_TSPAN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_TSPAN, SPTSpan))
+#define SP_TSPAN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_TSPAN, SPTSpanClass))
+#define SP_IS_TSPAN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_TSPAN))
+#define SP_IS_TSPAN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_TSPAN))
 
 #define SP_TYPE_STRING (sp_string_get_type ())
-#define SP_STRING(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_STRING, SPString))
-#define SP_STRING_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_STRING, SPStringClass))
-#define SP_IS_STRING(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_STRING))
-#define SP_IS_STRING_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_STRING))
+#define SP_STRING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_STRING, SPString))
+#define SP_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_STRING, SPStringClass))
+#define SP_IS_STRING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_STRING))
+#define SP_IS_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_STRING))
 
 /* Text specific flags */
 #define SP_TEXT_CONTENT_MODIFIED_FLAG SP_OBJECT_USER_MODIFIED_FLAG_A
@@ -53,10 +55,12 @@ struct _SPLayoutData {
 	SPSVGLength y;
 	SPSVGLength dx;
 	SPSVGLength dy;
-	guint rotate_set : 1;
-	gfloat rotate;
+	unsigned int rotate_set : 1;
+	float rotate;
+	float linespacing;
 };
 
+
 /*
  * The ultimate source of current mess is, that we have to derive string <- chars
  * This will be changed as soon as we have NRArenaGlyphList class
@@ -70,13 +74,22 @@ struct _SPString {
 	SPLayoutData *ly;
 	/* Content */
 	guchar *text;
+
+#ifdef SP_TEXT_NEW_CONTENT
+	/* New content stuff */
+	NRUShort *uchars;
+	unsigned int ulen;
+	unsigned int spstart : 1;
+	unsigned int spend : 1;
+#endif
+
 	NRPointF *p;
 	/* Bookkeeping */
 	guint start;
 	guint length;
 	/* Using current direction and style */
-	ArtDRect bbox;
-	ArtPoint advance;
+	NRRectF bbox;
+	NRPointF advance;
 };
 
 struct _SPStringClass {
@@ -85,7 +98,7 @@ struct _SPStringClass {
 
 #define SP_STRING_TEXT(s) (SP_STRING (s)->text)
 
-GtkType sp_string_get_type (void);
+GType sp_string_get_type (void);
 
 /* SPTSpan */
 
@@ -109,7 +122,7 @@ struct _SPTSpanClass {
 	SPItemClass parent_class;
 };
 
-GtkType sp_tspan_get_type (void);
+GType sp_tspan_get_type (void);
 
 /* SPText */
 
@@ -118,8 +131,6 @@ struct _SPText {
 
 	SPLayoutData ly;
 
-	SPObject *children;
-
 	guint relayout : 1;
 };
 
@@ -129,7 +140,7 @@ struct _SPTextClass {
 
 #define SP_TEXT_CHILD_STRING(c) (SP_IS_TSPAN (c) ? SP_TSPAN_STRING (c) : SP_STRING (c))
 
-GtkType sp_text_get_type (void);
+GType sp_text_get_type (void);
 
 int sp_text_is_empty (SPText *text);
 gchar *sp_text_get_string_multiline (SPText *text);
@@ -137,12 +148,6 @@ void sp_text_set_repr_text_multiline (SPText *text, const guchar *str);
 
 SPCurve *sp_text_normalized_bpath (SPText *text);
 
-#if 0
-/* fixme: Better place for these */
-gint sp_text_font_weight_to_gp (gint weight);
-#define sp_text_font_italic_to_gp(s) ((s) != SP_CSS_FONT_STYLE_NORMAL)
-#endif
-
 /* fixme: Think about these (Lauris) */
 
 SPTSpan *sp_text_append_line (SPText *text);
@@ -158,9 +163,11 @@ gint sp_text_delete (SPText *text, gint start, gint end);
 
 gint sp_text_up (SPText *text, gint pos);
 gint sp_text_down (SPText *text, gint pos);
+gint sp_text_start_of_line (SPText *text, gint pos);
+gint sp_text_end_of_line (SPText *text, gint pos);
 
 void sp_text_get_cursor_coords (SPText *text, gint position, ArtPoint *p0, ArtPoint *p1);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
