@@ -15,10 +15,15 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include <libart_lgpl/art_svp.h>
-#include <libart_lgpl/art_svp_wind.h>
+
+#include <libarikkei/arikkei-strlib.h>
+
 #include <gtk/gtksignal.h>
+
 #include "svg/svg.h"
+
+#include "enums.h"
+#include "attributes.h"
 #include "document.h"
 #include "uri-references.h"
 #include "sp-paint-server.h"
@@ -46,11 +51,9 @@ static SPTextStyle *sp_text_style_duplicate_unset (SPTextStyle *st);
 static guint sp_text_style_write (guchar *p, guint len, SPTextStyle *st);
 static void sp_style_privatize_text (SPStyle *style);
 
-static gint sp_style_property_index (const guchar *str);
-
 static void sp_style_read_ifloat (SPIFloat *val, const guchar *str);
 static void sp_style_read_iscale24 (SPIScale24 *val, const guchar *str);
-static void sp_style_read_ienum (SPIEnum *val, const guchar *str, const SPStyleEnum *dict, gboolean inherit);
+static void sp_style_read_ienum (SPIEnum *val, const guchar *str, const SPStyleEnum *dict, unsigned int inherit);
 static void sp_style_read_istring (SPIString *val, const guchar *str);
 static void sp_style_read_ilength (SPILength *val, const guchar *str);
 static void sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocument *document);
@@ -59,7 +62,7 @@ static void sp_style_read_ifontsize (SPIFontSize *val, const guchar *str);
 #if 0
 static void sp_style_read_pfloat (SPIFloat *val, SPRepr *repr, const guchar *key);
 #endif
-static void sp_style_read_penum (SPIEnum *val, SPRepr *repr, const guchar *key, const SPStyleEnum *dict, gboolean inherit);
+static void sp_style_read_penum (SPIEnum *val, SPRepr *repr, const guchar *key, const SPStyleEnum *dict, unsigned int inherit);
 static void sp_style_read_plength (SPILength *val, SPRepr *repr, const guchar *key);
 static void sp_style_read_pfontsize (SPIFontSize *val, SPRepr *repr, const guchar *key);
 
@@ -73,7 +76,7 @@ static gint sp_style_write_ifontsize (guchar *p, gint len, const guchar *key, SP
 
 static SPColor *sp_style_read_color_cmyk (SPColor *color, const guchar *str);
 
-static void sp_style_paint_clear (SPStyle *style, SPIPaint *paint, gboolean hunref, gboolean unset);
+static void sp_style_paint_clear (SPStyle *style, SPIPaint *paint, unsigned int hunref, unsigned int unset);
 
 #define SPS_READ_IFLOAT_IF_UNSET(v,s) if (!(v)->set) {sp_style_read_ifloat ((v), (s));}
 #define SPS_READ_PFLOAT_IF_UNSET(v,r,k) if (!(v)->set) {sp_style_read_pfloat ((v), (r), (k));}
@@ -93,22 +96,22 @@ struct _SPStyleEnum {
 };
 
 static const SPStyleEnum enum_fill_rule[] = {
-	{"nonzero", ART_WIND_RULE_NONZERO},
-	{"evenodd", ART_WIND_RULE_ODDEVEN},
+	{"nonzero", SP_WIND_RULE_NONZERO},
+	{"evenodd", SP_WIND_RULE_EVENODD},
 	{NULL, -1}
 };
 
 static const SPStyleEnum enum_stroke_linecap[] = {
-	{"butt", ART_PATH_STROKE_CAP_BUTT},
-	{"round", ART_PATH_STROKE_CAP_ROUND},
-	{"square", ART_PATH_STROKE_CAP_SQUARE},
+	{"butt", SP_STROKE_LINECAP_BUTT},
+	{"round", SP_STROKE_LINECAP_ROUND},
+	{"square", SP_STROKE_LINECAP_SQUARE},
 	{NULL, -1}
 };
 
 static const SPStyleEnum enum_stroke_linejoin[] = {
-	{"miter", ART_PATH_STROKE_JOIN_MITER},
-	{"round", ART_PATH_STROKE_JOIN_ROUND},
-	{"bevel", ART_PATH_STROKE_JOIN_BEVEL},
+	{"miter", SP_STROKE_LINEJOIN_MITER},
+	{"round", SP_STROKE_LINEJOIN_ROUND},
+	{"bevel", SP_STROKE_LINEJOIN_BEVEL},
 	{NULL, -1}
 };
 
@@ -188,7 +191,7 @@ static const SPStyleEnum enum_writing_mode[] = {
 };
 
 static void
-sp_style_object_released (GtkObject *object, SPStyle *style)
+sp_style_object_release (SPObject *object, SPStyle *style)
 {
 	style->object = NULL;
 }
@@ -221,7 +224,7 @@ sp_style_new_from_object (SPObject *object)
 	style = sp_style_new ();
 
 	style->object = object;
-	gtk_signal_connect (GTK_OBJECT (object), "release", GTK_SIGNAL_FUNC (sp_style_object_released), style);
+	g_signal_connect (G_OBJECT (object), "release", G_CALLBACK (sp_style_object_release), style);
 
 	return style;
 }
@@ -246,7 +249,8 @@ sp_style_unref (SPStyle *style)
 	style->refcount -= 1;
 
 	if (style->refcount < 1) {
-		if (style->object) gtk_signal_disconnect_by_data (GTK_OBJECT (style->object), style);
+/* 		if (style->object) gtk_signal_disconnect_by_data (GTK_OBJECT (style->object), style); */
+		if (style->object) g_signal_handlers_disconnect_matched (G_OBJECT(style->object), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, style);
 		if (style->text) sp_text_style_unref (style->text);
 		sp_style_paint_clear (style, &style->fill, TRUE, FALSE);
 		sp_style_paint_clear (style, &style->stroke, TRUE, FALSE);
@@ -279,14 +283,26 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	/* 2. Presentation-only attributes */
 	/* CMYK has precedence and can only be presentation attribute */
 	if (!style->fill.set || (style->fill.type == SP_PAINT_TYPE_COLOR)) {
+		/* backwards-compatability */
 		val = sp_repr_attr (repr, "fill-cmyk");
+		if (val) {
+			sp_repr_set_attr (repr, "sodipodi:fill-cmyk", val);
+			sp_repr_set_attr (repr, "fill-cmyk", NULL);
+		}
+		val = sp_repr_attr (repr, "sodipodi:fill-cmyk");
 		if (val && sp_style_read_color_cmyk (&style->fill.value.color, val)) {
 			style->fill.set = TRUE;
 			style->fill.inherit = FALSE;
 		}
 	}
 	if (!style->stroke.set || (style->stroke.type == SP_PAINT_TYPE_COLOR)) {
+		/* backwards-compatability */
 		val = sp_repr_attr (repr, "stroke-cmyk");
+		if (val) {
+			sp_repr_set_attr (repr, "sodipodi:stroke-cmyk", val);
+			sp_repr_set_attr (repr, "stroke-cmyk", NULL);
+		}
+		val = sp_repr_attr (repr, "sodipodi:stroke-cmyk");
 		if (val && sp_style_read_color_cmyk (&style->stroke.value.color, val)) {
 			style->stroke.set = TRUE;
 			style->stroke.inherit = FALSE;
@@ -352,7 +368,7 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	if (!style->stroke_dashoffset_set) {
 		/* fixme */
 		val = sp_repr_attr (repr, "stroke-dashoffset");
-		if (sp_svg_read_number_d (val, &style->stroke_dash.offset)) {
+		if (sp_svg_number_read_d (val, &style->stroke_dash.offset)) {
 			style->stroke_dashoffset_set = TRUE;
 		} else {
 			style->stroke_dashoffset_set = FALSE;
@@ -556,7 +572,7 @@ sp_style_merge_property (SPStyle *style, gint id, const guchar *val)
 	case SP_PROP_STROKE_DASHOFFSET:
 		if (!style->stroke_dashoffset_set) {
 			/* fixme */
-			if (sp_svg_read_number_d (val, &style->stroke_dash.offset)) {
+			if (sp_svg_number_read_d (val, &style->stroke_dash.offset)) {
 				style->stroke_dashoffset_set = TRUE;
 			} else {
 				style->stroke_dashoffset_set = FALSE;
@@ -628,8 +644,8 @@ sp_style_merge_from_style_string (SPStyle *style, const guchar *p)
 		}
 		e = strchr (p, ';');
 		if (!e) {
-			g_warning ("No end marker at style at: %s", p);
 			e = p + strlen (p);
+			if (*e) g_warning ("No end marker at style at: %s", p);
 		}
 		len = MIN (s - p, 4095);
 		if (len < 1) {
@@ -638,7 +654,7 @@ sp_style_merge_from_style_string (SPStyle *style, const guchar *p)
 		}
 		memcpy (c, p, len);
 		c[len] = '\0';
-		idx = sp_style_property_index (c);
+		idx = sp_attribute_lookup (c);
 		if (idx > 0) {
 			len = MIN (e - s - 1, 4095);
 			if (len > 0) memcpy (c, s + 1, len);
@@ -680,10 +696,12 @@ sp_style_merge_from_parent (SPStyle *style, SPStyle *parent)
 		}
 	} else if (style->font_size.type == SP_FONT_SIZE_PERCENTAGE) {
 		/* fixme: SVG and CSS do no specify clearly, whether we should use parent or viewport values here (Lauris) */
+#if 0
 		g_print ("Parent %g PC %g own %g\n",
 			 parent->font_size.computed,
 			 SP_F8_16_TO_FLOAT (style->font_size.value),
 			 style->font_size.computed);
+#endif
 		style->font_size.computed = parent->font_size.computed * SP_F8_16_TO_FLOAT (style->font_size.value);
 	}
 	/* 'font-style' */
@@ -839,10 +857,10 @@ sp_style_merge_ipaint (SPStyle *style, SPIPaint *paint, SPIPaint *parent)
 		paint->value.server = parent->value.server;
 		if (paint->value.server) {
 			sp_object_href (SP_OBJECT (paint->value.server), style);
-			gtk_signal_connect (GTK_OBJECT (paint->value.server), "release",
-					    GTK_SIGNAL_FUNC (sp_style_paint_server_release), style);
-			gtk_signal_connect (GTK_OBJECT (paint->value.server), "modified",
-					    GTK_SIGNAL_FUNC (sp_style_paint_server_modified), style);
+			g_signal_connect (G_OBJECT (paint->value.server), "release",
+					  G_CALLBACK (sp_style_paint_server_release), style);
+			g_signal_connect (G_OBJECT (paint->value.server), "modified",
+					  G_CALLBACK (sp_style_paint_server_modified), style);
 		}
 		break;
 	case SP_PAINT_TYPE_NONE:
@@ -889,14 +907,18 @@ sp_style_write_string (SPStyle *style)
 			gint i;
 			p += g_snprintf (p, c + BMAX - p, "stroke-dasharray:");
 			for (i = 0; i < style->stroke_dash.n_dash; i++) {
-				p += g_snprintf (p, c + BMAX - p, "%g ", style->stroke_dash.dash[i]);
+				unsigned char b[32];
+				sp_svg_number_write_d (b, style->stroke_dash.dash[i], 4, 2, FALSE);
+				p += g_snprintf (p, c + BMAX - p, "%s ", b);
 			}
 			p += g_snprintf (p, c + BMAX - p, ";");
 		}
 	}
 	/* fixme: */
 	if (style->stroke_dashoffset_set) {
-		p += g_snprintf (p, c + BMAX - p, "stroke-dashoffset:%g;", style->stroke_dash.offset);
+		unsigned char b[32];
+		sp_svg_number_write_d (b, style->stroke_dash.offset, 4, 2, FALSE);
+		p += g_snprintf (p, c + BMAX - p, "stroke-dashoffset:%s;", b);
 	}
 	p += sp_style_write_iscale24 (p, c + BMAX - p, "stroke-opacity", &style->stroke_opacity, NULL, SP_STYLE_FLAG_IFSET);
 
@@ -949,14 +971,18 @@ sp_style_write_difference (SPStyle *from, SPStyle *to)
 			gint i;
 			p += g_snprintf (p, c + BMAX - p, "stroke-dasharray:");
 			for (i = 0; i < from->stroke_dash.n_dash; i++) {
-				p += g_snprintf (p, c + BMAX - p, "%g ", from->stroke_dash.dash[i]);
+				unsigned char b[32];
+				sp_svg_number_write_d (b, from->stroke_dash.dash[i], 4, 2, FALSE);
+				p += g_snprintf (p, c + BMAX - p, "%s ", b);
 			}
 			p += g_snprintf (p, c + BMAX - p, ";");
 		}
 	}
 	/* fixme: */
 	if (from->stroke_dashoffset_set) {
-		p += g_snprintf (p, c + BMAX - p, "stroke-dashoffset:%g;", from->stroke_dash.offset);
+		unsigned char b[32];
+		sp_svg_number_write_d (b, from->stroke_dash.offset, 4, 2, FALSE);
+		p += g_snprintf (p, c + BMAX - p, "stroke-dashoffset:%s;", b);
 	}
 	p += sp_style_write_iscale24 (p, c + BMAX - p, "stroke-opacity", &from->stroke_opacity, &to->stroke_opacity, SP_STYLE_FLAG_IFDIFF);
 
@@ -976,7 +1002,7 @@ sp_style_clear (SPStyle *style)
 	SPObject *object;
 	gint refcount;
 	SPTextStyle *text;
-	gboolean text_private;
+	unsigned int text_private;
 
 	g_return_if_fail (style != NULL);
 
@@ -1020,7 +1046,7 @@ sp_style_clear (SPStyle *style)
 	style->fill.type = SP_PAINT_TYPE_COLOR;
 	sp_color_set_rgb_float (&style->fill.value.color, 0.0, 0.0, 0.0);
 	style->fill_opacity.value = SP_SCALE24_MAX;
-	style->fill_rule.value = ART_WIND_RULE_NONZERO;
+	style->fill_rule.value = SP_WIND_RULE_NONZERO;
 
 	style->stroke.set = FALSE;
 	style->stroke.type = SP_PAINT_TYPE_NONE;
@@ -1028,8 +1054,8 @@ sp_style_clear (SPStyle *style)
 	style->stroke_width.set = FALSE;
 	style->stroke_width.unit = SP_CSS_UNIT_NONE;
 	style->stroke_width.computed = 1.0;
-	style->stroke_linecap.value = ART_PATH_STROKE_CAP_BUTT;
-	style->stroke_linejoin.value = ART_PATH_STROKE_JOIN_MITER;
+	style->stroke_linecap.value = SP_STROKE_LINECAP_BUTT;
+	style->stroke_linejoin.value = SP_STROKE_LINEJOIN_MITER;
 	style->stroke_miterlimit.value = 4.0;
 	style->stroke_dash.n_dash = 0;
 	style->stroke_dash.dash = NULL;
@@ -1047,13 +1073,13 @@ sp_style_read_dash (ArtVpathDash *dash, const guchar *str)
 {
 	gint n_dash;
 	gdouble d[64];
-	guchar *e;
+	const unsigned char *e;
 
 	n_dash = 0;
 	e = NULL;
 
 	while (e != str && n_dash < 64) {
-		d[n_dash] = strtod (str, (char **) &e);
+		e = str + arikkei_strtod_simple (str, 1024, &d[n_dash]);
 		if (e != str) {
 			n_dash += 1;
 			str = e;
@@ -1069,7 +1095,7 @@ sp_style_read_dash (ArtVpathDash *dash, const guchar *str)
 }
 
 void
-sp_style_set_fill_color_rgba (SPStyle *style, gfloat r, gfloat g, gfloat b, gfloat a, gboolean fill_set, gboolean opacity_set)
+sp_style_set_fill_color_rgba (SPStyle *style, gfloat r, gfloat g, gfloat b, gfloat a, unsigned int fill_set, unsigned int opacity_set)
 {
 	g_return_if_fail (style != NULL);
 
@@ -1089,7 +1115,7 @@ sp_style_set_fill_color_rgba (SPStyle *style, gfloat r, gfloat g, gfloat b, gflo
 }
 
 void
-sp_style_set_fill_color_cmyka (SPStyle *style, gfloat c, gfloat m, gfloat y, gfloat k, gfloat a, gboolean fill_set, gboolean opacity_set)
+sp_style_set_fill_color_cmyka (SPStyle *style, gfloat c, gfloat m, gfloat y, gfloat k, gfloat a, unsigned int fill_set, unsigned int opacity_set)
 {
 	g_return_if_fail (style != NULL);
 
@@ -1109,7 +1135,7 @@ sp_style_set_fill_color_cmyka (SPStyle *style, gfloat c, gfloat m, gfloat y, gfl
 }
 
 void
-sp_style_set_stroke_color_rgba (SPStyle *style, gfloat r, gfloat g, gfloat b, gfloat a, gboolean stroke_set, gboolean opacity_set)
+sp_style_set_stroke_color_rgba (SPStyle *style, gfloat r, gfloat g, gfloat b, gfloat a, unsigned int stroke_set, unsigned int opacity_set)
 {
 	g_return_if_fail (style != NULL);
 
@@ -1129,7 +1155,7 @@ sp_style_set_stroke_color_rgba (SPStyle *style, gfloat r, gfloat g, gfloat b, gf
 }
 
 void
-sp_style_set_stroke_color_cmyka (SPStyle *style, gfloat c, gfloat m, gfloat y, gfloat k, gfloat a, gboolean stroke_set, gboolean opacity_set)
+sp_style_set_stroke_color_cmyka (SPStyle *style, gfloat c, gfloat m, gfloat y, gfloat k, gfloat a, unsigned int stroke_set, unsigned int opacity_set)
 {
 	g_return_if_fail (style != NULL);
 
@@ -1149,7 +1175,7 @@ sp_style_set_stroke_color_cmyka (SPStyle *style, gfloat c, gfloat m, gfloat y, g
 }
 
 void
-sp_style_set_opacity (SPStyle *style, gfloat opacity, gboolean opacity_set)
+sp_style_set_opacity (SPStyle *style, gfloat opacity, unsigned int opacity_set)
 {
 	g_return_if_fail (style != NULL);
 
@@ -1236,103 +1262,6 @@ sp_text_style_write (guchar *p, guint len, SPTextStyle *st)
 	return d;
 }
 
-typedef struct {
-	gint code;
-	guchar *name;
-} SPStyleProp;
-
-static const SPStyleProp props[] = {
-	/* CSS2 */
-	/* Font */
-	{SP_PROP_FONT, "font"},
-	{SP_PROP_FONT_FAMILY, "font-family"},
-	{SP_PROP_FONT_SIZE, "font-size"},
-	{SP_PROP_FONT_SIZE_ADJUST, "font-size-adjust"},
-	{SP_PROP_FONT_STRETCH, "font-stretch"},
-	{SP_PROP_FONT_STYLE, "font-style"},
-	{SP_PROP_FONT_VARIANT, "font-variant"},
-	{SP_PROP_FONT_WEIGHT, "font-weight"},
-	/* Text */
-	{SP_PROP_DIRECTION, "direction"},
-	{SP_PROP_LETTER_SPACING, "letter-spacing"},
-	{SP_PROP_TEXT_DECORATION, "text-decoration"},
-	{SP_PROP_UNICODE_BIDI, "unicode-bidi"},
-	{SP_PROP_WORD_SPACING, "word-spacing"},
-	/* Misc */
-	{SP_PROP_CLIP, "clip"},
-	{SP_PROP_COLOR, "color"},
-	{SP_PROP_CURSOR, "cursor"},
-	{SP_PROP_DISPLAY, "display"},
-	{SP_PROP_OVERFLOW, "overflow"},
-	{SP_PROP_VISIBILITY, "visibility"},
-	/* SVG */
-	/* Clip/Mask */
-	{SP_PROP_CLIP_PATH, "clip-path"},
-	{SP_PROP_CLIP_RULE, "clip-rule"},
-	{SP_PROP_MASK, "mask"},
-	{SP_PROP_OPACITY, "opacity"},
-	/* Filter */
-	{SP_PROP_ENABLE_BACKGROUND, "enable-background"},
-	{SP_PROP_FILTER, "filter"},
-	{SP_PROP_FLOOD_COLOR, "flood-color"},
-	{SP_PROP_FLOOD_OPACITY, "flood-opacity"},
-	{SP_PROP_LIGHTING_COLOR, "lighting-color"},
-	/* Gradient */
-	{SP_PROP_STOP_COLOR, "stop-color"},
-	{SP_PROP_STOP_OPACITY, "stop-opacity"},
-	/* Interactivity */
-	{SP_PROP_POINTER_EVENTS, "pointer-events"},
-	/* Paint */
-	{SP_PROP_COLOR_INTERPOLATION, "color-interpolation"},
-	{SP_PROP_COLOR_INTERPOLATION_FILTERS, "color-interpolation-filters"},
-	{SP_PROP_COLOR_PROFILE, "color-profile"},
-	{SP_PROP_COLOR_RENDERING, "color-rendering"},
-	{SP_PROP_FILL, "fill"},
-	{SP_PROP_FILL_OPACITY, "fill-opacity"},
-	{SP_PROP_FILL_RULE, "fill-rule"},
-	{SP_PROP_IMAGE_RENDERING, "image-rendering"},
-	{SP_PROP_MARKER, "marker"},
-	{SP_PROP_MARKER_END, "marker-end"},
-	{SP_PROP_MARKER_MID, "marker-mid"},
-	{SP_PROP_MARKER_START, "marker-start"},
-	{SP_PROP_SHAPE_RENDERING, "shape-rendering"},
-	{SP_PROP_STROKE, "stroke"},
-	{SP_PROP_STROKE_DASHARRAY, "stroke-dasharray"},
-	{SP_PROP_STROKE_DASHOFFSET, "stroke-dashoffset"},
-	{SP_PROP_STROKE_LINECAP, "stroke-linecap"},
-	{SP_PROP_STROKE_LINEJOIN, "stroke-linejoin"},
-	{SP_PROP_STROKE_MITERLIMIT, "stroke-miterlimit"},
-	{SP_PROP_STROKE_OPACITY, "stroke-opacity"},
-	{SP_PROP_STROKE_WIDTH, "stroke-width"},
-	{SP_PROP_TEXT_RENDERING, "text-rendering"},
-	/* Text */
-	{SP_PROP_ALIGNMENT_BASELINE, "alignment-baseline"},
-	{SP_PROP_BASELINE_SHIFT, "baseline-shift"},
-	{SP_PROP_DOMINANT_BASELINE, "dominant-baseline"},
-	{SP_PROP_GLYPH_ORIENTATION_HORIZONTAL, "glyph-orientation-horizontal"},
-	{SP_PROP_GLYPH_ORIENTATION_VERTICAL, "glyph-orientation-vertical"},
-	{SP_PROP_KERNING, "kerning"},
-	{SP_PROP_TEXT_ANCHOR, "text-anchor"},
-	{SP_PROP_WRITING_MODE, "writing-mode"},
-	{SP_PROP_INVALID, NULL}
-};
-
-static gint
-sp_style_property_index (const guchar *str)
-{
-	static GHashTable *propdict = NULL;
-
-	if (!propdict) {
-		const SPStyleProp *prop;
-		propdict = g_hash_table_new (g_str_hash, g_str_equal);
-		for (prop = props; prop->code != SP_PROP_INVALID; prop++) {
-			g_hash_table_insert (propdict, prop->name, GINT_TO_POINTER (prop->code));
-		}
-	}
-
-	return GPOINTER_TO_INT (g_hash_table_lookup (propdict, str));
-}
-
 static void
 sp_style_read_ifloat (SPIFloat *val, const guchar *str)
 {
@@ -1341,7 +1270,7 @@ sp_style_read_ifloat (SPIFloat *val, const guchar *str)
 		val->inherit = TRUE;
 	} else {
 		gfloat value;
-		if (sp_svg_read_number_f (str, &value)) {
+		if (sp_svg_number_read_f (str, &value)) {
 			val->set = TRUE;
 			val->inherit = FALSE;
 			val->value = value;
@@ -1357,7 +1286,7 @@ sp_style_read_iscale24 (SPIScale24 *val, const guchar *str)
 		val->inherit = TRUE;
 	} else {
 		gfloat value;
-		if (sp_svg_read_number_f (str, &value)) {
+		if (sp_svg_number_read_f (str, &value)) {
 			val->set = TRUE;
 			val->inherit = FALSE;
 			val->value = SP_SCALE24_FROM_FLOAT (value);
@@ -1367,7 +1296,7 @@ sp_style_read_iscale24 (SPIScale24 *val, const guchar *str)
 }
 
 static void
-sp_style_read_ienum (SPIEnum *val, const guchar *str, const SPStyleEnum *dict, gboolean inherit)
+sp_style_read_ienum (SPIEnum *val, const guchar *str, const SPStyleEnum *dict, unsigned int inherit)
 {
 	if (inherit && !strcmp (str, "inherit")) {
 		val->set = TRUE;
@@ -1410,10 +1339,10 @@ sp_style_read_ilength (SPILength *val, const guchar *str)
 		val->set = TRUE;
 		val->inherit = TRUE;
 	} else {
-		gdouble value;
-		gchar *e;
+		double value;
+		const unsigned char *e;
 		/* fixme: Move this to standard place (Lauris) */
-		value = strtod (str, &e);
+		e = str + arikkei_strtod_simple (str, 1024, &value);
 		if ((const guchar *) e != str) {
 			if (!*e) {
 				/* Userspace */
@@ -1485,10 +1414,10 @@ sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocu
 				paint->type = SP_PAINT_TYPE_PAINTSERVER;
 				paint->value.server = SP_PAINT_SERVER (ps);
 				sp_object_href (SP_OBJECT (paint->value.server), style);
-				gtk_signal_connect (GTK_OBJECT (paint->value.server), "release",
-						    GTK_SIGNAL_FUNC (sp_style_paint_server_release), style);
-				gtk_signal_connect (GTK_OBJECT (paint->value.server), "modified",
-						    GTK_SIGNAL_FUNC (sp_style_paint_server_modified), style);
+				g_signal_connect (G_OBJECT (paint->value.server), "release",
+						  G_CALLBACK (sp_style_paint_server_release), style);
+				g_signal_connect (G_OBJECT (paint->value.server), "modified",
+						  G_CALLBACK (sp_style_paint_server_modified), style);
 				paint->set = TRUE;
 				paint->inherit = FALSE;
 				return;
@@ -1533,9 +1462,9 @@ sp_style_read_ifontsize (SPIFontSize *val, const guchar *str)
 		return;
 	} else {
 		gdouble value;
-		gchar *e;
+		const unsigned char *e;
 		/* fixme: Move this to standard place (Lauris) */
-		value = strtod (str, &e);
+		e = str + arikkei_strtod_simple (str, 1024, &value);
 		if ((const guchar *) e != str) {
 			if (!*e) {
 				/* Userspace */
@@ -1590,7 +1519,7 @@ sp_style_read_pfloat (SPIFloat *val, SPRepr *repr, const guchar *key)
 #endif
 
 static void
-sp_style_read_penum (SPIEnum *val, SPRepr *repr, const guchar *key, const SPStyleEnum *dict, gboolean inherit)
+sp_style_read_penum (SPIEnum *val, SPRepr *repr, const guchar *key, const SPStyleEnum *dict, unsigned int inherit)
 {
 	const guchar *str;
 	str = sp_repr_attr (repr, key);
@@ -1627,7 +1556,9 @@ sp_style_write_ifloat (guchar *p, gint len, const guchar *key, SPIFloat *val, SP
 		if (val->inherit) {
 			return g_snprintf (p, len, "%s:inherit;", key);
 		} else {
-			return g_snprintf (p, len, "%s:%g;", key, val->value);
+			char b[32];
+			sp_svg_number_write_d (b, val->value, 6, 2, FALSE);
+			return g_snprintf (p, len, "%s:%s;", key, b);
 		}
 	}
 	return 0;
@@ -1641,7 +1572,9 @@ sp_style_write_iscale24 (guchar *p, gint len, const guchar *key, SPIScale24 *val
 		if (val->inherit) {
 			return g_snprintf (p, len, "%s:inherit;", key);
 		} else {
-			return g_snprintf (p, len, "%s:%g;", key, SP_SCALE24_TO_FLOAT (val->value));
+			char b[32];
+			sp_svg_number_write_d (b, SP_SCALE24_TO_FLOAT (val->value), 6, 2, FALSE);
+			return g_snprintf (p, len, "%s:%s;", key, b);
 		}
 	}
 	return 0;
@@ -1676,7 +1609,7 @@ sp_style_write_istring (guchar *p, gint len, const guchar *key, SPIString *val,
 	return 0;
 }
 
-static gboolean
+static unsigned int
 sp_length_differ (SPILength *a, SPILength *b)
 {
 	if (a->unit != b->unit) {
@@ -1699,47 +1632,62 @@ sp_style_write_ilength (guchar *p, gint len, const guchar *key, SPILength *val,
 		if (val->inherit) {
 			return g_snprintf (p, len, "%s:inherit;", key);
 		} else {
+			float r;
+			const unsigned char *u;
+			unsigned char b[32];
 			switch (val->unit) {
 			case SP_CSS_UNIT_NONE:
-				return g_snprintf (p, len, "%s:%g;", key, val->computed);
+				r = val->computed;
+				u = "";
 				break;
 			case SP_CSS_UNIT_PX:
-				return g_snprintf (p, len, "%s:%gpx;", key, val->computed);
+				r = val->computed;
+				u = "px";
 				break;
 			case SP_CSS_UNIT_PT:
-				return g_snprintf (p, len, "%s:%gpt;", key, val->computed / 1.25);
+				r = val->computed / 1.25;
+				u = "pt";
 				break;
 			case SP_CSS_UNIT_PC:
-				return g_snprintf (p, len, "%s:%gpc;", key, val->computed / 15.0);
+				r = val->computed / 15.0;
+				u = "pc";
 				break;
 			case SP_CSS_UNIT_MM:
-				return g_snprintf (p, len, "%s:%gmm;", key, val->computed / 3.543307);
+				r = val->computed / 3.543307;
+				u = "mm";
 				break;
 			case SP_CSS_UNIT_CM:
-				return g_snprintf (p, len, "%s:%gmm;", key, val->computed / 35.43307);
+				r = val->computed / 35.43307;
+				u = "cm";
 				break;
 			case SP_CSS_UNIT_IN:
-				return g_snprintf (p, len, "%s:%gin;", key, val->computed / 90.0);
+				r = val->computed / 90.0;
+				u = "in";
 				break;
 			case SP_CSS_UNIT_EM:
-				return g_snprintf (p, len, "%s:%gem;", key, val->value);
+				r = val->value;
+				u = "em";
 				break;
 			case SP_CSS_UNIT_EX:
-				return g_snprintf (p, len, "%s:%gex;", key, val->value);
+				r = val->value;
+				u = "ex";
 				break;
 			case SP_CSS_UNIT_PERCENT:
-				return g_snprintf (p, len, "%s:%g%%;", key, val->value);
+				r = val->value;
+				u = "%";
 				break;
 			default:
 				/* Invalid */
 				break;
 			}
+			sp_svg_number_write_d (b, r, 6, 2, FALSE);
+			return g_snprintf (p, len, "%s:%s;", key, b);
 		}
 	}
 	return 0;
 }
 
-static gboolean
+static unsigned int
 sp_paint_differ (SPIPaint *a, SPIPaint *b)
 {
 	if (a->type != b->type) return TRUE;
@@ -1751,7 +1699,7 @@ sp_paint_differ (SPIPaint *a, SPIPaint *b)
 static gint
 sp_style_write_ipaint (guchar *b, gint len, const guchar *key, SPIPaint *paint, SPIPaint *base, guint flags)
 {
-	gboolean set;
+	unsigned int set;
 
 	set = FALSE;
 	if (((flags & SP_STYLE_FLAG_IFSET) && paint->set) ||
@@ -1777,7 +1725,7 @@ sp_style_write_ipaint (guchar *b, gint len, const guchar *key, SPIPaint *paint,
 	return 0;
 }
 
-static gboolean
+static unsigned int
 sp_fontsize_differ (SPIFontSize *a, SPIFontSize *b)
 {
 	if (a->type != b->type) return TRUE;
@@ -1804,9 +1752,13 @@ sp_style_write_ifontsize (guchar *p, gint len, const guchar *key, SPIFontSize *v
 				}
 			}
 		} else if (val->type == SP_FONT_SIZE_LENGTH) {
-			return g_snprintf (p, len, "%s:%g;", key, val->computed);
+			unsigned char b[32];
+			sp_svg_number_write_d (b, val->computed, 4, 2, FALSE);
+			return g_snprintf (p, len, "%s:%s;", key, b);
 		} else if (val->type == SP_FONT_SIZE_PERCENTAGE) {
-			return g_snprintf (p, len, "%s:%g%%;", key, SP_F8_16_TO_FLOAT (val->value) * 100.0);
+			unsigned char b[32];
+			sp_svg_number_write_d (b, SP_F8_16_TO_FLOAT (val->value) * 100.0, 4, 2, FALSE);
+			return g_snprintf (p, len, "%s:%s%%;", key, b);
 		}
 	}
 	return 0;
@@ -1819,23 +1771,23 @@ sp_style_write_ifontsize (guchar *p, gint len, const guchar *key, SPIFontSize *v
 static SPColor *
 sp_style_read_color_cmyk (SPColor *color, const guchar *str)
 {
-	gdouble c, m, y, k;
+	double c, m, y, k;
 	gchar *cptr, *eptr;
 
 	g_return_val_if_fail (str != NULL, NULL);
 
 	c = m = y = k = 0.0;
 	cptr = (gchar *) str + 1;
-	c = strtod (cptr, &eptr);
+	eptr = cptr + arikkei_strtod_simple (cptr, 1024, &c);
 	if (eptr && (eptr != cptr)) {
 		cptr = eptr;
-		m = strtod (cptr, &eptr);
+		eptr = cptr + arikkei_strtod_simple (cptr, 1024, &m);
 		if (eptr && (eptr != cptr)) {
 			cptr = eptr;
-			y = strtod (cptr, &eptr);
+			eptr = cptr + arikkei_strtod_simple (cptr, 1024, &y);
 			if (eptr && (eptr != cptr)) {
 				cptr = eptr;
-				k = strtod (cptr, &eptr);
+				eptr = cptr + arikkei_strtod_simple (cptr, 1024, &k);
 				if (eptr && (eptr != cptr)) {
 					sp_color_set_cmyk_float (color, c, m, y, k);
 					return color;
@@ -1848,11 +1800,12 @@ sp_style_read_color_cmyk (SPColor *color, const guchar *str)
 }
 
 static void
-sp_style_paint_clear (SPStyle *style, SPIPaint *paint, gboolean hunref, gboolean unset)
+sp_style_paint_clear (SPStyle *style, SPIPaint *paint, unsigned int hunref, unsigned int unset)
 {
 	if ((paint->type == SP_PAINT_TYPE_PAINTSERVER) && paint->value.server) {
 		sp_object_hunref (SP_OBJECT (paint->value.server), style);
-		gtk_signal_disconnect_by_data (GTK_OBJECT (paint->value.server), style);
+/*  		gtk_signal_disconnect_by_data (GTK_OBJECT (paint->value.server), style); */
+		g_signal_handlers_disconnect_matched (G_OBJECT(paint->value.server), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, style);
 		paint->value.server = NULL;
 		paint->type = SP_PAINT_TYPE_NONE;
 	}
