@@ -14,6 +14,7 @@
 
 #include <libnr/nr-pixblock-pattern.h>
 #include <gtk/gtksignal.h>
+#include "macros.h"
 #include "../helper/nr-plain-stuff.h"
 #include "../helper/nr-plain-stuff-gdk.h"
 #include "gradient-image.h"
@@ -30,7 +31,7 @@ static void sp_gradient_image_size_request (GtkWidget *widget, GtkRequisition *r
 static void sp_gradient_image_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gint sp_gradient_image_expose (GtkWidget *widget, GdkEventExpose *event);
 
-static void sp_gradient_image_gradient_destroy (SPGradient *gr, SPGradientImage *im);
+static void sp_gradient_image_gradient_release (SPGradient *gr, SPGradientImage *im);
 static void sp_gradient_image_gradient_modified (SPGradient *gr, guint flags, SPGradientImage *im);
 static void sp_gradient_image_update (SPGradientImage *img);
 
@@ -91,7 +92,7 @@ sp_gradient_image_destroy (GtkObject *object)
 	image = SP_GRADIENT_IMAGE (object);
 
 	if (image->gradient) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (image->gradient), image);
+  		sp_signal_disconnect_by_data (image->gradient, image);
 		image->gradient = NULL;
 	}
 
@@ -216,26 +217,24 @@ void
 sp_gradient_image_set_gradient (SPGradientImage *image, SPGradient *gradient)
 {
 	if (image->gradient) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (image->gradient), image);
+  		sp_signal_disconnect_by_data (image->gradient, image);
 	}
 
 	image->gradient = gradient;
 
 	if (gradient) {
-		gtk_signal_connect (GTK_OBJECT (gradient), "destroy",
-				    GTK_SIGNAL_FUNC (sp_gradient_image_gradient_destroy), image);
-		gtk_signal_connect (GTK_OBJECT (gradient), "modified",
-				    GTK_SIGNAL_FUNC (sp_gradient_image_gradient_modified), image);
+		g_signal_connect (G_OBJECT (gradient), "release", G_CALLBACK (sp_gradient_image_gradient_release), image);
+		g_signal_connect (G_OBJECT (gradient), "modified", G_CALLBACK (sp_gradient_image_gradient_modified), image);
 	}
 
 	sp_gradient_image_update (image);
 }
 
 static void
-sp_gradient_image_gradient_destroy (SPGradient *gradient, SPGradientImage *image)
+sp_gradient_image_gradient_release (SPGradient *gradient, SPGradientImage *image)
 {
 	if (image->gradient) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (image->gradient), image);
+  		sp_signal_disconnect_by_data (image->gradient, image);
 	}
 
 	image->gradient = NULL;
