@@ -12,7 +12,12 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#include <libarikkei/arikkei-strlib.h>
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
@@ -22,8 +27,8 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkwindow.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
+#include "macros.h"
+#include "../helper/sp-intl.h"
 #include "../widgets/gradient-image.h"
 #include "../sodipodi.h"
 #include "../document-private.h"
@@ -40,8 +45,8 @@ static void sp_gradient_vector_selector_class_init (SPGradientVectorSelectorClas
 static void sp_gradient_vector_selector_init (SPGradientVectorSelector *gvs);
 static void sp_gradient_vector_selector_destroy (GtkObject *object);
 
-static void sp_gvs_gradient_destroy (SPGradient *gr, SPGradientVectorSelector *gvs);
-static void sp_gvs_defs_destroy (SPObject *defs, SPGradientVectorSelector *gvs);
+static void sp_gvs_gradient_release (SPGradient *gr, SPGradientVectorSelector *gvs);
+static void sp_gvs_defs_release (SPObject *defs, SPGradientVectorSelector *gvs);
 static void sp_gvs_defs_modified (SPObject *defs, guint flags, SPGradientVectorSelector *gvs);
 
 static void sp_gvs_rebuild_gui_full (SPGradientVectorSelector *gvs);
@@ -79,12 +84,11 @@ sp_gradient_vector_selector_class_init (SPGradientVectorSelectorClass *klass)
 
 	signals[VECTOR_SET] = gtk_signal_new ("vector_set",
 					      GTK_RUN_LAST,
-					      object_class->type,
+					      GTK_CLASS_TYPE(object_class),
 					      GTK_SIGNAL_OFFSET (SPGradientVectorSelectorClass, vector_set),
 					      gtk_marshal_NONE__POINTER,
 					      GTK_TYPE_NONE, 1,
 					      GTK_TYPE_POINTER);
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	object_class->destroy = sp_gradient_vector_selector_destroy;
 }
@@ -110,12 +114,12 @@ sp_gradient_vector_selector_destroy (GtkObject *object)
 	gvs = SP_GRADIENT_VECTOR_SELECTOR (object);
 
 	if (gvs->gr) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (gvs->gr), gvs);
+  		sp_signal_disconnect_by_data (gvs->gr, gvs);
 		gvs->gr = NULL;
 	}
 
 	if (gvs->doc) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DOCUMENT_DEFS (gvs->doc)), gvs);
+  		sp_signal_disconnect_by_data (SP_DOCUMENT_DEFS (gvs->doc), gvs);
 		gvs->doc = NULL;
 	}
 
@@ -160,25 +164,26 @@ sp_gradient_vector_selector_set_gradient (SPGradientVectorSelector *gvs, SPDocum
 	if (doc != gvs->doc) {
 		/* Disconnect signals */
 		if (gvs->gr) {
-			gtk_signal_disconnect_by_data (GTK_OBJECT (gvs->gr), gvs);
+  			sp_signal_disconnect_by_data (gvs->gr, gvs);
+		g_signal_handlers_disconnect_matched (G_OBJECT(gvs->gr), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gvs);
 			gvs->gr = NULL;
 		}
 		if (gvs->doc) {
-			gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DOCUMENT_DEFS (gvs->doc)), gvs);
+  			sp_signal_disconnect_by_data (SP_DOCUMENT_DEFS (gvs->doc), gvs);
 			gvs->doc = NULL;
 		}
 		/* Connect signals */
 		if (doc) {
-			gtk_signal_connect (GTK_OBJECT (SP_DOCUMENT_DEFS (doc)), "destroy", GTK_SIGNAL_FUNC (sp_gvs_defs_destroy), gvs);
-			gtk_signal_connect (GTK_OBJECT (SP_DOCUMENT_DEFS (doc)), "modified", GTK_SIGNAL_FUNC (sp_gvs_defs_modified), gvs);
+			g_signal_connect (G_OBJECT (SP_DOCUMENT_DEFS (doc)), "release", G_CALLBACK (sp_gvs_defs_release), gvs);
+			g_signal_connect (G_OBJECT (SP_DOCUMENT_DEFS (doc)), "modified", G_CALLBACK (sp_gvs_defs_modified), gvs);
 		}
 		if (gr) {
-			gtk_signal_connect (GTK_OBJECT (gr), "destroy", GTK_SIGNAL_FUNC (sp_gvs_gradient_destroy), gvs);
+			g_signal_connect (G_OBJECT (gr), "release", G_CALLBACK (sp_gvs_gradient_release), gvs);
 		}
 		gvs->doc = doc;
 		gvs->gr = gr;
 		sp_gvs_rebuild_gui_full (gvs);
-		if (!suppress) gtk_signal_emit (GTK_OBJECT (gvs), signals[VECTOR_SET], gr);
+		if (!suppress) g_signal_emit (G_OBJECT (gvs), signals[VECTOR_SET], 0, gr);
 	} else if (gr != gvs->gr) {
 		/* Harder case - keep document, rebuild menus and stuff */
 		/* fixme: (Lauris) */
@@ -186,7 +191,7 @@ sp_gradient_vector_selector_set_gradient (SPGradientVectorSelector *gvs, SPDocum
 		sp_gradient_vector_selector_set_gradient (gvs, NULL, NULL);
 		sp_gradient_vector_selector_set_gradient (gvs, doc, gr);
 		suppress = FALSE;
-		gtk_signal_emit (GTK_OBJECT (gvs), signals[VECTOR_SET], gr);
+		g_signal_emit (G_OBJECT (gvs), signals[VECTOR_SET], 0, gr);
 	}
 	/* The case of setting NULL -> NULL is not very interesting */
 }
@@ -270,8 +275,8 @@ sp_gvs_rebuild_gui_full (SPGradientVectorSelector *gvs)
 			/* Gradient name change */
 			i = gtk_menu_item_new ();
 			gtk_widget_show (i);
-			gtk_object_set_data (GTK_OBJECT (i), "gradient", gr);
-			gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_gvs_gradient_activate), gvs);
+			g_object_set_data (G_OBJECT (i), "gradient", gr);
+			g_signal_connect (G_OBJECT (i), "activate", G_CALLBACK (sp_gvs_gradient_activate), gvs);
 
 			w = sp_gradient_image_new (gr);
 			gtk_widget_show (w);
@@ -308,7 +313,7 @@ sp_gvs_gradient_activate (GtkMenuItem *mi, SPGradientVectorSelector *gvs)
 {
 	SPGradient *gr, *norm;
 
-	gr = gtk_object_get_data (GTK_OBJECT (mi), "gradient");
+	gr = g_object_get_data (G_OBJECT (mi), "gradient");
 	/* Hmmm... bad things may happen here, if actual gradient is something new */
 	/* Namely - menuitems etc. will be fucked up */
 	/* Hmmm - probably we can just re-set it as menuitem data (Lauris) */
@@ -319,24 +324,24 @@ sp_gvs_gradient_activate (GtkMenuItem *mi, SPGradientVectorSelector *gvs)
 	if (norm != gr) {
 		g_print ("SPGradientVectorSelector: become %s after normalization\n", SP_OBJECT_ID (norm));
 		/* But be careful that we do not have gradient saved anywhere else */
-		gtk_object_set_data (GTK_OBJECT (mi), "gradient", norm);
+		g_object_set_data (G_OBJECT (mi), "gradient", norm);
 	}
 
 	/* fixme: Really we would want to use _set_vector */
 	/* Detach old */
 	if (gvs->gr) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (gvs->gr), gvs);
+  		sp_signal_disconnect_by_data (gvs->gr, gvs);
 		gvs->gr = NULL;
 	}
 	/* Attach new */
 	if (norm) {
-		gtk_signal_connect (GTK_OBJECT (norm), "destroy", GTK_SIGNAL_FUNC (sp_gvs_gradient_destroy), gvs);
+		g_signal_connect (G_OBJECT (norm), "release", G_CALLBACK (sp_gvs_gradient_release), gvs);
 		/* fixme: Connect 'modified'? (Lauris) */
 		/* fixme: I think we do not need it (Lauris) */
 		gvs->gr = norm;
 	}
 
-	gtk_signal_emit (GTK_OBJECT (gvs), signals[VECTOR_SET], norm);
+	g_signal_emit (G_OBJECT (gvs), signals[VECTOR_SET], 0, norm);
 
 	if (norm != gr) {
 		/* We do extra undo push here */
@@ -346,7 +351,7 @@ sp_gvs_gradient_activate (GtkMenuItem *mi, SPGradientVectorSelector *gvs)
 }
 
 static void
-sp_gvs_gradient_destroy (SPGradient *gr, SPGradientVectorSelector *gvs)
+sp_gvs_gradient_release (SPGradient *gr, SPGradientVectorSelector *gvs)
 {
 	/* fixme: (Lauris) */
 	/* fixme: Not sure, what to do here (Lauris) */
@@ -354,12 +359,12 @@ sp_gvs_gradient_destroy (SPGradient *gr, SPGradientVectorSelector *gvs)
 }
 
 static void
-sp_gvs_defs_destroy (SPObject *defs, SPGradientVectorSelector *gvs)
+sp_gvs_defs_release (SPObject *defs, SPGradientVectorSelector *gvs)
 {
 	gvs->doc = NULL;
 	/* Disconnect gradient as well */
 	if (gvs->gr) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (gvs->gr), gvs);
+  		sp_signal_disconnect_by_data (gvs->gr, gvs);
 		gvs->gr = NULL;
 	}
 
@@ -387,7 +392,7 @@ static void sp_gradient_vector_dialog_close (GtkWidget *widget, GtkWidget *dialo
 static gint sp_gradient_vector_dialog_delete (GtkWidget *widget, GdkEvent *event, GtkWidget *dialog);
 
 static void sp_gradient_vector_widget_destroy (GtkObject *object, gpointer data);
-static void sp_gradient_vector_gradient_destroy (SPGradient *gradient, GtkWidget *widget);
+static void sp_gradient_vector_gradient_release (SPGradient *gradient, GtkWidget *widget);
 static void sp_gradient_vector_gradient_modified (SPGradient *gradient, guint flags, GtkWidget *widget);
 static void sp_gradient_vector_color_dragged (SPColorSelector *csel, GtkObject *object);
 static void sp_gradient_vector_color_changed (SPColorSelector *csel, GtkObject *object);
@@ -402,11 +407,10 @@ sp_gradient_vector_widget_new (SPGradient *gradient)
 	g_return_val_if_fail (!gradient || SP_IS_GRADIENT (gradient), NULL);
 
 	vb = gtk_vbox_new (FALSE, PAD);
-	gtk_signal_connect (GTK_OBJECT (vb), "destroy",
-			    GTK_SIGNAL_FUNC (sp_gradient_vector_widget_destroy), NULL);
+	g_signal_connect (G_OBJECT (vb), "destroy", G_CALLBACK (sp_gradient_vector_widget_destroy), NULL);
 
 	w = sp_gradient_image_new (gradient);
-	gtk_object_set_data (GTK_OBJECT (vb), "preview", w);
+	g_object_set_data (G_OBJECT (vb), "preview", w);
 	gtk_widget_show (w);
 	gtk_box_pack_start (GTK_BOX (vb), w, TRUE, TRUE, PAD);
 
@@ -414,25 +418,21 @@ sp_gradient_vector_widget_new (SPGradient *gradient)
 	gtk_widget_show (f);
 	gtk_box_pack_start (GTK_BOX (vb), f, TRUE, TRUE, PAD);
 	csel = sp_color_selector_new ();
-	gtk_object_set_data (GTK_OBJECT (vb), "start", csel);
+	g_object_set_data (G_OBJECT (vb), "start", csel);
 	gtk_widget_show (csel);
 	gtk_container_add (GTK_CONTAINER (f), csel);
-	gtk_signal_connect (GTK_OBJECT (csel), "dragged",
-			    GTK_SIGNAL_FUNC (sp_gradient_vector_color_dragged), vb);
-	gtk_signal_connect (GTK_OBJECT (csel), "changed",
-			    GTK_SIGNAL_FUNC (sp_gradient_vector_color_changed), vb);
+	g_signal_connect (G_OBJECT (csel), "dragged", G_CALLBACK (sp_gradient_vector_color_dragged), vb);
+	g_signal_connect (G_OBJECT (csel), "changed", G_CALLBACK (sp_gradient_vector_color_changed), vb);
 
 	f = gtk_frame_new (_("End color"));
 	gtk_widget_show (f);
 	gtk_box_pack_start (GTK_BOX (vb), f, TRUE, TRUE, PAD);
 	csel = sp_color_selector_new ();
-	gtk_object_set_data (GTK_OBJECT (vb), "end", csel);
+	g_object_set_data (G_OBJECT (vb), "end", csel);
 	gtk_widget_show (csel);
 	gtk_container_add (GTK_CONTAINER (f), csel);
-	gtk_signal_connect (GTK_OBJECT (csel), "dragged",
-			    GTK_SIGNAL_FUNC (sp_gradient_vector_color_dragged), vb);
-	gtk_signal_connect (GTK_OBJECT (csel), "changed",
-			    GTK_SIGNAL_FUNC (sp_gradient_vector_color_changed), vb);
+	g_signal_connect (G_OBJECT (csel), "dragged", G_CALLBACK (sp_gradient_vector_color_dragged), vb);
+	g_signal_connect (G_OBJECT (csel), "changed", G_CALLBACK (sp_gradient_vector_color_changed), vb);
 
 	gtk_widget_show (vb);
 
@@ -451,15 +451,15 @@ sp_gradient_vector_editor_new (SPGradient *gradient)
 		dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 		gtk_window_set_title (GTK_WINDOW (dialog), _("Gradient vector"));
 		gtk_container_set_border_width (GTK_CONTAINER (dialog), PAD);
-		gtk_signal_connect (GTK_OBJECT (dialog), "delete_event", GTK_SIGNAL_FUNC (sp_gradient_vector_dialog_delete), dialog);
+		g_signal_connect (G_OBJECT (dialog), "delete_event", G_CALLBACK (sp_gradient_vector_dialog_delete), dialog);
 		w = sp_gradient_vector_widget_new (gradient);
-		gtk_object_set_data (GTK_OBJECT (dialog), "gradient-vector-widget", w);
+		g_object_set_data (G_OBJECT (dialog), "gradient-vector-widget", w);
 		/* Connect signals */
 		gtk_widget_show (w);
 		gtk_container_add (GTK_CONTAINER (dialog), w);
 	} else {
 		GtkWidget *w;
-		w = gtk_object_get_data (GTK_OBJECT (dialog), "gradient-vector-widget");
+		w = g_object_get_data (G_OBJECT (dialog), "gradient-vector-widget");
 		sp_gradient_vector_widget_load_gradient (w, gradient);
 	}
 
@@ -473,20 +473,18 @@ sp_gradient_vector_widget_load_gradient (GtkWidget *widget, SPGradient *gradient
 	GtkWidget *w;
 	guint32 cs, ce;
 
-	old = gtk_object_get_data (GTK_OBJECT (widget), "gradient");
+	old = g_object_get_data (G_OBJECT (widget), "gradient");
 	if (old != gradient) {
 		if (old) {
-			gtk_signal_disconnect_by_data (GTK_OBJECT (old), widget);
+  			sp_signal_disconnect_by_data (old, widget);
 		}
 		if (gradient) {
-			gtk_signal_connect (GTK_OBJECT (gradient), "destroy",
-					    GTK_SIGNAL_FUNC (sp_gradient_vector_gradient_destroy), widget);
-			gtk_signal_connect (GTK_OBJECT (gradient), "modified",
-					    GTK_SIGNAL_FUNC (sp_gradient_vector_gradient_modified), widget);
+			g_signal_connect (G_OBJECT (gradient), "release", G_CALLBACK (sp_gradient_vector_gradient_release), widget);
+			g_signal_connect (G_OBJECT (gradient), "modified", G_CALLBACK (sp_gradient_vector_gradient_modified), widget);
 		}
 	}
 
-	gtk_object_set_data (GTK_OBJECT (widget), "gradient", gradient);
+	g_object_set_data (G_OBJECT (widget), "gradient", gradient);
 
 	if (gradient) {
 		sp_gradient_ensure_vector (gradient);
@@ -494,16 +492,16 @@ sp_gradient_vector_widget_load_gradient (GtkWidget *widget, SPGradient *gradient
 		ce = sp_color_get_rgba32_falpha (&gradient->vector->stops[1].color, gradient->vector->stops[1].opacity);
 
 		/* Set color selector values */
-		w = gtk_object_get_data (GTK_OBJECT (widget), "start");
+		w = g_object_get_data (G_OBJECT (widget), "start");
 		sp_color_selector_set_any_rgba32 (SP_COLOR_SELECTOR (w), cs);
-		w = gtk_object_get_data (GTK_OBJECT (widget), "end");
+		w = g_object_get_data (G_OBJECT (widget), "end");
 		sp_color_selector_set_any_rgba32 (SP_COLOR_SELECTOR (w), ce);
 
 		/* Fixme: Sensitivity */
 	}
 
 	/* Fill preview */
-	w = gtk_object_get_data (GTK_OBJECT (widget), "preview");
+	w = g_object_get_data (G_OBJECT (widget), "preview");
 	sp_gradient_image_set_gradient (SP_GRADIENT_IMAGE (w), gradient);
 }
 
@@ -526,19 +524,19 @@ sp_gradient_vector_dialog_delete (GtkWidget *widget, GdkEvent *event, GtkWidget
 static void
 sp_gradient_vector_widget_destroy (GtkObject *object, gpointer data)
 {
-	GtkObject *gradient;
+	GObject *gradient;
 
-	gradient = gtk_object_get_data (object, "gradient");
+	gradient = g_object_get_data (G_OBJECT (object), "gradient");
 
 	if (gradient) {
 		/* Remove signals connected to us */
 		/* fixme: may use _connect_while_alive as well */
-		gtk_signal_disconnect_by_data (gradient, object);
+  		sp_signal_disconnect_by_data (gradient, object);
 	}
 }
 
 static void
-sp_gradient_vector_gradient_destroy (SPGradient *gradient, GtkWidget *widget)
+sp_gradient_vector_gradient_release (SPGradient *gradient, GtkWidget *widget)
 {
 	sp_gradient_vector_widget_load_gradient (widget, NULL);
 }
@@ -562,7 +560,7 @@ sp_gradient_vector_color_dragged (SPColorSelector *csel, GtkObject *object)
 
 	if (blocked) return;
 
-	gradient = gtk_object_get_data (object, "gradient");
+	gradient = g_object_get_data (G_OBJECT (object), "gradient");
 	if (!gradient) return;
 
 	blocked = TRUE;
@@ -580,12 +578,12 @@ sp_gradient_vector_color_dragged (SPColorSelector *csel, GtkObject *object)
 	vector->start = ngr->vector->start;
 	vector->end = ngr->vector->end;
 
-	csel = gtk_object_get_data (object, "start");
+	csel = g_object_get_data (G_OBJECT (object), "start");
 	sp_color_selector_get_rgba_floatv (csel, c);
 	vector->stops[0].offset = 0.0;
 	sp_color_set_rgb_float (&vector->stops[0].color, c[0], c[1], c[2]);
 	vector->stops[0].opacity = c[3];
-	csel = gtk_object_get_data (object, "end");
+	csel = g_object_get_data (G_OBJECT (object), "end");
 	sp_color_selector_get_rgba_floatv (csel, c);
 	vector->stops[1].offset = 1.0;
 	sp_color_set_rgb_float (&vector->stops[1].color, c[0], c[1], c[2]);
@@ -603,11 +601,11 @@ sp_gradient_vector_color_changed (SPColorSelector *csel, GtkObject *object)
 	gdouble start, end;
 	SPObject *child;
 	guint32 color;
-	gchar c[256];
+	gchar c[256], c0[32];
 
 	if (blocked) return;
 
-	gradient = gtk_object_get_data (object, "gradient");
+	gradient = g_object_get_data (G_OBJECT (object), "gradient");
 	if (!gradient) return;
 
 	blocked = TRUE;
@@ -625,16 +623,17 @@ sp_gradient_vector_color_changed (SPColorSelector *csel, GtkObject *object)
 
 	/* Set start parameters */
 	/* We rely on normalized vector, i.e. stops HAVE to exist */
-	for (child = ngr->stops; child != NULL; child = child->next) {
+	for (child = ((SPObject *) ngr)->children; child != NULL; child = child->next) {
 		if (SP_IS_STOP (child)) break;
 	}
 	g_return_if_fail (child != NULL);
 
-	csel = gtk_object_get_data (object, "start");
+	csel = g_object_get_data (G_OBJECT (object), "start");
 	color = sp_color_selector_get_rgba32 (csel);
 
 	sp_repr_set_double_attribute (SP_OBJECT_REPR (child), "offset", start);
-	g_snprintf (c, 256, "stop-color:#%06x;stop-opacity:%g;", color >> 8, (gdouble) (color & 0xff) / 255.0);
+	arikkei_dtoa_simple (c0, 32, (double) (color & 0xff) / 255.0, 4, 0, FALSE);
+	g_snprintf (c, 256, "stop-color:#%06x;stop-opacity:%s;", color >> 8, c0);
 	sp_repr_set_attr (SP_OBJECT_REPR (child), "style", c);
 
 	for (child = child->next; child != NULL; child = child->next) {
@@ -642,11 +641,12 @@ sp_gradient_vector_color_changed (SPColorSelector *csel, GtkObject *object)
 	}
 	g_return_if_fail (child != NULL);
 
-	csel = gtk_object_get_data (object, "end");
+	csel = g_object_get_data (G_OBJECT (object), "end");
 	color = sp_color_selector_get_rgba32 (csel);
 
 	sp_repr_set_double_attribute (SP_OBJECT_REPR (child), "offset", end);
-	g_snprintf (c, 256, "stop-color:#%06x;stop-opacity:%g;", color >> 8, (gdouble) (color & 0xff) / 255.0);
+	arikkei_dtoa_simple (c0, 32, (double) (color & 0xff) / 255.0, 4, 0, FALSE);
+	g_snprintf (c, 256, "stop-color:#%06x;stop-opacity:%s;", color >> 8, c0);
 	sp_repr_set_attr (SP_OBJECT_REPR (child), "style", c);
 
 	/* Remove other stops */
