@@ -11,7 +11,9 @@
 
 #define noSP_PS_VERBOSE
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <math.h>
 #include <string.h>
@@ -27,15 +29,13 @@
 #include <gtk/gtkframe.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtkimage.h>
 
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-util.h>
-
-#include <libgnomeui/gnome-stock.h>
+#include "../helper/sp-intl.h"
 
+#include "icon.h"
+#include "menu.h"
 #include "../sp-item.h"
 #include "../sp-gradient.h"
 
@@ -105,35 +105,34 @@ sp_paint_selector_class_init (SPPaintSelectorClass *klass)
 
 	psel_signals[MODE_CHANGED] = gtk_signal_new ("mode_changed",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPPaintSelectorClass, mode_changed),
 						 gtk_marshal_NONE__UINT,
 						 GTK_TYPE_NONE, 1, GTK_TYPE_UINT);
 	psel_signals[GRABBED] =  gtk_signal_new ("grabbed",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPPaintSelectorClass, grabbed),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
 	psel_signals[DRAGGED] =  gtk_signal_new ("dragged",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPPaintSelectorClass, dragged),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
 	psel_signals[RELEASED] = gtk_signal_new ("released",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPPaintSelectorClass, released),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
 	psel_signals[CHANGED] =  gtk_signal_new ("changed",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPPaintSelectorClass, changed),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
-	gtk_object_class_add_signals (object_class, psel_signals, LAST_SIGNAL);
 
 	object_class->destroy = sp_paint_selector_destroy;
 }
@@ -155,18 +154,18 @@ sp_paint_selector_init (SPPaintSelector *psel)
 	gtk_box_pack_start (GTK_BOX (psel), psel->style, FALSE, FALSE, 0);
 
 	/* Buttons */
-	psel->none = sp_paint_selector_style_button_add (psel, "fill_none.xpm",
+	psel->none = sp_paint_selector_style_button_add (psel, "fill_none",
 							 SP_PAINT_SELECTOR_MODE_NONE, NULL);
-	psel->solid = sp_paint_selector_style_button_add (psel, "fill_solid.xpm",
+	psel->solid = sp_paint_selector_style_button_add (psel, "fill_solid",
 							  SP_PAINT_SELECTOR_MODE_COLOR_RGB, GTK_RADIO_BUTTON (psel->none));
-	psel->gradient = sp_paint_selector_style_button_add (psel, "fill_gradient.xpm",
+	psel->gradient = sp_paint_selector_style_button_add (psel, "fill_gradient",
 							     SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR, GTK_RADIO_BUTTON (psel->solid));
-	psel->radial = sp_paint_selector_style_button_add (psel, "fill_radial.xpm",
+	psel->radial = sp_paint_selector_style_button_add (psel, "fill_radial",
 							   SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL, GTK_RADIO_BUTTON (psel->gradient));
-	psel->pattern = sp_paint_selector_style_button_add (psel, "fill_pattern.xpm",
+	psel->pattern = sp_paint_selector_style_button_add (psel, "fill_pattern",
 							    SP_PAINT_SELECTOR_MODE_PATTERN, GTK_RADIO_BUTTON (psel->radial));
 	gtk_widget_hide (psel->pattern);
-	psel->fractal = sp_paint_selector_style_button_add (psel, "fill_fractal.xpm",
+	psel->fractal = sp_paint_selector_style_button_add (psel, "fill_fractal",
 							    SP_PAINT_SELECTOR_MODE_FRACTAL, GTK_RADIO_BUTTON (psel->pattern));
 	gtk_widget_hide (psel->fractal);
 
@@ -200,15 +199,12 @@ static GtkWidget *
 sp_paint_selector_style_button_add (SPPaintSelector *psel, const guchar *pixmap, SPPaintSelectorMode mode, GtkRadioButton *last)
 {
 	GtkWidget *b, *w;
-	gchar *path;
 
 	b = gtk_radio_button_new ((last) ? gtk_radio_button_group (last) : NULL);
 	gtk_widget_show (b);
 	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (b), FALSE);
 	gtk_object_set_data (GTK_OBJECT (b), "mode", GUINT_TO_POINTER (mode));
-	path = g_concat_dir_and_file (SODIPODI_GLADEDIR, pixmap);
-	w = gnome_stock_pixmap_widget (GTK_WIDGET (psel), path);
-	g_free (path);
+	w = sp_icon_new (SP_ICON_SIZE_BUTTON, pixmap);
 	gtk_widget_show (w);
 	gtk_container_add (GTK_CONTAINER (b), w);
 	gtk_box_pack_start (GTK_BOX (psel->style), b, FALSE, FALSE, 0);
@@ -443,7 +439,7 @@ sp_paint_selector_get_rgba_floatv (SPPaintSelector *psel, gfloat *rgba)
 
 	csel = gtk_object_get_data (GTK_OBJECT (psel->selector), "color-selector");
 
-	return sp_color_selector_get_rgba_floatv (csel, rgba);
+	sp_color_selector_get_rgba_floatv (csel, rgba);
 }
 
 void
@@ -455,7 +451,7 @@ sp_paint_selector_get_cmyka_floatv (SPPaintSelector *psel, gfloat *cmyka)
 
 	csel = gtk_object_get_data (GTK_OBJECT (psel->selector), "color-selector");
 
-	return sp_color_selector_get_cmyka_floatv (csel, cmyka);
+	sp_color_selector_get_cmyka_floatv (csel, cmyka);
 }
 
 SPGradient *
@@ -482,9 +478,9 @@ sp_paint_selector_get_gradient_position_floatv (SPPaintSelector *psel, gfloat *p
 	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
 
 	if (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) {
-		return sp_gradient_selector_get_lgradient_position_floatv (gsel, pos);
+		sp_gradient_selector_get_lgradient_position_floatv (gsel, pos);
 	} else {
-		return sp_gradient_selector_get_rgradient_position_floatv (gsel, pos);
+		sp_gradient_selector_get_rgradient_position_floatv (gsel, pos);
 	}
 }
 
@@ -492,8 +488,6 @@ void
 sp_paint_selector_write_lineargradient (SPPaintSelector *psel, SPLinearGradient *lg, SPItem *item)
 {
 	gfloat p[4], gp[4];
-	ArtDRect bbox;
-	gdouble ctm[6];
 	NRMatrixF fctm, gs2d, g2d, d2g, gs2g, g2gs;
 	NRRectF fbb;
 	double e;
@@ -508,18 +502,8 @@ sp_paint_selector_write_lineargradient (SPPaintSelector *psel, SPLinearGradient
 	/* Now position */
 	sp_paint_selector_get_gradient_position_floatv (psel, p);
 	/* Calculate raw gradient transform */
-	sp_item_invoke_bbox (item, &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
-	sp_item_i2doc_affine (item, ctm);
-	fctm.c[0] = ctm[0];
-	fctm.c[1] = ctm[1];
-	fctm.c[2] = ctm[2];
-	fctm.c[3] = ctm[3];
-	fctm.c[4] = ctm[4];
-	fctm.c[5] = ctm[5];
-	fbb.x0 = bbox.x0;
-	fbb.y0 = bbox.y0;
-	fbb.x1 = bbox.x1;
-	fbb.y1 = bbox.y1;
+	sp_item_invoke_bbox (item, &fbb, NULL, TRUE);
+	sp_item_i2doc_affine (item, &fctm);
 	sp_paint_selector_get_gradient_gs2d_matrix_f (psel, &gs2d);
 	sp_gradient_get_g2d_matrix_f (SP_GRADIENT (lg), &fctm, &fbb, &g2d);
 	nr_matrix_f_invert (&d2g, &g2d);
@@ -551,8 +535,6 @@ void
 sp_paint_selector_write_radialgradient (SPPaintSelector *psel, SPRadialGradient *rg, SPItem *item)
 {
 	gfloat p[5], gp[4];
-	ArtDRect bbox;
-	gdouble ctm[6];
 	NRMatrixF fctm, gs2d, g2d, d2g, gs2g, g2gs;
 	NRRectF fbb;
 	double e;
@@ -566,18 +548,8 @@ sp_paint_selector_write_radialgradient (SPPaintSelector *psel, SPRadialGradient
 	/* Now position */
 	sp_paint_selector_get_gradient_position_floatv (psel, p);
 	/* Calculate raw gradient transform */
-	sp_item_invoke_bbox (item, &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
-	sp_item_i2doc_affine (item, ctm);
-	fctm.c[0] = ctm[0];
-	fctm.c[1] = ctm[1];
-	fctm.c[2] = ctm[2];
-	fctm.c[3] = ctm[3];
-	fctm.c[4] = ctm[4];
-	fctm.c[5] = ctm[5];
-	fbb.x0 = bbox.x0;
-	fbb.y0 = bbox.y0;
-	fbb.x1 = bbox.x1;
-	fbb.y1 = bbox.y1;
+	sp_item_invoke_bbox (item, &fbb, NULL, TRUE);
+	sp_item_i2doc_affine (item, &fctm);
 	sp_paint_selector_get_gradient_gs2d_matrix_f (psel, &gs2d);
 	sp_gradient_get_g2d_matrix_f (SP_GRADIENT (rg), &fctm, &fbb, &g2d);
 	nr_matrix_f_invert (&d2g, &g2d);
@@ -612,6 +584,29 @@ sp_paint_selector_write_radialgradient (SPPaintSelector *psel, SPRadialGradient
 	sp_radialgradient_set_position (rg, p[0], p[1], p[2], p[3], p[4]);
 }
 
+void
+sp_paint_selector_system_color_set (SPPaintSelector *psel, const SPColor *color, float opacity)
+{
+	if ((psel->mode == SP_PAINT_SELECTOR_MODE_COLOR_RGB) ||
+	    (psel->mode == SP_PAINT_SELECTOR_MODE_COLOR_CMYK)) {
+		GtkToggleButton *gd;
+		gd = g_object_get_data (G_OBJECT (psel->selector), "get_dropper");
+		if (gtk_toggle_button_get_active (gd)) {
+			if (psel->mode == SP_PAINT_SELECTOR_MODE_COLOR_RGB) {
+				float rgba[4];
+				sp_color_get_rgb_floatv (color, rgba);
+				rgba[3] = opacity;
+				sp_paint_selector_set_color_rgba_floatv (psel, rgba);
+			} else {
+				float cmyka[5];
+				sp_color_get_cmyk_floatv (color, cmyka);
+				cmyka[4] = opacity;
+				sp_paint_selector_set_color_cmyka_floatv (psel, cmyka);
+			}
+		}
+	}
+}
+
 static void
 sp_paint_selector_set_mode_empty (SPPaintSelector *psel)
 {
@@ -659,14 +654,12 @@ sp_paint_selector_set_mode_none (SPPaintSelector *psel)
 static SPColorSelectorMode default_rgb_mode = SP_COLOR_SELECTOR_MODE_RGB;
 
 static void
-sp_paint_selector_color_mode_activate (GtkWidget *widget, SPPaintSelector *psel)
+sp_paint_selector_color_mode_select (GtkWidget *menu, SPColorSelectorMode cselmode, SPPaintSelector *psel)
 {
 	SPColorSelector *csel;
-	SPColorSelectorMode cselmode;
 	SPPaintSelectorMode pselmode;
 
 	csel = gtk_object_get_data (GTK_OBJECT (psel->selector), "color-selector");
-	cselmode = GPOINTER_TO_UINT (gtk_object_get_data (GTK_OBJECT (widget), "mode"));
 	/* We have to set it manually here, because RGB/HSV is ignored by psel */
 	sp_color_selector_set_mode (csel, cselmode);
 
@@ -728,7 +721,7 @@ sp_paint_selector_set_mode_color (SPPaintSelector *psel, SPPaintSelectorMode mod
 		csel = gtk_object_get_data (GTK_OBJECT (psel->selector), "color-selector");
 		cselmode = gtk_object_get_data (GTK_OBJECT (psel->selector), "mode-menu");
 	} else {
-		GtkWidget *vb, *hb, *l, *m, *i;
+		GtkWidget *vb, *hb, *l, *m, *cb;
 		if (psel->selector) {
 			gtk_widget_destroy (psel->selector);
 			psel->selector = NULL;
@@ -737,6 +730,7 @@ sp_paint_selector_set_mode_color (SPPaintSelector *psel, SPPaintSelectorMode mod
 		/* Create vbox */
 		vb = gtk_vbox_new (FALSE, 4);
 		gtk_widget_show (vb);
+
 		/* Create hbox */
 		hb = gtk_hbox_new (FALSE, 4);
 		gtk_widget_show (hb);
@@ -745,34 +739,28 @@ sp_paint_selector_set_mode_color (SPPaintSelector *psel, SPPaintSelectorMode mod
 		l = gtk_label_new (_("Mode:"));
 		gtk_widget_show (l);
 		gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-		gtk_box_pack_start (GTK_BOX (hb), l, TRUE, TRUE, 4);
+		gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 4);
 		/* Create option menu */
 		cselmode = gtk_option_menu_new ();
 		gtk_widget_show (cselmode);
 		/* Create menu */
-		m = gtk_menu_new ();
+		m = sp_menu_new ();
 		gtk_widget_show (m);
-		i = gtk_menu_item_new_with_label (_("RGB"));
-		gtk_widget_show (i);
-		gtk_object_set_data (GTK_OBJECT (i), "mode", GUINT_TO_POINTER (SP_COLOR_SELECTOR_MODE_RGB));
-		gtk_signal_connect (GTK_OBJECT (i), "activate",
-				    GTK_SIGNAL_FUNC (sp_paint_selector_color_mode_activate), psel);
-		gtk_menu_append (GTK_MENU (m), i);
-		i = gtk_menu_item_new_with_label (_("HSV"));
-		gtk_widget_show (i);
-		gtk_object_set_data (GTK_OBJECT (i), "mode", GUINT_TO_POINTER (SP_COLOR_SELECTOR_MODE_HSV));
-		gtk_signal_connect (GTK_OBJECT (i), "activate",
-				    GTK_SIGNAL_FUNC (sp_paint_selector_color_mode_activate), psel);
-		gtk_menu_append (GTK_MENU (m), i);
-		i = gtk_menu_item_new_with_label (_("CMYK"));
-		gtk_widget_show (i);
-		gtk_object_set_data (GTK_OBJECT (i), "mode", GUINT_TO_POINTER (SP_COLOR_SELECTOR_MODE_CMYK));
-		gtk_signal_connect (GTK_OBJECT (i), "activate",
-				    GTK_SIGNAL_FUNC (sp_paint_selector_color_mode_activate), psel);
-		gtk_menu_append (GTK_MENU (m), i);
+		sp_menu_append (SP_MENU (m), _("RGB"), _("RGB Colorspace"), GUINT_TO_POINTER (SP_COLOR_SELECTOR_MODE_RGB));
+		sp_menu_append (SP_MENU (m), _("HSV"), _("RGB Colorspace"), GUINT_TO_POINTER (SP_COLOR_SELECTOR_MODE_HSV));
+		sp_menu_append (SP_MENU (m), _("CMYK"), _("CMYK Colorspace"), GUINT_TO_POINTER (SP_COLOR_SELECTOR_MODE_CMYK));
+		gtk_signal_connect (GTK_OBJECT (m), "selected",
+				    GTK_SIGNAL_FUNC (sp_paint_selector_color_mode_select), psel);
 		gtk_option_menu_set_menu (GTK_OPTION_MENU (cselmode), m);
 		gtk_object_set_data (GTK_OBJECT (vb), "mode-menu", cselmode);
 		gtk_box_pack_start (GTK_BOX (hb), cselmode, FALSE, FALSE, 0);
+		/* Use dropper */
+		cb = gtk_check_button_new_with_label (_("Get from dropper"));
+		gtk_widget_show (cb);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb), TRUE);
+		gtk_box_pack_end (GTK_BOX (hb), cb, FALSE, FALSE, 4);
+		g_object_set_data (G_OBJECT (vb), "get_dropper", cb);
+
 		/* Color selector */
 		csel = sp_color_selector_new ();
 		gtk_widget_show (csel);
