@@ -165,14 +165,14 @@ sp_color_preview_paint (SPColorPreview *cp, GdkRectangle *area)
 
 	/* Draw shadow */
 	gtk_draw_shadow (widget->style, widget->window,
-			 widget->state, GTK_SHADOW_IN,
+			 widget->state, GTK_SHADOW_ETCHED_IN,
 			 warea.x, warea.y,
 			 warea.width, warea.height);
 
 	/* Transparent area */
 
-	bx = widget->style->klass->xthickness;
-	by = widget->style->klass->ythickness;
+	bx = widget->style->xthickness;
+	by = widget->style->ythickness;
 	w2 = warea.width / 2;
 
 	carea.x = warea.x + bx;
@@ -191,7 +191,7 @@ sp_color_preview_paint (SPColorPreview *cp, GdkRectangle *area)
 
 	carea.x = warea.x + w2;
 	carea.y = warea.y + by;
-	carea.width = warea.width - bx - carea.x;
+	carea.width = warea.width - bx - w2;
 	carea.height = warea.height - 2 * by;
 
 	if (gdk_rectangle_intersect (area, &carea, &cpaint)) {
@@ -202,27 +202,3 @@ sp_color_preview_paint (SPColorPreview *cp, GdkRectangle *area)
 	}
 }
 
-#if 0
-static void
-sp_color_preview_update (SPColorPreview *image)
-{
-	GtkAllocation *allocation;
-
-	if (!image->px) return;
-
-	allocation = &((GtkWidget *) image)->allocation;
-
-	if (image->gradient) {
-		nr_render_checkerboard_rgb (image->px, allocation->width, VBLOCK, 3 * allocation->width);
-		sp_gradient_render_vector_block_rgb (image->gradient,
-						     image->px, allocation->width, VBLOCK, 3 * allocation->width,
-						     0, allocation->width, TRUE);
-	} else {
-		nr_render_gray_garbage_rgb (image->px, allocation->width, VBLOCK, 3 * allocation->width);
-	}
-
-	if (GTK_WIDGET_DRAWABLE (image)) {
-		gtk_widget_queue_draw (GTK_WIDGET (image));
-	}
-}
-#endif
