@@ -8,7 +8,7 @@
  *
  * Copyright (C) 2001-2002 Lauris Kaplinski
  *
- * Released under GNU GPL, read the file 'COPYING' for more information
+ * This code is in public domain
  */
 
 #include <gdk/gdk.h>
@@ -36,9 +36,9 @@ static void sp_color_slider_destroy (GtkObject *object);
 static void sp_color_slider_realize (GtkWidget *widget);
 static void sp_color_slider_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void sp_color_slider_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
-static void sp_color_slider_draw (GtkWidget *widget, GdkRectangle *area);
-static void sp_color_slider_draw_focus (GtkWidget *widget);
-static void sp_color_slider_draw_default (GtkWidget *widget);
+/*  static void sp_color_slider_draw (GtkWidget *widget, GdkRectangle *area); */
+/*  static void sp_color_slider_draw_focus (GtkWidget *widget); */
+/*  static void sp_color_slider_draw_default (GtkWidget *widget); */
 
 static gint sp_color_slider_expose (GtkWidget *widget, GdkEventExpose *event);
 static gint sp_color_slider_button_press (GtkWidget *widget, GdkEventButton *event);
@@ -88,38 +88,37 @@ sp_color_slider_class_init (SPColorSliderClass *klass)
 
 	slider_signals[GRABBED] = gtk_signal_new ("grabbed",
 						  GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						  object_class->type,
+						  GTK_CLASS_TYPE(object_class),
 						  GTK_SIGNAL_OFFSET (SPColorSliderClass, grabbed),
 						  gtk_marshal_NONE__NONE,
 						  GTK_TYPE_NONE, 0);
 	slider_signals[DRAGGED] = gtk_signal_new ("dragged",
 						  GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						  object_class->type,
+						  GTK_CLASS_TYPE(object_class),
 						  GTK_SIGNAL_OFFSET (SPColorSliderClass, dragged),
 						  gtk_marshal_NONE__NONE,
 						  GTK_TYPE_NONE, 0);
 	slider_signals[RELEASED] = gtk_signal_new ("released",
 						  GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						  object_class->type,
+						  GTK_CLASS_TYPE(object_class),
 						  GTK_SIGNAL_OFFSET (SPColorSliderClass, released),
 						  gtk_marshal_NONE__NONE,
 						  GTK_TYPE_NONE, 0);
 	slider_signals[CHANGED] = gtk_signal_new ("changed",
 						  GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						  object_class->type,
+						  GTK_CLASS_TYPE(object_class),
 						  GTK_SIGNAL_OFFSET (SPColorSliderClass, changed),
 						  gtk_marshal_NONE__NONE,
 						  GTK_TYPE_NONE, 0);
-	gtk_object_class_add_signals (object_class, slider_signals, LAST_SIGNAL);
 
 	object_class->destroy = sp_color_slider_destroy;
 
 	widget_class->realize = sp_color_slider_realize;
 	widget_class->size_request = sp_color_slider_size_request;
 	widget_class->size_allocate = sp_color_slider_size_allocate;
-	widget_class->draw = sp_color_slider_draw;
-	widget_class->draw_focus = sp_color_slider_draw_focus;
-	widget_class->draw_default = sp_color_slider_draw_default;
+/*  	widget_class->draw = sp_color_slider_draw; */
+/*  	widget_class->draw_focus = sp_color_slider_draw_focus; */
+/*  	widget_class->draw_default = sp_color_slider_draw_default; */
 
 	widget_class->expose_event = sp_color_slider_expose;
 	widget_class->button_press_event = sp_color_slider_button_press;
@@ -202,7 +201,7 @@ sp_color_slider_realize (GtkWidget *widget)
 	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
 	gdk_window_set_user_data (widget->window, widget);
 
-	/* fixme: styling? */
+	widget->style = gtk_style_attach (widget->style, widget->window);
 }
 
 static void
@@ -212,8 +211,8 @@ sp_color_slider_size_request (GtkWidget *widget, GtkRequisition *requisition)
 
 	slider = SP_COLOR_SLIDER (widget);
 
-	requisition->width = SLIDER_WIDTH + widget->style->klass->xthickness * 2;
-	requisition->height = SLIDER_HEIGHT + widget->style->klass->ythickness * 2;
+	requisition->width = SLIDER_WIDTH + widget->style->xthickness * 2;
+	requisition->height = SLIDER_HEIGHT + widget->style->ythickness * 2;
 }
 
 static void
@@ -231,8 +230,8 @@ sp_color_slider_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	}
 }
 
-static void
-sp_color_slider_draw (GtkWidget *widget, GdkRectangle *area)
+static gint
+sp_color_slider_expose (GtkWidget *widget, GdkEventExpose *event)
 {
 	SPColorSlider *slider;
 
@@ -242,26 +241,8 @@ sp_color_slider_draw (GtkWidget *widget, GdkRectangle *area)
 		gint width, height;
 		width = widget->allocation.width;
 		height = widget->allocation.height;
-		sp_color_slider_paint (slider, area);
+		sp_color_slider_paint (slider, &event->area);
 	}
-}
-
-static void
-sp_color_slider_draw_focus (GtkWidget *widget)
-{
-	gtk_widget_draw (widget, NULL);
-}
-
-static void
-sp_color_slider_draw_default (GtkWidget *widget)
-{
-	gtk_widget_draw (widget, NULL);
-}
-
-static gint
-sp_color_slider_expose (GtkWidget *widget, GdkEventExpose *event)
-{
-	gtk_widget_draw (widget, NULL);
 
 	return FALSE;
 }
@@ -275,7 +256,7 @@ sp_color_slider_button_press (GtkWidget *widget, GdkEventButton *event)
 
 	if (event->button == 1) {
 		gint cx, cw;
-		cx = widget->style->klass->xthickness;
+		cx = widget->style->xthickness;
 		cw = widget->allocation.width - 2 * cx;
 		gtk_signal_emit (GTK_OBJECT (slider), slider_signals[GRABBED]);
 		slider->dragging = TRUE;
@@ -317,7 +298,7 @@ sp_color_slider_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 
 	if (slider->dragging) {
 		gint cx, cw;
-		cx = widget->style->klass->xthickness;
+		cx = widget->style->xthickness;
 		cw = widget->allocation.width - 2 * cx;
 		gtk_adjustment_set_value (slider->adjustment, CLAMP ((gfloat) (event->x - cx) / cw, 0.0, 1.0));
 		gtk_signal_emit (GTK_OBJECT (slider), slider_signals[DRAGGED]);
@@ -426,8 +407,8 @@ sp_color_slider_adjustment_value_changed (GtkAdjustment *adjustment, SPColorSlid
 
 	if (slider->value != adjustment->value) {
 		gint cx, cy, cw, ch;
-		cx = widget->style->klass->xthickness;
-		cy = widget->style->klass->ythickness;
+		cx = widget->style->xthickness;
+		cy = widget->style->ythickness;
 		cw = widget->allocation.width - 2 * cx;
 		ch = widget->allocation.height - 2 * cy;
 		if ((gint) (adjustment->value * cw) != (gint) (slider->value * cw)) {
@@ -435,34 +416,21 @@ sp_color_slider_adjustment_value_changed (GtkAdjustment *adjustment, SPColorSlid
 			gfloat value;
 			value = slider->value;
 			slider->value = adjustment->value;
-			ax = value * cw - ARROW_SIZE / 2 + cx;
-			ay = ch - ARROW_SIZE + cy * 2;
-			gtk_widget_queue_draw_area (widget, ax, ay, ARROW_SIZE, ARROW_SIZE);
-			ax = slider->value * cw - ARROW_SIZE / 2 + cx;
-			ay = ch - ARROW_SIZE + cy * 2;
-			gtk_widget_queue_draw_area (widget, ax, ay, ARROW_SIZE, ARROW_SIZE);
+			ax = cx + value * cw - ARROW_SIZE / 2 - 1;
+			ay = ch - ARROW_SIZE + cy * 2 - 1;
+			gtk_widget_queue_draw_area (widget, ax, ay, ARROW_SIZE + 2, ARROW_SIZE + 2);
+			ax = cx + slider->value * cw - ARROW_SIZE / 2 - 1;
+			ay = ch - ARROW_SIZE + cy * 2 - 1;
+			gtk_widget_queue_draw_area (widget, ax, ay, ARROW_SIZE + 2, ARROW_SIZE + 2);
 		} else {
 			slider->value = adjustment->value;
 		}
 	}
 }
 
-/* Area is widget relative */
-
-#if 0
-static void
-print_rect (const char *name, GdkRectangle *r)
-{
-	g_print ("%s: %d %d %d %d\n", name, r->x, r->y, r->width, r->height);
-}
-#endif
-
 static void
 sp_color_slider_paint (SPColorSlider *slider, GdkRectangle *area)
 {
-	static GdkPixmap *px;
-	static gint pxw = 0;
-	static gint pxh = 0;
 	GtkWidget *widget;
 	GdkRectangle warea, carea, aarea;
 	GdkRectangle wpaint, cpaint, apaint;
@@ -470,35 +438,30 @@ sp_color_slider_paint (SPColorSlider *slider, GdkRectangle *area)
 
 	widget = GTK_WIDGET (slider);
 
+	/* Widget area */
 	warea.x = 0;
 	warea.y = 0;
 	warea.width = widget->allocation.width;
 	warea.height = widget->allocation.height;
 
-	carea.x = widget->style->klass->xthickness;
-	carea.y = widget->style->klass->ythickness;
+	/* Color gradient area */
+	carea.x = widget->style->xthickness;
+	carea.y = widget->style->ythickness;
 	carea.width = widget->allocation.width - 2 * carea.x;
 	carea.height = widget->allocation.height - 2 * carea.y;
 
+	/* Arrow area */
 	aarea.x = slider->value * carea.width - ARROW_SIZE / 2 + carea.x;
 	aarea.width = ARROW_SIZE;
 	aarea.y = carea.height - ARROW_SIZE + carea.y * 2;
 	aarea.height = ARROW_SIZE;
 
+	/* Actual paintable area */
 	if (!gdk_rectangle_intersect (area, &warea, &wpaint)) return;
 
-	if (px && ((pxw < wpaint.width) || (pxh < wpaint.height))) {
-		gdk_pixmap_unref (px);
-		px = NULL;
-	}
-	if (!px) {
-		px = gdk_pixmap_new (widget->window, wpaint.width, wpaint.height, -1);
-		pxw = wpaint.width;
-		pxh = wpaint.height;
-	}
-
 	b = NULL;
 
+	/* Paintable part of color gradient area */
 	if (gdk_rectangle_intersect (area, &carea, &cpaint)) {
 		if (slider->map) {
 			gint s, d;
@@ -525,15 +488,16 @@ sp_color_slider_paint (SPColorSlider *slider, GdkRectangle *area)
 	}
 
 	/* Draw shadow */
-	gtk_draw_shadow (widget->style, px,
-			 widget->state, GTK_SHADOW_IN,
-			 0 - wpaint.x, 0 - wpaint.y,
-			 warea.width, warea.height);
+	gtk_paint_shadow (widget->style, widget->window,
+			  widget->state, GTK_SHADOW_IN,
+			  area, widget, "colorslider",
+			  0, 0,
+			  warea.width, warea.height);
 
 	/* Draw pixelstore */
 	if (b != NULL) {
-		gdk_draw_rgb_image (px, widget->style->black_gc,
-				    cpaint.x - wpaint.x, cpaint.y - wpaint.y,
+		gdk_draw_rgb_image (widget->window, widget->style->black_gc,
+				    cpaint.x, cpaint.y,
 				    cpaint.width, cpaint.height,
 				    GDK_RGB_DITHER_MAX,
 				    (guchar *) b, cpaint.width * 3);
@@ -541,20 +505,13 @@ sp_color_slider_paint (SPColorSlider *slider, GdkRectangle *area)
 
 	if (gdk_rectangle_intersect (area, &aarea, &apaint)) {
 		/* Draw arrow */
-		gtk_draw_arrow (widget->style, px,
-				widget->state, GTK_SHADOW_OUT,
-				GTK_ARROW_UP, TRUE,
-				aarea.x - wpaint.x, aarea.y - wpaint.y,
-				ARROW_SIZE, ARROW_SIZE);
+		gtk_paint_arrow (widget->style, widget->window,
+				 widget->state, GTK_SHADOW_IN,
+				 area, widget, "colorslider",
+				 GTK_ARROW_UP, TRUE,
+				 aarea.x, aarea.y,
+				 ARROW_SIZE, ARROW_SIZE);
 	}
-
-	/* Draw everything */
-	gdk_draw_pixmap (widget->window,
-			 widget->style->black_gc,
-			 px,
-			 0, 0,
-			 wpaint.x, wpaint.y,
-			 wpaint.width, wpaint.height);
 }
 
 /* Colors are << 16 */
