@@ -13,6 +13,7 @@
  */
 
 #include <gtk/gtksignal.h>
+#include "macros.h"
 #include "../xml/repr-private.h"
 #include "../sodipodi.h"
 #include "../desktop.h"
@@ -38,7 +39,7 @@ static void sp_widget_destroy (GtkObject *object);
 
 static void sp_widget_show (GtkWidget *widget);
 static void sp_widget_hide (GtkWidget *widget);
-static void sp_widget_draw (GtkWidget *widget, GdkRectangle *area);
+/*  static void sp_widget_draw (GtkWidget *widget, GdkRectangle *area); */
 static gint sp_widget_expose (GtkWidget *widget, GdkEventExpose *event);
 static void sp_widget_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void sp_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
@@ -59,7 +60,7 @@ spw_repr_destroy (SPRepr *repr, gpointer data)
 static void
 spw_repr_attr_changed (SPRepr *repr, const guchar *key, const guchar *oldval, const guchar *newval, gpointer data)
 {
-	gtk_signal_emit (GTK_OBJECT (data), signals[ATTR_CHANGED], key, oldval, newval);
+	g_signal_emit (G_OBJECT (data), signals[ATTR_CHANGED], 0, key, oldval, newval);
 }
 
 static void
@@ -115,50 +116,49 @@ sp_widget_class_init (SPWidgetClass *klass)
 
 	signals[CONSTRUCT] =        gtk_signal_new ("construct",
 						    GTK_RUN_FIRST,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPWidgetClass, construct),
 						    gtk_marshal_NONE__NONE,
 						    GTK_TYPE_NONE, 0);
 	signals[CHANGE_SELECTION] = gtk_signal_new ("change_selection",
 						    GTK_RUN_FIRST,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPWidgetClass, change_selection),
 						    gtk_marshal_NONE__POINTER,
 						    GTK_TYPE_NONE, 1,
 						    GTK_TYPE_POINTER);
 	signals[MODIFY_SELECTION] = gtk_signal_new ("modify_selection",
 						    GTK_RUN_FIRST,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPWidgetClass, modify_selection),
 						    gtk_marshal_NONE__POINTER_UINT,
 						    GTK_TYPE_NONE, 2,
 						    GTK_TYPE_POINTER, GTK_TYPE_UINT);
 	signals[SET_SELECTION] =    gtk_signal_new ("set_selection",
 						    GTK_RUN_FIRST,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPWidgetClass, set_selection),
 						    gtk_marshal_NONE__POINTER,
 						    GTK_TYPE_NONE, 1,
 						    GTK_TYPE_POINTER);
 	signals[ATTR_CHANGED] =     gtk_signal_new ("attr_changed",
 						    GTK_RUN_FIRST,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPWidgetClass, attr_changed),
 						    gtk_marshal_NONE__POINTER_POINTER_POINTER,
 						    GTK_TYPE_NONE, 3,
 						    GTK_TYPE_POINTER, GTK_TYPE_POINTER, GTK_TYPE_POINTER);
 	signals[SET_DIRTY] =        gtk_signal_new ("set_dirty",
 						    GTK_RUN_FIRST,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPWidgetClass, set_dirty),
 						    gtk_marshal_NONE__BOOL,
 						    GTK_TYPE_NONE, 1,
 						    GTK_TYPE_BOOL);
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	widget_class->show = sp_widget_show;
 	widget_class->hide = sp_widget_hide;
-	widget_class->draw = sp_widget_draw;
+/*  	widget_class->draw = sp_widget_draw; */
 	widget_class->expose_event = sp_widget_expose;
 	widget_class->size_request = sp_widget_size_request;
 	widget_class->size_allocate = sp_widget_size_allocate;
@@ -186,7 +186,7 @@ sp_widget_destroy (GtkObject *object)
 		/* This happens in ::hide (Lauris) */
 		/* It seems it does not (Lauris) */
 		/* Disconnect signals */
-		gtk_signal_disconnect_by_data (GTK_OBJECT (sodipodi), spw);
+		sp_signal_disconnect_by_data (sodipodi, spw);
 #endif
 		spw->sodipodi = NULL;
 	}
@@ -214,9 +214,9 @@ sp_widget_show (GtkWidget *widget)
 
 	if (spw->sodipodi) {
 		/* Connect signals */
-		gtk_signal_connect (GTK_OBJECT (sodipodi), "modify_selection", GTK_SIGNAL_FUNC (sp_widget_modify_selection), spw);
-		gtk_signal_connect (GTK_OBJECT (sodipodi), "change_selection", GTK_SIGNAL_FUNC (sp_widget_change_selection), spw);
-		gtk_signal_connect (GTK_OBJECT (sodipodi), "set_selection", GTK_SIGNAL_FUNC (sp_widget_set_selection), spw);
+		g_signal_connect (G_OBJECT (sodipodi), "modify_selection", G_CALLBACK (sp_widget_modify_selection), spw);
+		g_signal_connect (G_OBJECT (sodipodi), "change_selection", G_CALLBACK (sp_widget_change_selection), spw);
+		g_signal_connect (G_OBJECT (sodipodi), "set_selection", G_CALLBACK (sp_widget_set_selection), spw);
 	}
 
 	if (spw->repr) {
@@ -236,7 +236,7 @@ sp_widget_hide (GtkWidget *widget)
 
 	if (spw->sodipodi) {
 		/* Disconnect signals */
-		gtk_signal_disconnect_by_data (GTK_OBJECT (sodipodi), spw);
+		sp_signal_disconnect_by_data (sodipodi, spw);
 	}
 
 	if (spw->repr) {
@@ -247,12 +247,14 @@ sp_widget_hide (GtkWidget *widget)
 		(* ((GtkWidgetClass *) parent_class)->hide) (widget);
 }
 
+#if 0
 static void
 sp_widget_draw (GtkWidget *widget, GdkRectangle *area)
 {
 	if (((GtkBin *) widget)->child)
 		gtk_widget_draw (((GtkBin *) widget)->child, area);
 }
+#endif
 
 static gint
 sp_widget_expose (GtkWidget *widget, GdkEventExpose *event)
@@ -261,11 +263,14 @@ sp_widget_expose (GtkWidget *widget, GdkEventExpose *event)
 
 	bin = GTK_BIN (widget);
 
+	gtk_container_propagate_expose (GTK_CONTAINER(widget), bin->child, event);
+	/*
 	if ((bin->child) && (GTK_WIDGET_NO_WINDOW (bin->child))) {
 		GdkEventExpose ce;
 		ce = *event;
 		gtk_widget_event (bin->child, (GdkEvent *) &ce);
 	}
+	*/
 
 	return FALSE;
 }
@@ -293,9 +298,6 @@ sp_widget_new_global (Sodipodi *sodipodi)
 {
 	SPWidget *spw;
 
-	g_return_val_if_fail (sodipodi != NULL, NULL);
-	g_return_val_if_fail (SP_IS_SODIPODI (sodipodi), NULL);
-
 	spw = gtk_type_new (SP_TYPE_WIDGET);
 
 	if (!sp_widget_construct_global (spw, sodipodi)) {
@@ -326,10 +328,6 @@ sp_widget_new_repr (SPRepr *repr)
 GtkWidget *
 sp_widget_construct_global (SPWidget *spw, Sodipodi *sodipodi)
 {
-	g_return_val_if_fail (spw != NULL, NULL);
-	g_return_val_if_fail (SP_IS_WIDGET (spw), NULL);
-	g_return_val_if_fail (sodipodi != NULL, NULL);
-	g_return_val_if_fail (SP_IS_SODIPODI (sodipodi), NULL);
 	g_return_val_if_fail (!spw->sodipodi, NULL);
 
 	if (spw->repr) {
@@ -342,12 +340,12 @@ sp_widget_construct_global (SPWidget *spw, Sodipodi *sodipodi)
 
 	spw->sodipodi = sodipodi;
 	if (GTK_WIDGET_VISIBLE (spw)) {
-		gtk_signal_connect (GTK_OBJECT (sodipodi), "modify_selection", GTK_SIGNAL_FUNC (sp_widget_modify_selection), spw);
-		gtk_signal_connect (GTK_OBJECT (sodipodi), "change_selection", GTK_SIGNAL_FUNC (sp_widget_change_selection), spw);
-		gtk_signal_connect (GTK_OBJECT (sodipodi), "set_selection", GTK_SIGNAL_FUNC (sp_widget_set_selection), spw);
+		g_signal_connect (G_OBJECT (sodipodi), "modify_selection", G_CALLBACK (sp_widget_modify_selection), spw);
+		g_signal_connect (G_OBJECT (sodipodi), "change_selection", G_CALLBACK (sp_widget_change_selection), spw);
+		g_signal_connect (G_OBJECT (sodipodi), "set_selection", G_CALLBACK (sp_widget_set_selection), spw);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (spw), signals[CONSTRUCT]);
+	g_signal_emit (G_OBJECT (spw), signals[CONSTRUCT], 0);
 
 	return (GtkWidget *) spw;
 }
@@ -369,7 +367,7 @@ sp_widget_construct_repr (SPWidget *spw, SPRepr *repr)
 	}
 	if (spw->sodipodi) {
 		if (GTK_WIDGET_VISIBLE (spw)) {
-			gtk_signal_disconnect_by_data (GTK_OBJECT (sodipodi), spw);
+			sp_signal_disconnect_by_data (sodipodi, spw);
 		}
 		spw->sodipodi = NULL;
 	}
@@ -379,7 +377,7 @@ sp_widget_construct_repr (SPWidget *spw, SPRepr *repr)
 		sp_repr_add_listener (spw->repr, &spw_event_vector, spw);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (spw), signals[CONSTRUCT]);
+	g_signal_emit (G_OBJECT (spw), signals[CONSTRUCT], 0);
 
 	return (GtkWidget *) spw;
 }
@@ -387,20 +385,20 @@ sp_widget_construct_repr (SPWidget *spw, SPRepr *repr)
 static void
 sp_widget_modify_selection (Sodipodi *sodipodi, SPSelection *selection, guint flags, SPWidget *spw)
 {
-	gtk_signal_emit (GTK_OBJECT (spw), signals[MODIFY_SELECTION], selection, flags);
+	g_signal_emit (G_OBJECT (spw), signals[MODIFY_SELECTION], 0, selection, flags);
 }
 
 static void
 sp_widget_change_selection (Sodipodi *sodipodi, SPSelection *selection, SPWidget *spw)
 {
-	gtk_signal_emit (GTK_OBJECT (spw), signals[CHANGE_SELECTION], selection);
+	g_signal_emit (G_OBJECT (spw), signals[CHANGE_SELECTION], 0, selection);
 }
 
 static void
 sp_widget_set_selection (Sodipodi *sodipodi, SPSelection *selection, SPWidget *spw)
 {
 	/* Emit "set_selection" signal */
-	gtk_signal_emit (GTK_OBJECT (spw), signals[SET_SELECTION], selection);
+	g_signal_emit (G_OBJECT (spw), signals[SET_SELECTION], 0, selection);
 	/* Sodipodi will force "change_selection" anyways */
 }
 
@@ -410,12 +408,12 @@ sp_widget_set_dirty (SPWidget *spw, gboolean dirty)
 	if (dirty && !spw->dirty) {
 		spw->dirty = TRUE;
 		if (!spw->autoupdate) {
-			gtk_signal_emit (GTK_OBJECT (spw), signals[SET_DIRTY], TRUE);
+			g_signal_emit (G_OBJECT (spw), signals[SET_DIRTY], 0, TRUE);
 		}
 	} else if (!dirty && spw->dirty) {
 		spw->dirty = FALSE;
 		if (!spw->autoupdate) {
-			gtk_signal_emit (GTK_OBJECT (spw), signals[SET_DIRTY], FALSE);
+			g_signal_emit (G_OBJECT (spw), signals[SET_DIRTY], 0, FALSE);
 		}
 	}
 }
