@@ -3,6 +3,9 @@
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
+
+#include <glib.h>
+
 #include "repr.h"
 #include "repr-private.h"
 
@@ -10,7 +13,7 @@ struct _SPCSSAttr {
 	SPRepr repr;
 };
 
-static void sp_repr_css_add_components (SPCSSAttr * css, SPRepr * repr, const gchar * attr);
+static void sp_repr_css_add_components (SPCSSAttr * css, SPRepr * repr, const char * attr);
 
 SPCSSAttr *
 sp_repr_css_attr_new (void)
@@ -28,7 +31,7 @@ sp_repr_css_attr_unref (SPCSSAttr * css)
 	sp_repr_unref ((SPRepr *) css);
 }
 
-SPCSSAttr * sp_repr_css_attr (SPRepr * repr, const gchar * attr)
+SPCSSAttr * sp_repr_css_attr (SPRepr * repr, const char * attr)
 {
 	SPCSSAttr * css;
 
@@ -42,7 +45,7 @@ SPCSSAttr * sp_repr_css_attr (SPRepr * repr, const gchar * attr)
 	return css;
 }
 
-SPCSSAttr * sp_repr_css_attr_inherited (SPRepr * repr, const gchar * attr)
+SPCSSAttr * sp_repr_css_attr_inherited (SPRepr * repr, const char * attr)
 {
 	SPCSSAttr * css;
 	SPRepr * current;
@@ -64,13 +67,13 @@ SPCSSAttr * sp_repr_css_attr_inherited (SPRepr * repr, const gchar * attr)
 }
 
 static void
-sp_repr_css_add_components (SPCSSAttr * css, SPRepr * repr, const gchar * attr)
+sp_repr_css_add_components (SPCSSAttr * css, SPRepr * repr, const char * attr)
 {
-	const gchar * data;
-	gchar * new_str;
-	gchar * current;
-	gchar ** token, ** ctoken;
-	gchar * key, * val;
+	const char * data;
+	char * new_str;
+	char * current;
+	char ** token, ** ctoken;
+	char * key, * val;
 
 	g_assert (css != NULL);
 	g_assert (repr != NULL);
@@ -102,10 +105,10 @@ sp_repr_css_add_components (SPCSSAttr * css, SPRepr * repr, const gchar * attr)
 	return;
 }
 
-const gchar *
-sp_repr_css_property (SPCSSAttr * css, const gchar * name, const gchar * defval)
+const char *
+sp_repr_css_property (SPCSSAttr * css, const char * name, const char * defval)
 {
-	const gchar * attr;
+	const char * attr;
 
 	g_assert (css != NULL);
 	g_assert (name != NULL);
@@ -118,7 +121,7 @@ sp_repr_css_property (SPCSSAttr * css, const gchar * name, const gchar * defval)
 }
 
 void
-sp_repr_css_set_property (SPCSSAttr * css, const gchar * name, const gchar * value)
+sp_repr_css_set_property (SPCSSAttr * css, const char * name, const char * value)
 {
 	g_assert (css != NULL);
 	g_assert (name != NULL);
@@ -126,8 +129,8 @@ sp_repr_css_set_property (SPCSSAttr * css, const gchar * name, const gchar * val
 	sp_repr_set_attr ((SPRepr *) css, name, value);
 }
 
-gdouble
-sp_repr_css_double_property (SPCSSAttr * css, const gchar * name, gdouble defval)
+double
+sp_repr_css_double_property (SPCSSAttr * css, const char * name, double defval)
 {
 	g_assert (css != NULL);
 	g_assert (name != NULL);
@@ -136,12 +139,12 @@ sp_repr_css_double_property (SPCSSAttr * css, const gchar * name, gdouble defval
 }
 
 void
-sp_repr_css_set (SPRepr * repr, SPCSSAttr * css, const gchar * attr)
+sp_repr_css_set (SPRepr * repr, SPCSSAttr * css, const char * attr)
 {
 	SPReprAttr * a;
-	const gchar *key;
-	gchar *val;
-	gchar c[4096], *p;
+	const char *key;
+	char *val;
+	char c[4096], *p;
 
 	g_assert (repr != NULL);
 	g_assert (css != NULL);
@@ -155,7 +158,7 @@ sp_repr_css_set (SPRepr * repr, SPCSSAttr * css, const gchar * attr)
 		val = SP_REPR_ATTRIBUTE_VALUE (a);
 		p += g_snprintf (p, c + 4096 - p, "%s:%s;", key, val);
 	}
-	g_print ("style: %s\n", c);
+	/* g_print ("style: %s\n", c); */
 	sp_repr_set_attr (repr, attr, (c[0]) ? c : NULL);
 }
 
@@ -163,7 +166,7 @@ static void
 sp_repr_css_merge (SPCSSAttr * dst, SPCSSAttr * src)
 {
 	SPReprAttr * attr;
-	const gchar * key, * val;
+	const char * key, * val;
 
 	g_assert (dst != NULL);
 	g_assert (src != NULL);
@@ -177,7 +180,7 @@ sp_repr_css_merge (SPCSSAttr * dst, SPCSSAttr * src)
 }
 
 void
-sp_repr_css_change (SPRepr * repr, SPCSSAttr * css, const gchar * attr)
+sp_repr_css_change (SPRepr * repr, SPCSSAttr * css, const char * attr)
 {
 	SPCSSAttr * current;
 
@@ -193,7 +196,7 @@ sp_repr_css_change (SPRepr * repr, SPCSSAttr * css, const gchar * attr)
 }
 
 void
-sp_repr_css_change_recursive (SPRepr * repr, SPCSSAttr * css, const gchar * attr)
+sp_repr_css_change_recursive (SPRepr * repr, SPCSSAttr * css, const char * attr)
 {
 	SPRepr * child;
 
