@@ -15,24 +15,28 @@
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
-#include "repr.h"
-#include "repr-private.h"
-#include <xmlmemory.h>
-#include <parser.h>
-#include <tree.h>
 
-static const guchar *sp_svg_doctype_str =
+#include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
+#include <libxml/tree.h>
+
+#include <libarikkei/arikkei-strlib.h>
+#include <libarikkei/arikkei-iolib.h>
+
+#include <glib.h>
+
+#include "repr-private.h"
+
+static const unsigned char *sp_svg_doctype_str =
 "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\"\n"
-"\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\"\n"
-"[\n"
-" <!ATTLIST svg\n"
-"  xmlns:xlink CDATA #FIXED \"http://www.w3.org/1999/xlink\">\n"
-"]>\n";
-static const guchar *sp_comment_str =
+"\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">\n";
+static const unsigned char *sp_comment_str =
 "<!-- Created with Sodipodi (\"http://www.sodipodi.com/\") -->\n";
 
-static SPRepr * sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node);
-static void repr_write (SPRepr * repr, FILE * file, gint level);
+static SPReprDoc *sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns);
+static SPRepr * sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node, const gchar *default_ns, GHashTable *prefix_map);
+static void sp_repr_set_xmlns_attr (const unsigned char *prefix, const unsigned char *uri, SPRepr *repr);
+static gint sp_repr_qualified_name (unsigned char *p, gint len, xmlNsPtr ns, const xmlChar *name, const gchar *default_ns, GHashTable *prefix_map);
 
 #ifdef HAVE_LIBWMF
 static xmlDocPtr sp_wmf_convert (const char * file_name);
@@ -40,179 +44,216 @@ static char * sp_wmf_image_name (void * context);
 #endif /* HAVE_LIBWMF */
 
 SPReprDoc *
-sp_repr_read_file (const gchar * filename)
+sp_repr_read_file (const gchar * filename, const gchar *default_ns)
 {
+	const unsigned char *cdata;
+	int size;
 	xmlDocPtr doc;
-	xmlNodePtr node;
 	SPReprDoc * rdoc;
-	SPRepr * repr;
 
 	g_return_val_if_fail (filename != NULL, NULL);
 
+	xmlSubstituteEntitiesDefault(1);
+
 #ifdef HAVE_LIBWMF
 	if (strlen (filename) > 4) {
 		if ( (strcmp (filename + strlen (filename) - 4,".wmf") == 0)
-		  || (strcmp (filename + strlen (filename) - 4,".WMF") == 0))
+			|| (strcmp (filename + strlen (filename) - 4,".WMF") == 0)) {
 			doc = sp_wmf_convert (filename);
-		else
-			doc = xmlParseFile (filename);
-	}
-	else {
-		doc = xmlParseFile (filename);
+		} else {
+			cdata = arikkei_mmap (filename, &size, NULL);
+			if (!cdata) return NULL;
+			/* doc = xmlParseFile (filename); */
+			doc = xmlParseMemory ((char *) cdata, size);
+			arikkei_munmap (cdata, size);
+		}
+	} else {
+		cdata = arikkei_mmap (filename, st.&size, NULL);
+		if (!cdata) return NULL;
+		/* doc = xmlParseFile (filename); */
+		doc = xmlParseMemory ((char *) cdata, size);
+		arikkei_munmap (cdata, size);
 	}
 #else /* HAVE_LIBWMF */
-	doc = xmlParseFile (filename);
+	cdata = arikkei_mmap (filename, &size, NULL);
+	if (!cdata) return NULL;
+	/* doc = xmlParseFile (filename); */
+	doc = xmlParseMemory ((char *) cdata, size);
+	arikkei_munmap (cdata, size);
 #endif /* HAVE_LIBWMF */
-	if (doc == NULL) return NULL;
 
-	rdoc = sp_repr_document_new ("void");
-
-	repr = NULL;
-
-	for (node = xmlDocGetRootElement(doc); node != NULL; node = node->next) {
-		if (node->type == XML_ELEMENT_NODE) {
-			repr = sp_repr_svg_read_node (rdoc, node);
-			break;
-		}
+	rdoc = sp_repr_do_read (doc, default_ns);
+	if (doc) {
+		xmlFreeDoc (doc);
 	}
-
-	if (repr != NULL) {
-		sp_repr_document_set_root (rdoc, repr);
-		/* fixme: Another quick hack (Lauris) */
-		if (!strcmp (sp_repr_name (repr), "svg")) {
-			sp_repr_set_attr ((SPRepr *) rdoc, "doctype", sp_svg_doctype_str);
-			sp_repr_set_attr ((SPRepr *) rdoc, "comment", sp_comment_str);
-		}
-	}
-
-	xmlFreeDoc (doc);
-
 	return rdoc;
 }
 
+
+
 SPReprDoc *
-sp_repr_read_mem (const gchar * buffer, gint length)
+sp_repr_read_mem (const gchar * buffer, gint length, const gchar *default_ns)
 {
 	xmlDocPtr doc;
-	xmlNodePtr node;
 	SPReprDoc * rdoc;
-	SPRepr * repr;
+
+	xmlSubstituteEntitiesDefault(1);
 
 	g_return_val_if_fail (buffer != NULL, NULL);
 
 	doc = xmlParseMemory ((gchar *) buffer, length);
-	if (doc == NULL) return NULL;
 
+	rdoc = sp_repr_do_read (doc, default_ns);
+	if (doc) {
+		xmlFreeDoc (doc);
+	}
+	return rdoc;
+}
+
+SPReprDoc *
+sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns)
+{
+	SPReprDoc * rdoc;
+	SPRepr * repr;
+	GHashTable * prefix_map;
+	xmlNodePtr node;
+
+	if (doc == NULL) return NULL;
+	node = xmlDocGetRootElement (doc);
+	if (node == NULL) return NULL;
 	rdoc = sp_repr_document_new ("void");
 
-	repr = NULL;
 
+	prefix_map = g_hash_table_new (g_str_hash, g_str_equal);
+
+	repr = NULL;
 	for (node = xmlDocGetRootElement(doc); node != NULL; node = node->next) {
 		if (node->type == XML_ELEMENT_NODE) {
-			repr = sp_repr_svg_read_node (rdoc, node);
+			repr = sp_repr_svg_read_node (rdoc, node, default_ns, prefix_map);
 			break;
 		}
 	}
 
 	if (repr != NULL) {
-		sp_repr_document_set_root (rdoc, repr);
-		/* fixme: Another quick hack (Lauris) */
-		if (!strcmp (sp_repr_name (repr), "svg")) {
+		if (default_ns) {
+			sp_repr_set_attr (repr, "xmlns", default_ns);
+		}
+		g_hash_table_foreach (prefix_map, (GHFunc)sp_repr_set_xmlns_attr, repr);
+		/* always include Sodipodi namespace */
+		sp_repr_set_xmlns_attr (sp_xml_ns_uri_prefix (SP_SODIPODI_NS_URI, "sodipodi"), SP_SODIPODI_NS_URI, repr);
+
+		if (!strcmp (sp_repr_name (repr), "svg") && default_ns && !strcmp (default_ns, SP_SVG_NS_URI)) {
+			
 			sp_repr_set_attr ((SPRepr *) rdoc, "doctype", sp_svg_doctype_str);
 			sp_repr_set_attr ((SPRepr *) rdoc, "comment", sp_comment_str);
+			/* always include XLink namespace */
+			sp_repr_set_xmlns_attr (sp_xml_ns_uri_prefix (SP_XLINK_NS_URI, "xlink"), SP_XLINK_NS_URI, repr);
 		}
-	}
 
-	xmlFreeDoc (doc);
+		sp_repr_document_set_root (rdoc, repr);
+		sp_repr_unref (repr);
+	}
+	g_hash_table_destroy (prefix_map);
 
 	return rdoc;
 }
 
-static gint
-sp_repr_qualified_name (guchar *p, gint len, xmlNsPtr ns, const xmlChar *name)
+void
+sp_repr_set_xmlns_attr (const unsigned char *prefix, const unsigned char *uri, SPRepr *repr)
 {
-	if (ns && ns->prefix && strcmp (ns->prefix, "svg")) {
-		return g_snprintf (p, len, "%s:%s", ns->prefix, name);
+	gchar *name;
+	name = g_strconcat ("xmlns:", prefix, NULL);
+	sp_repr_set_attr (repr, name, uri);
+	g_free (name);
+}
+
+gint
+sp_repr_qualified_name (unsigned char *p, gint len, xmlNsPtr ns, const xmlChar *name, const gchar *default_ns, GHashTable *prefix_map)
+{
+	const gchar *prefix;
+	if (ns) {
+		if (!ns->href) {
+			prefix = ns->prefix;
+		} else if (default_ns && !strcmp (ns->href, default_ns)) {
+			prefix = NULL;
+		} else {
+			prefix = sp_xml_ns_uri_prefix (ns->href, ns->prefix);
+			g_hash_table_insert (prefix_map, (gpointer)prefix, (gpointer)ns->href);
+		}
+	} else {
+		prefix = NULL;
 	}
 
-	return g_snprintf (p, len, "%s", name);
+	if (prefix) {
+		return g_snprintf (p, len, "%s:%s", prefix, name);
+	} else {
+		return g_snprintf (p, len, "%s", name);
+	}
 }
 
 static SPRepr *
-sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node)
+sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node, const gchar *default_ns, GHashTable *prefix_map)
 {
 	SPRepr *repr, *crepr;
 	xmlAttrPtr prop;
 	xmlNodePtr child;
-	xmlNsPtr ns;
 	gchar c[256];
 
 #ifdef SP_REPR_IO_VERBOSE
 	g_print ("Node %d %s contains %s\n", node->type, node->name, node->content);
 #endif
 
-	if (node->type == XML_TEXT_NODE) {
+	if (node->type == XML_TEXT_NODE || node->type == XML_CDATA_SECTION_NODE)
+	{
 		xmlChar *p;
+		gboolean preserve;
+
+		preserve = (xmlNodeGetSpacePreserve (node) == 1);
+
 		for (p = node->content; p && *p; p++) {
-			if (!isspace (*p)) {
+			if (!isspace (*p) || preserve) {
 				xmlChar *e;
-				guchar *s;
+				unsigned char *s;
+				SPRepr *rdoc;
 				e = p + strlen (p) - 1;
-				while (*e && isspace (*e)) e -= 1;
-				s = g_new (guchar, e - p + 2);
+				if (! preserve)
+					while (*e && isspace (*e)) e -= 1;
+				s = g_new (unsigned char, e - p + 2);
 				memcpy (s, p, e - p + 1);
 				s[e - p + 1] = '\0';
-				return sp_xml_document_createTextNode (doc, s);
+				rdoc = sp_xml_document_createTextNode (doc, s);
 				g_free (s);
+				return rdoc;
 			}
 		}
 		return NULL;
 	}
 
 	if (node->type == XML_COMMENT_NODE) return NULL;
+	if (node->type == XML_ENTITY_DECL) return NULL;
 
-	sp_repr_qualified_name (c, 256, node->ns, node->name);
+	sp_repr_qualified_name (c, 256, node->ns, node->name, default_ns, prefix_map);
 	repr = sp_repr_new (c);
 
 	for (prop = node->properties; prop != NULL; prop = prop->next) {
-		if (prop->val) {
-			sp_repr_qualified_name (c, 256, prop->ns, prop->name);
-			sp_repr_set_attr (repr, c, prop->val->content);
+		if (prop->children) {
+			sp_repr_qualified_name (c, 256, prop->ns, prop->name, default_ns, prefix_map);
+			sp_repr_set_attr (repr, c, prop->children->content);
 		}
 	}
 
-	/* This is fuzzy piece of code, that adds namespaces back to tree */
-	/* We really should go more serious with XML */
-        for (ns = node->nsDef; ns; ns = ns->next) {
-                if (ns->prefix && strcmp (ns->prefix, "sodipodi") &&
-                                strcmp (ns->prefix, "xlink")) {
-                        snprintf (c, 256, "xlink:%s", ns->prefix);
-                        sp_repr_set_attr (repr, c, ns->href);
-                }
-        }
-
 	if (node->content)
 		sp_repr_set_content (repr, node->content);
 
 	child = node->xmlChildrenNode;
-#if 0
-	if ((child != NULL) &&
-        (child->name != NULL) &&
-		(strcmp (child->name, node->name) == 0) &&
-		(child->properties == NULL) &&
-		(child->content != NULL)) {
-
-		sp_repr_set_content (repr, child->content);
-
-	} else {
-#endif
 	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
-		crepr = sp_repr_svg_read_node (doc, child);
-		if (crepr) sp_repr_append_child (repr, crepr);
+		crepr = sp_repr_svg_read_node (doc, child, default_ns, prefix_map);
+		if (crepr) {
+			sp_repr_append_child (repr, crepr);
+			sp_repr_unref (crepr);
+		}
 	}
-#if 0
-	}
-#endif
+
 	return repr;
 }
 
@@ -220,11 +261,11 @@ void
 sp_repr_save_stream (SPReprDoc *doc, FILE *fp)
 {
 	SPRepr *repr;
-	const guchar *str;
+	const unsigned char *str;
 
 	/* fixme: do this The Right Way */
 
-	fputs ("<?xml version=\"1.0\" standalone=\"no\"?>\n", fp);
+	fputs ("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n", fp);
 
 	str = sp_repr_attr ((SPRepr *) doc, "doctype");
 	if (str) fputs (str, fp);
@@ -233,15 +274,34 @@ sp_repr_save_stream (SPReprDoc *doc, FILE *fp)
 
 	repr = sp_repr_document_root (doc);
 
-	repr_write (repr, fp, 0);
+	sp_repr_write_stream (repr, fp, 0);
 }
 
+#ifdef WIN32
+#include <windows.h>
+#include <tchar.h>
+#endif
+
 void
-sp_repr_save_file (SPReprDoc *doc, const gchar *filename)
+sp_repr_save_file (SPReprDoc *doc, const char *filename)
 {
+#ifdef WIN32
+	TCHAR *tfilename;
+#endif
 	FILE * file;
 
+#ifdef WIN32
+#ifdef _UNICODE
+	tfilename = arikkei_utf8_ucs2_strdup (filename);
+#else
+	tfilename = strdup (filename);
+#endif
+	file = _tfopen (tfilename, TEXT ("w"));
+	free (tfilename);
+#else
 	file = fopen (filename, "w");
+#endif
+
 	g_return_if_fail (file != NULL);
 
 	sp_repr_save_stream (doc, file);
@@ -254,7 +314,7 @@ sp_repr_save_file (SPReprDoc *doc, const gchar *filename)
 void
 sp_repr_print (SPRepr * repr)
 {
-	repr_write (repr, stdout, 0);
+	sp_repr_write_stream (repr, stdout, 0);
 
 	return;
 }
@@ -263,13 +323,6 @@ sp_repr_print (SPRepr * repr)
 static void
 repr_quote_write (FILE * file, const gchar * val)
 {
-	/* TODO: Presumably VAL is actually encoded in UTF8, no?
-	 * [Actually, preliminary tests suggest that this is not the
-	 * case; can sodipodi encode big charsets at all?]  If VAL is
-	 * UTF8, then this code will be buggy if e.g. `&' is the
-	 * second byte of a single utf8 "character".
-	 */
-
 	for (; *val != '\0'; val++) {
 		switch (*val) {
 		case '"': fputs ("&quot;", file); break;
@@ -281,8 +334,8 @@ repr_quote_write (FILE * file, const gchar * val)
 	}
 }
 
-static void
-repr_write (SPRepr * repr, FILE * file, gint level)
+void
+sp_repr_write_stream (SPRepr * repr, FILE * file, gint level)
 {
 	SPReprAttr *attr;
 	SPRepr *child;
@@ -306,23 +359,30 @@ repr_write (SPRepr * repr, FILE * file, gint level)
 		repr_quote_write (file, val);
 		putc ('"', file);
 	}
-	
-	if (repr->children || sp_repr_content (repr)) {
+	loose = TRUE;
+	for (child = repr->children; child != NULL; child = child->next) {
+		if (child->type == SP_XML_TEXT_NODE) {
+			loose = FALSE;
+			break;
+		}
+	}
+	if (repr->children /* || sp_repr_content (repr) */ ) {
+#if 0
 		if (sp_repr_content (repr)) {
 			fputs (">", file);
 			repr_quote_write (file, sp_repr_content (repr));
 		} else {
 			fputs (">\n", file);
 		}
-
-		loose = TRUE;
+#else
+		fputs (">", file);
+		if (loose) fputs ("\n", file);
+#endif
 		for (child = repr->children; child != NULL; child = child->next) {
 			if (child->type == SP_XML_TEXT_NODE) {
 				repr_quote_write (file, sp_repr_content (child));
-				loose = FALSE;
 			} else {
-				repr_write (child, file, (loose) ? (level + 1) : 0);
-				loose = TRUE;
+				sp_repr_write_stream (child, file, (loose) ? (level + 1) : 0);
 			}
 		}
 		
@@ -501,3 +561,4 @@ sp_wmf_image_name (void * context)
 }
 
 #endif /* HAVE_LIBWMF */
+
