@@ -16,12 +16,147 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+
+#include <libarikkei/arikkei-strlib.h>
+
+#include <glib.h>
+
+#ifndef FALSE
+#define FALSE 0
+#endif
+
+#ifndef TRUE
+#define TRUE (!FALSE)
+#endif
+
+#ifndef MAX
+#define MAX(a,b) (((a) < (b)) ? (b) : (a))
+#endif
+
 #include "repr-private.h"
 
+static void sp_xml_ns_register_defaults ();
+static char *sp_xml_ns_auto_prefix (const char *uri);
+
+/* SPXMLNs */
+
+static SPXMLNs *namespaces=NULL;
+
+void
+sp_xml_ns_register_defaults ()
+{
+	static SPXMLNs defaults[3];
+	defaults[0].uri = g_quark_from_static_string (SP_SODIPODI_NS_URI);
+	defaults[0].prefix = g_quark_from_static_string ("sodipodi");
+	defaults[0].next = &defaults[1];
+	defaults[1].uri = g_quark_from_static_string (SP_XLINK_NS_URI);
+	defaults[1].prefix = g_quark_from_static_string ("xlink");
+	defaults[1].next = &defaults[2];
+	defaults[2].uri = g_quark_from_static_string (SP_SVG_NS_URI);
+	defaults[2].prefix = g_quark_from_static_string ("svg");
+	defaults[2].next = NULL;
+	namespaces = &defaults[0];
+}
+
+char *
+sp_xml_ns_auto_prefix (const char *uri)
+{
+	const char *start, *end;
+	char *new_prefix;
+	start = uri;
+	while ((end = strpbrk (start, ":/"))) {
+		start = end + 1;
+	}
+	end = start + strspn (start, "abcdefghijklmnopqrstuvwxyz");
+	if (end == start) {
+		start = "ns";
+		end = start + 2;
+	}
+	new_prefix = g_strndup (start, end - start);
+	if (sp_xml_ns_prefix_uri (new_prefix)) {
+		char *temp;
+		int counter=0;
+		do {
+			temp = g_strdup_printf ("%s%d", new_prefix, counter++);
+		} while (sp_xml_ns_prefix_uri (temp));
+		g_free (new_prefix);
+		new_prefix = temp;
+	}
+	return new_prefix;
+}
+
+const unsigned char *
+sp_xml_ns_uri_prefix (const unsigned char *uri, const unsigned char *suggested)
+{
+	unsigned int key;
+	SPXMLNs *iter;
+	const char *prefix;
+
+	if (!uri) return NULL;
+
+	if (!namespaces) {
+		sp_xml_ns_register_defaults ();
+	}
+
+	key = g_quark_from_string (uri);
+	prefix = NULL;
+	for ( iter = namespaces ; iter ; iter = iter->next ) {
+		if ( iter->uri == key ) {
+			prefix = g_quark_to_string (iter->prefix);
+			break;
+		}
+	}
+	if (!prefix) {
+		const char *new_prefix;
+		SPXMLNs *ns;
+		if (suggested) {
+			new_prefix = suggested;
+		} else {
+			new_prefix = sp_xml_ns_auto_prefix (uri);
+		}
+		ns = g_new (SPXMLNs, 1);
+		if (ns) {
+			ns->uri = g_quark_from_string (uri);
+			ns->prefix = g_quark_from_string (new_prefix);
+			ns->next = namespaces;
+			namespaces = ns;
+			prefix = g_quark_to_string (ns->prefix);
+		}
+		if (!suggested) {
+			g_free ((char *)new_prefix);
+		}
+	}
+	return prefix;
+}
+
+const unsigned char *
+sp_xml_ns_prefix_uri (const unsigned char *prefix)
+{
+	unsigned int key;
+	SPXMLNs *iter;
+	const char *uri;
+
+	if (!prefix) return NULL;
+
+	if (!namespaces) {
+		sp_xml_ns_register_defaults ();
+	}
+
+	key = g_quark_from_string (prefix);
+	uri = NULL;
+	for ( iter = namespaces ; iter ; iter = iter->next ) {
+		if ( iter->prefix == key ) {
+			uri = g_quark_to_string (iter->uri);
+			break;
+		}
+	}
+	return uri;
+}
+
 /* SPXMLDocument */
 
 SPXMLText *
-sp_xml_document_createTextNode (SPXMLDocument *doc, const guchar *data)
+sp_xml_document_createTextNode (SPXMLDocument *doc, const unsigned char *data)
 {
 	SPXMLText *text;
 
@@ -33,13 +168,13 @@ sp_xml_document_createTextNode (SPXMLDocument *doc, const guchar *data)
 }
 
 SPXMLElement *
-sp_xml_document_createElement (SPXMLDocument *doc, const guchar *name)
+sp_xml_document_createElement (SPXMLDocument *doc, const unsigned char *name)
 {
 	return sp_repr_new (name);
 }
 
 SPXMLElement *
-sp_xml_document_createElementNS (SPXMLDocument *doc, const guchar *ns, const guchar *qname)
+sp_xml_document_createElementNS (SPXMLDocument *doc, const unsigned char *ns, const unsigned char *qname)
 {
 	if (!strncmp (qname, "svg:", 4)) qname += 4;
 
@@ -59,7 +194,7 @@ sp_xml_node_get_Document (SPXMLNode *node)
 /* SPXMLElement */
 
 void
-sp_xml_element_setAttributeNS (SPXMLElement *element, const guchar *nr, const guchar *qname, const guchar *val)
+sp_xml_element_setAttributeNS (SPXMLElement *element, const unsigned char *nr, const unsigned char *qname, const unsigned char *val)
 {
 	if (!strncmp (qname, "svg:", 4)) qname += 4;
 
@@ -80,63 +215,67 @@ sp_repr_next (SPRepr *repr)
 {
 	g_return_val_if_fail (repr != NULL, NULL);
 
-	return repr->children;
+	return repr->next;
 }
 
-gint sp_repr_attr_is_set (SPRepr * repr, const gchar * key)
+int sp_repr_attr_is_set (SPRepr * repr, const char * key)
 {
-	gchar * result;
+	char * result;
 
-	result = (gchar *) sp_repr_attr (repr, key);
+	result = (char *) sp_repr_attr (repr, key);
 
 	return (result != NULL);
 }
 
-gdouble sp_repr_get_double_attribute (SPRepr * repr, const gchar * key, gdouble def)
+double
+sp_repr_get_double_attribute (SPRepr * repr, const char * key, double def)
 {
-	gchar * result;
+	char *result;
+	double val;
 
 	g_return_val_if_fail (repr != NULL, def);
 	g_return_val_if_fail (key != NULL, def);
 
-	result = (gchar *) sp_repr_attr (repr, key);
+	result = (char *) sp_repr_attr (repr, key);
 
 	if (result == NULL) return def;
 
-	return atof (result);
+	val = def;
+	arikkei_strtod_exp (result, 256, &val);
+	return val;
 }
 
-gint sp_repr_get_int_attribute (SPRepr * repr, const gchar * key, gint def)
+int sp_repr_get_int_attribute (SPRepr * repr, const char * key, int def)
 {
-	gchar * result;
+	char * result;
 
 	g_return_val_if_fail (repr != NULL, def);
 	g_return_val_if_fail (key != NULL, def);
 
-	result = (gchar *) sp_repr_attr (repr, key);
+	result = (char *) sp_repr_attr (repr, key);
 
 	if (result == NULL) return def;
 
 	return atoi (result);
 }
 
-gboolean
-sp_repr_set_double_attribute (SPRepr * repr, const gchar * key, gdouble value)
+unsigned int
+sp_repr_set_double_attribute (SPRepr * repr, const char * key, double value)
 {
-	gchar c[32];
+	char c[32];
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
 
-	g_snprintf (c, 32, "%f", value);
+	arikkei_dtoa_exp (c, 32, value, 6, FALSE);
 
 	return sp_repr_set_attr (repr, key, c);
 }
 
-gboolean
-sp_repr_set_int_attribute (SPRepr * repr, const gchar * key, gint value)
+unsigned int
+sp_repr_set_int_attribute (SPRepr * repr, const char * key, int value)
 {
-	gchar c[32];
+	char c[32];
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -146,8 +285,8 @@ sp_repr_set_int_attribute (SPRepr * repr, const gchar * key, gint value)
 	return sp_repr_set_attr (repr, key, c);
 }
 
-const gchar *
-sp_repr_doc_attr (SPRepr * repr, const gchar * key)
+const char *
+sp_repr_doc_attr (SPRepr * repr, const char * key)
 {
 	SPRepr * p;
 
@@ -161,11 +300,11 @@ sp_repr_doc_attr (SPRepr * repr, const gchar * key)
 	return sp_repr_attr (repr, key);
 }
 
-gint
+int
 sp_repr_compare_position (SPRepr * first, SPRepr * second)
 {
 	SPRepr * parent;
-	gint p1, p2;
+	int p1, p2;
 
 	parent = sp_repr_parent (first);
 	g_assert (parent == sp_repr_parent (second));
@@ -178,12 +317,12 @@ sp_repr_compare_position (SPRepr * first, SPRepr * second)
 	return 0;
 }
 
-gint
+int
 sp_repr_position (SPRepr * repr)
 {
 	SPRepr * parent;
 	SPRepr * sibling;
-	gint pos;
+	int pos;
 
 	g_assert (repr != NULL);
 	parent = sp_repr_parent (repr);
@@ -201,9 +340,9 @@ sp_repr_position (SPRepr * repr)
 }
 
 void
-sp_repr_set_position_relative (SPRepr * repr, gint pos)
+sp_repr_set_position_relative (SPRepr * repr, int pos)
 {
-	gint cpos;
+	int cpos;
 
 	g_assert (repr != NULL);
 
@@ -211,11 +350,11 @@ sp_repr_set_position_relative (SPRepr * repr, gint pos)
 	sp_repr_set_position_absolute (repr, cpos + pos);
 }
 
-gint
+int
 sp_repr_n_children (SPRepr * repr)
 {
 	SPRepr * child;
-	gint n;
+	int n;
 
 	g_assert (repr != NULL);
 
@@ -288,11 +427,11 @@ sp_repr_remove_signals (SPRepr * repr)
 #endif
 }
 
-const guchar *
-sp_repr_attr_inherited (SPRepr *repr, const guchar *key)
+const unsigned char *
+sp_repr_attr_inherited (SPRepr *repr, const unsigned char *key)
 {
 	SPRepr *current;
-	const gchar *val;
+	const char *val;
 
 	g_assert (repr != NULL);
 	g_assert (key != NULL);
@@ -305,8 +444,8 @@ sp_repr_attr_inherited (SPRepr *repr, const guchar *key)
 	return NULL;
 }
 
-gboolean
-sp_repr_set_attr_recursive (SPRepr *repr, const guchar *key, const guchar *value)
+unsigned int
+sp_repr_set_attr_recursive (SPRepr *repr, const unsigned char *key, const unsigned char *value)
 {
 	SPRepr *child;
 
@@ -329,12 +468,12 @@ sp_repr_set_attr_recursive (SPRepr *repr, const guchar *key, const guchar *value
  */
 SPRepr *
 sp_repr_lookup_child (SPRepr       *repr,
-                      const guchar *key,
-                      const guchar *value)
+                      const unsigned char *key,
+                      const unsigned char *value)
 {
 	SPRepr *child;
 	SPReprAttr *attr;
-	GQuark quark;
+	unsigned int quark;
 
 	g_return_val_if_fail (repr != NULL, NULL);
 	g_return_val_if_fail (key != NULL, NULL);
@@ -354,10 +493,10 @@ sp_repr_lookup_child (SPRepr       *repr,
 }
 
 /* Convenience */
-gboolean
-sp_repr_get_boolean (SPRepr *repr, const guchar *key, gboolean *val)
+unsigned int
+sp_repr_get_boolean (SPRepr *repr, const unsigned char *key, unsigned int *val)
 {
-	const guchar *v;
+	const unsigned char *v;
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -366,9 +505,9 @@ sp_repr_get_boolean (SPRepr *repr, const guchar *key, gboolean *val)
 	v = sp_repr_attr (repr, key);
 
 	if (v != NULL) {
-		if (!strcasecmp (v, "true") ||
-		    !strcasecmp (v, "yes") ||
-		    !strcasecmp (v, "y") ||
+		if (!g_strcasecmp (v, "true") ||
+		    !g_strcasecmp (v, "yes") ||
+		    !g_strcasecmp (v, "y") ||
 		    (atoi (v) != 0)) {
 			*val = TRUE;
 		} else {
@@ -380,10 +519,10 @@ sp_repr_get_boolean (SPRepr *repr, const guchar *key, gboolean *val)
 	return FALSE;
 }
 
-gboolean
-sp_repr_get_int (SPRepr *repr, const guchar *key, gint *val)
+unsigned int
+sp_repr_get_int (SPRepr *repr, const unsigned char *key, int *val)
 {
-	const guchar *v;
+	const unsigned char *v;
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -399,10 +538,10 @@ sp_repr_get_int (SPRepr *repr, const guchar *key, gint *val)
 	return FALSE;
 }
 
-gboolean
-sp_repr_get_double (SPRepr *repr, const guchar *key, gdouble *val)
+unsigned int
+sp_repr_get_double (SPRepr *repr, const unsigned char *key, double *val)
 {
-	const guchar *v;
+	const unsigned char *v;
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -411,15 +550,14 @@ sp_repr_get_double (SPRepr *repr, const guchar *key, gdouble *val)
 	v = sp_repr_attr (repr, key);
 
 	if (v != NULL) {
-		*val = atof (v);
-		return TRUE;
+		return arikkei_strtod_exp (v, 256, val);
 	}
 
 	return FALSE;
 }
 
-gboolean
-sp_repr_set_boolean (SPRepr *repr, const guchar *key, gboolean val)
+unsigned int
+sp_repr_set_boolean (SPRepr *repr, const unsigned char *key, unsigned int val)
 {
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -427,10 +565,10 @@ sp_repr_set_boolean (SPRepr *repr, const guchar *key, gboolean val)
 	return sp_repr_set_attr (repr, key, (val) ? "true" : "false");
 }
 
-gboolean
-sp_repr_set_int (SPRepr *repr, const guchar *key, gint val)
+unsigned int
+sp_repr_set_int (SPRepr *repr, const unsigned char *key, int val)
 {
-	guchar c[32];
+	unsigned char c[32];
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -440,28 +578,21 @@ sp_repr_set_int (SPRepr *repr, const guchar *key, gint val)
 	return sp_repr_set_attr (repr, key, c);
 }
 
-#if 0
-static gint
-sp_svg_write_double (guchar *c, gdouble val)
+unsigned int
+sp_repr_set_double (SPRepr *repr, const unsigned char *key, double val)
 {
-}
-#endif
-
-gboolean
-sp_repr_set_double (SPRepr *repr, const guchar *key, gdouble val)
-{
-	guchar c[32];
+	unsigned char c[32];
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
 
-	g_snprintf (c, 32, "%g", val);
+	arikkei_dtoa_exp (c, 256, val, 8, FALSE);
 
 	return sp_repr_set_attr (repr, key, c);
 }
 
-gboolean
-sp_repr_set_double_default (SPRepr *repr, const guchar *key, gdouble val, gdouble def, gdouble e)
+unsigned int
+sp_repr_set_double_default (SPRepr *repr, const unsigned char *key, double val, double def, double e)
 {
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
