@@ -5,71 +5,129 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   MenTaLguY <mental@rydia.net>
  *
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2003 authors
  * Copyright (C) 2000-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#define noREPR_VERBOSE
+
 #include <string.h>
+#include <stddef.h>
+
+#include <glib.h>
+
 #include "repr-private.h"
 
 typedef struct _SPReprListener SPListener;
 
-static SPRepr * sp_repr_new_from_code (gint code);
+static void repr_init (SPRepr *repr);
+static void repr_doc_init (SPRepr *repr);
+
+static void repr_copy (SPRepr *to, const SPRepr *from);
+static void repr_doc_copy (SPRepr *to, const SPRepr *from);
+
+static void repr_finalize (SPRepr *repr);
+static void repr_doc_finalize (SPRepr *repr);
+
+static void bind_document (SPReprDoc *doc, SPRepr *repr);
+
+SPReprClass _sp_repr_xml_document_class = {
+	sizeof (SPReprDoc),
+	NULL,
+	repr_doc_init,
+	repr_doc_copy,
+	repr_doc_finalize
+};
+
+SPReprClass _sp_repr_xml_element_class = {
+	sizeof (SPRepr),
+	NULL,
+	repr_init,
+	repr_copy,
+	repr_finalize
+};
+
+SPReprClass _sp_repr_xml_text_class = {
+	sizeof (SPRepr),
+	NULL,
+	repr_init,
+	repr_copy,
+	repr_finalize
+};
+
+static SPRepr *sp_repr_new_from_code (SPReprClass *type, int code);
 static void sp_repr_remove_attribute (SPRepr *repr, SPReprAttr *attr);
-static void sp_node_remove_listener (SPXMLNode *node, SPListener *listener);
+static void sp_repr_remove_listener (SPRepr *repr, SPListener *listener);
 
-static SPXMLAttribute *sp_attribute_duplicate (const SPXMLAttribute *attr);
-static SPXMLAttribute *sp_attribute_new_from_quark (gint key, const guchar *value);
+static SPReprAttr *sp_attribute_duplicate (const SPReprAttr *attr);
+static SPReprAttr *sp_attribute_new_from_code (int key, const unsigned char *value);
 
-/* Helpers */
-static SPXMLNode * sp_node_alloc (void);
-static void sp_node_free (SPXMLNode *node);
-static SPXMLAttribute * sp_attribute_alloc (void);
-static void sp_attribute_free (SPXMLAttribute *attribute);
+static SPRepr * sp_repr_alloc (SPReprClass *type);
+static void sp_repr_free (SPRepr *repr);
+static SPReprAttr * sp_attribute_alloc (void);
+static void sp_attribute_free (SPReprAttr *attribute);
 static SPListener *sp_listener_alloc (void);
 static void sp_listener_free (SPListener *listener);
 
-SPRepr *
-sp_repr_new_from_code (gint code)
+static SPRepr *
+sp_repr_new_from_code (SPReprClass *type, int code)
 {
 	SPRepr * repr;
 
-	repr = sp_node_alloc ();
-
-	repr->refcount = 1;
+	repr = sp_repr_alloc (type);
 	repr->name = code;
-	repr->type = SP_XML_ELEMENT_NODE;
-	repr->parent = repr->next = repr->children = NULL;
-	repr->attributes = NULL;
-	repr->listeners = NULL;
-	repr->content = NULL;
+	repr->type->init (repr);
 
 	return repr;
 }
 
 SPRepr *
-sp_repr_new (const guchar *name)
+sp_repr_new (const unsigned char *name)
 {
 	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (*name != '\0', NULL);
 
-	return sp_repr_new_from_code (g_quark_from_string (name));
+	return sp_repr_new_from_code (SP_XML_ELEMENT_NODE, g_quark_from_string (name));
 }
 
 SPRepr *
-sp_repr_new_text (const guchar *content)
+sp_repr_new_text (const unsigned char *content)
 {
 	SPRepr * repr;
 	g_return_val_if_fail (content != NULL, NULL);
-	repr = sp_repr_new_from_code (g_quark_from_string ("text"));
+	repr = sp_repr_new_from_code (SP_XML_TEXT_NODE, g_quark_from_static_string ("text"));
 	repr->content = g_strdup (content);
 	repr->type = SP_XML_TEXT_NODE;
 	return repr;
 }
 
+static void
+repr_init (SPRepr *repr)
+{
+	repr->refcount = 1;
+	repr->doc = NULL;
+	repr->parent = repr->next = repr->children = NULL;
+	repr->attributes = NULL;
+	repr->listeners = NULL;
+	repr->content = NULL;
+}
+
+static void
+repr_doc_init (SPRepr *repr)
+{
+	SPReprDoc *doc=(SPReprDoc *)repr;
+
+	repr_init (repr);
+
+	repr->doc = doc;
+	doc->log = NULL;
+	doc->is_logging = 0;
+}
+
 SPRepr *
 sp_repr_ref (SPRepr *repr)
 {
@@ -90,29 +148,49 @@ sp_repr_unref (SPRepr *repr)
 	repr->refcount -= 1;
 
 	if (repr->refcount < 1) {
-		SPReprListener *rl;
-		/* Parents reference children */
-		g_assert (repr->parent == NULL);
-		g_assert (repr->next == NULL);
-		for (rl = repr->listeners; rl; rl = rl->next) {
-			if (rl->vector->destroy) (* rl->vector->destroy) (repr, rl->data);
-		}
-		while (repr->children) sp_repr_remove_child (repr, repr->children);
-		while (repr->attributes) sp_repr_remove_attribute (repr, repr->attributes);
-		if (repr->content) g_free (repr->content);
-		while (repr->listeners) sp_node_remove_listener (repr, repr->listeners);
-		sp_node_free (repr);
+		repr->type->finalize (repr);
+		sp_repr_free (repr);
 	}
 
 	return NULL;
 }
 
+static void
+repr_finalize (SPRepr *repr)
+{
+	SPReprListener *rl;
+	/* Parents reference children */
+	g_assert (repr->parent == NULL);
+	g_assert (repr->next == NULL);
+	repr->doc = NULL;
+
+	for (rl = repr->listeners; rl; rl = rl->next) {
+		if (rl->vector->destroy) (* rl->vector->destroy) (repr, rl->data);
+	}
+	while (repr->children) sp_repr_remove_child (repr, repr->children);
+	while (repr->attributes) sp_repr_remove_attribute (repr, repr->attributes);
+	if (repr->content) g_free (repr->content);
+	while (repr->listeners) sp_repr_remove_listener (repr, repr->listeners);
+}
+
+static void
+repr_doc_finalize (SPRepr *repr)
+{
+	SPReprDoc *doc = (SPReprDoc *)repr;
+	if (doc->log) {
+		sp_repr_free_log (doc->log);
+		doc->log = NULL;
+	}
+	repr_finalize (repr);
+}
+
 static SPRepr *
 sp_repr_attach (SPRepr *parent, SPRepr *child)
 {
 	g_assert (parent != NULL);
 	g_assert (child != NULL);
 	g_assert (child->parent == NULL);
+	g_assert (child->doc == NULL && parent->doc == NULL);
 
 	child->parent = parent;
 
@@ -123,38 +201,53 @@ SPRepr *
 sp_repr_duplicate (const SPRepr *repr)
 {
 	SPRepr *new;
-	SPRepr *child, *lastchild;
-	SPReprAttr *attr, *lastattr;
 
-	g_return_val_if_fail (repr != NULL, NULL);
+	new = sp_repr_new_from_code (repr->type, repr->name);
 
-	new = sp_repr_new_from_code (repr->name);
-	new->type = repr->type;
-
-	if (repr->content != NULL) new->content = g_strdup (repr->content);
-
-	lastchild = NULL;
-	for (child = repr->children; child != NULL; child = child->next) {
-		if (lastchild) {
-			lastchild = lastchild->next = sp_repr_attach (new, sp_repr_duplicate (child));
-		} else {
-			lastchild = new->children = sp_repr_attach (new, sp_repr_duplicate (child));
-		}
-	}
-
-	lastattr = NULL;
-	for (attr = repr->attributes; attr != NULL; attr = attr->next) {
-		if (lastattr) {
-			lastattr = lastattr->next = sp_attribute_duplicate (attr);
-		} else {
-			lastattr = new->attributes = sp_attribute_duplicate (attr);
-		}
-	}
+	repr->type->copy (new, repr);
 
 	return new;
 }
 
-const guchar *
+void
+repr_copy (SPRepr *to, const SPRepr *from)
+{
+	SPRepr *child, *lastchild;
+	SPReprAttr *attr, *lastattr;
+
+	g_return_if_fail (from != NULL);
+
+	if (from->content != NULL) to->content = g_strdup (from->content);
+
+	lastchild = NULL;
+	for (child = from->children; child != NULL; child = child->next) {
+		if (lastchild) {
+			lastchild = lastchild->next = sp_repr_attach (to, sp_repr_duplicate (child));
+		} else {
+			lastchild = to->children = sp_repr_attach (to, sp_repr_duplicate (child));
+		}
+	}
+
+	lastattr = NULL;
+	for (attr = from->attributes; attr != NULL; attr = attr->next) {
+		if (lastattr) {
+			lastattr = lastattr->next = sp_attribute_duplicate (attr);
+		} else {
+			lastattr = to->attributes = sp_attribute_duplicate (attr);
+		}
+	}
+}
+
+void
+repr_doc_copy (SPRepr *to, const SPRepr *from)
+{
+	SPReprDoc *to_doc=(SPReprDoc *)to;
+	repr_copy (to, from);
+	to_doc->log = NULL;
+	to_doc->is_logging = 0;
+}
+
+const unsigned char *
 sp_repr_name (const SPRepr *repr)
 {
 	g_return_val_if_fail (repr != NULL, NULL);
@@ -162,7 +255,7 @@ sp_repr_name (const SPRepr *repr)
 	return SP_REPR_NAME (repr);
 }
 
-const guchar *
+const unsigned char *
 sp_repr_content (const SPRepr *repr)
 {
 	g_assert (repr != NULL);
@@ -170,11 +263,11 @@ sp_repr_content (const SPRepr *repr)
 	return SP_REPR_CONTENT (repr);
 }
 
-const guchar *
-sp_repr_attr (const SPRepr *repr, const guchar *key)
+const unsigned char *
+sp_repr_attr (const SPRepr *repr, const unsigned char *key)
 {
 	SPReprAttr *ra;
-	GQuark q;
+	unsigned int q;
 
 	g_return_val_if_fail (repr != NULL, NULL);
 	g_return_val_if_fail (key != NULL, NULL);
@@ -186,12 +279,12 @@ sp_repr_attr (const SPRepr *repr, const guchar *key)
 	return NULL;
 }
 
-gboolean
-sp_repr_set_content (SPRepr *repr, const guchar *newcontent)
+unsigned int
+sp_repr_set_content (SPRepr *repr, const unsigned char *newcontent)
 {
 	SPReprListener *rl;
-	guchar *oldcontent;
-	gboolean allowed;
+	unsigned char *oldcontent;
+	unsigned int allowed;
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 
@@ -208,22 +301,27 @@ sp_repr_set_content (SPRepr *repr, const guchar *newcontent)
 		} else {
 			repr->content = NULL;
 		}
+		if ( repr->doc && repr->doc->is_logging ) {
+			repr->doc->log = sp_repr_log_chgcontent (repr->doc->log, repr, oldcontent, newcontent);
+		}
 		for (rl = repr->listeners; rl != NULL; rl = rl->next) {
 			if (rl->vector->content_changed) (* rl->vector->content_changed) (repr, oldcontent, newcontent, rl->data);
 		}
-		if (oldcontent) g_free (oldcontent);
+		if ( oldcontent && ( !repr->doc || !repr->doc->is_logging ) ) {
+			g_free (oldcontent);
+		}
 	}
 
 	return allowed;
 }
 
-static gboolean
-sp_repr_del_attr (SPRepr *repr, const guchar *key)
+static unsigned int
+sp_repr_del_attr (SPRepr *repr, const unsigned char *key)
 {
 	SPReprAttr *prev, *attr;
 	SPReprListener *rl;
-	gboolean allowed;
-	GQuark q;
+	unsigned int allowed;
+	unsigned int q;
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -236,7 +334,7 @@ sp_repr_del_attr (SPRepr *repr, const guchar *key)
 	for (attr = repr->attributes; attr && (attr->key != q); attr = attr->next) prev = attr;
 
 	if (attr) {
-		guchar *oldval;
+		unsigned char *oldval;
 
 		oldval = attr->value;
 
@@ -245,11 +343,21 @@ sp_repr_del_attr (SPRepr *repr, const guchar *key)
 		}
 
 		if (allowed) {
-			(prev) ? prev->next : repr->attributes = attr->next;
+			if (prev) {
+				prev->next = attr->next;
+			} else {
+				repr->attributes = attr->next;
+			}
 
+			if ( repr->doc && repr->doc->is_logging ) {
+				repr->doc->log = sp_repr_log_chgattr (repr->doc->log, repr, q, oldval, NULL);
+			}
 			for (rl = repr->listeners; rl != NULL; rl = rl->next) {
 				if (rl->vector->attr_changed) (* rl->vector->attr_changed) (repr, key, oldval, NULL, rl->data);
 			}
+			if ( repr->doc && repr->doc->is_logging ) {
+				attr->value = NULL;
+			}
 
 			sp_attribute_free (attr);
 		}
@@ -258,14 +366,14 @@ sp_repr_del_attr (SPRepr *repr, const guchar *key)
 	return allowed;
 }
 
-static gboolean
-sp_repr_chg_attr (SPRepr *repr, const guchar *key, const guchar *value)
+static unsigned int
+sp_repr_chg_attr (SPRepr *repr, const unsigned char *key, const unsigned char *value)
 {
 	SPReprAttr *prev, *attr;
 	SPReprListener *rl;
-	gboolean allowed;
-	gchar *oldval;
-	GQuark q;
+	unsigned int allowed;
+	char *oldval;
+	unsigned int q;
 
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -289,23 +397,32 @@ sp_repr_chg_attr (SPRepr *repr, const guchar *key, const guchar *value)
 		if (attr) {
 			attr->value = g_strdup (value);
 		} else {
-			attr = sp_attribute_new_from_quark (q, value);
+			attr = sp_attribute_new_from_code (q, value);
+			if (prev) {
+				prev->next = attr;
+			} else {
+				repr->attributes = attr;
+			}
+		}
 
-			(prev) ? prev->next : repr->attributes = attr;
+		if ( repr->doc && repr->doc->is_logging ) {
+			repr->doc->log = sp_repr_log_chgattr (repr->doc->log, repr, q, oldval, value);
 		}
 
 		for (rl = repr->listeners; rl != NULL; rl = rl->next) {
 			if (rl->vector->attr_changed) (* rl->vector->attr_changed) (repr, key, oldval, value, rl->data);
 		}
 
-		if (oldval) g_free (oldval);
+		if ( oldval && ( !repr->doc || !repr->doc->is_logging ) ) {
+			g_free (oldval);
+		}
 	}
 
 	return allowed;
 }
 
-gboolean
-sp_repr_set_attr (SPRepr *repr, const guchar *key, const guchar *value)
+unsigned int
+sp_repr_set_attr (SPRepr *repr, const unsigned char *key, const unsigned char *value)
 {
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -347,16 +464,17 @@ sp_repr_parent (SPRepr * repr)
 	return repr->parent;
 }
 
-gboolean
+unsigned int
 sp_repr_add_child (SPRepr *repr, SPRepr *child, SPRepr *ref)
 {
 	SPReprListener * rl;
-	gboolean allowed;
+	unsigned int allowed;
 
 	g_assert (repr != NULL);
 	g_assert (child != NULL);
 	g_assert (!ref || ref->parent == repr);
 	g_assert (child->parent == NULL);
+	g_assert (child->doc == NULL || child->doc == repr->doc);
 
 	allowed = TRUE;
 	for (rl = repr->listeners; rl != NULL; rl = rl->next) {
@@ -375,6 +493,12 @@ sp_repr_add_child (SPRepr *repr, SPRepr *child, SPRepr *ref)
 		child->parent = repr;
 		sp_repr_ref (child);
 
+		if (child->doc == NULL) bind_document (repr->doc, child);
+
+		if ( repr->doc && repr->doc->is_logging ) {
+			repr->doc->log = sp_repr_log_add (repr->doc->log, repr, child, ref);
+		}
+
 		for (rl = repr->listeners; rl != NULL; rl = rl->next) {
 			if (rl->vector->child_added) (* rl->vector->child_added) (repr, child, ref, rl->data);
 		}
@@ -383,12 +507,26 @@ sp_repr_add_child (SPRepr *repr, SPRepr *child, SPRepr *ref)
 	return allowed;
 }
 
-gboolean
+static void
+bind_document (SPReprDoc *doc, SPRepr *repr)
+{
+	SPRepr *child;
+
+	g_assert (repr->doc == NULL);
+
+	repr->doc = doc;
+
+	for ( child = repr->children ; child != NULL ; child = child->next ) {
+		bind_document (doc, child);
+	}
+}
+
+unsigned int
 sp_repr_remove_child (SPRepr *repr, SPRepr *child)
 {
 	SPReprListener *rl;
 	SPRepr *ref;
-	gboolean allowed;
+	unsigned int allowed;
 
 	g_assert (repr != NULL);
 	g_assert (child != NULL);
@@ -406,26 +544,39 @@ sp_repr_remove_child (SPRepr *repr, SPRepr *child)
 	}
 
 	if (allowed) {
-		(ref) ? ref->next : repr->children = child->next;
+		if (ref) {
+			ref->next = child->next;
+		} else {
+			repr->children = child->next;
+		}
 		child->parent = NULL;
 		child->next = NULL;
 
+		if ( repr->doc && repr->doc->is_logging ) {
+			repr->doc->log = sp_repr_log_remove (repr->doc->log, repr, child, ref);
+		}
+
 		for (rl = repr->listeners; rl != NULL; rl = rl->next) {
 			if (rl->vector->child_removed) (* rl->vector->child_removed) (repr, child, ref, rl->data);
 		}
 
+#ifdef REPR_VERBOSE
+		if (child->refcount > 1) {
+			g_warning ("Detaching repr with refcount > 1");
+		}
+#endif
 		sp_repr_unref (child);
 	}
 
 	return allowed;
 }
 
-gboolean
+unsigned int
 sp_repr_change_order (SPRepr *repr, SPRepr *child, SPRepr *ref)
 {
 	SPRepr * prev;
 	SPReprListener * rl;
-	gint allowed;
+	int allowed;
 
 	prev = NULL;
 	if (child != repr->children) {
@@ -454,6 +605,10 @@ sp_repr_change_order (SPRepr *repr, SPRepr *child, SPRepr *ref)
 			repr->children = child;
 		}
 
+		if ( repr->doc && repr->doc->is_logging ) {
+			repr->doc->log = sp_repr_log_chgorder (repr->doc->log, repr, child, prev, ref);
+		}
+
 		for (rl = repr->listeners; rl != NULL; rl = rl->next) {
 			if (rl->vector->order_changed) (* rl->vector->order_changed) (repr, child, prev, ref, rl->data);
 		}
@@ -463,14 +618,14 @@ sp_repr_change_order (SPRepr *repr, SPRepr *child, SPRepr *ref)
 }
 
 void
-sp_repr_set_position_absolute (SPRepr * repr, gint pos)
+sp_repr_set_position_absolute (SPRepr * repr, int pos)
 {
 	SPRepr * parent;
 	SPRepr * ref, * cur;
 
 	parent = repr->parent;
 
-	if (pos < 0) pos = G_MAXINT;
+	if (pos < 0) pos = 0x7fffffff;
 
 	ref = NULL;
 	cur = parent->children;
@@ -489,7 +644,7 @@ sp_repr_set_position_absolute (SPRepr * repr, gint pos)
 }
 
 void
-sp_repr_synthesize_events (SPRepr *repr, const SPReprEventVector *vector, gpointer data)
+sp_repr_synthesize_events (SPRepr *repr, const SPReprEventVector *vector, void * data)
 {
 	
 	if (vector->attr_changed) {
@@ -512,7 +667,7 @@ sp_repr_synthesize_events (SPRepr *repr, const SPReprEventVector *vector, gpoint
 }
 
 void
-sp_repr_add_listener (SPRepr *repr, const SPReprEventVector *vector, gpointer data)
+sp_repr_add_listener (SPRepr *repr, const SPReprEventVector *vector, void * data)
 {
 	SPReprListener *rl, *last;
 
@@ -538,13 +693,13 @@ sp_repr_add_listener (SPRepr *repr, const SPReprEventVector *vector, gpointer da
 }
 
 static void
-sp_node_remove_listener (SPXMLNode *node, SPListener *listener)
+sp_repr_remove_listener (SPRepr *repr, SPListener *listener)
 {
-	if (listener == node->listeners) {
-		node->listeners = listener->next;
+	if (listener == repr->listeners) {
+		repr->listeners = listener->next;
 	} else {
 		SPListener *prev;
-		prev = node->listeners;
+		prev = repr->listeners;
 		while (prev->next != listener) prev = prev->next;
 		prev->next = listener->next;
 	}
@@ -553,7 +708,7 @@ sp_node_remove_listener (SPXMLNode *node, SPListener *listener)
 }
 
 void
-sp_repr_remove_listener_by_data (SPRepr * repr, gpointer data)
+sp_repr_remove_listener_by_data (SPRepr *repr, void *data)
 {
 	SPReprListener * last, * rl;
 
@@ -573,7 +728,7 @@ sp_repr_remove_listener_by_data (SPRepr * repr, gpointer data)
 }
 
 SPRepr *
-sp_repr_nth_child (const SPRepr * repr, gint n)
+sp_repr_nth_child (const SPRepr * repr, int n)
 {
 	SPRepr * child;
 
@@ -591,29 +746,28 @@ sp_repr_nth_child (const SPRepr * repr, gint n)
 /* fixme: Do this somewhere, somehow The Right Way (TM) */
 
 SPReprDoc *
-sp_repr_document_new (const gchar * rootname)
+sp_repr_document_new (const char *rootname)
 {
-	SPRepr * repr, * root;
+	SPReprDoc * doc;
+	SPRepr * root;
 
-	repr = sp_repr_new ("xml");
+	doc = (SPReprDoc *)sp_repr_new_from_code (SP_XML_DOCUMENT_NODE, g_quark_from_static_string ("xml"));
 	if (!strcmp (rootname, "svg")) {
-		sp_repr_set_attr (repr, "version", "1.0");
-		sp_repr_set_attr (repr, "standalone", "no");
-		sp_repr_set_attr (repr, "doctype",
-				  "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\"\n"
-				  "\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\"\n"
-				  "[\n"
-				  " <!ATTLIST svg\n"
-				  "  xmlns:xlink CDATA #FIXED \"http://www.w3.org/1999/xlink\">\n"
-				  "]>\n");
-		sp_repr_set_attr (repr, "comment",
+		sp_repr_set_attr (&doc->repr, "version", "1.0");
+		sp_repr_set_attr (&doc->repr, "standalone", "no");
+		sp_repr_set_attr (&doc->repr, "doctype",
+				  "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\"\n"
+				  "\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">\n");
+		sp_repr_set_attr (&doc->repr, "comment",
 				  "<!-- Created with Sodipodi (\"http://www.sodipodi.com/\") -->\n");
 	}
 
 	root = sp_repr_new (rootname);
-	sp_repr_add_child (repr, root, 0);
 
-	return (SPReprDoc *) repr;
+	sp_repr_add_child (&doc->repr, root, 0);
+	sp_repr_unref (root);
+
+	return doc;
 }
 
 void
@@ -629,39 +783,28 @@ sp_repr_document_unref (SPReprDoc * doc)
 }
 
 void
-sp_repr_document_set_root (SPReprDoc * doc, SPRepr * repr)
+sp_repr_document_set_root (SPReprDoc *doc, SPRepr *repr)
 {
-	SPRepr * rdoc;
-
 	g_assert (doc != NULL);
 	g_assert (repr != NULL);
+	g_assert (doc->repr.children != NULL);
+	g_assert (repr->doc == NULL || repr->doc == doc);
 
-	rdoc = (SPRepr *) doc;
-
-	g_assert (rdoc->children != NULL);
-
-	sp_repr_remove_child (rdoc, rdoc->children);
-	sp_repr_add_child (rdoc, repr, NULL);
+	sp_repr_remove_child (&doc->repr, doc->repr.children);
+	sp_repr_add_child (&doc->repr, repr, NULL);
 }
 
 SPReprDoc *
 sp_repr_document (const SPRepr *repr)
 {
-	/* fixme: */
-	while (repr->parent) repr = repr->parent;
-
-	return (SPReprDoc *) repr;
+	return repr->doc;
 }
 
 SPRepr *
 sp_repr_document_root (const SPReprDoc *doc)
 {
-	SPRepr * repr;
-
-	repr = (SPRepr *) doc;
-	g_return_val_if_fail (repr->children != NULL, NULL);
-
-	return repr->children;
+	g_assert (doc != NULL);
+	return doc->repr.children;
 }
 
 /*
@@ -669,12 +812,12 @@ sp_repr_document_root (const SPReprDoc *doc)
  * Does NOT erase original attributes and children
  */
 
-gboolean
-sp_repr_document_merge (SPReprDoc *doc, const SPReprDoc *src, const guchar *key)
+unsigned int
+sp_repr_document_merge (SPReprDoc *doc, const SPReprDoc *src, const unsigned char *key)
 {
 	SPRepr *rdoc;
 	const SPRepr *rsrc;
-	
+
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (src != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
@@ -690,8 +833,8 @@ sp_repr_document_merge (SPReprDoc *doc, const SPReprDoc *src, const guchar *key)
  * Does NOT erase original attributes and children
  */
 
-gboolean
-sp_repr_merge (SPRepr *repr, const SPRepr *src, const guchar *key)
+unsigned int
+sp_repr_merge (SPRepr *repr, const SPRepr *src, const unsigned char *key)
 {
 	SPRepr *child;
 	SPReprAttr *attr;
@@ -710,7 +853,7 @@ sp_repr_merge (SPRepr *repr, const SPRepr *src, const guchar *key)
 	
 	for (child = src->children; child != NULL; child = child->next) {
 		SPRepr *rch;
-		const guchar *id;
+		const unsigned char *id;
 		id = sp_repr_attr (child, key);
 		if (id) {
 			rch = sp_repr_lookup_child (repr, key, id);
@@ -733,10 +876,10 @@ sp_repr_merge (SPRepr *repr, const SPRepr *src, const guchar *key)
 	return TRUE;
 }
 
-static SPXMLAttribute *
-sp_attribute_duplicate (const SPXMLAttribute *attr)
+static SPReprAttr *
+sp_attribute_duplicate (const SPReprAttr *attr)
 {
-	SPXMLAttribute *new;
+	SPReprAttr *new;
 
 	new = sp_attribute_alloc ();
 
@@ -747,10 +890,10 @@ sp_attribute_duplicate (const SPXMLAttribute *attr)
 	return new;
 }
 
-static SPXMLAttribute *
-sp_attribute_new_from_quark (gint key, const guchar *value)
+static SPReprAttr *
+sp_attribute_new_from_code (int key, const unsigned char *value)
 {
-	SPXMLAttribute *new;
+	SPReprAttr *new;
 
 	new = sp_attribute_alloc ();
 
@@ -761,50 +904,62 @@ sp_attribute_new_from_quark (gint key, const guchar *value)
 	return new;
 }
 
-/* Helpers */
-#define SP_NODE_ALLOC_SIZE 32
-static SPXMLNode *free_node = NULL;
+#define SP_REPR_CHUNK_SIZE 32
 
-static SPXMLNode *
-sp_node_alloc (void)
+static SPRepr *
+sp_repr_alloc (SPReprClass *type)
 {
-	SPXMLNode *node;
-	gint i;
+	SPRepr *repr;
 
-	if (free_node) {
-		node = free_node;
-		free_node = free_node->next;
+	if (type->pool) {
+		repr = type->pool;
+		type->pool = repr->next;
 	} else {
-		node = g_new (SPXMLNode, SP_NODE_ALLOC_SIZE);
-		for (i = 1; i < SP_NODE_ALLOC_SIZE - 1; i++) node[i].next = node + i + 1;
-		node[SP_NODE_ALLOC_SIZE - 1].next = NULL;
-		free_node = node + 1;
-	}
+		char *chunk;
+		size_t max, offset;
 
-	return node;
+		chunk = (char *)g_malloc (type->size * SP_REPR_CHUNK_SIZE);
+
+		max = type->size * ( SP_REPR_CHUNK_SIZE - 1 );
+		for ( offset = type->size ;
+		      offset < max ;
+		      offset += type->size )
+		{
+			repr = (SPRepr *)(chunk+offset);
+			repr->next = (SPRepr *)(chunk+offset+type->size);
+		}
+
+		((SPRepr *)(chunk+offset))->next = NULL;
+		type->pool = (SPRepr *)(chunk+type->size);
+
+		repr = (SPRepr *)chunk;
+	}
+	repr->type = type;
+
+	return repr;
 }
 
 static void
-sp_node_free (SPXMLNode *node)
+sp_repr_free (SPRepr *repr)
 {
-	node->next = free_node;
-	free_node = node;
+	repr->next = repr->type->pool;
+	repr->type->pool = repr;
 }
 
 #define SP_ATTRIBUTE_ALLOC_SIZE 32
-static SPXMLAttribute *free_attribute = NULL;
+static SPReprAttr *free_attribute = NULL;
 
-static SPXMLAttribute *
+static SPReprAttr *
 sp_attribute_alloc (void)
 {
-	SPXMLAttribute *attribute;
-	gint i;
+	SPReprAttr *attribute;
+	int i;
 
 	if (free_attribute) {
 		attribute = free_attribute;
 		free_attribute = free_attribute->next;
 	} else {
-		attribute = g_new (SPXMLAttribute, SP_ATTRIBUTE_ALLOC_SIZE);
+		attribute = g_new (SPReprAttr, SP_ATTRIBUTE_ALLOC_SIZE);
 		for (i = 1; i < SP_ATTRIBUTE_ALLOC_SIZE - 1; i++) attribute[i].next = attribute + i + 1;
 		attribute[SP_ATTRIBUTE_ALLOC_SIZE - 1].next = NULL;
 		free_attribute = attribute + 1;
@@ -814,7 +969,7 @@ sp_attribute_alloc (void)
 }
 
 static void
-sp_attribute_free (SPXMLAttribute *attribute)
+sp_attribute_free (SPReprAttr *attribute)
 {
 	attribute->next = free_attribute;
 	free_attribute = attribute;
@@ -827,7 +982,7 @@ static SPListener *
 sp_listener_alloc (void)
 {
 	SPListener *listener;
-	gint i;
+	int i;
 
 	if (free_listener) {
 		listener = free_listener;
